@@ -23,7 +23,6 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: funcs.h,v 1.16 2004/02/06 00:58:35 taniwha Exp $
 */
 
 #ifndef __QF_GL_funcs_h_
@@ -43,13 +42,8 @@
 #undef QFGL_NEED
 #undef QFGL_WANT
 
-extern void *libgl_handle;
-
 qboolean GLF_Init (void);
 qboolean GLF_FindFunctions (void);
 void *QFGL_ProcAddress (void *handle, const char *name, qboolean);
 
-void *QFGL_LoadLibrary (void); 
-void *QFGL_GetProcAddress (void *handle, const char *name);
-
 #endif // __QF_GL_funcs_h_
