@@ -25,10 +25,7 @@
 		Free Software Foundation, Inc.
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
-
-	$Id: image.h,v 1.4 2003/09/04 18:59:36 taniwha Exp $
 */
-
 #ifndef __QF_image_h
 #define __QF_image_h
 
@@ -44,12 +41,12 @@ typedef struct tex_s {
 } tex_t;
 
 #define tex_palette 0
-#define tex_l GL_LUMINANCE
-#define tex_a GL_ALPHA
+#define tex_l 0x1909 //GL_LUMINANCE
+#define tex_a 0x1906 //GL_ALPHA
 #define tex_la 2
 #define tex_rgb 3
 #define tex_rgba 4
 
 tex_t *LoadImage (const char *imageFile);
 
-#endif//__QF_image_h
+#endif //__QF_image_h
