@@ -24,15 +24,21 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: va.h,v 1.6 2003/01/15 15:31:17 taniwha Exp $
 */
 
 #ifndef __va_h
 #define __va_h
 
+/** \addtogroup misc
+	Formatted printing.
+*/
+//@{
+
 // does a varargs printf into a temp buffer
 char	*va(const char *format, ...) __attribute__((format(printf,1,2)));
 // does a varargs printf into a malloced buffer
 char	*nva(const char *format, ...) __attribute__((format(printf,1,2)));
 
+//@}
+
 #endif // __va_h
