@@ -23,7 +23,6 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: net_loop.h,v 1.1 2003/02/11 22:48:56 taniwha Exp $
 */
 
 #ifndef __net_loop_h
@@ -32,6 +31,11 @@
 #include "QF/qtypes.h"
 #include "netmain.h"
 
+/** \defgroup nq-loop NetQuake loopback network driver.
+	\ingroup nq-nd
+*/
+//@{
+
 int			Loop_Init (void);
 void		Loop_Listen (qboolean state);
 void		Loop_SearchForHosts (qboolean xmit);
@@ -45,4 +49,6 @@ qboolean	Loop_CanSendUnreliableMessage (qsocket_t *sock);
 void		Loop_Close (qsocket_t *sock);
 void		Loop_Shutdown (void);
 
+//@}
+
 #endif//__net_loop_h
