@@ -23,10 +23,14 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: net_vcr.h,v 1.1 2003/02/11 22:48:56 taniwha Exp $
 */
 
 
+/** \defgroup nq-vcr NetQuake VCR network driver.
+	\ingroup nq-nd
+*/
+//@{
+
 #define VCR_OP_CONNECT					1
 #define VCR_OP_GETMESSAGE				2
 #define VCR_OP_SENDMESSAGE				3
@@ -43,3 +47,5 @@ int			VCR_SendMessage (qsocket_t *sock, sizebuf_t *data);
 qboolean	VCR_CanSendMessage (qsocket_t *sock);
 void		VCR_Close (qsocket_t *sock);
 void		VCR_Shutdown (void);
+
+//@}
