@@ -23,16 +23,15 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: msg_ucmd.h,v 1.1 2004/02/21 05:09:00 taniwha Exp $
 */
 #ifndef __msg_ucmd_h
 #define __msg_ucmd_h
 
-#include "QF/sizebuf.h"
-
 extern struct usercmd_s nullcmd;
+struct msg_s;
+struct sizebuf_s;
 
-void MSG_WriteDeltaUsercmd (sizebuf_t *sb, struct usercmd_s *from, struct usercmd_s *cmd);
-void MSG_ReadDeltaUsercmd (struct usercmd_s *from, struct usercmd_s *cmd);
+void MSG_WriteDeltaUsercmd (struct sizebuf_s *sb, struct usercmd_s *from, struct usercmd_s *cmd);
+void MSG_ReadDeltaUsercmd (struct msg_s *msg, struct usercmd_s *from, struct usercmd_s *cmd);
 
 #endif // __msg_ucmd_h
