@@ -23,10 +23,9 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: sbar.h,v 1.8 2004/03/02 03:55:16 taniwha Exp $
 */
 
-// the status bar is only redrawn if something has changed, but if anything
+// the status bar is redrawn only if something has changed, but if anything
 // does, the entire thing will be redrawn for the next vid.numpages frames.
 
 #ifndef _SBAR_H
