@@ -23,16 +23,11 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: varrays.h,v 1.7 2003/03/20 19:58:17 despair Exp $
 */
 
 #ifndef __qf_varrays_h
 #define __qf_varrays_h
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 #ifdef HAVE_WINDOWS_H
 # include <windows.h>
 #endif
@@ -55,24 +50,6 @@ typedef struct varray_t2f_c4f_n3f_v3f_s {
 extern qboolean gl_va_capable;
 extern int vaelements;
 
-//extern varray_t2f_c4f_n3f_v3f_t *modelVertexArray
-//extern int mVAsize;
-
-extern varray_t2f_c4ub_v3f_t *particleVertexArray;
-extern int pVAsize;
-
-extern float *textCoords;
-extern float *textVertices;
-extern int tVAsize;
-
-extern varray_t2f_c4ub_v3f_t *spriteVertexArray;
-extern int sVAsize;
-
-/*
-extern varray_t2f_c4ub_v3f_t *polyVertexArray;
-extern float *polyCoords;
-extern float *polyVertices
-extern int polyVAsize;
-*/
+extern varray_t2f_c4ub_v3f_t *gl_spriteVertexArray;
 
 #endif // __qf_varrays_h
