@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: client.c,v 1.71 2004/03/02 03:55:17 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -54,7 +51,6 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/gib.h"
 #include "QF/input.h"
 #include "QF/keys.h"
-#include "QF/plugin.h"
 #include "QF/qargs.h"
 #include "QF/quakefs.h"
 #include "QF/render.h"
@@ -64,14 +60,11 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/vid.h"
 #include "QF/view.h"
 
-#include "compat.h"
+#include "QF/plugin/general.h"
+#include "QF/plugin/console.h"
+#include "QF/plugin/vid_render.h"
 
-// XXX check InputLine.h in ruamoko/include
-typedef struct {
-	int         x, y;
-	int         xbase, ybase;
-	int         cursor;
-} il_data_t;
+#include "compat.h"
 
 static general_data_t plugin_info_general_data;
 console_data_t con_data;
@@ -86,7 +79,7 @@ static cvar_t *con_notifytime;				// seconds
 static cvar_t *con_alpha;
 static cvar_t *con_size;
 static cvar_t *con_speed;
-static cvar_t *cl_chatmode;
+static cvar_t *cl_conmode;
 
 #define	NUM_CON_TIMES 4
 static float con_times[NUM_CON_TIMES];	// realtime time the line was generated
@@ -110,6 +103,7 @@ static view_t *hud_view;
 
 static qboolean con_initialized;
 
+static keydest_t con_keydest;
 
 static void
 ClearNotify (void)
@@ -126,12 +120,10 @@ ToggleConsole_f (void)
 {
 	Con_ClearTyping (input_line, 0);
 
-	if (key_dest == key_console && !con_data.force_commandline) {
-		key_dest = key_game;
-		game_target = IMT_0;
+	if (con_keydest == key_console && !con_data.force_commandline) {
+		Key_SetKeyDest (key_game);
 	} else {
-		key_dest = key_console;
-		game_target = IMT_CONSOLE;
+		Key_SetKeyDest (key_console);
 	}
 
 	ClearNotify ();
@@ -142,12 +134,10 @@ ToggleChat_f (void)
 {
 	Con_ClearTyping (input_line, 0);
 
-	if (key_dest == key_console && !con_data.force_commandline) {
-		key_dest = key_game;
-		game_target = IMT_0;
+	if (con_keydest == key_console && !con_data.force_commandline) {
+		Key_SetKeyDest (key_game);
 	} else {
-		key_dest = key_console;
-		game_target = IMT_CONSOLE;
+		Key_SetKeyDest (key_console);
 	}
 
 	ClearNotify ();
@@ -169,8 +159,7 @@ MessageMode_f (void)
 	if (con_data.force_commandline)
 		return;
 	chat_team = false;
-	key_dest = key_message;
-	game_target = IMT_CONSOLE;
+	Key_SetKeyDest (key_message);
 }
 
 static void
@@ -179,8 +168,7 @@ MessageMode2_f (void)
 	if (con_data.force_commandline)
 		return;
 	chat_team = true;
-	key_dest = key_message;
-	game_target = IMT_CONSOLE;
+	Key_SetKeyDest (key_message);
 }
 
 static void
@@ -189,7 +177,7 @@ Resize (old_console_t *con)
 	char		tbuf[CON_TEXTSIZE];
 	int			width, oldwidth, oldtotallines, numlines, numchars, i, j;
 
-	width = (vid.width >> 3) - 2;
+	width = (r_data->vid->conwidth >> 3) - 2;
 
 	if (width < 1) {					// video hasn't been initialized yet
 		width = 38;
@@ -227,7 +215,7 @@ Resize (old_console_t *con)
 	say_team_line->width = con_linewidth - 9;
 	say_line->width = con_linewidth - 4;
 	input_line->width = con_linewidth;
-							
+
 	con->current = con_totallines - 1;
 	con->display = con->current;
 }
@@ -243,7 +231,7 @@ C_CheckResize (void)
 	Resize (&con_main);
 	Resize (&con_chat);
 
-	view_resize (con_data.view, vid.conwidth, vid.conheight);
+	view_resize (con_data.view, r_data->vid->conwidth, r_data->vid->conheight);
 }
 
 static void
@@ -252,22 +240,21 @@ Condump_f (void)
 	int         line = con->current - con->numlines;
 	const char *start, *end;
 	QFile      *file;
-	char        name[MAX_OSPATH];
+	char       *name;
 
 	if (Cmd_Argc () != 2) {
-		Con_Printf ("usage: condump <filename>\n");
+		Sys_Printf ("usage: condump <filename>\n");
 		return;
 	}
 
 	if (strchr (Cmd_Argv (1), '/') || strchr (Cmd_Argv (1), '\\')) {
-		Con_Printf ("invalid character in filename\n");
+		Sys_Printf ("invalid character in filename\n");
 		return;
 	}
-	snprintf (name, sizeof (name), "%s/%s.txt", qfs_gamedir->dir.def,
-			  Cmd_Argv (1));
+	name = va ("%s/%s.txt", qfs_gamedir->dir.def, Cmd_Argv (1));
 
 	if (!(file = QFS_WOpen (name, 0))) {
-		Con_Printf ("could not open %s for writing: %s\n", name,
+		Sys_Printf ("could not open %s for writing: %s\n", name,
 					strerror (errno));
 		return;
 	}
@@ -284,74 +271,73 @@ Condump_f (void)
 	Qclose (file);
 }
 
-static qboolean
-CheckForCommand (const char *line)
+static int
+cl_exec_line_command (void *data, const char *line)
 {
-	char        command[128];
-	const char *cmd;
-	int         i;
-
-	for (i = 0; i < 127; i++) {
-		if (line[i] <= ' ')
-			break;
-		else
-			command[i] = line[i];
-	}
-	command[i] = 0;
-	
-	cmd = Cmd_CompleteCommand (command);
-	if (!cmd || strcmp (cmd, command))
-		cmd = Cvar_CompleteVariable (command);
-	if (!cmd || strcmp (cmd, command))
-		return false;					// just a chat message
-	return true;
-}
-
-static void
-C_ExecLine (const char *line)
-{
-	if (line[0] == '/' && line [1] == '/')
-		goto no_lf;
-	else if (line[0] == '|')
-		Cbuf_AddText (con_data.cbuf, line);
-	else if (line[0] == '\\' || line[0] == '/')
-		Cbuf_AddText (con_data.cbuf, line + 1);
-	else if (cl_chatmode->int_val != 1 && CheckForCommand (line))
-		Cbuf_AddText (con_data.cbuf, line);
-	else if (cl_chatmode->int_val) {
-		Cbuf_AddText (con_data.cbuf, "say ");
-		Cbuf_AddText (con_data.cbuf, line);
-	} else
-		Cbuf_AddText (con_data.cbuf, line);
+	Cbuf_AddText (con_data.cbuf, line);
 	Cbuf_AddText (con_data.cbuf, "\n");
-  no_lf:
-	Con_Printf ("%s\n", line);
+	return 1;
+}
+
+static int
+cl_exec_line_chat (void *data, const char *line)
+{
+	Cbuf_AddText (con_data.cbuf, "say ");
+	Cbuf_AddText (con_data.cbuf, line);
+	Cbuf_AddText (con_data.cbuf, "\n");
+	return 0;
+}
+
+static int
+cl_exec_line_rcon (void *data, const char *line)
+{
+	Cbuf_AddText (con_data.cbuf, "rcon ");
+	Cbuf_AddText (con_data.cbuf, line);
+	Cbuf_AddText (con_data.cbuf, "\n");
+	Sys_Printf ("rcon %s\n", line);
+	return 0;
 }
 
 static void
-C_Say (const char *line)
+cl_conmode_f (cvar_t *var)
 {
+	if (!strcmp (var->string, "command")) {
+		con_data.exec_line = cl_exec_line_command;
+	} else if (!strcmp (var->string, "chat")) {
+		con_data.exec_line = cl_exec_line_chat;
+	} else if (!strcmp (var->string, "rcon")) {
+		con_data.exec_line = cl_exec_line_rcon;
+	} else {
+		Sys_Printf ("mode must be one of \"command\", \"chat\" or \"rcon\"\n");
+		Sys_Printf ("    forcing \"command\"\n");
+		Cvar_Set (var, "command");
+	}
+}
+
+static void
+C_Say (inputline_t *il)
+{
+	const char *line = il->line;
 	if (!*line)
 		return;
 
 	Cbuf_AddText (con_data.cbuf, "say \"");
 	Cbuf_AddText (con_data.cbuf, line);
 	Cbuf_AddText (con_data.cbuf, "\"\n");
-	key_dest = key_game;
-	game_target = IMT_0;
+	Key_SetKeyDest (key_game);
 }
 
 static void
-C_SayTeam (const char *line)
+C_SayTeam (inputline_t *il)
 {
+	const char *line = il->line;
 	if (!*line)
 		return;
 
 	Cbuf_AddText (con_data.cbuf, "say_team \"");
 	Cbuf_AddText (con_data.cbuf, line);
 	Cbuf_AddText (con_data.cbuf, "\"\n");
-	key_dest = key_game;
-	game_target = IMT_0;
+	Key_SetKeyDest (key_game);
 }
 
 static void
@@ -423,7 +409,7 @@ C_Print (const char *fmt, va_list args)
 		if (!con->x) {
 			Linefeed ();
 			// mark time for transparent overlay
-			if (con->current >= 0)
+			if (con->current >= 0 && con_data.realtime)
 				con_times[con->current % NUM_CON_TIMES] = *con_data.realtime;
 		}
 
@@ -463,39 +449,45 @@ C_KeyEvent (knum_t key, short unicode, qboolean down)
 	if (!down)
 		return;
 
-	if (down && (key == QFK_ESCAPE || unicode == '\x1b')) {
-		switch (key_dest) {
-			case key_menu:
-				Menu_Leave ();
-				return;
-			case key_message:
-				if (chat_team) {
-					Con_ClearTyping (say_team_line, 1);
-				} else {
-					Con_ClearTyping (say_line, 1);
-				}
-				key_dest = key_game;
-				game_target = IMT_0;
-				return;
-			case key_console:
-				if (!con_data.force_commandline) {
-					Cbuf_AddText (con_data.cbuf, "toggleconsole\n");
+	if (con_keydest == key_menu) {
+		if (Menu_KeyEvent (key, unicode, down))
+			return;
+	}
+
+	if (down) {
+		if (key == key_togglemenu) {
+			switch (con_keydest) {
+				case key_menu:
+					Menu_Leave ();
 					return;
-				}
-			case key_game:
-				Menu_Enter ();
-				return;
-			default:
-				Sys_Error ("Bad key_dest");
+				case key_message:
+					if (chat_team) {
+						Con_ClearTyping (say_team_line, 1);
+					} else {
+						Con_ClearTyping (say_line, 1);
+					}
+					Key_SetKeyDest (key_game);
+					return;
+				case key_console:
+					if (!con_data.force_commandline) {
+						Cbuf_AddText (con_data.cbuf, "toggleconsole\n");
+						return;
+					}
+				case key_game:
+					Menu_Enter ();
+					return;
+				default:
+					Sys_Error ("Bad con_keydest");
+			}
+		} else if (key == key_toggleconsole) {
+			ToggleConsole_f ();
+			return;
 		}
 	}
 
-	if (key_dest == key_menu) {
-		Menu_KeyEvent (key, unicode, down);
+	if (con_keydest == key_menu) {
 		return;
-	}
-
-	if (key_dest == key_message) {
+	} else if (con_keydest == key_message) {
 		if (chat_team) {
 			il = say_team_line;
 		} else {
@@ -556,32 +548,30 @@ DrawInputLine (int x, int y, int cursor, inputline_t *il)
 	const char *s = il->lines[il->edit_line] + il->scroll;
 
 	if (il->scroll) {
-		Draw_Character (x, y, '<' | 0x80);
-		Draw_nString (x + 8, y, s + 1, il->width - 2);
+		r_funcs->Draw_Character (x, y, '<' | 0x80);
+		r_funcs->Draw_nString (x + 8, y, s + 1, il->width - 2);
 	} else {
-		Draw_nString (x, y, s, il->width - 1);
+		r_funcs->Draw_nString (x, y, s, il->width - 1);
 	}
-	if (cursor) {
+	if (cursor && con_data.realtime) {
 		float       t = *con_data.realtime * con_cursorspeed;
 		int         ch = 10 + ((int) (t) & 1);
-		Draw_Character (x + ((il->linepos - il->scroll) << 3), y, ch);
+		r_funcs->Draw_Character (x + ((il->linepos - il->scroll) << 3), y, ch);
 	}
 	if (strlen (s) >= il->width)
-		Draw_Character (x + ((il->width - 1) << 3), y, '>' | 0x80);
+		r_funcs->Draw_Character (x + ((il->width - 1) << 3), y, '>' | 0x80);
 }
 
 void
 C_DrawInputLine (inputline_t *il)
 {
-	il_data_t   *data = il->user_data;
-	DrawInputLine (data->xbase + data->x, data->ybase + data->y, data->cursor,
-				   il);
+	DrawInputLine (il->x, il->y, il->cursor, il);
 }
 
 static void
 draw_input (view_t *view)
 {
-	if (key_dest != key_console)// && !con_data.force_commandline)
+	if (con_keydest != key_console)// && !con_data.force_commandline)
 		return;				// don't draw anything (always draw if not active)
 
 	DrawInputLine (view->xabs + 8, view->yabs, 1, input_line);
@@ -594,10 +584,10 @@ draw_download (view_t *view)
 	const char *text;
 	size_t		i, j, x, y, n;
 
-	if (!con_data.dl_name || !*con_data.dl_name)
+	if (!con_data.dl_name || !*con_data.dl_name->str)
 		return;
 
-	text = QFS_SkipPath(con_data.dl_name);
+	text = QFS_SkipPath(con_data.dl_name->str);
 
 	x = con_linewidth - ((con_linewidth * 7) / 40);
 	y = x - strlen (text) - 8;
@@ -629,7 +619,7 @@ draw_download (view_t *view)
 			  " %02d%%", *con_data.dl_percent);
 
 	// draw it
-	Draw_String (view->xabs, view->yabs, dlbar);
+	r_funcs->Draw_String (view->xabs, view->yabs, dlbar);
 }
 
 static void
@@ -647,12 +637,12 @@ draw_console_text (view_t *view)
 	if (con->display != con->current) {
 		// draw arrows to show the buffer is backscrolled
 		for (i = 0; i < con_linewidth; i += 4)
-			Draw_Character (x + (i << 3), y, '^');
+			r_funcs->Draw_Character (x + (i << 3), y, '^');
 
 		y -= 8;
 		rows--;
 	}
-	
+
 	row = con->display;
 	for (i = 0; i < rows; i++, y -= 8, row--) {
 		if (row < 0)
@@ -662,7 +652,7 @@ draw_console_text (view_t *view)
 
 		text = con->text + (row % con_totallines) * con_linewidth;
 
-		Draw_nString(x, y, text, con_linewidth);
+		r_funcs->Draw_nString(x, y, text, con_linewidth);
 	}
 }
 
@@ -674,12 +664,12 @@ draw_console (view_t *view)
 	if (con_data.force_commandline) {
 		alpha = 255;
 	} else {
-		float       y = vid.height * con_size->value;
+		float       y = r_data->vid->conheight * con_size->value;
 		alpha = 255 * con_alpha->value * view->ylen / y;
 		alpha = min (alpha, 255);
 	}
 	// draw the background
-	Draw_ConsoleBackground (view->ylen, alpha);
+	r_funcs->Draw_ConsoleBackground (view->ylen, alpha);
 
 	// draw everything else
 	view_draw (view);
@@ -688,14 +678,13 @@ draw_console (view_t *view)
 static void
 draw_say (view_t *view)
 {
-	clearnotify = 0;
-	scr_copytop = 1;
+	r_data->scr_copytop = 1;
 
 	if (chat_team) {
-		Draw_String (view->xabs + 8, view->yabs, "say_team:");
+		r_funcs->Draw_String (view->xabs + 8, view->yabs, "say_team:");
 		DrawInputLine (view->xabs + 80, view->yabs, 1, say_team_line);
 	} else {
-		Draw_String (view->xabs + 8, view->yabs, "say:");
+		r_funcs->Draw_String (view->xabs + 8, view->yabs, "say:");
 		DrawInputLine (view->xabs + 40, view->yabs, 1, say_line);
 	}
 }
@@ -707,6 +696,9 @@ draw_notify (view_t *view)
 	char	   *text;
 	float		time;
 
+	if (!con_data.realtime)
+		return;
+
 	x = view->xabs + 8;
 	y = view->yabs;
 	for (i = con->current - NUM_CON_TIMES + 1; i <= con->current; i++) {
@@ -720,10 +712,9 @@ draw_notify (view_t *view)
 			continue;
 		text = con->text + (i % con_totallines) * con_linewidth;
 
-		clearnotify = 0;
-		scr_copytop = 1;
+		r_data->scr_copytop = 1;
 
-		Draw_nString (x, y, text, con_linewidth);
+		r_funcs->Draw_nString (x, y, text, con_linewidth);
 		y += 8;
 	}
 }
@@ -734,22 +725,26 @@ setup_console (void)
 	float       lines;
 
 	if (con_data.force_commandline) {
-		lines = con_data.lines = vid.height;
-	} else if (key_dest == key_console) {
-		lines = vid.height * bound (0.2, con_size->value, 1);
+		lines = con_data.lines = r_data->vid->conheight;
+	} else if (con_keydest == key_console) {
+		lines = r_data->vid->conheight * bound (0.2, con_size->value, 1);
 	} else {
 		lines = 0;
 	}
 
-	if (lines < con_data.lines) {
-		con_data.lines -= max (0.2, con_speed->value) * *con_data.frametime;
-		con_data.lines = max (con_data.lines, lines);
-	} else if (lines > con_data.lines) {
-		con_data.lines += max (0.2, con_speed->value) * *con_data.frametime;
-		con_data.lines = min (con_data.lines, lines);
+	if (con_speed->value) {
+		if (lines < con_data.lines) {
+			con_data.lines -= max (0.2, con_speed->value) * *con_data.frametime;
+			con_data.lines = max (con_data.lines, lines);
+		} else if (lines > con_data.lines) {
+			con_data.lines += max (0.2, con_speed->value) * *con_data.frametime;
+			con_data.lines = min (con_data.lines, lines);
+		}
+	} else {
+		con_data.lines = lines;
 	}
-	if (con_data.lines >= vid.height - r_lineadj)
-		scr_copyeverything = 1;
+	if (con_data.lines >= r_data->vid->conheight - r_data->lineadj)
+		r_data->scr_copyeverything = 1;
 }
 
 static void
@@ -760,11 +755,11 @@ C_DrawConsole (void)
 	if (console_view->ylen != con_data.lines)
 		view_resize (console_view, console_view->xlen, con_data.lines);
 
-	say_view->visible = key_dest == key_message;
+	say_view->visible = con_keydest == key_message;
 	console_view->visible = con_data.lines != 0;
-	menu_view->visible = key_dest == key_menu;
+	menu_view->visible = con_keydest == key_menu;
 
-	view_draw (con_data.view);
+	con_data.view->draw (con_data.view);
 }
 
 static void
@@ -775,15 +770,13 @@ C_ProcessInput (void)
 static void
 C_NewMap (void)
 {
-	static int  first_time = 1;
-	static char old_gamedir[MAX_OSPATH];
+	static dstring_t *old_gamedir = 0;
 
-	if (first_time || !strequal (old_gamedir, qfs_gamedir->gamedir)) {
-		first_time = 0;
+	if (!old_gamedir || !strequal (old_gamedir->str, qfs_gamedir->gamedir))
 		Menu_Load ();
-	}
-	strncpy (old_gamedir, qfs_gamedir->gamedir, sizeof (old_gamedir));
-	old_gamedir[sizeof (old_gamedir) - 1] = 0;
+	if (!old_gamedir)
+		old_gamedir = dstring_newstr ();
+	dstring_copystr (old_gamedir, qfs_gamedir->gamedir);
 }
 
 static void
@@ -798,6 +791,19 @@ C_GIB_HUD_Disable_f (void)
 	hud_view->visible = 0;
 }
 
+static void
+exec_line (inputline_t *il)
+{
+	Con_ExecLine (il->line);
+}
+
+static void
+con_keydest_callback (keydest_t kd)
+{
+	// simply cache the value
+	con_keydest = kd;
+}
+
 static void
 C_Init (void)
 {
@@ -807,14 +813,12 @@ C_Init (void)
 	setlocale (LC_ALL, "C-TRADITIONAL");
 #endif
 
+	Key_KeydestCallback (con_keydest_callback);
 	Menu_Init ();
 
 	con_notifytime = Cvar_Get ("con_notifytime", "3", CVAR_NONE, NULL,
 							   "How long in seconds messages are displayed "
 							   "on screen");
-	cl_chatmode = Cvar_Get ("cl_chatmode", "2", CVAR_NONE, NULL,
-							"Controls when console text will be treated as a "
-							"chat message: 0 - never, 1 - always, 2 - smart");
 
 	con_alpha = Cvar_Get ("con_alpha", "0.6", CVAR_ARCHIVE, NULL,
 						  "alpha value for the console background");
@@ -823,16 +827,18 @@ C_Init (void)
 						 "down");
 	con_speed = Cvar_Get ("con_speed", "300", CVAR_ARCHIVE, NULL,
 						  "How quickly the console scrolls up or down");
+	cl_conmode = Cvar_Get ("cl_conmode", "command", CVAR_ARCHIVE, cl_conmode_f,
+						   "Set the console input mode (command, chat, rcon)");
 
 	con_debuglog = COM_CheckParm ("-condebug");
 
+	// The console will get resized, so assume initial size is 320x200
 	con_data.view = view_new (0, 0, 320, 200, grav_northeast);
-
 	console_view = view_new (0, 0, 320, 200, grav_northwest);
 	say_view     = view_new (0, 0, 320, 8, grav_northwest);
 	notify_view  = view_new (0, 8, 320, 32, grav_northwest);
 	menu_view    = view_new (0, 0, 320, 200, grav_center);
-	hud_view     = view_new (0, 0, vid.conwidth, vid.conheight, grav_northeast);
+	hud_view     = view_new (0, 0, 320, 200, grav_northeast);
 
 	view_add (con_data.view, say_view);
 	view_add (con_data.view, notify_view);
@@ -877,7 +883,7 @@ C_Init (void)
 
 	input_line = Con_CreateInputLine (32, MAXCMDLINE, ']');
 	input_line->complete = Con_BasicCompleteCommandLine;
-	input_line->enter = C_ExecLine;
+	input_line->enter = exec_line;
 	input_line->width = con_linewidth;
 	input_line->user_data = 0;
 	input_line->draw = 0;
@@ -898,7 +904,7 @@ C_Init (void)
 
 	C_CheckResize ();
 
-	Con_Printf ("Console initialized.\n");
+	Sys_Printf ("Console initialized.\n");
 
 	// register our commands
 	Cmd_AddCommand ("toggleconsole", ToggleConsole_f,
@@ -912,7 +918,7 @@ C_Init (void)
 	Cmd_AddCommand ("clear", Clear_f, "Clear the console");
 	Cmd_AddCommand ("condump", Condump_f, "dump the console text to a "
 					"file");
-					
+
 	// register GIB builtins
 	GIB_Builtin_Add ("HUD::enable", C_GIB_HUD_Enable_f);
 	GIB_Builtin_Add ("HUD::disable", C_GIB_HUD_Disable_f);
