@@ -32,10 +32,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused))
-const char  rcsid[] =
-	"$Id: gib_builtin.c,v 1.56 2004/01/19 07:47:38 taniwha Exp $";
-
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
@@ -68,7 +64,7 @@ const char  rcsid[] =
 #include "gib_classes.h"
 
 static char _gib_null_string[] = "";
-char * const gib_null_string = _gib_null_string;
+VISIBLE char * const gib_null_string = _gib_null_string;
 
 hashtab_t  *gib_builtins;
 
@@ -76,7 +72,7 @@ hashtab_t  *gib_builtins;
 	Hashtable callbacks
 */
 static const char *
-GIB_Builtin_Get_Key (void *ele, void *ptr)
+GIB_Builtin_Get_Key (const void *ele, void *ptr)
 {
 	return ((gib_builtin_t *) ele)->name;
 }
@@ -92,11 +88,11 @@ GIB_Builtin_Free (void *ele, void *ptr)
 
 /*
 	GIB_Builtin_Add
-	
+
 	Registers a new builtin GIB command.
 */
 
-void
+VISIBLE void
 GIB_Builtin_Add (const char *name, void (*func) (void))
 {
 	gib_builtin_t *new;
@@ -111,7 +107,7 @@ GIB_Builtin_Add (const char *name, void (*func) (void))
 	Hash_Add (gib_builtins, new);
 }
 
-void
+VISIBLE void
 GIB_Builtin_Remove (const char *name)
 {
 	gib_builtin_t *del;
@@ -120,7 +116,7 @@ GIB_Builtin_Remove (const char *name)
 		Hash_Free (gib_builtins, del);
 }
 
-qboolean
+VISIBLE qboolean
 GIB_Builtin_Exists (const char *name)
 {
 	return Hash_Find (gib_builtins, name) ? true : false;
@@ -128,12 +124,12 @@ GIB_Builtin_Exists (const char *name)
 
 /*
 	GIB_Builtin_Find
-	
+
 	Looks up the builtin name in the builtin hash,
 	returning a pointer to the struct on success,
 	zero otherwise.
 */
-gib_builtin_t *
+VISIBLE gib_builtin_t *
 GIB_Builtin_Find (const char *name)
 {
 	if (!gib_builtins)
@@ -141,7 +137,7 @@ GIB_Builtin_Find (const char *name)
 	return (gib_builtin_t *) Hash_Find (gib_builtins, name);
 }
 
-dstring_t *
+VISIBLE dstring_t *
 GIB_Return (const char *str)
 {
 	dstring_t  *dstr;
@@ -157,7 +153,7 @@ GIB_Return (const char *str)
 	return 0;
 }
 
-void
+VISIBLE void
 GIB_Error (const char *type, const char *fmt, ...)
 {
 	va_list     args;
@@ -169,7 +165,7 @@ GIB_Error (const char *type, const char *fmt, ...)
 
 /*
 	GIB Builtin functions
-	
+
 	See GIB docs for information.
 */
 static void
@@ -178,38 +174,39 @@ GIB_Function_f (void)
 	gib_tree_t *program;
 	gib_function_t *func;
 	int i;
-	
-	if (GIB_Argc () < 3)
+	int argc = GIB_Argc ();
+
+	if (argc < 3) {
 		GIB_USAGE ("name [arg1 arg2 ...] program");
-	else {
+	} else {
 		// Is the function program already tokenized?
-		if (GIB_Argm (GIB_Argc()-1)->delim != '{') {
+		if (GIB_Argm (argc-1)->delim != '{') {
 			// Parse on the fly
 			if (!(program = GIB_Parse_Lines (GIB_Argv
-							(GIB_Argc()-1), 0))) {
+							(argc-1), 0))) {
 				// Error!
 				GIB_Error ("ParseError", "Parse error while defining function '%s'.",
 						   GIB_Argv (1));
 				return;
 			}
 		} else
-			program = GIB_Argm (GIB_Argc()-1)->children;
-		func = GIB_Function_Define (GIB_Argv (1), GIB_Argv (GIB_Argc()-1), program,
+			program = GIB_Argm (argc-1)->children;
+		func = GIB_Function_Define (GIB_Argv (1), GIB_Argv (argc-1), program,
 							 GIB_DATA (cbuf_active)->script,
 							 GIB_DATA (cbuf_active)->globals);
 		llist_flush (func->arglist);
-		for (i = 2; i < GIB_Argc()-1; i++)
+		for (i = 2; i < argc-1; i++)
 			llist_append (func->arglist, strdup (GIB_Argv(i)));
-		func->minargs = GIB_Argc()-2;
+		func->minargs = argc-2;
 	}
 }
 
 static void
 GIB_Function_Get_f (void)
 {
-	if (GIB_Argc () != 2)
+	if (GIB_Argc () != 2) {
 		GIB_USAGE ("name");
-	else {
+	} else {
 		gib_function_t *f;
 
 		if ((f = GIB_Function_Find (GIB_Argv (1))))
@@ -227,9 +224,9 @@ GIB_Local_f (void)
 	int i;
 	static hashtab_t  *zero = 0;
 
-	if (GIB_Argc () < 2)
+	if (GIB_Argc () < 2) {
 		GIB_USAGE ("var [= value1 value2 ...] || var [var2 var3 ...]");
-	else if (!strcmp (GIB_Argv(2), "=")) {
+	} else if (!strcmp (GIB_Argv(2), "=")) {
 		var = GIB_Var_Get_Complex (&GIB_DATA (cbuf_active)->locals, &zero,
 								 GIB_Argv (1), &index, true);
 		if (GIB_Argc () >= 3)
@@ -252,9 +249,9 @@ GIB_Shared_f (void)
 	int i;
 	static hashtab_t  *zero = 0;
 
-	if (GIB_Argc () < 2)
+	if (GIB_Argc () < 2) {
 		GIB_USAGE ("var [= value1 value2 ...] || var [var2 var3 ...]");
-	else if (!strcmp (GIB_Argv(2), "=")) {
+	} else if (!strcmp (GIB_Argv(2), "=")) {
 		var = GIB_Var_Get_Complex (&GIB_DATA (cbuf_active)->globals, &zero,
 								 GIB_Argv (1), &index, true);
 		if (GIB_Argc () >= 3)
@@ -265,7 +262,7 @@ GIB_Shared_f (void)
 				GIB_Return (GIB_Argv(i));
 	} else for (i = 1; i < GIB_Argc(); i++)
 		var = GIB_Var_Get_Complex (&GIB_DATA (cbuf_active)->globals, &zero,
-								 GIB_Argv (i), &index, true);						 
+								 GIB_Argv (i), &index, true);
 }
 
 static void
@@ -276,10 +273,10 @@ GIB_Delete_f (void)
 	int i;
 	hashtab_t *source;
 	char *c;
-	
-	if (GIB_Argc () < 2)
+
+	if (GIB_Argc () < 2) {
 		GIB_USAGE ("var [var2 var2 ...]");
-	else for (i = 1; i < GIB_Argc(); i++) {
+	} else for (i = 1; i < GIB_Argc(); i++) {
 		if ((c = strrchr (GIB_Argv(i), '.'))) {
 			*(c++) = 0;
 			if (!(var = GIB_Var_Get_Complex (&GIB_DATA (cbuf_active)->locals,
@@ -329,9 +326,9 @@ GIB_Return_f (void)
 
 		for (i = 1; i < GIB_Argc(); i++)
 			argv[i-1] = GIB_Argv(i);
-		
+
 		GIB_Reply (g->reply.obj, g->reply.mesg, GIB_Argc()-1, argv);
-		free (argv);
+		free ((void*)argv);
 		g->dnotify = NULL;
 	} else if (GIB_Argc () > 1 && sp && sp->interpreter == &gib_interp
 		&& GIB_DATA (sp)->waitret) {
@@ -369,11 +366,11 @@ GIB_Runexported_f (void)
 	gib_function_t *f;
 	const char **args;
 
-	if (!(f = GIB_Function_Find (Cmd_Argv (0))))
+	if (!(f = GIB_Function_Find (Cmd_Argv (0)))) {
 		Sys_Printf ("Error:  No function found for exported command \"%s\".\n"
 					"This is most likely a bug, please report it to"
 					"The QuakeForge developers.", Cmd_Argv (0));
-	else {
+	} else {
 		cbuf_t     *sub = Cbuf_PushStack (&gib_interp);
 		int i;
 
@@ -381,7 +378,7 @@ GIB_Runexported_f (void)
 		for (i = 0; i < Cmd_Argc(); i++)
 			args[i] = Cmd_Argv(i);
 		GIB_Function_Execute (sub, f, args, Cmd_Argc());
-		free (args);
+		free ((void*)args);
 	}
 }
 
@@ -440,7 +437,7 @@ GIB_Count_f (void)
 	if (GIB_CanReturn())
 		dsprintf (GIB_Return(0), "%u", GIB_Argc() - 1);
 }
-		
+
 
 static void
 GIB_Contains_f (void)
@@ -448,7 +445,7 @@ GIB_Contains_f (void)
 	int i;
 	if (GIB_Argc () < 2)
 		GIB_USAGE ("needle [straw1 straw2 ...]");
-	else if (GIB_CanReturn())
+	else if (GIB_CanReturn ())
 		for (i = 2; i < GIB_Argc(); i++)
 			if (!strcmp(GIB_Argv(1), GIB_Argv(i))) {
 				GIB_Return("1");
@@ -463,9 +460,9 @@ GIB_Slice_f (void)
 	dstring_t  *ret;
 	int         start, end, len;
 
-	if (GIB_Argc () < 3 || GIB_Argc () > 4)
+	if (GIB_Argc () < 3 || GIB_Argc () > 4) {
 		GIB_USAGE ("string start [end]");
-	else {
+	} else {
 		len = strlen (GIB_Argv (1));
 		start = atoi (GIB_Argv (2));
 		end = *GIB_Argv (3) ? atoi (GIB_Argv (3)) : len;
@@ -493,14 +490,14 @@ GIB_Slice_Find_f (void)
 	if (GIB_Argc () != 3) {
 		GIB_USAGE ("haystack needle");
 		return;
-	} else if (!GIB_CanReturn ())
+	} else if (!GIB_CanReturn ()) {
 		return;
-	else if ((res = strstr (GIB_Argv (1), GIB_Argv (2)))) {
+	} else if ((res = strstr (GIB_Argv (1), GIB_Argv (2)))) {
 		dsprintf (GIB_Return (0), "%lu",
 				  (unsigned long int) (res - GIB_Argv (1)));
 		dsprintf (GIB_Return (0), "%lu",
-				  (unsigned long int) (res - GIB_Argv (1)) +
-				  strlen (GIB_Argv (2)));
+				  (unsigned long int) (res - GIB_Argv (1) +
+									   strlen (GIB_Argv (2))));
 	}
 }
 
@@ -566,11 +563,8 @@ GIB_Regex_Match_f (void)
 		return;
 	}
 
-	if (!
-		(reg =
-		 GIB_Regex_Compile (GIB_Argv (2),
-							REG_EXTENDED |
-							GIB_Regex_Translate_Options (GIB_Argv (3)))))
+	if (!(reg =  GIB_Regex_Compile (GIB_Argv (2), REG_EXTENDED |
+					GIB_Regex_Translate_Options (GIB_Argv (3)))))
 		GIB_Error ("RegexError", "%s: %s", GIB_Argv (0), GIB_Regex_Error ());
 	else if (regexec (reg, GIB_Argv (1), 0, 0, GIB_Regex_Translate_Runtime_Options (GIB_Argv (3))))
 		GIB_Return ("0");
@@ -582,7 +576,7 @@ static void
 GIB_Regex_Replace_f (void)
 {
 	regex_t    *reg;
-	int         ofs, len;
+	int         ofs;
 	regmatch_t  match[10];
 
 	if (GIB_Argc () != 5) {
@@ -591,7 +585,6 @@ GIB_Regex_Replace_f (void)
 	}
 
 	ofs = 0;
-	len = strlen (GIB_Argv (4));
 
 	if (!
 		(reg =
@@ -719,7 +712,7 @@ GIB_Text_From_Decimal_f (void)
 	if (GIB_Argc () < 2)
 		GIB_USAGE ("num1 [...]");
 	else if (GIB_CanReturn ()) {
-		unsigned int i;
+		int i;
 		dstring_t *dstr;
 		char *str;
 
@@ -956,10 +949,10 @@ static void
 GIB_Class_f (void)
 {
 	if (GIB_Object_Get (GIB_Argv(1))) {
-		GIB_Error ("ClassRedefinitionError", 
+		GIB_Error ("ClassRedefinitionError",
 				"Class '%s' already exists", GIB_Argv(1));
 	} else if (GIB_Argc () == 5)
-		GIB_Classes_Build_Scripted (GIB_Argv(1), GIB_Argv(3), 
+		GIB_Classes_Build_Scripted (GIB_Argv(1), GIB_Argv(3),
 				GIB_Argm (4)->children,
 				GIB_DATA(cbuf_active)->script);
 	else
@@ -980,14 +973,14 @@ GIB_Emit_f (void)
 	} else {
 		int i;
 		const char **argv = malloc (GIB_Argc () - 1);
-		
+
 		for (i = 1; i < GIB_Argc (); i ++)
 			argv[i-1] = GIB_Argv (1);
-		
+
 		GIB_Object_Signal_Emit (GIB_DATA(cbuf_active)->reply.obj,
 				GIB_Argc () - 1, argv);
 
-		free (argv);
+		free ((void*)argv);
 	}
 }
 
@@ -1020,10 +1013,10 @@ GIB_New_f (void)
 	} else if (
 			   !(class = GIB_Object_Get(GIB_Argv(1)))
 			|| classobj->class->classobj != classobj) {
-		GIB_Error ("UnknownClassError", "Class '%s' does not exist", 
+		GIB_Error ("UnknownClassError", "Class '%s' does not exist",
 				GIB_Argv(1));
 	} else {
-		GIB_Send (classobj, 
+		GIB_Send (classobj,
 */
 
 static void
