@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: tga.c,v 1.3 2003/09/04 18:59:36 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -235,7 +232,7 @@ skip_colormap (TargaHeader *targa, byte *data)
 	int         bpe;
 	if (!targa->colormap_type)
 		return data;
-	Sys_DPrintf ("LoadTGA: skipping colormap\n");
+	Sys_MaskPrintf (SYS_DEV, "LoadTGA: skipping colormap\n");
 	bpe = (targa->pixel_size +7) / 8;
 	return data + bpe * targa->colormap_length;
 }
@@ -627,14 +624,15 @@ LoadTGA (QFile *fin)
 {
 	byte       *dataByte;
 	decoder_t   decode;
-	int         numPixels, targa_mark;
+	int         fsize = Qfilesize (fin);
+	int			numPixels, targa_mark;
 	TargaHeader *targa;
 	tex_t      *tex;
 
 	targa_mark = Hunk_LowMark ();
-	targa = Hunk_AllocName (qfs_filesize, "TGA");
-	Qread (fin, targa, qfs_filesize);
-	
+	targa = Hunk_AllocName (fsize, "TGA");
+	Qread (fin, targa, fsize);
+
 	targa->colormap_index = LittleShort (targa->colormap_index);
 	targa->colormap_length = LittleShort (targa->colormap_length);
 	targa->x_origin = LittleShort (targa->x_origin);
@@ -662,9 +660,10 @@ LoadTGA (QFile *fin)
 	return tex;
 }
 
-void
+VISIBLE void
 WriteTGAfile (const char *tganame, byte *data, int width, int height)
 {
+	QFile      *qfile;
 	TargaHeader header;
 
 	memset (&header, 0, sizeof (header));
@@ -673,6 +672,12 @@ WriteTGAfile (const char *tganame, byte *data, int width, int height)
 	header.height = LittleShort (height);
 	header.pixel_size = 24;
 
-	QFS_WriteBuffers (tganame, 2, &header, sizeof (header), data,
-					  width * height * 3);
+	qfile = QFS_WOpen (tganame, 0);
+	if (!qfile) {
+		Sys_Printf ("Error opening %s", tganame);
+		return;
+	}
+	Qwrite (qfile, &header, sizeof (header));
+	Qwrite (qfile, data, width * height * 3);
+	Qclose (qfile);
 }
