@@ -29,8 +29,8 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: gl_lightmap.c,v 1.26 2004/03/17 04:47:55 taniwha Exp $";
+#define NH_DEFINE
+#include "namehack.h"
 
 #ifdef HAVE_STRING_H
 # include <string.h>
@@ -42,7 +42,6 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include <math.h>
 #include <stdio.h>
 
-#include "QF/console.h"
 #include "QF/cvar.h"
 #include "QF/render.h"
 #include "QF/sys.h"
@@ -55,30 +54,27 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/GL/qf_vid.h"
 
 #include "compat.h"
-#include "r_cvar.h"
-#include "r_local.h"
-#include "r_shared.h"
+#include "r_internal.h"
 
-int          active_lightmaps;
-int          dlightdivtable[8192];
-int			 gl_internalformat;				// 1 or 3
-int          lightmap_bytes;				// 1, 3, or 4
-int          lightmap_textures;
+static int          dlightdivtable[8192];
+static int			 gl_internalformat;				// 1 or 3
+static int          lightmap_bytes;				// 1, 3, or 4
+int          gl_lightmap_textures;
 
 // keep lightmap texture data in main memory so texsubimage can update properly
 // LordHavoc: changed to be allocated at runtime (typically lower memory usage)
-byte        *lightmaps[MAX_LIGHTMAPS];
+static byte        *lightmaps[MAX_LIGHTMAPS];
 
-unsigned int blocklights[34 * 34 * 3];	//FIXME make dynamic
-int          allocated[MAX_LIGHTMAPS][BLOCK_WIDTH];
+static unsigned int blocklights[34 * 34 * 3];	//FIXME make dynamic
+static int          allocated[MAX_LIGHTMAPS][BLOCK_WIDTH];
 
-qboolean	 lightmap_modified[MAX_GLTEXTURES];
-glpoly_t	*lightmap_polys[MAX_LIGHTMAPS];
-glRect_t	 lightmap_rectchange[MAX_LIGHTMAPS];
+qboolean	 gl_lightmap_modified[MAX_LIGHTMAPS];
+instsurf_t	*gl_lightmap_polys[MAX_LIGHTMAPS];
+glRect_t	 gl_lightmap_rectchange[MAX_LIGHTMAPS];
 
 static int	 lmshift = 7;
 
-void (*R_BuildLightMap) (msurface_t *surf);
+void (*gl_R_BuildLightMap) (msurface_t *surf);
 
 extern void gl_multitexture_f (cvar_t *var);
 
@@ -93,7 +89,7 @@ gl_lightmap_init (void)
 	for (s = 1; s < 8192; s++)
 		dlightdivtable[s] = 1048576 / (s << 7);
 }
-
+/*
 static void
 R_RecursiveLightUpdate (mnode_t *node)
 {
@@ -109,12 +105,13 @@ R_RecursiveLightUpdate (mnode_t *node)
 			 c--, surf++)
 			surf->cached_dlight = true;
 }
-
+*/
 static inline void
 R_AddDynamicLights_1 (msurface_t *surf)
 {
 	float			dist;
-	int				maxdist, maxdist2, maxdist3, smax, smax_bytes, tmax,
+	unsigned int	maxdist, maxdist2, maxdist3;
+	int             smax, smax_bytes, tmax,
 					grey, s, t;
 	unsigned int	lnum, td, i, j;
 	unsigned int    sdtable[18];
@@ -126,7 +123,7 @@ R_AddDynamicLights_1 (msurface_t *surf)
 	tmax = (surf->extents[1] >> 4) + 1;
 
 	for (lnum = 0; lnum < r_maxdlights; lnum++) {
-		if (!(surf->dlightbits & (1 << lnum)))
+		if (!(surf->dlightbits[lnum / 32] & (1 << (lnum % 32))))
 			continue;					// not lit by this light
 
 		VectorSubtract (r_dlights[lnum].origin, currententity->origin, local);
@@ -178,7 +175,8 @@ static inline void
 R_AddDynamicLights_3 (msurface_t *surf)
 {
 	float			dist;
-	int				maxdist, maxdist2, maxdist3, smax, smax_bytes, tmax,
+	unsigned int	maxdist, maxdist2, maxdist3;
+	int             smax, smax_bytes, tmax,
 					red, green, blue, s, t;
 	unsigned int	lnum, td, i, j;
 	unsigned int    sdtable[18];
@@ -190,7 +188,7 @@ R_AddDynamicLights_3 (msurface_t *surf)
 	tmax = (surf->extents[1] >> 4) + 1;
 
 	for (lnum = 0; lnum < r_maxdlights; lnum++) {
-		if (!(surf->dlightbits & (1 << lnum)))
+		if (!(surf->dlightbits[lnum / 32] & (1 << (lnum % 32))))
 			continue;					// not lit by this light
 
 		VectorSubtract (r_dlights[lnum].origin, currententity->origin, local);
@@ -439,7 +437,7 @@ do_subimage_2 (int i)
 {
 	byte       *block, *lm, *b;
 	int         stride, width;
-	glRect_t   *rect = &lightmap_rectchange[i];
+	glRect_t   *rect = &gl_lightmap_rectchange[i];
 
 	width = rect->w * lightmap_bytes;
 	stride = BLOCK_WIDTH * lightmap_bytes;
@@ -462,10 +460,10 @@ GL_UploadLightmap (int i)
 		do_subimage_2 (i);
 		break;
 	case 1:
-		qfglTexSubImage2D (GL_TEXTURE_2D, 0, 0, lightmap_rectchange[i].t,
-						   BLOCK_WIDTH, lightmap_rectchange[i].h,
+		qfglTexSubImage2D (GL_TEXTURE_2D, 0, 0, gl_lightmap_rectchange[i].t,
+						   BLOCK_WIDTH, gl_lightmap_rectchange[i].h,
 						   gl_lightmap_format, GL_UNSIGNED_BYTE,
-						   lightmaps[i] + (lightmap_rectchange[i].t *
+						   lightmaps[i] + (gl_lightmap_rectchange[i].t *
 										   BLOCK_WIDTH) * lightmap_bytes);
 		break;
 	default:
@@ -478,46 +476,50 @@ GL_UploadLightmap (int i)
 }
 
 void
-R_CalcLightmaps (void)
+gl_R_CalcLightmaps (void)
 {
 	int         i;
-	glpoly_t   *p;
 
 	for (i = 0; i < MAX_LIGHTMAPS; i++) {
-		p = lightmap_polys[i];
-		if (!p)
+		if (!gl_lightmap_polys[i])
 			continue;
-		if (lightmap_modified[i]) {
-			qfglBindTexture (GL_TEXTURE_2D, lightmap_textures + i);
+		if (gl_lightmap_modified[i]) {
+			qfglBindTexture (GL_TEXTURE_2D, gl_lightmap_textures + i);
 			GL_UploadLightmap (i);
-			lightmap_modified[i] = false;
+			gl_lightmap_modified[i] = false;
 		}
 	}
 }
 
 void
-R_BlendLightmaps (void)
+gl_R_BlendLightmaps (void)
 {
 	float      *v;
 	int         i, j;
+	instsurf_t *sc;
 	glpoly_t   *p;
 
 	qfglDepthMask (GL_FALSE);					// don't bother writing Z
 	qfglBlendFunc (lm_src_blend, lm_dest_blend);
 
 	for (i = 0; i < MAX_LIGHTMAPS; i++) {
-		p = lightmap_polys[i];
-		if (!p)
-			continue;
-		qfglBindTexture (GL_TEXTURE_2D, lightmap_textures + i);
-		for (; p; p = p->chain) {
-			qfglBegin (GL_POLYGON);
-			v = p->verts[0];
-			for (j = 0; j < p->numverts; j++, v += VERTEXSIZE) {
-				qfglTexCoord2fv (&v[5]);
-				qfglVertex3fv (v);
+		for (sc = gl_lightmap_polys[i]; sc; sc = sc->lm_chain) {
+			qfglBindTexture (GL_TEXTURE_2D, gl_lightmap_textures + i);
+			if (sc->transform) {
+				qfglPushMatrix ();
+				qfglLoadMatrixf (sc->transform);
 			}
-			qfglEnd ();
+			for (p = sc->surface->polys; p; p = p->next) {
+				qfglBegin (GL_POLYGON);
+				v = p->verts[0];
+				for (j = 0; j < p->numverts; j++, v += VERTEXSIZE) {
+					qfglTexCoord2fv (&v[5]);
+					qfglVertex3fv (v);
+				}
+				qfglEnd ();
+			}
+			if (sc->transform)
+				qfglPopMatrix ();
 		}
 	}
 
@@ -532,19 +534,20 @@ gl_overbright_f (cvar_t *var)
 	int			 num, i, j;
 	model_t		*m;
 	msurface_t  *fa;
+	entity_t    *ent;
 
 	if (!var)
 		return;
 
 	if (var->int_val) {
 		if (!gl_combine_capable && gl_mtex_capable) {
-			Con_Printf ("Warning: gl_overbright has no effect with "
+			Sys_Printf ("Warning: gl_overbright has no effect with "
 						"gl_multitexture enabled if you don't have "
 						"GL_COMBINE support in your driver.\n");
 			lm_src_blend = GL_ZERO;
 			lm_dest_blend = GL_SRC_COLOR;
 			lmshift = 7;
-			rgb_scale = 1.0;
+			gl_rgb_scale = 1.0;
 		} else {
 			lm_src_blend = GL_DST_COLOR;
 			lm_dest_blend = GL_SRC_COLOR;
@@ -552,15 +555,15 @@ gl_overbright_f (cvar_t *var)
 			switch (var->int_val) {
 			case 2:
 				lmshift = 9;
-				rgb_scale = 4.0;
+				gl_rgb_scale = 4.0;
 				break;
 			case 1:
 				lmshift = 8;
-				rgb_scale = 2.0;
+				gl_rgb_scale = 2.0;
 				break;
 			default:
 				lmshift = 7;
-				rgb_scale = 1.0;
+				gl_rgb_scale = 1.0;
 				break;
 			}
 		}
@@ -568,17 +571,17 @@ gl_overbright_f (cvar_t *var)
 		lm_src_blend = GL_ZERO;
 		lm_dest_blend = GL_SRC_COLOR;
 		lmshift = 7;
-		rgb_scale = 1.0;
+		gl_rgb_scale = 1.0;
 	}
 
 	if (gl_multitexture)
 		gl_multitexture_f (gl_multitexture);
 
-	if (!R_BuildLightMap)
+	if (!gl_R_BuildLightMap)
 		return;
 
-	for (i = 0; i < r_numvisedicts; i++) {
-		m = r_visedicts[i]->model;
+	for (ent = r_ent_queue; ent; ent = ent->next) {
+		m = ent->model;
 
 		if (m->type != mod_brush)
 			continue;
@@ -590,13 +593,13 @@ gl_overbright_f (cvar_t *var)
 				continue;
 
 			num = fa->lightmaptexturenum;
-			lightmap_modified[num] = true;
-			lightmap_rectchange[num].l = 0;
-			lightmap_rectchange[num].t = 0;
-			lightmap_rectchange[num].w = BLOCK_WIDTH;
-			lightmap_rectchange[num].h = BLOCK_HEIGHT;
+			gl_lightmap_modified[num] = true;
+			gl_lightmap_rectchange[num].l = 0;
+			gl_lightmap_rectchange[num].t = 0;
+			gl_lightmap_rectchange[num].w = BLOCK_WIDTH;
+			gl_lightmap_rectchange[num].h = BLOCK_HEIGHT;
 
-			R_BuildLightMap (fa);
+			gl_R_BuildLightMap (fa);
 		}
 	}
 
@@ -607,51 +610,16 @@ gl_overbright_f (cvar_t *var)
 			continue;
 
 		num = fa->lightmaptexturenum;
-		lightmap_modified[num] = true;
-		lightmap_rectchange[num].l = 0;
-		lightmap_rectchange[num].t = 0;
-		lightmap_rectchange[num].w = BLOCK_WIDTH;
-		lightmap_rectchange[num].h = BLOCK_HEIGHT;
+		gl_lightmap_modified[num] = true;
+		gl_lightmap_rectchange[num].l = 0;
+		gl_lightmap_rectchange[num].t = 0;
+		gl_lightmap_rectchange[num].w = BLOCK_WIDTH;
+		gl_lightmap_rectchange[num].h = BLOCK_HEIGHT;
 
-		R_BuildLightMap (fa);
+		gl_R_BuildLightMap (fa);
 	}
 }
 
-void
-R_CalcAndBlendLightmaps (void)
-{
-	float      *v;
-	int         i, j;
-	glpoly_t   *p;
-
-	qfglDepthMask (GL_FALSE);					// don't bother writing Z
-	qfglBlendFunc (lm_src_blend, lm_dest_blend);
-
-	for (i = 0; i < MAX_LIGHTMAPS; i++) {
-		p = lightmap_polys[i];
-		if (!p)
-			continue;
-		qfglBindTexture (GL_TEXTURE_2D, lightmap_textures + i);
-		if (lightmap_modified[i]) {
-			GL_UploadLightmap (i);
-			lightmap_modified[i] = false;
-		}
-		for (; p; p = p->chain) {
-			qfglBegin (GL_POLYGON);
-			v = p->verts[0];
-			for (j = 0; j < p->numverts; j++, v += VERTEXSIZE) {
-				qfglTexCoord2fv (&v[5]);
-				qfglVertex3fv (v);
-			}
-			qfglEnd ();
-		}
-	}
-
-	// Return to normal blending  --KB
-	qfglBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	qfglDepthMask (GL_TRUE);					// back to normal Z buffering
-}
-
 // LIGHTMAP ALLOCATION ========================================================
 
 // returns a texture number and the position inside it
@@ -709,7 +677,7 @@ GL_CreateSurfaceLightmap (msurface_t *surf)
 
 	surf->lightmaptexturenum =
 		AllocBlock (smax, tmax, &surf->light_s, &surf->light_t);
-	R_BuildLightMap (surf);
+	gl_R_BuildLightMap (surf);
 }
 
 /*
@@ -727,9 +695,9 @@ GL_BuildLightmaps (model_t **models, int num_models)
 
 	r_framecount = 1;					// no dlightcache
 
-	if (!lightmap_textures) {
-		lightmap_textures = texture_extension_number;
-		texture_extension_number += MAX_LIGHTMAPS;
+	if (!gl_lightmap_textures) {
+		gl_lightmap_textures = gl_texture_number;
+		gl_texture_number += MAX_LIGHTMAPS;
 	}
 
 	switch (r_lightmap_components->int_val) {
@@ -737,26 +705,26 @@ GL_BuildLightmaps (model_t **models, int num_models)
 		gl_internalformat = 1;
 		gl_lightmap_format = GL_LUMINANCE;
 		lightmap_bytes = 1;
-		R_BuildLightMap = R_BuildLightMap_1;
+		gl_R_BuildLightMap = R_BuildLightMap_1;
 		break;
 	case 3:
 		gl_internalformat = 3;
-		if (use_bgra)
+		if (gl_use_bgra)
 			gl_lightmap_format = GL_BGR;
 		else
 			gl_lightmap_format = GL_RGB;
 		lightmap_bytes = 3;
-		R_BuildLightMap = R_BuildLightMap_3;
+		gl_R_BuildLightMap = R_BuildLightMap_3;
 		break;
 	case 4:
 	default:
 		gl_internalformat = 3;
-		if (use_bgra)
+		if (gl_use_bgra)
 			gl_lightmap_format = GL_BGRA;
 		else
 			gl_lightmap_format = GL_RGBA;
 		lightmap_bytes = 4;
-		R_BuildLightMap = R_BuildLightMap_4;
+		gl_R_BuildLightMap = R_BuildLightMap_4;
 		break;
 	}
 
@@ -764,10 +732,13 @@ GL_BuildLightmaps (model_t **models, int num_models)
 		m = models[j];
 		if (!m)
 			break;
-		if (m->name[0] == '*')
+		if (m->name[0] == '*') {
+			// sub model surfaces are processed as part of the main model
 			continue;
+		}
 		r_pcurrentvertbase = m->vertexes;
-		currentmodel = m;
+		gl_currentmodel = m;
+		// non-bsp models don't have surfaces.
 		for (i = 0; i < m->numsurfaces; i++) {
 			if (m->surfaces[i].flags & SURF_DRAWTURB)
 				continue;
@@ -775,7 +746,7 @@ GL_BuildLightmaps (model_t **models, int num_models)
 										   SURF_DRAWSKY))
 				continue;
 			GL_CreateSurfaceLightmap (m->surfaces + i);
-			BuildSurfaceDisplayList (m->surfaces + i);
+			GL_BuildSurfaceDisplayList (m->surfaces + i);
 		}
 	}
 
@@ -783,14 +754,17 @@ GL_BuildLightmaps (model_t **models, int num_models)
 	for (i = 0; i < MAX_LIGHTMAPS; i++) {
 		if (!allocated[i][0])
 			break;						// no more used
-		lightmap_modified[i] = false;
-		lightmap_rectchange[i].l = BLOCK_WIDTH;
-		lightmap_rectchange[i].t = BLOCK_HEIGHT;
-		lightmap_rectchange[i].w = 0;
-		lightmap_rectchange[i].h = 0;
-		qfglBindTexture (GL_TEXTURE_2D, lightmap_textures + i);
+		gl_lightmap_modified[i] = false;
+		gl_lightmap_rectchange[i].l = BLOCK_WIDTH;
+		gl_lightmap_rectchange[i].t = BLOCK_HEIGHT;
+		gl_lightmap_rectchange[i].w = 0;
+		gl_lightmap_rectchange[i].h = 0;
+		qfglBindTexture (GL_TEXTURE_2D, gl_lightmap_textures + i);
 		qfglTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 		qfglTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+		if (gl_Anisotropy)
+			qfglTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT,
+							   gl_aniso);
 		qfglTexImage2D (GL_TEXTURE_2D, 0, lightmap_bytes, BLOCK_WIDTH,
 						BLOCK_HEIGHT, 0, gl_lightmap_format,
 						GL_UNSIGNED_BYTE, lightmaps[i]);
