@@ -29,8 +29,8 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: gl_rsurf.c,v 1.79 2004/02/24 20:50:55 despair Exp $";
+#define NH_DEFINE
+#include "namehack.h"
 
 #ifdef HAVE_STRING_H
 # include <string.h>
@@ -39,6 +39,8 @@ static __attribute__ ((unused)) const char rcsid[] =
 # include <strings.h>
 #endif
 
+#include "qfalloca.h"
+
 #include <math.h>
 #include <stdio.h>
 
@@ -55,68 +57,114 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/GL/qf_vid.h"
 
 #include "compat.h"
-#include "r_cvar.h"
-#include "r_local.h"
-#include "r_shared.h"
+#include "r_internal.h"
 
-int			 skytexturenum;
+static instsurf_t  *waterchain = NULL;
+static instsurf_t **waterchain_tail = &waterchain;
+static instsurf_t  *sky_chain;
+static instsurf_t **sky_chain_tail;
 
-glpoly_t    *fullbright_polys[MAX_GLTEXTURES];
-
-msurface_t  *waterchain = NULL;
-msurface_t **waterchain_tail = &waterchain;
-msurface_t  *sky_chain;
-msurface_t **sky_chain_tail;
-
-#define CHAIN_SURF_F2B(surf,chain)				\
-	do { 										\
-		*(chain##_tail) = (surf);				\
-		(chain##_tail) = &(surf)->texturechain;	\
-		*(chain##_tail) = 0;					\
+#define CHAIN_SURF_F2B(surf,chain)							\
+	do { 													\
+		instsurf_t *inst = (surf)->instsurf;				\
+		if (__builtin_expect(!inst, 1))						\
+			(surf)->tinst = inst = get_instsurf ();			\
+		inst->surface = (surf);								\
+		*(chain##_tail) = inst;								\
+		(chain##_tail) = &inst->tex_chain;					\
+		*(chain##_tail) = 0;								\
 	} while (0)
 
-#define CHAIN_SURF_B2F(surf,chain) 				\
-	do { 										\
-		(surf)->texturechain = (chain);			\
-		(chain) = (surf);						\
+#define CHAIN_SURF_B2F(surf,chain) 							\
+	do { 													\
+		instsurf_t *inst = (surf)->instsurf;				\
+		if (__builtin_expect(!inst, 1))						\
+			(surf)->tinst = inst = get_instsurf ();			\
+		inst->surface = (surf);								\
+		inst->tex_chain = (chain);							\
+		(chain) = inst;										\
 	} while (0)
 
-extern int			lightmap_textures;
-extern qboolean		lightmap_modified[MAX_LIGHTMAPS];
-extern glpoly_t    *lightmap_polys[MAX_LIGHTMAPS];
-extern glRect_t		lightmap_rectchange[MAX_LIGHTMAPS];
+static texture_t **r_texture_chains;
+static int  r_num_texture_chains;
+static int  max_texture_chains;
 
+static instsurf_t *static_chains;
+static instsurf_t *free_instsurfs;
+static instsurf_t *alloced_instsurfs;
+static instsurf_t **alloced_instsurfs_tail = &alloced_instsurfs;
+#define NUM_INSTSURFS (64 * 6)	// most brush models are simple boxes.
 
-/*
-  R_TextureAnimation
-
-  Returns the proper texture for a given time and base texture
-*/
-texture_t  *
-R_TextureAnimation (msurface_t *surf)
+static inline instsurf_t *
+get_instsurf (void)
 {
-	texture_t  *base = surf->texinfo->texture;
-	int         count, relative;
+	instsurf_t *instsurf;
 
-	if (currententity->frame) {
-		if (base->alternate_anims)
-			base = base->alternate_anims;
+	if (!free_instsurfs) {
+		int         i;
+
+		free_instsurfs = calloc (NUM_INSTSURFS, sizeof (instsurf_t));
+		for (i = 0; i < NUM_INSTSURFS - 1; i++)
+			free_instsurfs[i]._next = &free_instsurfs[i + 1];
 	}
-
-	relative = (int) (r_realtime * 10) % base->anim_total;
-
-	count = 0;
-	while (base->anim_min > relative || base->anim_max <= relative) {
-		base = base->anim_next;
-		if (!base)
-			Sys_Error ("R_TextureAnimation: broken cycle");
-		if (++count > 100)
-			Sys_Error ("R_TextureAnimation: infinite cycle");
-	}
-
-	return base;
+	instsurf = free_instsurfs;
+	free_instsurfs = instsurf->_next;
+	instsurf->_next = 0;
+	//build the chain of allocated instance surfaces so they can all be freed
+	//in one go
+	*alloced_instsurfs_tail = instsurf;
+	alloced_instsurfs_tail = &instsurf->_next;
+	return instsurf;
 }
 
+static inline void
+release_instsurfs (void)
+{
+	if (alloced_instsurfs) {
+		*alloced_instsurfs_tail = free_instsurfs;
+		free_instsurfs = alloced_instsurfs;
+		alloced_instsurfs = 0;
+		alloced_instsurfs_tail = &alloced_instsurfs;
+	}
+}
+
+void
+gl_R_AddTexture (texture_t *tex)
+{
+	int         i;
+	if (r_num_texture_chains == max_texture_chains) {
+		max_texture_chains += 64;
+		r_texture_chains = realloc (r_texture_chains,
+								  max_texture_chains * sizeof (texture_t *));
+		for (i = r_num_texture_chains; i < max_texture_chains; i++)
+			r_texture_chains[i] = 0;
+	}
+	r_texture_chains[r_num_texture_chains++] = tex;
+	tex->tex_chain = NULL;
+	tex->tex_chain_tail = &tex->tex_chain;
+}
+
+void
+gl_R_InitSurfaceChains (model_t *model)
+{
+	int         i;
+
+	if (static_chains)
+		free (static_chains);
+	static_chains = calloc (model->nummodelsurfaces, sizeof (instsurf_t));
+	for (i = 0; i < model->nummodelsurfaces; i++)
+		model->surfaces[i].instsurf = static_chains + i;
+
+	release_instsurfs ();
+}
+
+void
+gl_R_ClearTextures (void)
+{
+	r_num_texture_chains = 0;
+}
+
+
 // BRUSH MODELS ===============================================================
 
 static void
@@ -125,19 +173,34 @@ R_RenderFullbrights (void)
 	float      *v;
 	int         i, j;
 	glpoly_t   *p;
+	instsurf_t *sc;
+	texture_t  *tex;
 
-	for (i = 1; i < MAX_GLTEXTURES; i++) {
-		if (!fullbright_polys[i])
+	for (i = 0; i < r_num_texture_chains; i++) {
+		if (!(tex = r_texture_chains[i]) || !tex->gl_fb_texturenum)
 			continue;
-		qfglBindTexture (GL_TEXTURE_2D, i);
-		for (p = fullbright_polys[i]; p; p = p->fb_chain) {
-			qfglBegin (GL_POLYGON);
-			for (j = 0, v = p->verts[0]; j < p->numverts; j++, v += VERTEXSIZE)
-			{
-				qfglTexCoord2fv (&v[3]);
-				qfglVertex3fv (v);
+		qfglBindTexture (GL_TEXTURE_2D, tex->gl_fb_texturenum);
+		for (sc = tex->tex_chain; sc; sc = sc->tex_chain) {
+			if (sc->transform) {
+				qfglPushMatrix ();
+				qfglLoadMatrixf (sc->transform);
 			}
-			qfglEnd ();
+			if (sc->color)
+				qfglColor4fv (sc->color);
+			for (p = sc->surface->polys; p; p = p->next) {
+				qfglBegin (GL_POLYGON);
+				for (j = 0, v = p->verts[0]; j < p->numverts;
+					 j++, v += VERTEXSIZE)
+				{
+					qfglTexCoord2fv (&v[3]);
+					qfglVertex3fv (v);
+				}
+				qfglEnd ();
+			}
+			if (sc->transform)
+				qfglPopMatrix ();
+			if (sc->color)
+				qfglColor3ubv (color_white);
 		}
 	}
 }
@@ -148,7 +211,7 @@ R_RenderBrushPoly_3 (msurface_t *fa)
 	float      *v;
 	int			i;
 
-	c_brush_polys++;
+	gl_c_brush_polys++;
 
 	qfglBegin (GL_POLYGON);
 	v = fa->polys->verts[0];
@@ -169,7 +232,7 @@ R_RenderBrushPoly_2 (msurface_t *fa)
 	float      *v;
 	int			i;
 
-	c_brush_polys++;
+	gl_c_brush_polys++;
 
 	qfglBegin (GL_POLYGON);
 	v = fa->polys->verts[0];
@@ -189,7 +252,7 @@ R_RenderBrushPoly_1 (msurface_t *fa)
 	float      *v;
 	int			i;
 
-	c_brush_polys++;
+	gl_c_brush_polys++;
 
 	qfglBegin (GL_POLYGON);
 	v = fa->polys->verts[0];
@@ -207,11 +270,14 @@ R_AddToLightmapChain (msurface_t *fa)
 {
 	int		maps, smax, tmax;
 	glRect_t 	*theRect;
+	instsurf_t *sc;
 
 	// add the poly to the proper lightmap chain
+	if (!(sc = fa->instsurf))
+		sc = fa->tinst;
 
-	fa->polys->chain = lightmap_polys[fa->lightmaptexturenum];
-	lightmap_polys[fa->lightmaptexturenum] = fa->polys;
+	sc->lm_chain = gl_lightmap_polys[fa->lightmaptexturenum];
+	gl_lightmap_polys[fa->lightmaptexturenum] = sc;
 
 	// check for lightmap modification
 	for (maps = 0; maps < MAXLIGHTMAPS && fa->styles[maps] != 255; maps++)
@@ -221,8 +287,8 @@ R_AddToLightmapChain (msurface_t *fa)
 	if ((fa->dlightframe == r_framecount) || fa->cached_dlight) {
 	  dynamic:
 		if (r_dynamic->int_val) {
-			lightmap_modified[fa->lightmaptexturenum] = true;
-			theRect = &lightmap_rectchange[fa->lightmaptexturenum];
+			gl_lightmap_modified[fa->lightmaptexturenum] = true;
+			theRect = &gl_lightmap_rectchange[fa->lightmaptexturenum];
 			if (fa->light_t < theRect->t) {
 				if (theRect->h)
 					theRect->h += theRect->t - fa->light_t;
@@ -239,52 +305,61 @@ R_AddToLightmapChain (msurface_t *fa)
 				theRect->w = (fa->light_s - theRect->l) + smax;
 			if ((theRect->h + theRect->t) < (fa->light_t + tmax))
 				theRect->h = (fa->light_t - theRect->t) + tmax;
-			R_BuildLightMap (fa);
+			gl_R_BuildLightMap (fa);
 		}
 	}
 }
 
 void
-R_DrawWaterSurfaces (void)
+gl_R_DrawWaterSurfaces (void)
 {
 	int         i;
-	msurface_t *s;
+	instsurf_t *s;
+	msurface_t *fa;
+	float       wateralpha = max (vr_data.min_wateralpha, r_wateralpha->value);
 
 	if (!waterchain)
 		return;
 
 	// go back to the world matrix
-	qfglLoadMatrixf (r_world_matrix);
+	qfglLoadMatrixf (gl_r_world_matrix);
 
-	if (cl_wateralpha < 1.0) {
+	if (wateralpha < 1.0) {
 		qfglDepthMask (GL_FALSE);
-		color_white[3] = cl_wateralpha * 255;
+		color_white[3] = wateralpha * 255;
 		qfglColor4ubv (color_white);
 	}
 
 	i = -1;
-	for (s = waterchain; s; s = s->texturechain) {
-		if (i != s->texinfo->texture->gl_texturenum) {
-			i = s->texinfo->texture->gl_texturenum;
+	for (s = waterchain; s; s = s->tex_chain) {
+		fa = s->surface;
+		if (s->transform)
+			qfglLoadMatrixf (s->transform);
+		else
+			qfglLoadMatrixf (gl_r_world_matrix);
+		if (i != fa->texinfo->texture->gl_texturenum) {
+			i = fa->texinfo->texture->gl_texturenum;
 			qfglBindTexture (GL_TEXTURE_2D, i);
 		}
-		EmitWaterPolys (s);
+		GL_EmitWaterPolys (fa);
 	}
+	qfglLoadMatrixf (gl_r_world_matrix);
 
 	waterchain = NULL;
 	waterchain_tail = &waterchain;
 
-	if (cl_wateralpha < 1.0) {
+	if (wateralpha < 1.0) {
 		qfglDepthMask (GL_TRUE);
 		qfglColor3ubv (color_white);
 	}
 }
 
-static inline void
-DrawTextureChains (void)
+static void
+DrawTextureChains (int disable_blend, int do_bind)
 {
 	int			i;
-	msurface_t *s;
+	instsurf_t *s;
+	msurface_t *fa;
 	texture_t  *tex;
 
 	if (gl_mtex_active_tmus >= 2) {
@@ -296,8 +371,8 @@ DrawTextureChains (void)
 		qglActiveTexture (gl_mtex_enum + 0);
 		qfglTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
-		for (i = 0; i < r_worldentity.model->numtextures; i++) {
-			tex = r_worldentity.model->textures[i];
+		for (i = 0; i < r_num_texture_chains; i++) {
+			tex = r_texture_chains[i];
 			if (!tex)
 				continue;
 			qfglBindTexture (GL_TEXTURE_2D, tex->gl_texturenum);
@@ -308,11 +383,23 @@ DrawTextureChains (void)
 				qfglBindTexture (GL_TEXTURE_2D, tex->gl_fb_texturenum);
 
 				qglActiveTexture (gl_mtex_enum + 1);
-				for (s = tex->texturechain; s; s = s->texturechain) {
-					qfglBindTexture (GL_TEXTURE_2D, lightmap_textures +
-									 s->lightmaptexturenum);
+				for (s = tex->tex_chain; s; s = s->tex_chain) {
+					fa = s->surface;
+					if (s->transform) {
+						qfglPushMatrix ();
+						qfglLoadMatrixf (s->transform);
+					}
+					if (s->color && do_bind)
+						qfglColor4fv (s->color);
+					qfglBindTexture (GL_TEXTURE_2D, gl_lightmap_textures +
+									 fa->lightmaptexturenum);
 
-					R_RenderBrushPoly_3 (s);
+					R_RenderBrushPoly_3 (fa);
+
+					if (s->transform)
+						qfglPopMatrix ();
+					if (s->color && do_bind)
+						qfglColor3ubv (color_white);
 				}
 
 				qglActiveTexture (gl_mtex_enum + 2);
@@ -321,17 +408,27 @@ DrawTextureChains (void)
 				qglActiveTexture (gl_mtex_enum + 0);
 			} else {
 				qglActiveTexture (gl_mtex_enum + 1);
-				for (s = tex->texturechain; s; s = s->texturechain) {
-					qfglBindTexture (GL_TEXTURE_2D, lightmap_textures +
-									 s->lightmaptexturenum);
+				for (s = tex->tex_chain; s; s = s->tex_chain) {
+					fa = s->surface;
+					qfglBindTexture (GL_TEXTURE_2D, gl_lightmap_textures +
+									 fa->lightmaptexturenum);
 
-					R_RenderBrushPoly_2 (s);
+					if (s->transform) {
+						qfglPushMatrix ();
+						qfglLoadMatrixf (s->transform);
+					}
+					if (s->color && do_bind)
+						qfglColor4fv (s->color);
+					R_RenderBrushPoly_2 (fa);
+
+					if (s->transform)
+						qfglPopMatrix ();
+					if (s->color && do_bind)
+						qfglColor3ubv (color_white);
 				}
 
 				qglActiveTexture (gl_mtex_enum + 0);
 			}
-			tex->texturechain = NULL;
-			tex->texturechain_tail = &tex->texturechain;
 		}
 		// Turn off lightmaps for other entities
 		qglActiveTexture (gl_mtex_enum + 1);
@@ -341,43 +438,95 @@ DrawTextureChains (void)
 		qglActiveTexture (gl_mtex_enum + 0);
 		qfglTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 	} else {
-		qfglDisable (GL_BLEND);
-		for (i = 0; i < r_worldentity.model->numtextures; i++) {
-			tex = r_worldentity.model->textures[i];
+		if (disable_blend)
+			qfglDisable (GL_BLEND);
+		for (i = 0; i < r_num_texture_chains; i++) {
+			tex = r_texture_chains[i];
 			if (!tex)
 				continue;
-			qfglBindTexture (GL_TEXTURE_2D, tex->gl_texturenum);
+			if (do_bind)
+				qfglBindTexture (GL_TEXTURE_2D, tex->gl_texturenum);
 
-			for (s = tex->texturechain; s; s = s->texturechain)
-				R_RenderBrushPoly_1 (s);
+			for (s = tex->tex_chain; s; s = s->tex_chain) {
+				if (s->transform) {
+					qfglPushMatrix ();
+					qfglLoadMatrixf (s->transform);
+				}
+				R_RenderBrushPoly_1 (s->surface);
 
-			tex->texturechain = NULL;
-			tex->texturechain_tail = &tex->texturechain;
+				if (s->transform)
+					qfglPopMatrix ();
+				if (s->color && do_bind)
+					qfglColor3ubv (color_white);
+			}
 		}
-		qfglEnable (GL_BLEND);
+		if (disable_blend)
+			qfglEnable (GL_BLEND);
 	}
+}
 
+static void
+clear_texture_chains (void)
+{
+	int			i;
+	texture_t  *tex;
+
+	for (i = 0; i < r_num_texture_chains; i++) {
+		tex = r_texture_chains[i];
+		if (!tex)
+			continue;
+		tex->tex_chain = NULL;
+		tex->tex_chain_tail = &tex->tex_chain;
+	}
 	tex = r_notexture_mip;
-	tex->texturechain = NULL;
-	tex->texturechain_tail = &tex->texturechain;
+	tex->tex_chain = NULL;
+	tex->tex_chain_tail = &tex->tex_chain;
+	release_instsurfs ();
+
+	memset (gl_lightmap_polys, 0, sizeof (gl_lightmap_polys));
+}
+
+static inline void
+chain_surface (msurface_t *surf, vec_t *transform, float *color)
+{
+	instsurf_t *sc;
+
+	if (surf->flags & SURF_DRAWTURB) {
+		CHAIN_SURF_B2F (surf, waterchain);
+	} else if (surf->flags & SURF_DRAWSKY) {
+		CHAIN_SURF_F2B (surf, sky_chain);
+	} else {
+		texture_t  *tex;
+
+		if (!surf->texinfo->texture->anim_total)
+			tex = surf->texinfo->texture;
+		else
+			tex = R_TextureAnimation (surf);
+		CHAIN_SURF_F2B (surf, tex->tex_chain);
+
+		R_AddToLightmapChain (surf);
+	}
+	if (!(sc = surf->instsurf))
+		sc = surf->tinst;
+	sc->transform = transform;
+	sc->color = color;
 }
 
 void
-R_DrawBrushModel (entity_t *e)
+gl_R_DrawBrushModel (entity_t *e)
 {
 	float       dot, radius;
-	float		color[4], watercolor[4];
 	int         i;
 	unsigned int k;
 	model_t    *model;
-	mplane_t   *pplane;
+	plane_t    *pplane;
 	msurface_t *psurf;
 	qboolean    rotated;
 	vec3_t      mins, maxs;
 
 	model = e->model;
 
-	if (e->angles[0] || e->angles[1] || e->angles[2]) {
+	if (e->transform[0] != 1 || e->transform[5] != 1 || e->transform[10] != 1) {
 		rotated = true;
 		radius = model->radius;
 #if 0 //QSG FIXME
@@ -400,35 +549,23 @@ R_DrawBrushModel (entity_t *e)
 			return;
 	}
 
-	VectorCopy (e->colormod, color);
-	VectorCopy (color, watercolor);
-	color[3] = e->colormod[3];
-	qfglColor4fv (color);
-	if (color[3] < 1.0)
-		qfglDepthMask (GL_FALSE);
-
-	memset (lightmap_polys, 0, sizeof (lightmap_polys));
-	memset (fullbright_polys, 0, sizeof (fullbright_polys));
-
 	VectorSubtract (r_refdef.vieworg, e->origin, modelorg);
 	if (rotated) {
-		vec3_t      temp, forward, right, up;
+		vec3_t      temp;
 
 		VectorCopy (modelorg, temp);
-		AngleVectors (e->angles, forward, right, up);
-		modelorg[0] = DotProduct (temp, forward);
-		modelorg[1] = -DotProduct (temp, right);
-		modelorg[2] = DotProduct (temp, up);
+		modelorg[0] = DotProduct (temp, e->transform + 0);
+		modelorg[1] = DotProduct (temp, e->transform + 4);
+		modelorg[2] = DotProduct (temp, e->transform + 8);
 	}
 
-	psurf = &model->surfaces[model->firstmodelsurface];
-
 	// calculate dynamic lighting for bmodel if it's not an instanced model
 	if (model->firstmodelsurface != 0 && r_dlight_lightmap->int_val) {
 		vec3_t      lightorigin;
 
 		for (k = 0; k < r_maxdlights; k++) {
-			if ((r_dlights[k].die < r_realtime) || (!r_dlights[k].radius))
+			if ((r_dlights[k].die < vr_data.realtime)
+				 || (!r_dlights[k].radius))
 				continue;
 
 			VectorSubtract (r_dlights[k].origin, e->origin, lightorigin);
@@ -438,19 +575,9 @@ R_DrawBrushModel (entity_t *e)
 	}
 
 	qfglPushMatrix ();
-	e->angles[0] = -e->angles[0];		// stupid quake bug
-	R_RotateForEntity (e);
-	e->angles[0] = -e->angles[0];		// stupid quake bug
-
-	// Build lightmap chains
-	for (i = 0; i < model->nummodelsurfaces; i++, psurf++) {
-		if (psurf->flags & (SURF_DRAWTURB | SURF_DRAWSKY))
-			continue;
-		R_AddToLightmapChain (psurf);
-	}
-
-	if (gl_mtex_active_tmus >= 2)
-		R_CalcLightmaps ();
+	gl_R_RotateForEntity (e);
+	qfglGetFloatv (GL_MODELVIEW_MATRIX, e->full_transform);
+	qfglPopMatrix ();
 
 	psurf = &model->surfaces[model->firstmodelsurface];
 
@@ -464,97 +591,9 @@ R_DrawBrushModel (entity_t *e)
 		// draw the polygon
 		if (((psurf->flags & SURF_PLANEBACK) && (dot < -BACKFACE_EPSILON)) ||
 			(!(psurf->flags & SURF_PLANEBACK) && (dot > BACKFACE_EPSILON))) {
-			if (psurf->flags & SURF_DRAWTURB) {
-				qfglBindTexture (GL_TEXTURE_2D,
-								 psurf->texinfo->texture->gl_texturenum);
-				if (cl_wateralpha < 1.0) {
-					if (color[3] >= 1.0)
-						qfglDepthMask (GL_FALSE);
-					watercolor[3] = color[3] * cl_wateralpha;
-					qfglColor4fv (watercolor);
-					EmitWaterPolys (psurf);
-					qfglColor4fv (color);
-					if (color[3] >= 1.0)
-						qfglDepthMask (GL_TRUE);
-				} else {
-					EmitWaterPolys (psurf);
-				}
-			} else if (psurf->flags & SURF_DRAWSKY) {
-// QSG FIXME: add modelalpha support for sky brushes
-				CHAIN_SURF_F2B (psurf, sky_chain);
-			} else {
-				texture_t  *tex;
-
-				if (!psurf->texinfo->texture->anim_total)
-					tex = psurf->texinfo->texture;
-				else
-					tex = R_TextureAnimation (psurf);
-
-				if (gl_mtex_active_tmus >= 2) {
-					if (tex->gl_fb_texturenum && gl_mtex_fullbright) {
-						qglActiveTexture (gl_mtex_enum + 2);
-						qfglEnable (GL_TEXTURE_2D);
-						qfglBindTexture (GL_TEXTURE_2D, tex->gl_fb_texturenum);
-
-						qglActiveTexture (gl_mtex_enum + 1);
-						qfglEnable (GL_TEXTURE_2D);
-						qfglBindTexture (GL_TEXTURE_2D, lightmap_textures +
-										 psurf->lightmaptexturenum);
-
-						qglActiveTexture (gl_mtex_enum + 0);
-						qfglBindTexture (GL_TEXTURE_2D, tex->gl_texturenum);
-						R_RenderBrushPoly_3 (psurf);
-
-						qglActiveTexture (gl_mtex_enum + 2);
-						qfglDisable (GL_TEXTURE_2D);
-
-						qglActiveTexture (gl_mtex_enum + 1);
-						qfglDisable (GL_TEXTURE_2D);
-
-						qglActiveTexture (gl_mtex_enum + 0);
-					} else {
-						qglActiveTexture (gl_mtex_enum + 1);
-						qfglEnable (GL_TEXTURE_2D);
-						qfglBindTexture (GL_TEXTURE_2D, lightmap_textures +
-										 psurf->lightmaptexturenum);
-
-						qglActiveTexture (gl_mtex_enum + 0);
-						qfglBindTexture (GL_TEXTURE_2D, tex->gl_texturenum);
-						R_RenderBrushPoly_2 (psurf);
-
-						qglActiveTexture (gl_mtex_enum + 1);
-						qfglDisable (GL_TEXTURE_2D);
-
-						qglActiveTexture (gl_mtex_enum + 0);
-					}
-				} else {
-					qfglBindTexture (GL_TEXTURE_2D, tex->gl_texturenum);
-					R_RenderBrushPoly_1 (psurf);
-
-					if (tex->gl_fb_texturenum && gl_mtex_fullbright) {
-						psurf->polys->fb_chain =
-							fullbright_polys[tex->gl_fb_texturenum];
-						fullbright_polys[tex->gl_fb_texturenum] = psurf->polys;
-					}
-				}
-			}
+			chain_surface (psurf, e->full_transform, e->colormod);
 		}
 	}
-
-	if (gl_mtex_active_tmus >= 2) {
-		qfglTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-	} else {
-		R_CalcAndBlendLightmaps ();
-	}
-
-	if (gl_fb_bmodels->int_val && !gl_mtex_fullbright)
-		R_RenderFullbrights ();
-
-	qfglPopMatrix ();
-
-	if (color[3] < 1.0)
-		qfglDepthMask (GL_TRUE);
-	qfglColor3ubv (color_white);
 }
 
 // WORLD MODEL ================================================================
@@ -564,14 +603,14 @@ visit_leaf (mleaf_t *leaf)
 {
 	// deal with model fragments in this leaf
 	if (leaf->efrags)
-		R_StoreEfrags (&leaf->efrags);
+		R_StoreEfrags (leaf->efrags);
 }
 
 static inline int
 get_side (mnode_t *node)
 {
 	// find which side of the node we are on
-	mplane_t   *plane = node->plane;
+	plane_t    *plane = node->plane;
 
 	if (plane->type < 3)
 		return (modelorg[plane->type] - plane->dist) < 0;
@@ -597,27 +636,7 @@ visit_node (mnode_t *node, int side)
 			if (side ^ (surf->flags & SURF_PLANEBACK))
 				continue;				// wrong side
 
-			if (surf->flags & SURF_DRAWTURB) {
-				CHAIN_SURF_B2F (surf, waterchain);
-			} else if (surf->flags & SURF_DRAWSKY) {
-				CHAIN_SURF_F2B (surf, sky_chain);
-			} else {
-				texture_t  *tex;
-
-				R_AddToLightmapChain (surf);
-
-				if (!surf->texinfo->texture->anim_total)
-					tex = surf->texinfo->texture;
-				else
-					tex = R_TextureAnimation (surf);
-				if (gl_fb_bmodels->int_val && tex->gl_fb_texturenum
-					&& !gl_mtex_fullbright) {
-					surf->polys->fb_chain =
-						fullbright_polys[tex->gl_fb_texturenum];
-					fullbright_polys[tex->gl_fb_texturenum] = surf->polys;
-				}
-				CHAIN_SURF_F2B (surf, tex->texturechain);
-			}
+			chain_surface (surf, 0, 0);
 		}
 	}
 }
@@ -634,17 +653,22 @@ test_node (mnode_t *node)
 	return 1;
 }
 
-// FIXME: R_IterativeWorldNode
 static void
-R_RecursiveWorldNode (mnode_t *node)
+R_VisitWorldNodes (model_t *model)
 {
-	struct {
+	typedef struct {
 		mnode_t    *node;
 		int         side;
-	}          *node_ptr, node_stack[256];
+	} rstack_t;
+	rstack_t   *node_ptr;
+	rstack_t   *node_stack;
+	mnode_t    *node;
 	mnode_t    *front;
 	int         side;
 
+	node = model->nodes;
+	// +2 for paranoia
+	node_stack = alloca ((model->depth + 2) * sizeof (rstack_t));
 	node_ptr = node_stack;
 
 	while (1) {
@@ -652,9 +676,6 @@ R_RecursiveWorldNode (mnode_t *node)
 			side = get_side (node);
 			front = node->children[side];
 			if (test_node (front)) {
-				if (node_ptr - node_stack
-					== sizeof (node_stack) / sizeof (node_stack[0]))
-					Sys_Error ("node_stack overflow");
 				node_ptr->node = node;
 				node_ptr->side = side;
 				node_ptr++;
@@ -683,102 +704,120 @@ R_RecursiveWorldNode (mnode_t *node)
 }
 
 void
-R_DrawWorld (void)
+gl_R_DrawWorld (void)
 {
-	entity_t    ent;
+	entity_t    worldent;
 
-	memset (&ent, 0, sizeof (ent));
-	ent.model = r_worldentity.model;
+	memset (&worldent, 0, sizeof (worldent));
+	worldent.model = r_worldentity.model;
 
 	VectorCopy (r_refdef.vieworg, modelorg);
 
-	currententity = &ent;
-
-	memset (lightmap_polys, 0, sizeof (lightmap_polys));
-	memset (fullbright_polys, 0, sizeof (fullbright_polys));
+	currententity = &worldent;
 
 	sky_chain = 0;
 	sky_chain_tail = &sky_chain;
 	if (!gl_sky_clip->int_val) {
-		R_DrawSky ();
+		gl_R_DrawSky ();
 	}
 
-	R_RecursiveWorldNode (r_worldentity.model->nodes);
+	R_VisitWorldNodes (r_worldentity.model);
+	if (r_drawentities->int_val) {
+		entity_t   *ent;
+		for (ent = r_ent_queue; ent; ent = ent->next) {
+			if (ent->model->type != mod_brush)
+				continue;
+			currententity = ent;
 
-	R_CalcLightmaps ();
-
-	R_DrawSkyChain (sky_chain);
-
-	DrawTextureChains ();
-
-	if (gl_mtex_active_tmus <= 1)
-		R_BlendLightmaps ();
-
-	if (gl_fb_bmodels->int_val && !gl_mtex_fullbright)
-		R_RenderFullbrights ();
-}
-
-void
-R_MarkLeaves (void)
-{
-	byte         solid[4096];
-	byte        *vis;
-	int			 c;
-	unsigned int i;
-	mleaf_t     *leaf;
-	mnode_t     *node;
-	msurface_t **mark;
-
-	if (r_oldviewleaf == r_viewleaf && !r_novis->int_val)
-		return;
-
-	r_visframecount++;
-	r_oldviewleaf = r_viewleaf;
-
-	if (r_novis->int_val) {
-		vis = solid;
-		memset (solid, 0xff, (r_worldentity.model->numleafs + 7) >> 3);
-	} else
-		vis = Mod_LeafPVS (r_viewleaf, r_worldentity.model);
-
-	for (i = 0; (int) i < r_worldentity.model->numleafs; i++) {
-		if (vis[i >> 3] & (1 << (i & 7))) {
-			leaf = &r_worldentity.model->leafs[i + 1];
-			if ((c = leaf->nummarksurfaces)) {
-				mark = leaf->firstmarksurface;
-				do {
-					(*mark)->visframe = r_visframecount;
-					mark++;
-				} while (--c);
-			}
-			node = (mnode_t *) leaf;
-			do {
-				if (node->visframe == r_visframecount)
-					break;
-				node->visframe = r_visframecount;
-				node = node->parent;
-			} while (node);
+			gl_R_DrawBrushModel (currententity);
 		}
 	}
+
+	gl_R_CalcLightmaps ();
+
+	gl_R_DrawSkyChain (sky_chain);
+
+	if (!gl_Fog_GetDensity ()
+		|| (gl_fb_bmodels->int_val && gl_mtex_fullbright)
+		|| gl_mtex_active_tmus > 1) {
+		// we have enough active TMUs to render everything in one go
+		// or we're not doing fog
+		DrawTextureChains (1, 1);
+
+		if (gl_mtex_active_tmus <= 1)
+			gl_R_BlendLightmaps ();
+
+		if (gl_fb_bmodels->int_val && !gl_mtex_fullbright)
+			R_RenderFullbrights ();
+	} else {
+		if (gl_mtex_active_tmus > 1) {
+			// textures and lightmaps in one pass
+			// black fog
+			// no blending
+			gl_Fog_StartAdditive ();
+			DrawTextureChains (1, 1);
+			// buf = fTL + (1-f)0
+			//     = fTL
+		} else {
+			// texture pass + lightmap pass
+			// no fog
+			// no blending
+			gl_Fog_DisableGFog ();
+			DrawTextureChains (1, 1);
+			// buf = T
+			// black fog
+			// blend: buf = zero, src (non-overbright)
+			// FIXME overbright broken?
+			gl_Fog_EnableGFog ();
+			gl_Fog_StartAdditive ();
+			gl_R_BlendLightmaps ();	//leaves blending as As, 1-As
+			// buf = I*0 + buf*I
+			//     = T*C
+			//     = T(fL + (1-f)0)
+			//     = fTL
+		}
+		// fullbright pass
+		// fog is still black
+		R_RenderFullbrights ();
+		// buf = aI + (1-a)buf
+		//     = aC + (1-a)fTL
+		//     = a(fG + (1-f)0) + (1-a)fTL
+		//     = afG + (1-a)fTL
+		//     = f((1-a)TL + aG)
+		gl_Fog_StopAdditive ();		// use fog color
+		qfglDepthMask (GL_FALSE);	// don't write Z
+		qfglBlendFunc (GL_ONE, GL_ONE);
+		// draw black polys
+		qfglColor4f (0, 0, 0, 1);
+		DrawTextureChains (0, 0);
+		// buf = I + buf
+		//     = C + f((1-a)TL + aG)
+		//     = (f0 + (1-f)F) + f((1-a)TL + aG)
+		//     = (1-f)F + f((1-a)TL + aG)
+		//     = f((1-a)TL + aG) + (1-f)F
+		// restore state
+		qfglColor4ubv (color_white);
+		qfglBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		qfglDepthMask (GL_TRUE);
+	}
+
+	clear_texture_chains ();
 }
 
-int         nColinElim;
-model_t    *currentmodel;
-mvertex_t  *r_pcurrentvertbase;
+model_t    *gl_currentmodel;
 
 void
-BuildSurfaceDisplayList (msurface_t *fa)
+GL_BuildSurfaceDisplayList (msurface_t *fa)
 {
 	float       s, t;
 	float      *vec;
-	int         lindex, lnumverts, vertpage, i;
+	int         lindex, lnumverts, i;
 	glpoly_t   *poly;
 	medge_t    *pedges, *r_pedge;
 
 	// reconstruct the polygon
-	pedges = currentmodel->edges;
+	pedges = gl_currentmodel->edges;
 	lnumverts = fa->numedges;
-	vertpage = 0;
 
 	// draw texture
 	poly = Hunk_Alloc (sizeof (glpoly_t) + (lnumverts - 4) *
@@ -789,7 +828,7 @@ BuildSurfaceDisplayList (msurface_t *fa)
 	poly->numverts = lnumverts;
 
 	for (i = 0; i < lnumverts; i++) {
-		lindex = currentmodel->surfedges[fa->firstedge + i];
+		lindex = gl_currentmodel->surfedges[fa->firstedge + i];
 
 		if (lindex > 0) {
 			r_pedge = &pedges[lindex];
@@ -854,7 +893,6 @@ BuildSurfaceDisplayList (msurface_t *fa)
 						poly->verts[j - 1][k] = poly->verts[j][k];
 				}
 				--lnumverts;
-				++nColinElim;
 				// retry next vertex next time, which is now current vertex
 				--i;
 			}
