@@ -28,10 +28,8 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: d_zpoint.c,v 1.3 2003/01/15 15:31:24 taniwha Exp $";
-
 #include "d_local.h"
+#include "r_internal.h"
 
 
 void
