@@ -28,27 +28,24 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: nonintel.c,v 1.8 2003/02/12 23:41:20 despair Exp $";
-
 #ifdef PIC
 #undef USE_INTEL_ASM //XXX asm pic hack
 #endif
 
 #ifndef USE_INTEL_ASM
-#include "r_local.h"
+#include "r_internal.h"
 
 int         r_bmodelactive;
 
 void
 R_SurfPatch (void)
 {
-	// we only patch code on Intel
+	// we patch code only on Intel
 }
 
 void
 R_SurfacePatch (void)
 {
-	// we only patch code on Intel
+	// we patch code only on Intel
 }
 #endif // !USE_INTEL_ASM
