@@ -28,24 +28,20 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sw_ralias.c,v 1.18 2004/02/08 02:49:37 despair Exp $";
+#include <stdlib.h>
 
-#include "QF/console.h"
 #include "QF/image.h"
 #include "QF/render.h"
 #include "QF/skin.h"
 #include "QF/sys.h"
 
 #include "d_ifacea.h"
-#include "r_local.h"
-#include "stdlib.h"
+#include "r_internal.h"
 
 #define LIGHT_MIN	5					// lowest light value we'll allow, to
 										// avoid the need for inner-loop light
 										// clamping
 
-mtriangle_t *ptriangles;
 affinetridesc_t r_affinetridesc;
 
 void       *acolormap;					// FIXME: should go away
@@ -53,14 +49,14 @@ void       *acolormap;					// FIXME: should go away
 trivertx_t *r_apverts;
 
 // TODO: these probably will go away with optimized rasterization
-mdl_t      *pmdl;
+static mdl_t      *pmdl;
 vec3_t      r_plightvec;
 int         r_ambientlight;
 float       r_shadelight;
-aliashdr_t *paliashdr;
+static aliashdr_t *paliashdr;
 finalvert_t *pfinalverts;
 auxvert_t  *pauxverts;
-static float ziscale;
+float ziscale;
 static model_t *pmodel;
 
 static vec3_t alias_forward, alias_right, alias_up;
@@ -84,11 +80,6 @@ static aedge_t aedges[12] = {
 	{0, 5}, {1, 4}, {2, 7}, {3, 6}
 };
 
-float       r_avertexnormals[NUMVERTEXNORMALS][3] = {
-#include "anorms.h"
-};
-
-
 qboolean
 R_AliasCheckBBox (void)
 {
@@ -105,7 +96,8 @@ R_AliasCheckBBox (void)
 	// expand, rotate, and translate points into worldspace
 	currententity->trivial_accept = 0;
 	pmodel = currententity->model;
-	pahdr = Cache_Get (&pmodel->cache);
+	if (!(pahdr = pmodel->aliashdr))
+		pahdr = Cache_Get (&pmodel->cache);
 	pmdl = (mdl_t *) ((byte *) pahdr + pahdr->model);
 
 	R_AliasSetUpTransform (0);
@@ -114,7 +106,7 @@ R_AliasCheckBBox (void)
 	frame = currententity->frame;
 // TODO: don't repeat this check when drawing?
 	if ((frame >= pmdl->numframes) || (frame < 0)) {
-		Con_DPrintf ("No such frame %d %s\n", frame, pmodel->name);
+		Sys_MaskPrintf (SYS_DEV, "No such frame %d %s\n", frame, pmodel->name);
 		frame = 0;
 	}
 
@@ -158,7 +150,8 @@ R_AliasCheckBBox (void)
 	}
 
 	if (zfullyclipped) {
-		Cache_Release (&pmodel->cache);
+		if (!pmodel->aliashdr)
+			Cache_Release (&pmodel->cache);
 		return false;					// everything was near-z-clipped
 	}
 
@@ -220,7 +213,8 @@ R_AliasCheckBBox (void)
 	}
 
 	if (allclip) {
-		Cache_Release (&pmodel->cache);
+		if (!pmodel->aliashdr)
+			Cache_Release (&pmodel->cache);
 		return false;					// trivial reject off one side
 	}
 
@@ -232,7 +226,8 @@ R_AliasCheckBBox (void)
 		}
 	}
 
-	Cache_Release (&pmodel->cache);
+	if (!pmodel->aliashdr)
+		Cache_Release (&pmodel->cache);
 	return true;
 }
 
@@ -244,7 +239,7 @@ R_AliasTransformVector (vec3_t in, vec3_t out)
 	out[2] = DotProduct (in, aliastransform[2]) + aliastransform[2][3];
 }
 
-static void
+void
 R_AliasClipAndProjectFinalVert (finalvert_t *fv, auxvert_t *av)
 {
 	if (av->fv[2] < ALIAS_Z_CLIP_PLANE) {
@@ -265,8 +260,7 @@ R_AliasClipAndProjectFinalVert (finalvert_t *fv, auxvert_t *av)
 }
 
 static void
-R_AliasTransformFinalVert16 (finalvert_t *fv, auxvert_t *av,
-							 trivertx_t *pverts)
+R_AliasTransformFinalVert16 (auxvert_t *av, trivertx_t *pverts)
 {
 	trivertx_t  * pextra;
 	float       vextra[3];
@@ -284,7 +278,7 @@ R_AliasTransformFinalVert16 (finalvert_t *fv, auxvert_t *av,
 }
 
 static void
-R_AliasTransformFinalVert8 (finalvert_t *fv, auxvert_t *av, trivertx_t *pverts)
+R_AliasTransformFinalVert8 (auxvert_t *av, trivertx_t *pverts)
 {
 	av->fv[0] = DotProduct (pverts->v, aliastransform[0]) +
 		aliastransform[0][3];
@@ -317,15 +311,15 @@ R_AliasPreparePoints (void)
 	if (pmdl->ident == HEADER_MDL16) {
 		for (i = 0; i < r_anumverts; i++, fv++, av++, r_apverts++,
 				 pstverts++) {
-			R_AliasTransformFinalVert16 (fv, av, r_apverts);
-			R_AliasTransformFinalVert (fv, av, r_apverts, pstverts);
+			R_AliasTransformFinalVert16 (av, r_apverts);
+			R_AliasTransformFinalVert (fv, r_apverts, pstverts);
 			R_AliasClipAndProjectFinalVert (fv, av);
 		}
 	} else {
 		for (i = 0; i < r_anumverts; i++, fv++, av++, r_apverts++,
 				 pstverts++) {
-			R_AliasTransformFinalVert8 (fv, av, r_apverts);
-			R_AliasTransformFinalVert (fv, av, r_apverts, pstverts);
+			R_AliasTransformFinalVert8 (av, r_apverts);
+			R_AliasTransformFinalVert (fv, r_apverts, pstverts);
 			R_AliasClipAndProjectFinalVert (fv, av);
 		}
 	}
@@ -339,12 +333,12 @@ R_AliasPreparePoints (void)
 		pfv[1] = &pfinalverts[ptri->vertindex[1]];
 		pfv[2] = &pfinalverts[ptri->vertindex[2]];
 
-		if (pfv[0]->flags & pfv[1]->flags & pfv[2]->
-			flags & (ALIAS_XY_CLIP_MASK | ALIAS_Z_CLIP))
+		if (pfv[0]->flags & pfv[1]->flags & pfv[2]->flags
+			& (ALIAS_XY_CLIP_MASK | ALIAS_Z_CLIP))
 			continue;					// completely clipped
 
-		if (!((pfv[0]->flags | pfv[1]->flags | pfv[2]->flags) &
-			  (ALIAS_XY_CLIP_MASK | ALIAS_Z_CLIP))) {	// totally unclipped
+		if (!((pfv[0]->flags | pfv[1]->flags | pfv[2]->flags)
+			  & (ALIAS_XY_CLIP_MASK | ALIAS_Z_CLIP))) {	// totally unclipped
 			r_affinetridesc.pfinalverts = pfinalverts;
 			r_affinetridesc.ptriangles = ptri;
 			D_PolysetDraw ();
@@ -361,16 +355,10 @@ R_AliasSetUpTransform (int trivial_accept)
 	float       rotationmatrix[3][4], t2matrix[3][4];
 	static float tmatrix[3][4];
 	static float viewmatrix[3][4];
-	vec3_t      angles;
 
-// TODO: should really be stored with the entity instead of being reconstructed
-// TODO: should use a look-up table
-// TODO: could cache lazily, stored in the entity
-
-	angles[ROLL] = currententity->angles[ROLL];
-	angles[PITCH] = -currententity->angles[PITCH];
-	angles[YAW] = currententity->angles[YAW];
-	AngleVectors (angles, alias_forward, alias_right, alias_up);
+	VectorCopy (currententity->transform + 0, alias_forward);
+	VectorNegate (currententity->transform + 4, alias_right);
+	VectorCopy (currententity->transform + 8, alias_up);
 
 	tmatrix[0][0] = pmdl->scale[0];
 	tmatrix[1][1] = pmdl->scale[1];
@@ -398,7 +386,7 @@ R_AliasSetUpTransform (int trivial_accept)
 // TODO: should be global, set when vright, etc., set
 	VectorCopy (vright, viewmatrix[0]);
 	VectorCopy (vup, viewmatrix[1]);
-	VectorInverse (viewmatrix[1]);
+	VectorNegate (viewmatrix[1], viewmatrix[1]);
 	VectorCopy (vpn, viewmatrix[2]);
 
 //	viewmatrix[0][3] = 0;
@@ -431,8 +419,8 @@ R_AliasSetUpTransform (int trivial_accept)
 	lighting actual 3D transform is done by R_AliasTransformFinalVert8/16
 	functions above */
 void
-R_AliasTransformFinalVert (finalvert_t *fv, auxvert_t *av,
-						   trivertx_t *pverts, stvert_t *pstverts)
+R_AliasTransformFinalVert (finalvert_t *fv, trivertx_t *pverts,
+						   stvert_t *pstverts)
 {
 	int         temp;
 	float       lightcos, *plightnormal;
@@ -552,59 +540,35 @@ R_AliasPrepareUnclippedPoints (void)
 static void
 R_AliasSetupSkin (void)
 {
-	int         numskins, skinnum, i;
-	float       skintargettime, skintime;
-	float      *pskinintervals, fullskininterval;
-	maliasskingroup_t *paliasskingroup;
+	int         skinnum;
 
 	skinnum = currententity->skinnum;
 	if ((skinnum >= pmdl->numskins) || (skinnum < 0)) {
-		Con_DPrintf ("R_AliasSetupSkin: no such skin # %d\n", skinnum);
+		Sys_MaskPrintf (SYS_DEV, "R_AliasSetupSkin: no such skin # %d\n",
+						skinnum);
 		skinnum = 0;
 	}
 
-	pskindesc = ((maliasskindesc_t *)
-				 ((byte *) paliashdr + paliashdr->skindesc)) + skinnum;
+	pskindesc = R_AliasGetSkindesc (skinnum, paliashdr);
+
 	a_skinwidth = pmdl->skinwidth;
 
-	if (pskindesc->type == ALIAS_SKIN_GROUP) {
-		paliasskingroup = (maliasskingroup_t *) ((byte *) paliashdr +
-												 pskindesc->skin);
-		pskinintervals = (float *)
-			((byte *) paliashdr + paliasskingroup->intervals);
-		numskins = paliasskingroup->numskins;
-		fullskininterval = pskinintervals[numskins - 1];
-
-		skintime = r_realtime + currententity->syncbase;
-
-		// when loading in Mod_LoadAliasSkinGroup, we guaranteed all interval
-		// values are positive, so we don't have to worry about division by 0
-		skintargettime = skintime -
-			((int) (skintime / fullskininterval)) * fullskininterval;
-
-		for (i = 0; i < (numskins - 1); i++) {
-			if (pskinintervals[i] > skintargettime)
-				break;
-		}
-
-		pskindesc = &paliasskingroup->skindescs[i];
-	}
-
-	r_affinetridesc.pskindesc = pskindesc;
 	r_affinetridesc.pskin = (void *) ((byte *) paliashdr + pskindesc->skin);
 	r_affinetridesc.skinwidth = a_skinwidth;
 	r_affinetridesc.seamfixupX16 = (a_skinwidth >> 1) << 16;
 	r_affinetridesc.skinheight = pmdl->skinheight;
 
+	acolormap = vid.colormap8;
 	if (currententity->skin) {
 		tex_t      *base;
 
-		base = currententity->skin->data.texels;
+		base = currententity->skin->texels;
 		if (base) {
 			r_affinetridesc.pskin = base->data;
 			r_affinetridesc.skinwidth = base->width;
 			r_affinetridesc.skinheight = base->height;
 		}
+		acolormap = currententity->skin->colormap;
 	}
 }
 
@@ -645,42 +609,10 @@ R_AliasSetupLighting (alight_t *plighting)
 static void
 R_AliasSetupFrame (void)
 {
-	int         frame, numframes, i;
-	float		fullinterval, targettime, time;
-	float      *pintervals;
-	maliasgroup_t *paliasgroup;
+	maliasframedesc_t *frame;
 
-	frame = currententity->frame;
-	if ((frame >= pmdl->numframes) || (frame < 0)) {
-		Con_DPrintf ("R_AliasSetupFrame: no such frame %d\n", frame);
-		frame = 0;
-	}
-
-	if (paliashdr->frames[frame].type == ALIAS_SINGLE) {
-		r_apverts = (trivertx_t *)
-			((byte *) paliashdr + paliashdr->frames[frame].frame);
-		return;
-	}
-
-	paliasgroup = (maliasgroup_t *)
-		((byte *) paliashdr + paliashdr->frames[frame].frame);
-	pintervals = (float *) ((byte *) paliashdr + paliasgroup->intervals);
-	numframes = paliasgroup->numframes;
-	fullinterval = pintervals[numframes - 1];
-
-	time = r_realtime + currententity->syncbase;
-
-	// when loading in Mod_LoadAliasGroup, we guaranteed all interval values
-	// are positive, so we don't have to worry about division by 0
-	targettime = time - ((int) (time / fullinterval)) * fullinterval;
-
-	for (i = 0; i < (numframes - 1); i++) {
-		if (pintervals[i] > targettime)
-			break;
-	}
-
-	r_apverts = (trivertx_t *)
-		((byte *) paliashdr + paliasgroup->frames[i].frame);
+	frame = R_AliasGetFramedesc (currententity->frame, paliashdr);
+	r_apverts = (trivertx_t *) ((byte *) paliashdr + frame->frame);
 }
 
 
@@ -692,7 +624,8 @@ R_AliasDrawModel (alight_t *plighting)
 
 	r_amodels_drawn++;
 
-	paliashdr = Cache_Get (&currententity->model->cache);
+	if (!(paliashdr = currententity->model->aliashdr))
+		paliashdr = Cache_Get (&currententity->model->cache);
 	pmdl = (mdl_t *) ((byte *) paliashdr + paliashdr->model);
 
 	size = (CACHE_SIZE - 1)
@@ -704,7 +637,7 @@ R_AliasDrawModel (alight_t *plighting)
 
 	// cache align
 	pfinalverts = (finalvert_t *)
-		(((long) &finalverts[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
+		(((intptr_t) &finalverts[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
 	pauxverts = (auxvert_t *) &pfinalverts[pmdl->numverts + 1];
 
 	R_AliasSetupSkin ();
@@ -712,23 +645,21 @@ R_AliasDrawModel (alight_t *plighting)
 	R_AliasSetupLighting (plighting);
 	R_AliasSetupFrame ();
 
-	if (!currententity->colormap)
-		Sys_Error ("R_AliasDrawModel: !currententity->colormap");
-
 	r_affinetridesc.drawtype = (currententity->trivial_accept == 3) &&
 		r_recursiveaffinetriangles;
 
+	if (!acolormap)
+		acolormap = vid.colormap8;
+
 	if (r_affinetridesc.drawtype) {
 		D_PolysetUpdateTables ();		// FIXME: precalc...
 	} else {
 #ifdef USE_INTEL_ASM
-		D_Aff8Patch (currententity->colormap);
+		D_Aff8Patch (acolormap);
 #endif
 	}
 
-	acolormap = currententity->colormap;
-
-	if (currententity != r_view_model)
+	if (currententity != vr_data.view_model)
 		ziscale = (float) 0x8000 *(float) 0x10000;
 	else
 		ziscale = (float) 0x8000 *(float) 0x10000 *3.0;
@@ -738,5 +669,6 @@ R_AliasDrawModel (alight_t *plighting)
 	else
 		R_AliasPreparePoints ();
 
-	Cache_Release (&currententity->model->cache);
+	if (!currententity->model->aliashdr)
+		Cache_Release (&currententity->model->cache);
 }
