@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sw_rmain.c,v 1.28 2004/04/16 03:24:10 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -44,7 +41,6 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include <math.h>
 
 #include "QF/cmd.h"
-#include "QF/console.h"
 #include "QF/cvar.h"
 #include "QF/locs.h"
 #include "QF/mathlib.h"
@@ -53,18 +49,18 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/sound.h"
 #include "QF/sys.h"
 
-#include "r_cvar.h"
-#include "r_dynamic.h"
-#include "r_local.h"
-#include "view.h"
+#include "compat.h"
+#include "mod_internal.h"
+#include "r_internal.h"
+#include "vid_internal.h"
 
 #ifdef PIC
 # undef USE_INTEL_ASM //XXX asm pic hack
 #endif
 
 void       *colormap;
-vec3_t      viewlightvec;
-alight_t    r_viewlighting = { 128, 192, viewlightvec };
+static vec3_t      viewlightvec;
+static alight_t    r_viewlighting = { 128, 192, viewlightvec };
 int         r_numallocatededges;
 qboolean    r_drawpolys;
 qboolean    r_drawculledpolys;
@@ -74,35 +70,20 @@ int         r_pixbytes = 1;
 float       r_aliasuvscale = 1.0;
 int         r_outofsurfaces;
 int         r_outofedges;
-int			r_init = 0;
 
 qboolean    r_dowarp, r_dowarpold, r_viewchanged;
 
-int         numbtofpolys;
-btofpoly_t *pbtofpolys;
-mvertex_t  *r_pcurrentvertbase;
-
 int         c_surf;
-int         r_maxsurfsseen, r_maxedgesseen, r_cnumsurfs;
-qboolean    r_surfsonstack;
+int         r_maxsurfsseen, r_maxedgesseen;
+static int  r_cnumsurfs;
+static qboolean    r_surfsonstack;
 int         r_clipflags;
 
 byte       *r_warpbuffer;
 
-byte       *r_stack_start;
-
-qboolean    r_fov_greater_than_90;
-
-entity_t    r_worldentity;
-
-// view origin
-vec3_t      vup, base_vup;
-vec3_t      vpn, base_vpn;
-vec3_t      vright, base_vright;
-vec3_t      r_origin;
+static byte       *r_stack_start;
 
 // screen size info
-refdef_t    r_refdef;
 float       xcenter, ycenter;
 float       xscale, yscale;
 float       xscaleinv, yscaleinv;
@@ -112,44 +93,37 @@ float       aliasxscale, aliasyscale, aliasxcenter, aliasycenter;
 int         screenwidth;
 
 float       pixelAspect;
-float       screenAspect;
-float       verticalFieldOfView;
-float       xOrigin, yOrigin;
+static float       screenAspect;
+static float       verticalFieldOfView;
+static float       xOrigin, yOrigin;
 
-mplane_t    screenedge[4];
+plane_t     screenedge[4];
 
 // refresh flags
-int         r_framecount = 1;	// so frame counts initialized to 0 don't match
-int         r_visframecount;
-int         d_spanpixcount;
 int         r_polycount;
 int         r_drawnpolycount;
-int         r_wholepolycount;
 
 int        *pfrustum_indexes[4];
 int         r_frustum_indexes[4 * 6];
 
-int         reinit_surfcache = 1;	// if 1, surface cache is currently empty
-									// and must be reinitialized for current
-									// cache size
-
-mleaf_t    *r_viewleaf, *r_oldviewleaf;
-
 float       r_aliastransition, r_resfudge;
 
-int         d_lightstylevalue[256];		// 8.8 fraction of base light value
-
-float       dp_time1, dp_time2, db_time1, db_time2, rw_time1, rw_time2;
-float       se_time1, se_time2, de_time1, de_time2, dv_time1, dv_time2;
+static float dp_time1, dp_time2, db_time1, db_time2, rw_time1, rw_time2;
+static float se_time1, se_time2, de_time1, de_time2, dv_time1, dv_time2;
 
 void
-R_Init (void)
+sw_R_Init (void)
 {
 	int         dummy;
 
 	// get stack position so we can guess if we are going to overflow
 	r_stack_start = (byte *) & dummy;
 
+	R_Init_Cvars ();
+	R_Particles_Init_Cvars ();
+
+	Draw_Init ();
+	SCR_Init ();
 	R_SetFPCW ();
 #ifdef USE_INTEL_ASM
 	R_InitVars ();
@@ -182,6 +156,8 @@ R_Init (void)
 						   (long) R_EdgeCodeEnd - (long) R_EdgeCodeStart);
 #endif // USE_INTEL_ASM
 	D_Init ();
+
+	Skin_Init ();
 }
 
 void
@@ -192,12 +168,20 @@ R_NewMap (model_t *worldmodel, struct model_s **models, int num_models)
 	memset (&r_worldentity, 0, sizeof (r_worldentity));
 	r_worldentity.model = worldmodel;
 
+	R_FreeAllEntities ();
+
 	// clear out efrags in case the level hasn't been reloaded
 	// FIXME: is this one short?
 	for (i = 0; i < r_worldentity.model->numleafs; i++)
 		r_worldentity.model->leafs[i].efrags = NULL;
 
+	if (worldmodel->skytexture)
+		R_InitSky (worldmodel->skytexture);
+
+	// Force a vis update
 	r_viewleaf = NULL;
+	R_MarkLeaves ();
+
 	R_ClearParticles ();
 
 	r_cnumsurfs = r_maxsurfs->int_val;
@@ -278,11 +262,11 @@ R_ViewChanged (float aspect)
 	r_refdef.aliasvrectbottom = r_refdef.aliasvrect.y +
 		r_refdef.aliasvrect.height;
 
-	pixelAspect = (float)r_refdef.vrect.height/(float)r_refdef.vrect.width;
+	pixelAspect = vid.aspect;
 	xOrigin = r_refdef.xOrigin;
 	yOrigin = r_refdef.yOrigin;
 
-	screenAspect = 1.0;
+	screenAspect = r_refdef.vrect.width * pixelAspect / r_refdef.vrect.height;
 	// 320*200 1.0 pixelAspect = 1.6 screenAspect
 	// 320*240 1.0 pixelAspect = 1.3333 screenAspect
 	// proper 320*200 pixelAspect = 0.8333333
@@ -346,11 +330,6 @@ R_ViewChanged (float aspect)
 	r_aliastransition = r_aliastransbase->value * res_scale;
 	r_resfudge = r_aliastransadj->value * res_scale;
 
-	if (scr_fov->value <= 90.0)
-		r_fov_greater_than_90 = false;
-	else
-		r_fov_greater_than_90 = true;
-
 // TODO: collect 386-specific code in one place
 #ifdef USE_INTEL_ASM
 	Sys_MakeCodeWriteable ((long) R_Surf8Start,
@@ -362,90 +341,25 @@ R_ViewChanged (float aspect)
 	D_ViewChanged ();
 }
 
-void
-R_MarkLeaves (void)
-{
-	byte       *vis;
-	mnode_t    *node;
-	mleaf_t    *leaf;
-	msurface_t **mark;
-	int         c;
-	int         i;
-
-	if (r_oldviewleaf == r_viewleaf)
-		return;
-
-	r_visframecount++;
-	r_oldviewleaf = r_viewleaf;
-
-	vis = Mod_LeafPVS (r_viewleaf, r_worldentity.model);
-
-	for (i = 0; i < r_worldentity.model->numleafs; i++) {
-		if (vis[i >> 3] & (1 << (i & 7))) {
-			leaf = &r_worldentity.model->leafs[i + 1];
-			mark = leaf->firstmarksurface;
-			c = leaf->nummarksurfaces;
-			if (c) {
-				do {
-					(*mark)->visframe = r_visframecount;
-					mark++;
-				} while (--c);
-			}
-
-			node = (mnode_t *) leaf;
-			do {
-				if (node->visframe == r_visframecount)
-					break;
-				node->visframe = r_visframecount;
-				node = node->parent;
-			} while (node);
-		}
-	}
-}
-#if 0
-static void
-R_ShowNearestLoc (void)
-{
-	dlight_t   *dl;
-	location_t *nearloc;
-	vec3_t      trueloc;
-	
-	if (r_drawentities->int_val)
-		return;
-
-	nearloc = locs_find (r_origin);
-	if (nearloc) {
-		dl = R_AllocDlight (4096);
-		if (dl) {
-			VectorCopy (nearloc->loc, dl->origin);
-			dl->radius = 200;
-			dl->die = r_realtime + 0.1;
-			dl->color[1] = 1;
-		}
-
-		VectorCopy(nearloc->loc, trueloc);
-		R_RunParticleEffect(trueloc, vec3_origin, 252, 10);
-	}
-}
-#endif
-
 static void
 R_DrawEntitiesOnList (void)
 {
-	int         i, j;
+	int         j;
 	unsigned int lnum;
 	alight_t    lighting;
+	entity_t   *ent;
 
 	// FIXME: remove and do real lighting
 	float       lightvec[3] = { -1, 0, 0 };
 	vec3_t      dist;
 	float       add;
+	float       minlight = 0;
 
 	if (!r_drawentities->int_val)
 		return;
 
-	for (i = 0; i < r_numvisedicts; i++) {
-		currententity = r_visedicts[i];
+	for (ent = r_ent_queue; ent; ent = ent->next) {
+		currententity = ent;
 
 		switch (currententity->model->type) {
 			case mod_sprite:
@@ -455,13 +369,19 @@ R_DrawEntitiesOnList (void)
 				break;
 
 			case mod_alias:
+			case mod_iqm:
 				VectorCopy (currententity->origin, r_entorigin);
 				VectorSubtract (r_origin, r_entorigin, modelorg);
 
+				minlight = max (currententity->model->min_light, currententity->min_light);
+
 				// see if the bounding box lets us trivially reject, also
 				// sets trivial accept status
-				if (R_AliasCheckBBox ()) {
-					j = R_LightPoint (currententity->origin);
+				currententity->trivial_accept = 0;	//FIXME
+				if (currententity->model->type == mod_iqm//FIXME
+					|| R_AliasCheckBBox ()) {
+					// 128 instead of 255 due to clamping below
+					j = max (R_LightPoint (currententity->origin), minlight * 128);
 
 					lighting.ambientlight = j;
 					lighting.shadelight = j;
@@ -469,7 +389,7 @@ R_DrawEntitiesOnList (void)
 					lighting.plightvec = lightvec;
 
 					for (lnum = 0; lnum < r_maxdlights; lnum++) {
-						if (r_dlights[lnum].die >= r_realtime) {
+						if (r_dlights[lnum].die >= vr_data.realtime) {
 							VectorSubtract (currententity->origin,
 											r_dlights[lnum].origin, dist);
 							add = r_dlights[lnum].radius - VectorLength (dist);
@@ -485,7 +405,10 @@ R_DrawEntitiesOnList (void)
 					if (lighting.ambientlight + lighting.shadelight > 192)
 						lighting.shadelight = 192 - lighting.ambientlight;
 
-					R_AliasDrawModel (&lighting);
+					if (currententity->model->type == mod_iqm)
+						R_IQMDrawModel (&lighting);
+					else
+						R_AliasDrawModel (&lighting);
 				}
 
 				break;
@@ -505,14 +428,15 @@ R_DrawViewModel (void)
 	unsigned int lnum;
 	vec3_t      dist;
 	float       add;
+	float       minlight;
 	dlight_t   *dl;
 
-	if (r_inhibit_viewmodel
+	if (vr_data.inhibit_viewmodel
 		|| !r_drawviewmodel->int_val
 		|| !r_drawentities->int_val)
 		return;
 
-	currententity = r_view_model;
+	currententity = vr_data.view_model;
 	if (!currententity->model)
 		return;
 
@@ -520,23 +444,23 @@ R_DrawViewModel (void)
 	VectorSubtract (r_origin, r_entorigin, modelorg);
 
 	VectorCopy (vup, viewlightvec);
-	VectorInverse (viewlightvec);
+	VectorNegate (viewlightvec, viewlightvec);
 
-	j = R_LightPoint (currententity->origin);
+	minlight = max (currententity->min_light, currententity->model->min_light);
+
+	j = max (R_LightPoint (currententity->origin), minlight * 128);
 
-	if (j < 24)
-		j = 24;							// always give some light on gun
 	r_viewlighting.ambientlight = j;
 	r_viewlighting.shadelight = j;
 
-	// add dynamic lights       
+	// add dynamic lights
 	for (lnum = 0; lnum < r_maxdlights; lnum++) {
 		dl = &r_dlights[lnum];
 		if (!dl->radius)
 			continue;
 		if (!dl->radius)
 			continue;
-		if (dl->die < r_realtime)
+		if (dl->die < vr_data.realtime)
 			continue;
 
 		VectorSubtract (currententity->origin, dl->origin, dist);
@@ -565,8 +489,8 @@ R_BmodelCheckBBox (model_t *clmodel, float *minmaxs)
 
 	clipflags = 0;
 
-	if (currententity->angles[0] || currententity->angles[1]
-		|| currententity->angles[2]) {
+	if (currententity->transform[0] != 1 || currententity->transform[5] != 1
+		|| currententity->transform[10] != 1) {
 		for (i = 0; i < 4; i++) {
 			d = DotProduct (currententity->origin, view_clipplanes[i].normal);
 			d -= view_clipplanes[i].dist;
@@ -613,11 +537,12 @@ R_BmodelCheckBBox (model_t *clmodel, float *minmaxs)
 static void
 R_DrawBEntitiesOnList (void)
 {
-	int         i, j, clipflags;
+	int         j, clipflags;
 	unsigned int k;
 	vec3_t      oldorigin;
 	model_t    *clmodel;
 	float       minmaxs[6];
+	entity_t   *ent;
 
 	if (!r_drawentities->int_val)
 		return;
@@ -625,8 +550,8 @@ R_DrawBEntitiesOnList (void)
 	VectorCopy (modelorg, oldorigin);
 	insubmodel = true;
 
-	for (i = 0; i < r_numvisedicts; i++) {
-		currententity = r_visedicts[i];
+	for (ent = r_ent_queue; ent; ent = ent->next) {
+		currententity = ent;
 
 		switch (currententity->model->type) {
 			case mod_brush:
@@ -659,7 +584,7 @@ R_DrawBEntitiesOnList (void)
 						vec3_t      lightorigin;
 
 						for (k = 0; k < r_maxdlights; k++) {
-							if ((r_dlights[k].die < r_realtime) ||
+							if ((r_dlights[k].die < vr_data.realtime) ||
 								(!r_dlights[k].radius)) continue;
 
 							VectorSubtract (r_dlights[k].origin,
@@ -676,20 +601,11 @@ R_DrawBEntitiesOnList (void)
 					if (r_drawpolys | r_drawculledpolys) {
 						R_ZDrawSubmodelPolys (clmodel);
 					} else {
-						r_pefragtopnode = NULL;
+						if (currententity->topnode) {
+							mnode_t    *topnode = currententity->topnode;
 
-						for (j = 0; j < 3; j++) {
-							r_emins[j] = minmaxs[j];
-							r_emaxs[j] = minmaxs[3 + j];
-						}
-
-						R_SplitEntityOnNode2 (r_worldentity.model->nodes);
-
-						if (r_pefragtopnode) {
-							currententity->topnode = r_pefragtopnode;
-
-							if (r_pefragtopnode->contents >= 0) {
-								// not a leaf; has to be clipped to the world 
+							if (topnode->contents >= 0) {
+								// not a leaf; has to be clipped to the world
 								// BSP
 								r_clipflags = clipflags;
 								R_DrawSolidClippedSubmodelPolygons (clmodel);
@@ -700,11 +616,10 @@ R_DrawBEntitiesOnList (void)
 								R_DrawSubmodelPolygons (clmodel, clipflags);
 							}
 
-							currententity->topnode = NULL;
 						}
 					}
 
-					// put back world rotation and frustum clipping     
+					// put back world rotation and frustum clipping
 					// FIXME: R_RotateBmodel should just work off base_vxx
 					VectorCopy (base_vpn, vpn);
 					VectorCopy (base_vup, vup);
@@ -722,6 +637,28 @@ R_DrawBEntitiesOnList (void)
 	insubmodel = false;
 }
 
+static void
+R_PrintDSpeeds (void)
+{
+	float       ms, dp_time, r_time2, rw_time, db_time, se_time, de_time,
+
+		dv_time;
+
+	r_time2 = Sys_DoubleTime ();
+
+	dp_time = (dp_time2 - dp_time1) * 1000;
+	rw_time = (rw_time2 - rw_time1) * 1000;
+	db_time = (db_time2 - db_time1) * 1000;
+	se_time = (se_time2 - se_time1) * 1000;
+	de_time = (de_time2 - de_time1) * 1000;
+	dv_time = (dv_time2 - dv_time1) * 1000;
+	ms = (r_time2 - r_time1) * 1000;
+
+	Sys_Printf ("%3i %4.1fp %3iw %4.1fb %3is %4.1fe %4.1fv\n",
+				(int) ms, dp_time, (int) rw_time, db_time, (int) se_time,
+				de_time, dv_time);
+}
+
 static void
 R_EdgeDrawing (void)
 {
@@ -734,12 +671,12 @@ R_EdgeDrawing (void)
 		r_edges = auxedges;
 	} else {
 		r_edges = (edge_t *)
-			(((long) &ledges[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
+			(((intptr_t) &ledges[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
 	}
 
 	if (r_surfsonstack) {
 		surfaces = (surf_t *)
-			(((long) &lsurfs[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
+			(((intptr_t) &lsurfs[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
 		surf_max = &surfaces[r_cnumsurfs];
 		// surface 0 doesn't really exist; it's just a dummy because index 0
 		// is used to indicate no edge attached to surface
@@ -858,8 +795,6 @@ R_RenderView_ (void)
 	if (r_dowarp)
 		D_WarpScreen ();
 
-	V_SetContentsColor (r_viewleaf->contents);
-
 	if (r_timegraph->int_val)
 		R_TimeGraph ();
 
@@ -876,10 +811,10 @@ R_RenderView_ (void)
 		R_PrintDSpeeds ();
 
 	if (r_reportsurfout->int_val && r_outofsurfaces)
-		Con_Printf ("Short %d surfaces\n", r_outofsurfaces);
+		Sys_Printf ("Short %d surfaces\n", r_outofsurfaces);
 
 	if (r_reportedgeout->int_val && r_outofedges)
-		Con_Printf ("Short roughly %d edges\n", r_outofedges * 2 / 3);
+		Sys_Printf ("Short roughly %d edges\n", r_outofedges * 2 / 3);
 
 	// back to high floating-point precision
 	R_HighFPPrecision ();
@@ -900,10 +835,10 @@ R_RenderView (void)
 	if (Hunk_LowMark () & 3)
 		Sys_Error ("Hunk is missaligned");
 
-	if ((long) (&dummy) & 3)
+	if ((intptr_t) (&dummy) & 3)
 		Sys_Error ("Stack is missaligned");
 
-	if ((long) (&r_warpbuffer) & 3)
+	if ((intptr_t) (&r_warpbuffer) & 3)
 		Sys_Error ("Globals are missaligned");
 
 	if (!scr_fisheye->int_val)
@@ -1148,8 +1083,8 @@ renderside (byte* bufs, double yaw, double pitch, double roll, int side)
 static void
 renderlookup (byte **offs, byte* bufs)
 {
-	byte *p = (byte*)vid.buffer;
-	unsigned int x, y;
+	byte       *p = (byte*)vid.buffer;
+	int        x, y;
 	for (y = 0; y < vid.height; y++) {
 		for (x = 0; x < vid.width; x++, offs++)
 		    p[x] = **offs;
@@ -1211,6 +1146,9 @@ R_RenderViewFishEye (void)
 }
 
 void
-gl_overbright_f (cvar_t *un)
+R_ClearState (void)
 {
+	R_ClearEfrags ();
+	R_ClearDlights ();
+	R_ClearParticles ();
 }
