@@ -28,30 +28,35 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sw_rpart.c,v 1.33 2003/10/23 03:53:59 despair Exp $";
-
 #include <stdlib.h>
+#ifdef HAVE_STRING_H
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
 
-#include "QF/console.h"
 #include "QF/cvar.h"
+#include "QF/mersenne.h"
 #include "QF/qargs.h"
 #include "QF/quakefs.h"
 #include "QF/render.h"
+#include "QF/sys.h"
+#include "QF/va.h"
 
 #include "compat.h"
-#include "r_cvar.h"
-#include "r_dynamic.h"
-#include "r_local.h"
+#include "r_internal.h"
 
-int			ramp1[8] = { 0x6f, 0x6d, 0x6b, 0x69, 0x67, 0x65, 0x63, 0x61 };
-int			ramp2[8] = { 0x6f, 0x6e, 0x6d, 0x6c, 0x6b, 0x6a, 0x68, 0x66 };
-int			ramp3[8] = { 0x6d, 0x6b, 6, 5, 4, 3 };
+static int		ramp1[8] = { 0x6f, 0x6d, 0x6b, 0x69, 0x67, 0x65, 0x63, 0x61 };
+//static int	ramp2[8] = { 0x6f, 0x6e, 0x6d, 0x6c, 0x6b, 0x6a, 0x68, 0x66 };
+static int		ramp3[8] = { 0x6d, 0x6b, 6, 5, 4, 3 };
 
+static mtstate_t mt;	// private PRNG state
 
 void
 R_InitParticles (void)
 {
+	mtwist_seed (&mt, 0xdeadbeef);
 }
 
 void
@@ -75,17 +80,24 @@ R_ReadPointFile_f (void)
 	vec3_t      org;
 	int         c, r;
 	particle_t *p;
-	char        name[MAX_OSPATH];
+	const char *name;
+	char       *mapname;
 
-	// FIXME    snprintf (name, sizeof (name), "maps/%s.pts", sv.name);
+	mapname = strdup (r_worldentity.model->name);
+	if (!mapname)
+		Sys_Error ("Can't duplicate mapname!");
+	QFS_StripExtension (mapname, mapname);
+
+	name = va ("maps/%s.pts", mapname);
+	free (mapname);
 
 	QFS_FOpenFile (name, &f);
 	if (!f) {
-		Con_Printf ("couldn't open %s\n", name);
+		Sys_Printf ("couldn't open %s\n", name);
 		return;
 	}
 
-	Con_Printf ("Reading %s...\n", name);
+	Sys_Printf ("Reading %s...\n", name);
 	c = 0;
 	for (;;) {
 		char        buf[64];
@@ -97,7 +109,7 @@ R_ReadPointFile_f (void)
 		c++;
 
 		if (!free_particles) {
-			Con_Printf ("Not enough free particles\n");
+			Sys_Printf ("Not enough free particles\n");
 			break;
 		}
 		p = free_particles;
@@ -108,12 +120,13 @@ R_ReadPointFile_f (void)
 		p->die = 99999;
 		p->color = (-c) & 15;
 		p->type = pt_static;
+		p->phys = R_ParticlePhysics (p->type);
 		VectorZero (p->vel);
 		VectorCopy (org, p->org);
 	}
 
 	Qclose (f);
-	Con_Printf ("%i points read\n", c);
+	Sys_Printf ("%i points read\n", c);
 }
 
 static void
@@ -133,16 +146,17 @@ R_ParticleExplosion_QF (const vec3_t org)
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 5;
+		p->die = vr_data.realtime + 5;
 		p->color = ramp1[0];
-		p->ramp = rand () & 3;
+		p->ramp = mtwist_rand (&mt) & 3;
 		if (i & 1)
 			p->type = pt_explode;
 		else
 			p->type = pt_explode2;
+		p->phys = R_ParticlePhysics (p->type);
 		for (j = 0; j < 3; j++) {
-			p->org[j] = org[j] + ((rand () % 32) - 16);
-			p->vel[j] = (rand () % 512) - 256;
+			p->org[j] = org[j] + ((mtwist_rand (&mt) % 32) - 16);
+			p->vel[j] = (mtwist_rand (&mt) % 512) - 256;
 		}
 	}
 }
@@ -162,14 +176,15 @@ R_ParticleExplosion2_QF (const vec3_t org, int colorStart, int colorLength)
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 0.3;
+		p->die = vr_data.realtime + 0.3;
 		p->color = colorStart + (colorMod % colorLength);
 		colorMod++;
 
 		p->type = pt_blob;
+		p->phys = R_ParticlePhysics (p->type);
 		for (j=0 ; j<3 ; j++) {
-			p->org[j] = org[j] + ((rand()%32)-16);
-			p->vel[j] = (rand()%512)-256;
+			p->org[j] = org[j] + ((mtwist_rand (&mt)%32)-16);
+			p->vel[j] = (mtwist_rand (&mt)%512)-256;
 		}
 	}
 }
@@ -191,18 +206,19 @@ R_BlobExplosion_QF (const vec3_t org)
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 1 + (rand () & 8) * 0.05;
+		p->die = vr_data.realtime + 1 + (mtwist_rand (&mt) & 8) * 0.05;
 
 		if (i & 1) {
 			p->type = pt_blob;
-			p->color = 66 + rand () % 6;
+			p->color = 66 + mtwist_rand (&mt) % 6;
 		} else {
 			p->type = pt_blob2;
-			p->color = 150 + rand () % 6;
+			p->color = 150 + mtwist_rand (&mt) % 6;
 		}
+		p->phys = R_ParticlePhysics (p->type);
 		for (j = 0; j < 3; j++) {
-			p->org[j] = org[j] + ((rand () % 32) - 16);
-			p->vel[j] = (rand () % 512) - 256;
+			p->org[j] = org[j] + ((mtwist_rand (&mt) % 32) - 16);
+			p->vel[j] = (mtwist_rand (&mt) % 512) - 256;
 		}
 	}
 }
@@ -225,12 +241,13 @@ R_RunParticleEffect_QF (const vec3_t org, const vec3_t dir, int color,
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 0.1 * (rand () % 5);
-		p->color = (color & ~7) + (rand () & 7);
+		p->die = vr_data.realtime + 0.1 * (mtwist_rand (&mt) % 5);
+		p->color = (color & ~7) + (mtwist_rand (&mt) & 7);
 		p->type = pt_grav;
+		p->phys = R_ParticlePhysics (p->type);
 		for (j = 0; j < 3; j++) {
-			p->org[j] = org[j] + ((rand () & 15) - 8);
-			p->vel[j] = dir[j];	// + (rand()%300)-150;
+			p->org[j] = org[j] + ((mtwist_rand (&mt) & 15) - 8);
+			p->vel[j] = dir[j];	// + (mtwist_rand (&mt)%300)-150;
 		}
 	}
 }
@@ -244,7 +261,7 @@ R_SpikeEffect_QF (const vec3_t org)
 static void
 R_SuperSpikeEffect_QF (const vec3_t org)
 {
-	R_RunParticleEffect (org, vec3_origin, 0, 20);
+	R_RunParticleEffect_QF (org, vec3_origin, 0, 20);
 }
 
 static void
@@ -298,20 +315,21 @@ R_LavaSplash_QF (const vec3_t org)
 				p->next = active_particles;
 				active_particles = p;
 
-				p->die = r_realtime + 2 + (rand () & 31) * 0.02;
-				p->color = 224 + (rand () & 7);
+				p->die = vr_data.realtime + 2 + (mtwist_rand (&mt) & 31) * 0.02;
+				p->color = 224 + (mtwist_rand (&mt) & 7);
 				p->type = pt_grav;
+				p->phys = R_ParticlePhysics (p->type);
 
-				dir[0] = j * 8 + (rand () & 7);
-				dir[1] = i * 8 + (rand () & 7);
+				dir[0] = j * 8 + (mtwist_rand (&mt) & 7);
+				dir[1] = i * 8 + (mtwist_rand (&mt) & 7);
 				dir[2] = 256;
 
 				p->org[0] = org[0] + dir[0];
 				p->org[1] = org[1] + dir[1];
-				p->org[2] = org[2] + (rand () & 63);
+				p->org[2] = org[2] + (mtwist_rand (&mt) & 63);
 
 				VectorNormalize (dir);
-				vel = 50 + (rand () & 63);
+				vel = 50 + (mtwist_rand (&mt) & 63);
 				VectorScale (dir, vel, p->vel);
 			}
 }
@@ -337,20 +355,21 @@ R_TeleportSplash_QF (const vec3_t org)
 				p->next = active_particles;
 				active_particles = p;
 
-				p->die = r_realtime + 0.2 + (rand () & 7) * 0.02;
-				p->color = 7 + (rand () & 7);
+				p->die = vr_data.realtime + 0.2 + (mtwist_rand (&mt) & 7) * 0.02;
+				p->color = 7 + (mtwist_rand (&mt) & 7);
 				p->type = pt_grav;
+				p->phys = R_ParticlePhysics (p->type);
 
 				dir[0] = j * 8;
 				dir[1] = i * 8;
 				dir[2] = k * 8;
 
-				p->org[0] = org[0] + i + (rand () & 3);
-				p->org[1] = org[1] + j + (rand () & 3);
-				p->org[2] = org[2] + k + (rand () & 3);
+				p->org[0] = org[0] + i + (mtwist_rand (&mt) & 3);
+				p->org[1] = org[1] + j + (mtwist_rand (&mt) & 3);
+				p->org[2] = org[2] + k + (mtwist_rand (&mt) & 3);
 
 				VectorNormalize (dir);
-				vel = 50 + (rand () & 63);
+				vel = 50 + (mtwist_rand (&mt) & 63);
 				VectorScale (dir, vel, p->vel);
 			}
 }
@@ -380,11 +399,12 @@ R_DarkFieldParticles_ID (const entity_t *ent)
                 p->next = active_particles;
 				active_particles = p;
 
-				rnd = rand ();
+				rnd = mtwist_rand (&mt);
 
-				p->die = r_realtime + 0.2 + (rnd & 7) * 0.02;
-				p->color = 150 + rand () % 6;
+				p->die = vr_data.realtime + 0.2 + (rnd & 7) * 0.02;
+				p->color = 150 + mtwist_rand (&mt) % 6;
 				p->type = pt_slowgrav;
+				p->phys = R_ParticlePhysics (p->type);
 				dir[0] = j * 8;
 				dir[1] = i * 8;
 				dir[2] = k * 8;
@@ -417,18 +437,18 @@ R_EntityParticles_ID (const entity_t *ent)
 
 	if (!avelocities[0][0]) {
 		for (i = 0; i < NUMVERTEXNORMALS * 3; i++)
-			avelocities[0][i] = (rand () & 255) * 0.01;
+			avelocities[0][i] = (mtwist_rand (&mt) & 255) * 0.01;
 	}
 
 	for (i = 0; i < NUMVERTEXNORMALS; i++) {
-		angle = r_realtime * avelocities[i][0];
+		angle = vr_data.realtime * avelocities[i][0];
 		cy = cos (angle);
 		sy = sin (angle);
-		angle = r_realtime * avelocities[i][1];
+		angle = vr_data.realtime * avelocities[i][1];
 		cp = cos (angle);
 		sp = sin (angle);
 // Next 3 lines results aren't currently used, may be in future. --Despair
-//		angle = r_realtime * avelocities[i][2];
+//		angle = vr_data.realtime * avelocities[i][2];
 //		sr = sin (angle);
 //		cr = cos (angle);
 
@@ -443,9 +463,10 @@ R_EntityParticles_ID (const entity_t *ent)
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 0.01;
+		p->die = vr_data.realtime + 0.01;
 		p->color = 0x6f;
 		p->type = pt_explode;
+		p->phys = R_ParticlePhysics (p->type);
 
 		p->org[0] = ent->origin[0] + r_avertexnormals[i][0] * dist +
 			forward[0] * beamlength;
@@ -483,12 +504,13 @@ R_RocketTrail_QF (const entity_t *ent)
 
 		VectorZero (p->vel);
 
-		p->die = r_realtime + 2;
-		p->ramp = (rand () & 3);
+		p->die = vr_data.realtime + 2;
+		p->ramp = (mtwist_rand (&mt) & 3);
 		p->color = ramp3[(int) p->ramp];
 		p->type = pt_fire;
+		p->phys = R_ParticlePhysics (p->type);
 		for (j = 0; j < 3; j++)
-			p->org[j] = old_origin[j] + ((rand () % 6) - 3);
+			p->org[j] = old_origin[j] + ((mtwist_rand (&mt) % 6) - 3);
 
 		VectorAdd (old_origin, vec, old_origin);
 	}
@@ -521,12 +543,13 @@ R_GrenadeTrail_QF (const entity_t *ent)
 
 		VectorZero (p->vel);
 
-		p->die = r_realtime + 2;
-		p->ramp = (rand () & 3) + 2;
+		p->die = vr_data.realtime + 2;
+		p->ramp = (mtwist_rand (&mt) & 3) + 2;
 		p->color = ramp3[(int) p->ramp];
 		p->type = pt_fire;
+		p->phys = R_ParticlePhysics (p->type);
 		for (j = 0; j < 3; j++)
-			p->org[j] = old_origin[j] + ((rand () % 6) - 3);
+			p->org[j] = old_origin[j] + ((mtwist_rand (&mt) % 6) - 3);
 
 		VectorAdd (old_origin, vec, old_origin);
 	}
@@ -559,11 +582,12 @@ R_BloodTrail_QF (const entity_t *ent)
 
 		VectorZero (p->vel);
 
-		p->die = r_realtime + 2;
+		p->die = vr_data.realtime + 2;
 		p->type = pt_slowgrav;
-		p->color = 67 + (rand () & 3);
+		p->phys = R_ParticlePhysics (p->type);
+		p->color = 67 + (mtwist_rand (&mt) & 3);
 		for (j = 0; j < 3; j++)
-			p->org[j] = old_origin[j] + ((rand () % 6) - 3);
+			p->org[j] = old_origin[j] + ((mtwist_rand (&mt) % 6) - 3);
 		break;
 
 		VectorAdd (old_origin, vec, old_origin);
@@ -597,11 +621,12 @@ R_SlightBloodTrail_QF (const entity_t *ent)
 
 		VectorZero (p->vel);
 
-		p->die = r_realtime + 2;
+		p->die = vr_data.realtime + 2;
 		p->type = pt_slowgrav;
-		p->color = 67 + (rand () & 3);
+		p->phys = R_ParticlePhysics (p->type);
+		p->color = 67 + (mtwist_rand (&mt) & 3);
 		for (j = 0; j < 3; j++)
-			p->org[j] = old_origin[j] + ((rand () % 6) - 3);
+			p->org[j] = old_origin[j] + ((mtwist_rand (&mt) % 6) - 3);
 
 		VectorAdd (old_origin, vec, old_origin);
 	}
@@ -633,8 +658,9 @@ R_WizTrail_QF (const entity_t *ent)
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 0.5;
+		p->die = vr_data.realtime + 0.5;
 		p->type = pt_static;
+		p->phys = R_ParticlePhysics (p->type);
 		p->color = 52 + ((tracercount & 4) << 1);
 
 		tracercount++;
@@ -679,8 +705,9 @@ R_FlameTrail_QF (const entity_t *ent)
 		p->next = active_particles;
 		active_particles = p;
 
-		p->die = r_realtime + 0.5;
+		p->die = vr_data.realtime + 0.5;
 		p->type = pt_static;
+		p->phys = R_ParticlePhysics (p->type);
 		p->color = 230 + ((tracercount & 4) << 1);
 
 		tracercount++;
@@ -726,11 +753,12 @@ R_VoorTrail_QF (const entity_t *ent)
 
 		VectorZero (p->vel);
 
-		p->die = r_realtime + 0.3;
+		p->die = vr_data.realtime + 0.3;
 		p->type = pt_static;
-		p->color = 9 * 16 + 8 + (rand () & 3);
+		p->phys = R_ParticlePhysics (p->type);
+		p->color = 9 * 16 + 8 + (mtwist_rand (&mt) & 3);
 		for (j = 0; j < 3; j++)
-			p->org[j] = old_origin[j] + ((rand () & 15) - 8);
+			p->org[j] = old_origin[j] + ((mtwist_rand (&mt) & 15) - 8);
 
 		VectorAdd (old_origin, vec, old_origin);
 	}
@@ -739,25 +767,14 @@ R_VoorTrail_QF (const entity_t *ent)
 void
 R_DrawParticles (void)
 {
-	int         i;
-	float       dvel, frametime, grav, time1, time2, time3;
 	particle_t *p, **particle;
 
-	D_StartParticles ();
-
 	VectorScale (vright, xscaleshrink, r_pright);
 	VectorScale (vup, yscaleshrink, r_pup);
 	VectorCopy (vpn, r_ppn);
 
-	frametime = r_frametime;
-	time3 = frametime * 15;
-	time2 = frametime * 10;				// 15;
-	time1 = frametime * 5;
-	grav = frametime * r_gravity * 0.05;
-	dvel = 4 * frametime;
-
 	for (particle = &active_particles; *particle;) {
-		if ((*particle)->die < r_realtime) {
+		if ((*particle)->die < vr_data.realtime) {
 			p = (*particle)->next;
 			(*particle)->next = free_particles;
 			free_particles = (*particle);
@@ -768,62 +785,9 @@ R_DrawParticles (void)
 
 			D_DrawParticle (p);
 
-			p->org[0] += p->vel[0] * frametime;
-			p->org[1] += p->vel[1] * frametime;
-			p->org[2] += p->vel[2] * frametime;
-
-			switch (p->type) {
-				case pt_static:
-					break;
-				case pt_fire:
-					p->ramp += time1;
-					if (p->ramp >= 6)
-						p->die = -1;
-					else
-						p->color = ramp3[(int) p->ramp];
-					p->vel[2] += grav;
-					break;
-				case pt_explode:
-					p->ramp += time2;
-					if (p->ramp >= 8)
-						p->die = -1;
-					else
-						p->color = ramp1[(int) p->ramp];
-					for (i = 0; i < 3; i++)
-						p->vel[i] += p->vel[i] * dvel;
-					p->vel[2] -= grav;
-					break;
-				case pt_explode2:
-					p->ramp += time3;
-					if (p->ramp >= 8)
-						p->die = -1;
-					else
-						p->color = ramp2[(int) p->ramp];
-					for (i = 0; i < 3; i++)
-						p->vel[i] -= p->vel[i] * frametime;
-					p->vel[2] -= grav;
-					break;
-				case pt_blob:
-					for (i = 0; i < 3; i++)
-						p->vel[i] += p->vel[i] * dvel;
-					p->vel[2] -= grav;
-					break;
-				case pt_blob2:
-					for (i = 0; i < 2; i++)
-						p->vel[i] -= p->vel[i] * dvel;
-					p->vel[2] -= grav;
-					break;
-				case pt_slowgrav:
-				case pt_grav:
-					p->vel[2] -= grav;
-					break;
-				default:
-					Con_DPrintf ("unhandled particle type %d\n", p->type);
-					break;
-			}
+			p->phys (p);
 		}
 	}
-	D_EndParticles ();
 }
 
 void
@@ -836,38 +800,122 @@ r_particles_style_f (cvar_t *var)
 {
 }
 
+static vid_particle_funcs_t particles_QF = {
+	R_RocketTrail_QF,
+	R_GrenadeTrail_QF,
+	R_BloodTrail_QF,
+	R_SlightBloodTrail_QF,
+	R_WizTrail_QF,
+	R_FlameTrail_QF,
+	R_VoorTrail_QF,
+	0,//R_GlowTrail_QF,
+	R_RunParticleEffect_QF,
+	R_BloodPuffEffect_QF,
+	R_GunshotEffect_QF,
+	R_LightningBloodEffect_QF,
+	R_SpikeEffect_QF,
+	R_KnightSpikeEffect_QF,
+	R_SuperSpikeEffect_QF,
+	R_WizSpikeEffect_QF,
+	R_BlobExplosion_QF,
+	R_ParticleExplosion_QF,
+	R_ParticleExplosion2_QF,
+	R_LavaSplash_QF,
+	R_TeleportSplash_QF,
+	R_DarkFieldParticles_ID,
+	R_EntityParticles_ID,
+	R_Particle_New,
+	R_Particle_NewRandom,
+};
+
 static void
 R_ParticleFunctionInit (void)
 {
-	R_BlobExplosion = R_BlobExplosion_QF;
-	R_ParticleExplosion = R_ParticleExplosion_QF;
-	R_ParticleExplosion2 = R_ParticleExplosion2_QF;
-	R_LavaSplash = R_LavaSplash_QF;
-	R_TeleportSplash = R_TeleportSplash_QF;
-	R_DarkFieldParticles = R_DarkFieldParticles_ID;
-	R_EntityParticles = R_EntityParticles_ID;
+	sw_vid_render_funcs.particles = &particles_QF;
+}
 
-	R_BloodPuffEffect = R_BloodPuffEffect_QF;
-	R_GunshotEffect = R_GunshotEffect_QF;
-	R_LightningBloodEffect = R_LightningBloodEffect_QF;
+static void
+r_particles_nearclip_f (cvar_t *var)
+{
+	Cvar_SetValue (r_particles_nearclip, bound (r_nearclip->value, var->value,
+												r_farclip->value));
+}
 
-	R_RunParticleEffect = R_RunParticleEffect_QF;
-	R_SpikeEffect = R_SpikeEffect_QF;
-	R_SuperSpikeEffect = R_SuperSpikeEffect_QF;
-	R_KnightSpikeEffect = R_KnightSpikeEffect_QF;
-	R_WizSpikeEffect = R_WizSpikeEffect_QF;
+static void
+r_particles_f (cvar_t *var)
+{
+	R_MaxParticlesCheck (var, r_particles_max);
+}
 
-	R_RocketTrail = R_RocketTrail_QF;
-	R_GrenadeTrail = R_GrenadeTrail_QF;
-	R_BloodTrail = R_BloodTrail_QF;
-	R_SlightBloodTrail = R_SlightBloodTrail_QF;
-	R_WizTrail = R_WizTrail_QF;
-	R_FlameTrail = R_FlameTrail_QF;
-	R_VoorTrail = R_VoorTrail_QF;
+static void
+r_particles_max_f (cvar_t *var)
+{
+	R_MaxParticlesCheck (r_particles, var);
 }
 
 void
 R_Particles_Init_Cvars (void)
 {
+	easter_eggs = Cvar_Get ("easter_eggs", "0", CVAR_NONE, r_easter_eggs_f,
+							"Enables easter eggs.");
+	r_particles = Cvar_Get ("r_particles", "1", CVAR_ARCHIVE, r_particles_f,
+							"Toggles drawing of particles.");
+	r_particles_max = Cvar_Get ("r_particles_max", "2048", CVAR_ARCHIVE,
+								r_particles_max_f, "Maximum amount of "
+								"particles to display. No maximum, minimum "
+								"is 0.");
+	r_particles_nearclip = Cvar_Get ("r_particles_nearclip", "32",
+									 CVAR_ARCHIVE, r_particles_nearclip_f,
+									 "Distance of the particle near clipping "
+									 "plane from the player.");
+	r_particles_style = Cvar_Get ("r_particles_style", "1", CVAR_ARCHIVE,
+								  r_particles_style_f, "Sets particle style. "
+								  "0 for Id, 1 for QF.");
 	R_ParticleFunctionInit ();
 }
+
+void
+R_Particle_New (ptype_t type, int texnum, const vec3_t org, float scale,
+			    const vec3_t vel, float die, int color, float alpha, float ramp)
+{
+	particle_t *p;
+
+	if (!free_particles)
+		return;
+	p = free_particles;
+	free_particles = p->next;
+	p->next = active_particles;
+	active_particles = p;
+
+	VectorCopy (org, p->org);
+	p->color = color;
+	p->tex = texnum;
+	p->scale = scale;
+	p->alpha = alpha;
+	VectorCopy (vel, p->vel);
+	p->type = type;
+	p->phys = R_ParticlePhysics (p->type);
+	p->die = die;
+	p->ramp = ramp;
+}
+
+void
+R_Particle_NewRandom (ptype_t type, int texnum, const vec3_t org, int org_fuzz,
+					  float scale, int vel_fuzz, float die, int color,
+					  float alpha, float ramp)
+{
+	float       o_fuzz = org_fuzz, v_fuzz = vel_fuzz;
+	int         rnd;
+	vec3_t      porg, pvel;
+
+	rnd = mtwist_rand (&mt);
+	porg[0] = o_fuzz * ((rnd & 63) - 31.5) / 63.0 + org[0];
+	porg[1] = o_fuzz * (((rnd >> 6) & 63) - 31.5) / 63.0 + org[1];
+	porg[2] = o_fuzz * (((rnd >> 10) & 63) - 31.5) / 63.0 + org[2];
+	rnd = mtwist_rand (&mt);
+	pvel[0] = v_fuzz * ((rnd & 63) - 31.5) / 63.0;
+	pvel[1] = v_fuzz * (((rnd >> 6) & 63) - 31.5) / 63.0;
+	pvel[2] = v_fuzz * (((rnd >> 10) & 63) - 31.5) / 63.0;
+
+	R_Particle_New (type, texnum, porg, scale, pvel, die, color, alpha, ramp);
+}
