@@ -28,17 +28,16 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sw32_ralias.c,v 1.15 2004/02/08 02:49:38 despair Exp $";
+#define NH_DEFINE
+#include "namehack.h"
 
-#include "QF/console.h"
 #include "QF/image.h"
 #include "QF/render.h"
 #include "QF/skin.h"
 #include "QF/sys.h"
 
 #include "d_ifacea.h"
-#include "r_local.h"
+#include "r_internal.h"
 
 #include "stdlib.h"
 
@@ -46,33 +45,32 @@ static __attribute__ ((unused)) const char rcsid[] =
 										// avoid the need for inner-loop light
 										// clamping
 
-mtriangle_t *ptriangles;
-affinetridesc_t r_affinetridesc;
+affinetridesc_t sw32_r_affinetridesc;
 
-void       *acolormap;					// FIXME: should go away
+void       *sw32_acolormap;					// FIXME: should go away
 
-trivertx_t *r_apverts;
+trivertx_t *sw32_r_apverts;
 
 // TODO: these probably will go away with optimized rasterization
-mdl_t      *pmdl;
-vec3_t      r_plightvec;
-int         r_ambientlight;
-float       r_shadelight;
-aliashdr_t *paliashdr;
-finalvert_t *pfinalverts;
-auxvert_t  *pauxverts;
-static float ziscale;
+static mdl_t      *pmdl;
+vec3_t      sw32_r_plightvec;
+int         sw32_r_ambientlight;
+float       sw32_r_shadelight;
+static aliashdr_t *paliashdr;
+finalvert_t *sw32_pfinalverts;
+auxvert_t  *sw32_pauxverts;
+float sw32_ziscale;
 static model_t *pmodel;
 
 static vec3_t alias_forward, alias_right, alias_up;
 
 static maliasskindesc_t *pskindesc;
 
-int         r_amodels_drawn;
-int         a_skinwidth;
-int         r_anumverts;
+int         sw32_r_amodels_drawn;
+static int         a_skinwidth;
+static int         r_anumverts;
 
-float       aliastransform[3][4];
+float       sw32_aliastransform[3][4];
 
 typedef struct {
 	int         index0;
@@ -85,12 +83,8 @@ static aedge_t aedges[12] = {
 	{0, 5}, {1, 4}, {2, 7}, {3, 6}
 };
 
-float       r_avertexnormals[NUMVERTEXNORMALS][3] = {
-#include "anorms.h"
-};
-
 qboolean
-R_AliasCheckBBox (void)
+sw32_R_AliasCheckBBox (void)
 {
 	int         i, flags, frame, numv;
 	aliashdr_t *pahdr;
@@ -105,16 +99,17 @@ R_AliasCheckBBox (void)
 	// expand, rotate, and translate points into worldspace
 	currententity->trivial_accept = 0;
 	pmodel = currententity->model;
-	pahdr = Cache_Get (&pmodel->cache);
+	if (!(pahdr = pmodel->aliashdr))
+		pahdr = Cache_Get (&pmodel->cache);
 	pmdl = (mdl_t *) ((byte *) pahdr + pahdr->model);
 
-	R_AliasSetUpTransform (0);
+	sw32_R_AliasSetUpTransform (0);
 
 	// construct the base bounding box for this frame
 	frame = currententity->frame;
 // TODO: don't repeat this check when drawing?
 	if ((frame >= pmdl->numframes) || (frame < 0)) {
-		Con_DPrintf ("No such frame %d %s\n", frame, pmodel->name);
+		Sys_MaskPrintf (SYS_DEV, "No such frame %d %s\n", frame, pmodel->name);
 		frame = 0;
 	}
 
@@ -143,7 +138,7 @@ R_AliasCheckBBox (void)
 
 	minz = 9999;
 	for (i = 0; i < 8; i++) {
-		R_AliasTransformVector (&basepts[i][0], &viewaux[i].fv[0]);
+		sw32_R_AliasTransformVector (&basepts[i][0], &viewaux[i].fv[0]);
 
 		if (viewaux[i].fv[2] < ALIAS_Z_CLIP_PLANE) {
 			// we must clip points that are closer than the near clip plane
@@ -158,7 +153,8 @@ R_AliasCheckBBox (void)
 	}
 
 	if (zfullyclipped) {
-		Cache_Release (&pmodel->cache);
+		if (!pmodel->aliashdr)
+			Cache_Release (&pmodel->cache);
 		return false;					// everything was near-z-clipped
 	}
 
@@ -201,8 +197,8 @@ R_AliasCheckBBox (void)
 		zi = 1.0 / viewaux[i].fv[2];
 
 		// FIXME: do with chop mode in ASM, or convert to float
-		v0 = (viewaux[i].fv[0] * xscale * zi) + xcenter;
-		v1 = (viewaux[i].fv[1] * yscale * zi) + ycenter;
+		v0 = (viewaux[i].fv[0] * sw32_xscale * zi) + sw32_xcenter;
+		v1 = (viewaux[i].fv[1] * sw32_yscale * zi) + sw32_ycenter;
 
 		flags = 0;
 
@@ -220,41 +216,46 @@ R_AliasCheckBBox (void)
 	}
 
 	if (allclip) {
-		Cache_Release (&pmodel->cache);
+		if (!pmodel->aliashdr)
+			Cache_Release (&pmodel->cache);
 		return false;					// trivial reject off one side
 	}
 
 	currententity->trivial_accept = !anyclip & !zclipped;
 
 	if (currententity->trivial_accept) {
-		if (minz > (r_aliastransition + (pmdl->size * r_resfudge))) {
+		if (minz > (sw32_r_aliastransition + (pmdl->size * sw32_r_resfudge))) {
 			currententity->trivial_accept |= 2;
 		}
 	}
 
-	Cache_Release (&pmodel->cache);
+	if (!pmodel->aliashdr)
+		Cache_Release (&pmodel->cache);
 	return true;
 }
 
 
 void
-R_AliasTransformVector (vec3_t in, vec3_t out)
+sw32_R_AliasTransformVector (vec3_t in, vec3_t out)
 {
-	out[0] = DotProduct (in, aliastransform[0]) + aliastransform[0][3];
-	out[1] = DotProduct (in, aliastransform[1]) + aliastransform[1][3];
-	out[2] = DotProduct (in, aliastransform[2]) + aliastransform[2][3];
+	out[0] = DotProduct (in, sw32_aliastransform[0])
+		+ sw32_aliastransform[0][3];
+	out[1] = DotProduct (in, sw32_aliastransform[1])
+		+ sw32_aliastransform[1][3];
+	out[2] = DotProduct (in, sw32_aliastransform[2])
+		+ sw32_aliastransform[2][3];
 }
 
 
-static void
-R_AliasClipAndProjectFinalVert (finalvert_t *fv, auxvert_t *av)
+void
+sw32_R_AliasClipAndProjectFinalVert (finalvert_t *fv, auxvert_t *av)
 {
 	if (av->fv[2] < ALIAS_Z_CLIP_PLANE) {
 		fv->flags |= ALIAS_Z_CLIP;
 		return;
 	}
 
-	R_AliasProjectFinalVert (fv, av);
+	sw32_R_AliasProjectFinalVert (fv, av);
 
 	if (fv->v[0] < r_refdef.aliasvrect.x)
 		fv->flags |= ALIAS_LEFT_CLIP;
@@ -267,7 +268,7 @@ R_AliasClipAndProjectFinalVert (finalvert_t *fv, auxvert_t *av)
 }
 
 static void
-R_AliasTransformFinalVert16 (finalvert_t *fv, auxvert_t *av, trivertx_t *pverts)
+R_AliasTransformFinalVert16 (auxvert_t *av, trivertx_t *pverts)
 {
 	trivertx_t  * pextra;
 	float       vextra[3];
@@ -276,23 +277,23 @@ R_AliasTransformFinalVert16 (finalvert_t *fv, auxvert_t *av, trivertx_t *pverts)
 	vextra[0] = pverts->v[0] + pextra->v[0] / (float)256;
 	vextra[1] = pverts->v[1] + pextra->v[1] / (float)256;
 	vextra[2] = pverts->v[2] + pextra->v[2] / (float)256;
-	av->fv[0] = DotProduct (vextra, aliastransform[0]) +
-		aliastransform[0][3];
-	av->fv[1] = DotProduct (vextra, aliastransform[1]) +
-		aliastransform[1][3];
-	av->fv[2] = DotProduct (vextra, aliastransform[2]) +
-		aliastransform[2][3];
+	av->fv[0] = DotProduct (vextra, sw32_aliastransform[0]) +
+		sw32_aliastransform[0][3];
+	av->fv[1] = DotProduct (vextra, sw32_aliastransform[1]) +
+		sw32_aliastransform[1][3];
+	av->fv[2] = DotProduct (vextra, sw32_aliastransform[2]) +
+		sw32_aliastransform[2][3];
 }
 
 static void
-R_AliasTransformFinalVert8 (finalvert_t *fv, auxvert_t *av, trivertx_t *pverts)
+R_AliasTransformFinalVert8 (auxvert_t *av, trivertx_t *pverts)
 {
-	av->fv[0] = DotProduct (pverts->v, aliastransform[0]) +
-		aliastransform[0][3];
-	av->fv[1] = DotProduct (pverts->v, aliastransform[1]) +
-		aliastransform[1][3];
-	av->fv[2] = DotProduct (pverts->v, aliastransform[2]) +
-		aliastransform[2][3];
+	av->fv[0] = DotProduct (pverts->v, sw32_aliastransform[0]) +
+		sw32_aliastransform[0][3];
+	av->fv[1] = DotProduct (pverts->v, sw32_aliastransform[1]) +
+		sw32_aliastransform[1][3];
+	av->fv[2] = DotProduct (pverts->v, sw32_aliastransform[2]) +
+		sw32_aliastransform[2][3];
 }
 
 /*
@@ -313,27 +314,27 @@ R_AliasPreparePoints (void)
 	pstverts = (stvert_t *) ((byte *) paliashdr + paliashdr->stverts);
 	r_anumverts = pmdl->numverts;
 	fv = pfinalverts;
-	av = pauxverts;
+	av = sw32_pauxverts;
 
 	if (pmdl->ident == HEADER_MDL16) {
-		for (i = 0; i < r_anumverts; i++, fv++, av++, r_apverts++,
+		for (i = 0; i < r_anumverts; i++, fv++, av++, sw32_r_apverts++,
 				pstverts++) {
-			R_AliasTransformFinalVert16 (fv, av, r_apverts);
-			R_AliasTransformFinalVert (fv, av, r_apverts, pstverts);
+			R_AliasTransformFinalVert16 (av, sw32_r_apverts);
+			sw32_R_AliasTransformFinalVert (fv, sw32_r_apverts, pstverts);
 			R_AliasClipAndProjectFinalVert (fv, av);
 		}
 	}
 	else {
-		for (i = 0; i < r_anumverts; i++, fv++, av++, r_apverts++,
+		for (i = 0; i < r_anumverts; i++, fv++, av++, sw32_r_apverts++,
 				pstverts++) {
-			R_AliasTransformFinalVert8 (fv, av, r_apverts);
-			R_AliasTransformFinalVert (fv, av, r_apverts, pstverts);
+			R_AliasTransformFinalVert8 (av, sw32_r_apverts);
+			sw32_R_AliasTransformFinalVert (fv, sw32_r_apverts, pstverts);
 			R_AliasClipAndProjectFinalVert (fv, av);
 		}
 	}
 
 	// clip and draw all triangles
-	r_affinetridesc.numtriangles = 1;
+	sw32_r_affinetridesc.numtriangles = 1;
 
 	ptri = (mtriangle_t *) ((byte *) paliashdr + paliashdr->triangles);
 	for (i = 0; i < pmdl->numtris; i++, ptri++) {
@@ -347,33 +348,27 @@ R_AliasPreparePoints (void)
 
 		if (!((pfv[0]->flags | pfv[1]->flags | pfv[2]->flags) &
 			  (ALIAS_XY_CLIP_MASK | ALIAS_Z_CLIP))) {	// totally unclipped
-			r_affinetridesc.pfinalverts = pfinalverts;
-			r_affinetridesc.ptriangles = ptri;
-			D_PolysetDraw ();
+			sw32_r_affinetridesc.pfinalverts = pfinalverts;
+			sw32_r_affinetridesc.ptriangles = ptri;
+			sw32_D_PolysetDraw ();
 		} else {						// partially clipped
-			R_AliasClipTriangle (ptri);
+			sw32_R_AliasClipTriangle (ptri);
 		}
 	}
 }
 
 
 void
-R_AliasSetUpTransform (int trivial_accept)
+sw32_R_AliasSetUpTransform (int trivial_accept)
 {
 	int         i;
 	float       rotationmatrix[3][4], t2matrix[3][4];
 	static float tmatrix[3][4];
 	static float viewmatrix[3][4];
-	vec3_t      angles;
 
-// TODO: should really be stored with the entity instead of being reconstructed
-// TODO: should use a look-up table
-// TODO: could cache lazily, stored in the entity
-
-	angles[ROLL] = currententity->angles[ROLL];
-	angles[PITCH] = -currententity->angles[PITCH];
-	angles[YAW] = currententity->angles[YAW];
-	AngleVectors (angles, alias_forward, alias_right, alias_up);
+	VectorCopy (currententity->transform + 0, alias_forward);
+	VectorNegate (currententity->transform + 4, alias_right);
+	VectorCopy (currententity->transform + 8, alias_up);
 
 	tmatrix[0][0] = pmdl->scale[0];
 	tmatrix[1][1] = pmdl->scale[1];
@@ -401,14 +396,14 @@ R_AliasSetUpTransform (int trivial_accept)
 // TODO: should be global, set when vright, etc., set
 	VectorCopy (vright, viewmatrix[0]);
 	VectorCopy (vup, viewmatrix[1]);
-	VectorInverse (viewmatrix[1]);
+	VectorNegate (viewmatrix[1], viewmatrix[1]);
 	VectorCopy (vpn, viewmatrix[2]);
 
 //	viewmatrix[0][3] = 0;
 //	viewmatrix[1][3] = 0;
 //	viewmatrix[2][3] = 0;
 
-	R_ConcatTransforms (viewmatrix, rotationmatrix, aliastransform);
+	R_ConcatTransforms (viewmatrix, rotationmatrix, sw32_aliastransform);
 
 // do the scaling up of x and y to screen coordinates as part of the transform
 // for the unclipped case (it would mess up clipping in the clipped case).
@@ -417,24 +412,24 @@ R_AliasSetUpTransform (int trivial_accept)
 // FIXME: make this work for clipped case too?
 	if (trivial_accept) {
 		for (i = 0; i < 4; i++) {
-			aliastransform[0][i] *= aliasxscale *
+			sw32_aliastransform[0][i] *= sw32_aliasxscale *
 				(1.0 / ((float) 0x8000 * 0x10000));
-			aliastransform[1][i] *= aliasyscale *
+			sw32_aliastransform[1][i] *= sw32_aliasyscale *
 				(1.0 / ((float) 0x8000 * 0x10000));
-			aliastransform[2][i] *= 1.0 / ((float) 0x8000 * 0x10000);
+			sw32_aliastransform[2][i] *= 1.0 / ((float) 0x8000 * 0x10000);
 		}
 	}
 }
 
 /*
-R_AliasTransformFinalVert
+sw32_R_AliasTransformFinalVert
 
 now this function just copies the texture coordinates and calculates lighting
 actual 3D transform is done by R_AliasTransformFinalVert8/16 functions above
 */
 void
-R_AliasTransformFinalVert (finalvert_t *fv, auxvert_t *av,
-						   trivertx_t *pverts, stvert_t *pstverts)
+sw32_R_AliasTransformFinalVert (finalvert_t *fv, trivertx_t *pverts,
+								stvert_t *pstverts)
 {
 	int         temp;
 	float       lightcos, *plightnormal;
@@ -447,11 +442,11 @@ R_AliasTransformFinalVert (finalvert_t *fv, auxvert_t *av,
 	// lighting
 	// LordHavoc: flipped lightcos so it is + for bright, not -
 	plightnormal = r_avertexnormals[pverts->lightnormalindex];
-	lightcos = -DotProduct (plightnormal, r_plightvec);
-	temp = r_ambientlight;
+	lightcos = -DotProduct (plightnormal, sw32_r_plightvec);
+	temp = sw32_r_ambientlight;
 
 	if (lightcos > 0) {
-		temp += (int) (r_shadelight * lightcos);
+		temp += (int) (sw32_r_shadelight * lightcos);
 
 		// clamp; because we limited the minimum ambient and shading light,
 		// we don't have to clamp low light, just bright
@@ -463,28 +458,28 @@ R_AliasTransformFinalVert (finalvert_t *fv, auxvert_t *av,
 }
 
 void
-R_AliasTransformAndProjectFinalVerts (finalvert_t *fv, stvert_t *pstverts)
+sw32_R_AliasTransformAndProjectFinalVerts (finalvert_t *fv, stvert_t *pstverts)
 {
 	int         i, temp;
 	float       lightcos, *plightnormal, zi;
 	trivertx_t *pverts;
 
-	pverts = r_apverts;
+	pverts = sw32_r_apverts;
 
 	for (i = 0; i < r_anumverts; i++, fv++, pverts++, pstverts++) {
 		// transform and project
-		zi = 1.0 / (DotProduct (pverts->v, aliastransform[2]) +
-					aliastransform[2][3]);
+		zi = 1.0 / (DotProduct (pverts->v, sw32_aliastransform[2]) +
+					sw32_aliastransform[2][3]);
 
 		// x, y, and z are scaled down by 1/2**31 in the transform, so 1/z is
 		// scaled up by 1/2**31, and the scaling cancels out for x and y in
 		// the projection
 		fv->v[5] = zi;
 
-		fv->v[0] = ((DotProduct (pverts->v, aliastransform[0]) +
-					 aliastransform[0][3]) * zi) + aliasxcenter;
-		fv->v[1] = ((DotProduct (pverts->v, aliastransform[1]) +
-					 aliastransform[1][3]) * zi) + aliasycenter;
+		fv->v[0] = ((DotProduct (pverts->v, sw32_aliastransform[0]) +
+					 sw32_aliastransform[0][3]) * zi) + sw32_aliasxcenter;
+		fv->v[1] = ((DotProduct (pverts->v, sw32_aliastransform[1]) +
+					 sw32_aliastransform[1][3]) * zi) + sw32_aliasycenter;
 
 		fv->v[2] = pstverts->s;
 		fv->v[3] = pstverts->t;
@@ -493,11 +488,11 @@ R_AliasTransformAndProjectFinalVerts (finalvert_t *fv, stvert_t *pstverts)
 		// lighting
 		// LordHavoc: flipped lightcos so it is + for bright, not -
 		plightnormal = r_avertexnormals[pverts->lightnormalindex];
-		lightcos = -DotProduct (plightnormal, r_plightvec);
-		temp = r_ambientlight;
+		lightcos = -DotProduct (plightnormal, sw32_r_plightvec);
+		temp = sw32_r_ambientlight;
 
 		if (lightcos > 0) {
-			temp += (int) (r_shadelight * lightcos);
+			temp += (int) (sw32_r_shadelight * lightcos);
 
 			// clamp; because we limited the minimum ambient and shading
 			// light, we don't have to clamp low light, just bright
@@ -510,17 +505,17 @@ R_AliasTransformAndProjectFinalVerts (finalvert_t *fv, stvert_t *pstverts)
 }
 
 void
-R_AliasProjectFinalVert (finalvert_t *fv, auxvert_t *av)
+sw32_R_AliasProjectFinalVert (finalvert_t *fv, auxvert_t *av)
 {
 	float       zi;
 
 	// project points
 	zi = 1.0 / av->fv[2];
 
-	fv->v[5] = zi * ziscale;
+	fv->v[5] = zi * sw32_ziscale;
 
-	fv->v[0] = (av->fv[0] * aliasxscale * zi) + aliasxcenter;
-	fv->v[1] = (av->fv[1] * aliasyscale * zi) + aliasycenter;
+	fv->v[0] = (av->fv[0] * sw32_aliasxscale * zi) + sw32_aliasxcenter;
+	fv->v[1] = (av->fv[1] * sw32_aliasyscale * zi) + sw32_aliasycenter;
 }
 
 
@@ -532,14 +527,14 @@ R_AliasPrepareUnclippedPoints (void)
 	pstverts = (stvert_t *) ((byte *) paliashdr + paliashdr->stverts);
 	r_anumverts = pmdl->numverts;
 
-	R_AliasTransformAndProjectFinalVerts (pfinalverts, pstverts);
+	sw32_R_AliasTransformAndProjectFinalVerts (pfinalverts, pstverts);
 
-	r_affinetridesc.pfinalverts = pfinalverts;
-	r_affinetridesc.ptriangles = (mtriangle_t *)
+	sw32_r_affinetridesc.pfinalverts = pfinalverts;
+	sw32_r_affinetridesc.ptriangles = (mtriangle_t *)
 		((byte *) paliashdr + paliashdr->triangles);
-	r_affinetridesc.numtriangles = pmdl->numtris;
+	sw32_r_affinetridesc.numtriangles = pmdl->numtris;
 
-	D_PolysetDraw ();
+	sw32_D_PolysetDraw ();
 }
 
 
@@ -547,59 +542,33 @@ static void
 R_AliasSetupSkin (void)
 {
 	int         skinnum;
-	int         i, numskins;
-	maliasskingroup_t *paliasskingroup;
-	float      *pskinintervals, fullskininterval;
-	float       skintargettime, skintime;
 
 	skinnum = currententity->skinnum;
 	if ((skinnum >= pmdl->numskins) || (skinnum < 0)) {
-		Con_DPrintf ("R_AliasSetupSkin: no such skin # %d\n", skinnum);
+		Sys_MaskPrintf (SYS_DEV, "R_AliasSetupSkin: no such skin # %d\n",
+						skinnum);
 		skinnum = 0;
 	}
 
-	pskindesc = ((maliasskindesc_t *)
-				 ((byte *) paliashdr + paliashdr->skindesc)) + skinnum;
+	pskindesc = R_AliasGetSkindesc (skinnum, paliashdr);
 	a_skinwidth = pmdl->skinwidth;
 
-	if (pskindesc->type == ALIAS_SKIN_GROUP) {
-		paliasskingroup = (maliasskingroup_t *) ((byte *) paliashdr +
-												 pskindesc->skin);
-		pskinintervals = (float *)
-			((byte *) paliashdr + paliasskingroup->intervals);
-		numskins = paliasskingroup->numskins;
-		fullskininterval = pskinintervals[numskins - 1];
-
-		skintime = r_realtime + currententity->syncbase;
-
-		// when loading in Mod_LoadAliasSkinGroup, we guaranteed all interval
-		// values are positive, so we don't have to worry about division by 0
-		skintargettime = skintime -
-			((int) (skintime / fullskininterval)) * fullskininterval;
-
-		for (i = 0; i < (numskins - 1); i++) {
-			if (pskinintervals[i] > skintargettime)
-				break;
-		}
-
-		pskindesc = &paliasskingroup->skindescs[i];
-	}
-
-	r_affinetridesc.pskindesc = pskindesc;
-	r_affinetridesc.pskin = (void *) ((byte *) paliashdr + pskindesc->skin);
-	r_affinetridesc.skinwidth = a_skinwidth;
-	r_affinetridesc.seamfixupX16 = (a_skinwidth >> 1) << 16;
-	r_affinetridesc.skinheight = pmdl->skinheight;
+	sw32_r_affinetridesc.pskin = (void *) ((byte *) paliashdr + pskindesc->skin);
+	sw32_r_affinetridesc.skinwidth = a_skinwidth;
+	sw32_r_affinetridesc.seamfixupX16 = (a_skinwidth >> 1) << 16;
+	sw32_r_affinetridesc.skinheight = pmdl->skinheight;
 
+	sw32_acolormap = vid.colormap8;
 	if (currententity->skin) {
 		tex_t      *base;
 
-		base = currententity->skin->data.texels;
+		base = currententity->skin->texels;
 		if (base) {
-			r_affinetridesc.pskin = base->data;
-			r_affinetridesc.skinwidth = base->width;
-			r_affinetridesc.skinheight = base->height;
+			sw32_r_affinetridesc.pskin = base->data;
+			sw32_r_affinetridesc.skinwidth = base->width;
+			sw32_r_affinetridesc.skinheight = base->height;
 		}
+		sw32_acolormap = currententity->skin->colormap;
 	}
 }
 
@@ -608,86 +577,55 @@ R_AliasSetupLighting (alight_t *plighting)
 {
 	// guarantee that no vertex will ever be lit below LIGHT_MIN, so we don't
 	// have to clamp off the bottom
-	r_ambientlight = plighting->ambientlight;
+	sw32_r_ambientlight = plighting->ambientlight;
 
-	if (r_ambientlight < LIGHT_MIN)
-		r_ambientlight = LIGHT_MIN;
+	if (sw32_r_ambientlight < LIGHT_MIN)
+		sw32_r_ambientlight = LIGHT_MIN;
 
-	r_ambientlight = (/*255 -*/ r_ambientlight) << VID_CBITS;
+	sw32_r_ambientlight = (/*255 -*/ sw32_r_ambientlight) << VID_CBITS;
 
-//	if (r_ambientlight < LIGHT_MIN)
-//		r_ambientlight = LIGHT_MIN;
+//	if (sw32_r_ambientlight < LIGHT_MIN)
+//		sw32_r_ambientlight = LIGHT_MIN;
 
-	r_shadelight = plighting->shadelight;
+	sw32_r_shadelight = plighting->shadelight;
 
-	if (r_shadelight < 0)
-		r_shadelight = 0;
+	if (sw32_r_shadelight < 0)
+		sw32_r_shadelight = 0;
 
-	r_shadelight *= VID_GRADES;
+	sw32_r_shadelight *= VID_GRADES;
 
 	// rotate the lighting vector into the model's frame of reference
-	r_plightvec[0] = DotProduct (plighting->plightvec, alias_forward);
-	r_plightvec[1] = -DotProduct (plighting->plightvec, alias_right);
-	r_plightvec[2] = DotProduct (plighting->plightvec, alias_up);
+	sw32_r_plightvec[0] = DotProduct (plighting->plightvec, alias_forward);
+	sw32_r_plightvec[1] = -DotProduct (plighting->plightvec, alias_right);
+	sw32_r_plightvec[2] = DotProduct (plighting->plightvec, alias_up);
 }
 
 
 /*
 	R_AliasSetupFrame
 
-	set r_apverts
+	set sw32_r_apverts
 */
 static void
 R_AliasSetupFrame (void)
 {
-	int         frame;
-	int         i, numframes;
-	maliasgroup_t *paliasgroup;
-	float      *pintervals, fullinterval, targettime, time;
+	maliasframedesc_t *frame;
 
-	frame = currententity->frame;
-	if ((frame >= pmdl->numframes) || (frame < 0)) {
-		Con_DPrintf ("R_AliasSetupFrame: no such frame %d\n", frame);
-		frame = 0;
-	}
-
-	if (paliashdr->frames[frame].type == ALIAS_SINGLE) {
-		r_apverts = (trivertx_t *)
-			((byte *) paliashdr + paliashdr->frames[frame].frame);
-		return;
-	}
-
-	paliasgroup = (maliasgroup_t *)
-		((byte *) paliashdr + paliashdr->frames[frame].frame);
-	pintervals = (float *) ((byte *) paliashdr + paliasgroup->intervals);
-	numframes = paliasgroup->numframes;
-	fullinterval = pintervals[numframes - 1];
-
-	time = r_realtime + currententity->syncbase;
-
-	// when loading in Mod_LoadAliasGroup, we guaranteed all interval values
-	// are positive, so we don't have to worry about division by 0
-	targettime = time - ((int) (time / fullinterval)) * fullinterval;
-
-	for (i = 0; i < (numframes - 1); i++) {
-		if (pintervals[i] > targettime)
-			break;
-	}
-
-	r_apverts = (trivertx_t *)
-		((byte *) paliashdr + paliasgroup->frames[i].frame);
+	frame = R_AliasGetFramedesc (currententity->frame, paliashdr);
+	sw32_r_apverts = (trivertx_t *) ((byte *) paliashdr + frame->frame);
 }
 
 
 void
-R_AliasDrawModel (alight_t *plighting)
+sw32_R_AliasDrawModel (alight_t *plighting)
 {
 	int         size;
 	finalvert_t *finalverts;
 
-	r_amodels_drawn++;
+	sw32_r_amodels_drawn++;
 
-	paliashdr = Cache_Get (&currententity->model->cache);
+	if (!(paliashdr = currententity->model->aliashdr))
+		paliashdr = Cache_Get (&currententity->model->cache);
 	pmdl = (mdl_t *) ((byte *) paliashdr + paliashdr->model);
 
 	size = (CACHE_SIZE - 1)
@@ -699,38 +637,37 @@ R_AliasDrawModel (alight_t *plighting)
 
 	// cache align
 	pfinalverts = (finalvert_t *)
-		(((long) &finalverts[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
-	pauxverts = (auxvert_t *) &pfinalverts[pmdl->numverts + 1];
+		(((intptr_t) &finalverts[0] + CACHE_SIZE - 1) & ~(CACHE_SIZE - 1));
+	sw32_pauxverts = (auxvert_t *) &pfinalverts[pmdl->numverts + 1];
 
 	R_AliasSetupSkin ();
-	R_AliasSetUpTransform (currententity->trivial_accept);
+	sw32_R_AliasSetUpTransform (currententity->trivial_accept);
 	R_AliasSetupLighting (plighting);
 	R_AliasSetupFrame ();
 
-	if (!currententity->colormap)
-		Sys_Error ("R_AliasDrawModel: !currententity->colormap");
-
-	acolormap = currententity->colormap;
-	if (acolormap == &vid.colormap8 && r_pixbytes != 1)
+	if (!sw32_acolormap)
+		sw32_acolormap = vid.colormap8;
+	if (sw32_acolormap == &vid.colormap8 && sw32_r_pixbytes != 1)
 	{
-		if (r_pixbytes == 2)
-			acolormap = vid.colormap16;
-		else if (r_pixbytes == 4)
-			acolormap = vid.colormap32;
+		if (sw32_r_pixbytes == 2)
+			sw32_acolormap = vid.colormap16;
+		else if (sw32_r_pixbytes == 4)
+			sw32_acolormap = vid.colormap32;
 		else
 			Sys_Error("R_AliasDrawmodel: unsupported r_pixbytes %i",
-					  r_pixbytes);
+					  sw32_r_pixbytes);
 	}
 
-	if (currententity != r_view_model)
-		ziscale = (float) 0x8000 *(float) 0x10000;
+	if (currententity != vr_data.view_model)
+		sw32_ziscale = (float) 0x8000 *(float) 0x10000;
 	else
-		ziscale = (float) 0x8000 *(float) 0x10000 *3.0;
+		sw32_ziscale = (float) 0x8000 *(float) 0x10000 *3.0;
 
-	if (currententity->trivial_accept && pmdl->ident != HEADER_MDL16)
+	if (currententity->trivial_accept)
 		R_AliasPrepareUnclippedPoints ();
 	else
 		R_AliasPreparePoints ();
 
-	Cache_Release (&currententity->model->cache);
+	if (!currententity->model->aliashdr)
+		Cache_Release (&currententity->model->cache);
 }
