@@ -28,29 +28,41 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sw32_rsurf.c,v 1.12 2003/04/17 00:01:42 taniwha Exp $";
+#define NH_DEFINE
+#include "namehack.h"
 
 #include "QF/render.h"
 #include "QF/sys.h"
 
 #include "compat.h"
-#include "r_local.h"
+#include "r_internal.h"
 
-drawsurf_t  r_drawsurf;
+drawsurf_t  sw32_r_drawsurf;
 
-int         lightleft, sourcesstep, blocksize, sourcetstep;
-int         lightdelta, lightdeltastep;
-int         lightright, lightleftstep, lightrightstep, blockdivshift;
-unsigned int blockdivmask;
-byte       *prowdestbase;
-byte       *psource;
-int         surfrowbytes;				// used by ASM files
-int        *r_lightptr;
-int         r_stepback;
-int         r_lightwidth;
-int         r_numhblocks, r_numvblocks;
-byte       *r_source, *r_sourcemax;
+static int         lightleft, blocksize, sourcetstep;
+static int         lightright, lightleftstep, lightrightstep, blockdivshift;
+static unsigned int blockdivmask;
+static byte       *prowdestbase;
+static byte       *psource;
+static int         surfrowbytes;
+static int        *r_lightptr;
+static int         r_stepback;
+static int         r_lightwidth;
+static int         r_numhblocks, r_numvblocks;
+static byte       *r_source, *r_sourcemax;
+
+static void R_DrawSurfaceBlock8_mip0 (void);
+static void R_DrawSurfaceBlock8_mip1 (void);
+static void R_DrawSurfaceBlock8_mip2 (void);
+static void R_DrawSurfaceBlock8_mip3 (void);
+static void R_DrawSurfaceBlock16_mip0 (void);
+static void R_DrawSurfaceBlock16_mip1 (void);
+static void R_DrawSurfaceBlock16_mip2 (void);
+static void R_DrawSurfaceBlock16_mip3 (void);
+static void R_DrawSurfaceBlock32_mip0 (void);
+static void R_DrawSurfaceBlock32_mip1 (void);
+static void R_DrawSurfaceBlock32_mip2 (void);
+static void R_DrawSurfaceBlock32_mip3 (void);
 
 static void (*surfmiptable8[4]) (void) = {
 	R_DrawSurfaceBlock8_mip0,
@@ -73,7 +85,7 @@ static void (*surfmiptable32[4]) (void) = {
 	R_DrawSurfaceBlock32_mip3
 };
 
-int blocklights[34 * 34];	//FIXME make dynamic
+static int blocklights[34 * 34];	//FIXME make dynamic
 
 
 static void
@@ -89,13 +101,13 @@ R_AddDynamicLights (void)
 	int         smax, tmax;
 	mtexinfo_t *tex;
 
-	surf = r_drawsurf.surf;
+	surf = sw32_r_drawsurf.surf;
 	smax = (surf->extents[0] >> 4) + 1;
 	tmax = (surf->extents[1] >> 4) + 1;
 	tex = surf->texinfo;
 
 	for (lnum = 0; lnum < r_maxdlights; lnum++) {
-		if (!(surf->dlightbits & (1 << lnum)))
+		if (!(surf->dlightbits[lnum / 32] & (1 << (lnum % 32))))
 			continue;					// not lit by this light
 
 		VectorSubtract (r_dlights[lnum].origin, currententity->origin,
@@ -153,7 +165,7 @@ R_BuildLightMap (void)
 	int         maps;
 	msurface_t *surf;
 
-	surf = r_drawsurf.surf;
+	surf = sw32_r_drawsurf.surf;
 
 	smax = (surf->extents[0] >> 4) + 1;
 	tmax = (surf->extents[1] >> 4) + 1;
@@ -172,7 +184,7 @@ R_BuildLightMap (void)
 	// add all the lightmaps
 	if (lightmap)
 		for (maps = 0; maps < MAXLIGHTMAPS && surf->styles[maps] != 255; maps++) {
-			scale = r_drawsurf.lightadj[maps];	// 8.8 fraction     
+			scale = sw32_r_drawsurf.lightadj[maps];	// 8.8 fraction
 			for (i = 0; i < size; i++)
 				blocklights[i] += lightmap[i] * scale;
 			lightmap += size;			// skip to next lightmap
@@ -181,13 +193,12 @@ R_BuildLightMap (void)
 	if (surf->dlightframe == r_framecount)
 		R_AddDynamicLights ();
 
-	// LordHavoc: changed to positive (not inverse) lighting
-	for (i = 0; i < size; i++) {
-		t = bound(256, blocklights[i] >> (8 - VID_CBITS),
-				  256 * (VID_GRADES - 1));
-		blocklights[i] = t;
-	}
-/*
+//	// LordHavoc: changed to positive (not inverse) lighting
+//	for (i = 0; i < size; i++) {
+//		t = bound(256, blocklights[i] >> (8 - VID_CBITS),
+//				  256 * (VID_GRADES - 1));
+//		blocklights[i] = t;
+//	}
 	// bound, invert, and shift
 	for (i = 0; i < size; i++) {
 		t = (255 * 256 - blocklights[i]) >> (8 - VID_CBITS);
@@ -197,45 +208,10 @@ R_BuildLightMap (void)
 
 		blocklights[i] = t;
 	}
-*/
-}
-
-/*
-	R_TextureAnimation
-
-	Returns the proper texture for a given time and base texture
-*/
-texture_t *
-R_TextureAnimation (msurface_t *surf)
-{
-	texture_t  *base = surf->texinfo->texture;
-	int         relative;
-	int         count;
-
-	if (currententity->frame) {
-		if (base->alternate_anims)
-			base = base->alternate_anims;
-	}
-
-	if (!base->anim_total)
-		return base;
-
-	relative = (int) (r_realtime * 10) % base->anim_total;
-
-	count = 0;
-	while (base->anim_min > relative || base->anim_max <= relative) {
-		base = base->anim_next;
-		if (!base)
-			Sys_Error ("R_TextureAnimation: broken cycle");
-		if (++count > 100)
-			Sys_Error ("R_TextureAnimation: infinite cycle");
-	}
-
-	return base;
 }
 
 void
-R_DrawSurface (void)
+sw32_R_DrawSurface (void)
 {
 	byte       *basetptr;
 	int         smax, tmax, twidth;
@@ -249,62 +225,62 @@ R_DrawSurface (void)
 	// calculate the lightings
 	R_BuildLightMap ();
 
-	surfrowbytes = r_drawsurf.rowbytes;
+	surfrowbytes = sw32_r_drawsurf.rowbytes;
 
-	mt = r_drawsurf.texture;
+	mt = sw32_r_drawsurf.texture;
 
-	r_source = (byte *) mt + mt->offsets[r_drawsurf.surfmip];
+	r_source = (byte *) mt + mt->offsets[sw32_r_drawsurf.surfmip];
 
 	// the fractional light values should range from 0 to
 	// (VID_GRADES - 1) << 16 from a source range of 0 - 255
 
-	texwidth = mt->width >> r_drawsurf.surfmip;
+	texwidth = mt->width >> sw32_r_drawsurf.surfmip;
 
-	blocksize = 16 >> r_drawsurf.surfmip;
-	blockdivshift = 4 - r_drawsurf.surfmip;
+	blocksize = 16 >> sw32_r_drawsurf.surfmip;
+	blockdivshift = 4 - sw32_r_drawsurf.surfmip;
 	blockdivmask = (1 << blockdivshift) - 1;
 
-	r_lightwidth = (r_drawsurf.surf->extents[0] >> 4) + 1;
+	r_lightwidth = (sw32_r_drawsurf.surf->extents[0] >> 4) + 1;
 
-	r_numhblocks = r_drawsurf.surfwidth >> blockdivshift;
-	r_numvblocks = r_drawsurf.surfheight >> blockdivshift;
+	r_numhblocks = sw32_r_drawsurf.surfwidth >> blockdivshift;
+	r_numvblocks = sw32_r_drawsurf.surfheight >> blockdivshift;
 
 //==============================
 
-	smax = mt->width >> r_drawsurf.surfmip;
+	smax = mt->width >> sw32_r_drawsurf.surfmip;
 	twidth = texwidth;
-	tmax = mt->height >> r_drawsurf.surfmip;
+	tmax = mt->height >> sw32_r_drawsurf.surfmip;
 	sourcetstep = texwidth;
 	r_stepback = tmax * twidth;
 
-	soffset = r_drawsurf.surf->texturemins[0];
-	basetoffset = r_drawsurf.surf->texturemins[1];
+	soffset = sw32_r_drawsurf.surf->texturemins[0];
+	basetoffset = sw32_r_drawsurf.surf->texturemins[1];
 
-	switch (r_pixbytes) {
+	switch (sw32_r_pixbytes) {
 	case 1:
-		pblockdrawer = surfmiptable8[r_drawsurf.surfmip];
+		pblockdrawer = surfmiptable8[sw32_r_drawsurf.surfmip];
 		break;
 	case 2:
-		pblockdrawer = surfmiptable16[r_drawsurf.surfmip];
+		pblockdrawer = surfmiptable16[sw32_r_drawsurf.surfmip];
 		break;
 	case 4:
-		pblockdrawer = surfmiptable32[r_drawsurf.surfmip];
+		pblockdrawer = surfmiptable32[sw32_r_drawsurf.surfmip];
 		break;
 	default:
-		Sys_Error("R_DrawSurface: unsupported r_pixbytes %i", r_pixbytes);
+		Sys_Error("R_DrawSurface: unsupported r_pixbytes %i", sw32_r_pixbytes);
 		pblockdrawer = NULL;
 	}
 
-	horzblockstep = blocksize * r_pixbytes;
+	horzblockstep = blocksize * sw32_r_pixbytes;
 
 	r_sourcemax = r_source + (tmax * smax);
 
 	// << 16 components are to guarantee positive values for %
-	basetptr = r_source + (((basetoffset >> r_drawsurf.surfmip) +
+	basetptr = r_source + (((basetoffset >> sw32_r_drawsurf.surfmip) +
 							(tmax << 16)) % tmax) * twidth;
-	soffset = (((soffset >> r_drawsurf.surfmip) + (smax << 16)) % smax);
+	soffset = (((soffset >> sw32_r_drawsurf.surfmip) + (smax << 16)) % smax);
 
-	pcolumndest = (byte *) r_drawsurf.surfdat;
+	pcolumndest = (byte *) sw32_r_drawsurf.surfdat;
 
 	for (u = 0; u < r_numhblocks; u++) {
 		r_lightptr = blocklights + u;
@@ -325,7 +301,7 @@ R_DrawSurface (void)
 
 //=============================================================================
 
-void
+static void
 R_DrawSurfaceBlock8_mip0 (void)
 {
 	int         v, i, b, lightstep, light;
@@ -364,7 +340,7 @@ R_DrawSurfaceBlock8_mip0 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock8_mip1 (void)
 {
 	int         v, i, b, lightstep, light;
@@ -403,7 +379,7 @@ R_DrawSurfaceBlock8_mip1 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock8_mip2 (void)
 {
 	int         v, i, b, lightstep, light;
@@ -442,7 +418,7 @@ R_DrawSurfaceBlock8_mip2 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock8_mip3 (void)
 {
 	int         v, i, b, lightstep, light;
@@ -481,7 +457,7 @@ R_DrawSurfaceBlock8_mip3 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock16_mip0 (void)
 {
 	int         k, v;
@@ -546,7 +522,7 @@ R_DrawSurfaceBlock16_mip0 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock16_mip1 (void)
 {
 	int         k, v;
@@ -595,7 +571,7 @@ R_DrawSurfaceBlock16_mip1 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock16_mip2 (void)
 {
 	int         k, v;
@@ -636,7 +612,7 @@ R_DrawSurfaceBlock16_mip2 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock16_mip3 (void)
 {
 	int v;
@@ -671,7 +647,7 @@ R_DrawSurfaceBlock16_mip3 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock32_mip0 (void)
 {
 	int         k, v;
@@ -736,7 +712,7 @@ R_DrawSurfaceBlock32_mip0 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock32_mip1 (void)
 {
 	int         k, v;
@@ -785,7 +761,7 @@ R_DrawSurfaceBlock32_mip1 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock32_mip2 (void)
 {
 	int         k, v;
@@ -826,7 +802,7 @@ R_DrawSurfaceBlock32_mip2 (void)
 	}
 }
 
-void
+static void
 R_DrawSurfaceBlock32_mip3 (void)
 {
 	int v;
@@ -902,8 +878,3 @@ R_DrawSurfaceBlock32 (void)
 	}
 }
 */
-
-void
-R_SurfacePatch (void)
-{
-}
