@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: keys.c,v 1.38 2004/03/16 04:12:28 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -56,53 +53,44 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "compat.h"
 #include "old_keys.h"
 
-#define U __attribute__ ((unused))
+#define U __attribute__ ((used))
 static U void (*const key_progs_init)(struct progs_s *) = Key_Progs_Init;
 #undef U
 
 /*  key up events are sent even if in console mode */
 
-cvar_t     *in_bind_imt;
+static keydest_t    key_dest = key_console;
+static keytarget_t  key_targets[key_last];
+VISIBLE knum_t      key_togglemenu = QFK_ESCAPE;
+VISIBLE knum_t      key_toggleconsole = QFK_BACKQUOTE;
 
-keydest_t   key_dest = key_console;
-imt_t		game_target = IMT_CONSOLE;
+#define KEYDEST_CALLBACK_CHUNK 16
+static keydest_callback_t **keydest_callbacks;
+static int num_keydest_callbacks;
+static int max_keydest_callbacks;
 
-struct keybind_s keybindings[IMT_LAST][QFK_LAST];
-int			keydown[QFK_LAST];
+VISIBLE int			keydown[QFK_LAST];
 
 static int  keyhelp;
 static cbuf_t *cbuf;
 
+static const char *keydest_names[] = {
+	"key_unfocused",
+	"key_game",
+	"key_demo",
+	"key_console",
+	"key_message",
+	"key_menu",
+
+	"key_last"
+};
+
+
 typedef struct {
 	const char *name;
 	imt_t	imtnum;
 } imtname_t;
 
-imtname_t   imtnames[] = {
-	{"IMT_CONSOLE",	IMT_CONSOLE},
-	{"IMT_0",		IMT_0},
-	{"IMT_1",		IMT_1},
-	{"IMT_2",		IMT_2},
-	{"IMT_3",		IMT_3},
-	{"IMT_4",		IMT_4},
-	{"IMT_5",		IMT_5},
-	{"IMT_6",		IMT_6},
-	{"IMT_7",		IMT_7},
-	{"IMT_8",		IMT_8},
-	{"IMT_9",		IMT_9},
-	{"IMT_10",		IMT_10},
-	{"IMT_11",		IMT_11},
-	{"IMT_12",		IMT_12},
-	{"IMT_13",		IMT_13},
-	{"IMT_14",		IMT_14},
-	{"IMT_15",		IMT_15},
-	{"IMT_16",		IMT_16},
-
-	{"IMT_DEFAULT",	IMT_0},
-
-	{NULL, 0}
-};
-
 typedef struct {
 	const char *name;
 	knum_t	keynum;
@@ -346,6 +334,49 @@ keyname_t   keynames[] = {
 	{ "K_MENU",			QFK_MENU },
 	{ "K_POWER",		QFK_POWER },
 	{ "K_EURO",			QFK_EURO },
+
+	{ "K_KANJI", 		QFK_KANJI },
+	{ "K_MUHENKAN", 	QFK_MUHENKAN },
+	{ "K_HENKAN", 		QFK_HENKAN },
+	{ "K_ROMAJI", 		QFK_ROMAJI },
+	{ "K_HIRAGANA", 	QFK_HIRAGANA },
+	{ "K_KATAKANA", 	QFK_KATAKANA },
+	{ "K_HIRAGANA_KATAKANA",	QFK_HIRAGANA_KATAKANA },
+	{ "K_ZENKAKU", 		QFK_ZENKAKU },
+	{ "K_HANKAKU", 		QFK_HANKAKU },
+	{ "K_ZENKAKU_HANKAKU",	QFK_ZENKAKU_HANKAKU },
+	{ "K_TOUROKU", 		QFK_TOUROKU },
+	{ "K_MASSYO", 		QFK_MASSYO },
+	{ "K_KANA_LOCK", 	QFK_KANA_LOCK },
+	{ "K_KANA_SHIFT", 	QFK_KANA_SHIFT },
+	{ "K_EISU_SHIFT", 	QFK_EISU_SHIFT },
+	{ "K_EISU_TOGGLE", 	QFK_EISU_TOGGLE },
+	{ "K_KANJI_BANGOU", QFK_KANJI_BANGOU },
+	{ "K_ZEN_KOHO", 	QFK_ZEN_KOHO },
+	{ "K_MAE_KOHO", 	QFK_MAE_KOHO },
+
+	{ "K_HOMEPAGE",		QFK_HOMEPAGE },
+	{ "K_SEARCH",		QFK_SEARCH },
+	{ "K_MAIL",			QFK_MAIL },
+	{ "K_FAVORITES",	QFK_FAVORITES },
+	{ "K_AUDIOMUTE",	QFK_AUDIOMUTE },
+	{ "K_AUDIOLOWERVOLUME",	QFK_AUDIOLOWERVOLUME },
+	{ "K_AUDIORAISEVOLUME",	QFK_AUDIORAISEVOLUME },
+	{ "K_AUDIOPLAY",		QFK_AUDIOPLAY },
+	{ "K_CALCULATOR",	QFK_CALCULATOR },
+	{ "K_UNDO",			QFK_UNDO },
+	{ "K_REDO",			QFK_REDO },
+	{ "K_NEW",			QFK_NEW },
+	{ "K_RELOAD",		QFK_RELOAD },
+	{ "K_OPEN",			QFK_OPEN },
+	{ "K_CLOSE",		QFK_CLOSE },
+	{ "K_REPLY",		QFK_REPLY },
+	{ "K_MAILFORWARD",	QFK_MAILFORWARD },
+	{ "K_SEND",			QFK_SEND },
+	{ "K_SAVE",			QFK_SAVE },
+	{ "K_BACK",			QFK_BACK },
+	{ "K_FORWARD",		QFK_FORWARD },
+
 	{ "M_BUTTON1",		QFM_BUTTON1 },
 	{ "M_BUTTON2",		QFM_BUTTON2 },
 	{ "M_BUTTON3",		QFM_BUTTON3 },
@@ -353,6 +384,31 @@ keyname_t   keynames[] = {
 	{ "M_WHEEL_DOWN",	QFM_WHEEL_DOWN },
 	{ "M_BUTTON6",		QFM_BUTTON6 },
 	{ "M_BUTTON7",		QFM_BUTTON7 },
+	{ "M_BUTTON8",		QFM_BUTTON8 },
+	{ "M_BUTTON9",		QFM_BUTTON9 },
+	{ "M_BUTTON10",		QFM_BUTTON10 },
+	{ "M_BUTTON11",		QFM_BUTTON11 },
+	{ "M_BUTTON12",		QFM_BUTTON12 },
+	{ "M_BUTTON13",		QFM_BUTTON13 },
+	{ "M_BUTTON14",		QFM_BUTTON14 },
+	{ "M_BUTTON15",		QFM_BUTTON15 },
+	{ "M_BUTTON16",		QFM_BUTTON16 },
+	{ "M_BUTTON17",		QFM_BUTTON17 },
+	{ "M_BUTTON18",		QFM_BUTTON18 },
+	{ "M_BUTTON19",		QFM_BUTTON19 },
+	{ "M_BUTTON20",		QFM_BUTTON20 },
+	{ "M_BUTTON21",		QFM_BUTTON21 },
+	{ "M_BUTTON22",		QFM_BUTTON22 },
+	{ "M_BUTTON23",		QFM_BUTTON23 },
+	{ "M_BUTTON24",		QFM_BUTTON24 },
+	{ "M_BUTTON25",		QFM_BUTTON25 },
+	{ "M_BUTTON26",		QFM_BUTTON26 },
+	{ "M_BUTTON27",		QFM_BUTTON27 },
+	{ "M_BUTTON28",		QFM_BUTTON28 },
+	{ "M_BUTTON29",		QFM_BUTTON29 },
+	{ "M_BUTTON30",		QFM_BUTTON30 },
+	{ "M_BUTTON31",		QFM_BUTTON31 },
+	{ "M_BUTTON32",		QFM_BUTTON32 },
 
 	{ "J_BUTTON1",		QFJ_BUTTON1 },
 	{ "J_BUTTON2",		QFJ_BUTTON2 },
@@ -390,6 +446,107 @@ keyname_t   keynames[] = {
 	{NULL, 0}
 };
 
+static imt_t *
+key_target_find_imt (keytarget_t *kt, const char *imt_name)
+{
+	imt_t      *imt;
+	for (imt = kt->imts; imt; imt = imt->next) {
+		if (!strcasecmp (imt->name, imt_name)) {
+			return imt;
+		}
+	}
+	return 0;
+}
+
+VISIBLE imt_t *
+Key_FindIMT (const char *imt_name)
+{
+	keydest_t   kd;
+	imt_t      *imt = 0;
+
+	for (kd = key_unfocused; !imt && kd < key_last; kd++) {
+		imt = key_target_find_imt (&key_targets[kd], imt_name);
+	}
+	return imt;
+}
+
+void
+Key_CreateIMT (keydest_t kd, const char *imt_name, const char *chain_imt_name)
+{
+	imt_t      *imt;
+	imt_t      *chain_imt = 0;
+	keytarget_t *kt = &key_targets[kd];
+
+	imt = Key_FindIMT (imt_name);
+	if (imt) {
+		Sys_Printf ("imt error: imt %s already exists\n", imt_name);
+		return;
+	}
+	if (chain_imt_name) {
+		chain_imt = Key_FindIMT (chain_imt_name);
+		if (!chain_imt) {
+			Sys_Printf ("imt error: chain imt %s does not exist\n",
+						chain_imt_name);
+			return;
+		}
+		imt = key_target_find_imt (kt, chain_imt_name);
+		if (!imt) {
+			Sys_Printf ("imt error: chain imt %s not on target key "
+						"destination\n", chain_imt_name);
+			return;
+		}
+	}
+	imt = calloc (1, sizeof (imt_t));
+	imt->name = strdup (imt_name);
+	imt->chain = chain_imt;
+	imt->next = kt->imts;
+	kt->imts = imt;
+	if (!kt->active) {
+		kt->active = imt;
+	}
+}
+
+VISIBLE void
+Key_SetBinding (imt_t *imt, knum_t keynum, const char *binding)
+{
+	if (keynum == (knum_t) -1)
+		return;
+
+	if (imt->bindings[keynum].str) {
+		free (imt->bindings[keynum].str);
+		imt->bindings[keynum].str = NULL;
+	}
+	if (binding) {
+		imt->bindings[keynum].str = strdup(binding);
+	}
+}
+
+static void
+Key_CallDestCallbacks (keydest_t kd)
+{
+	int         i;
+
+	for (i = 0; i < num_keydest_callbacks; i++)
+		keydest_callbacks[i] (kd);
+}
+
+static void
+process_binding (knum_t key, const char *kb)
+{
+	char        cmd[1024];
+
+	if (kb[0] == '+') {
+		if (keydown[key])
+			snprintf (cmd, sizeof (cmd), "%s %d\n", kb, key);
+		else
+			snprintf (cmd, sizeof (cmd), "-%s %d\n", kb + 1, key);
+	} else {
+		if (!keydown[key])
+			return;
+		snprintf (cmd, sizeof (cmd), "%s\n", kb);
+	}
+	Cbuf_AddText (cbuf, cmd);
+}
 
 /*
   Key_Game
@@ -400,34 +557,22 @@ static qboolean
 Key_Game (knum_t key, short unicode)
 {
 	const char *kb;
-	char        cmd[1024];
+	imt_t      *imt = key_targets[key_dest].active;
 
-	kb = Key_GetBinding(game_target, key);
-	if (!kb && (game_target > IMT_0))
-		kb = Key_GetBinding(IMT_0, key);
-
-	/*
-	Con_Printf("kb %p, game_target %d, key_dest %d, key %d\n", kb,
-			game_target, key_dest, key);
-			*/
-	if (!kb)
-		return false;
-
-	if (keydown[key] > 1) 
-		return true;
-
-	if (kb[0] == '+') {
-		if (keydown[key])
-			snprintf (cmd, sizeof (cmd), "%s %d\n", kb, key);
-		else
-			snprintf (cmd, sizeof (cmd), "-%s %d\n", kb + 1, key);
-	} else {
-		if (!keydown[key])
+	while (imt) {
+		kb = imt->bindings[key].str;
+		if (kb) {
+			if (keydown[key] <= 1)
+				process_binding (key, kb);
 			return true;
-		snprintf (cmd, sizeof (cmd), "%s\n", kb);
+		}
+		imt = imt->chain;
 	}
-	Cbuf_AddText (cbuf, cmd);
-	return true;
+	return false;
+/*
+	Sys_DPrintf("kb %p, key_target %d, key_dest %d, key %d\n", kb,
+				key_target, key_dest, key);
+*/
 }
 
 /*
@@ -447,48 +592,6 @@ Key_Console (knum_t key, short unicode)
 
 //============================================================================
 
-/*
-  Key_StringToIMTnum
-  Returns an imt number to be used to index imtbindings[] by looking at
-  the given string.  Single ascii characters return themselves, while
-  the QFK_* names are matched up.
-*/
-static int
-Key_StringToIMTnum (const char *str)
-{
-	imtname_t  *kn;
-
-	if (!str || !str[0])
-		return -1;
-
-	for (kn = imtnames; kn->name; kn++) {
-		if (!strcasecmp (str, kn->name))
-			return kn->imtnum;
-	}
-	return -1;
-}
-
-/*
-  Key_IMTnumToString
-
-  Returns a string (a QFK_* name) for the given imtnum.
-  FIXME: handle quote special (general escape sequence?)
-*/
-static const char *
-Key_IMTnumToString (const imt_t imtnum)
-{
-	imtname_t  *kn;
-
-	if (imtnum == (imt_t) -1)
-		return "<IMT NOT FOUND>";
-
-	for (kn = imtnames; kn->name; kn++)
-		if (imtnum == kn->imtnum)
-			return kn->name;
-
-	return "<UNKNOWN IMTNUM>";
-}
-
 /*
   Key_StringToKeynum
 
@@ -517,7 +620,7 @@ Key_StringToKeynum (const char *str)
   Returns a string (a QFK_* name) for the given keynum.
   FIXME: handle quote special (general escape sequence?)
 */
-const char *
+VISIBLE const char *
 Key_KeynumToString (knum_t keynum)
 {
 	keyname_t  *kn;
@@ -533,30 +636,31 @@ Key_KeynumToString (knum_t keynum)
 }
 
 static void
-Key_In_Unbind (const char *imt, const char *key)
+Key_In_Unbind (const char *imt_name, const char *key_name)
 {
-	int t, b;
+	imt_t      *imt;
+	int         key;
 
-	t = Key_StringToIMTnum (imt);
-	if (t == -1) {
-		Con_Printf ("\"%s\" isn't a valid imt\n", imt);
+	imt = Key_FindIMT (imt_name);
+	if (!imt) {
+		Sys_Printf ("\"%s\" isn't a valid imt\n", imt_name);
 		return;
 	}
 
-	b = Key_StringToKeynum (key);
-	if (b == -1) {
-		Con_Printf ("\"%s\" isn't a valid key\n", key);
+	key = Key_StringToKeynum (key_name);
+	if (key == -1) {
+		Sys_Printf ("\"%s\" isn't a valid key\n", key_name);
 		return;
 	}
 
-	Key_SetBinding (t, b, NULL);
+	Key_SetBinding (imt, key, NULL);
 }
 
 static void
 Key_In_Unbind_f (void)
 {
 	if (Cmd_Argc () != 3) {
-		Con_Printf ("in_unbind <imt> <key> : remove commands from a key\n");
+		Sys_Printf ("in_unbind <imt> <key> : remove commands from a key\n");
 		return;
 	}
 	Key_In_Unbind (Cmd_Argv (1), Cmd_Argv (2));
@@ -565,39 +669,118 @@ Key_In_Unbind_f (void)
 static void
 Key_Unbindall_f (void)
 {
-	int         i, j;
+	keydest_t   kd;
+	imt_t      *imt;
+	int         i;
 
-	for (j = 0; j < IMT_LAST; j++)
-		for (i = 0; i < QFK_LAST; i++)
-			Key_SetBinding (j, i, NULL);
+	for (kd = key_unfocused; kd < key_last; kd++) {
+		for (imt = key_targets[kd].imts; imt; imt = imt->next) {
+			for (i = 0; i < QFK_LAST; i++) {
+				Key_SetBinding (imt, i, 0);
+			}
+		}
+	}
 }
 
 static void
-Key_In_Bind (const char *imt, const char *key, const char *cmd)
+Key_In_Clear (void)
 {
-	int t, b;
+	int         err = 0;
+	imt_t      *imt;
+	int         i, j;
 
-	t = Key_StringToIMTnum (imt);
-	if (t == -1) {
-		Con_Printf ("\"%s\" isn't a valid imt\n", imt);
+	if (Cmd_Argc () == 1) {
+		Sys_Printf ("in_clear <imt> ...\n");
+		return;
+	}
+	for (i = 1; i < Cmd_Argc (); i++) {
+		if (!Key_FindIMT (Cmd_Argv (i))) {
+			Sys_Printf ("\"%s\" isn't a valid imt\n", Cmd_Argv (i));
+			err = 1;
+		}
+	}
+	if (err)
+		return;
+	for (i = 1; i < Cmd_Argc (); i++) {
+		imt = Key_FindIMT (Cmd_Argv (i));
+		for (j = 0; j < QFK_LAST; j++)
+			Key_SetBinding (imt, j, NULL);
+	}
+}
+
+static void
+Key_IMT_Create_f (void)
+{
+	const char *keydest;
+	const char *imt_name;
+	const char *chain_imt_name = 0;
+	keydest_t   kd;
+
+	if (Cmd_Argc () < 3 || Cmd_Argc () > 4) {
+		Sys_Printf ("see help imt_create\n");
+		return;
+	}
+	keydest = Cmd_Argv (1);
+	imt_name = Cmd_Argv (2);
+	if (Cmd_Argc () == 4) {
+		chain_imt_name = Cmd_Argv (3);
+	}
+	for (kd = key_game; kd < key_last; kd++) {
+		if (!strcasecmp (keydest_names[kd], keydest)) {
+			break;
+		}
+	}
+	if (kd == key_last) {
+		Sys_Printf ("imt error: invalid keydest: %s\n", keydest);
+		return;
+	}
+	Key_CreateIMT (kd, imt_name, chain_imt_name);
+}
+
+static void
+Key_IMT_Drop_All_f (void)
+{
+	keydest_t   kd;
+	imt_t       *imt;
+
+	for (kd = key_unfocused; kd < key_last; kd++) {
+		while (key_targets[kd].imts) {
+			imt = key_targets[kd].imts;
+			key_targets[kd].imts = imt->next;
+			free ((char *) imt->name);
+			free (imt);
+		}
+		key_targets[kd].active = 0;
+	}
+}
+
+static void
+Key_In_Bind (const char *imt_name, const char *key_name, const char *cmd)
+{
+	imt_t      *imt;
+	int         key;
+
+	imt = Key_FindIMT (imt_name);
+	if (!imt) {
+		Sys_Printf ("\"%s\" isn't a valid imt\n", imt_name);
 		return;
 	}
 
-	b = Key_StringToKeynum (key);
-	if (b == -1) {
-		Con_Printf ("\"%s\" isn't a valid key\n", key);
+	key = Key_StringToKeynum (key_name);
+	if (key == -1) {
+		Sys_Printf ("\"%s\" isn't a valid key\n", key_name);
 		return;
 	}
 
 	if (!cmd) {
-		if (Key_GetBinding(t, b))
-			Con_Printf ("%s %s \"%s\"\n", imt, key,
-						Key_GetBinding(t, b));
+		if (imt->bindings[key].str)
+			Sys_Printf ("%s %s \"%s\"\n", imt_name, key_name,
+						imt->bindings[key].str);
 		else
-			Con_Printf ("%s %s is not bound\n", imt, key);
+			Sys_Printf ("%s %s is not bound\n", imt_name, key_name);
 		return;
 	}
-	Key_SetBinding (t, b, cmd);
+	Key_SetBinding (imt, key, cmd);
 }
 
 static void
@@ -610,7 +793,7 @@ Key_In_Bind_f (void)
 	c = Cmd_Argc ();
 
 	if (c < 3) {
-		Con_Printf ("in_bind <imt> <key> [command] : attach a command to a "
+		Sys_Printf ("in_bind <imt> <key> [command] : attach a command to a "
 					"key\n");
 		return;
 	}
@@ -639,29 +822,27 @@ Key_Unbind_f (void)
 	const char *key;
 
 	if (Cmd_Argc () != 2) {
-		Con_Printf ("unbind <key> : remove commands from a key\n");
+		Sys_Printf ("unbind <key> : remove commands from a key\n");
 		return;
 	}
 	key = OK_TranslateKeyName (Cmd_Argv (1));
-	Key_In_Unbind (in_bind_imt->string, key);
+	Key_In_Unbind ("imt_mod", key);
 }
 
 static void
 Key_Bind_f (void)
 {
 	int         c, i;
-	const char *imt, *key, *cmd = 0;
+	const char *key, *cmd = 0;
 	char        cmd_buf[1024];
 
 	c = Cmd_Argc ();
 
 	if (c < 2) {
-		Con_Printf ("bind <key> [command] : attach a command to a key\n");
+		Sys_Printf ("bind <key> [command] : attach a command to a key\n");
 		return;
 	}
 
-	imt = in_bind_imt->string;
-
 	key = OK_TranslateKeyName (Cmd_Argv (1));
 
 	if (c >= 3) {
@@ -675,96 +856,201 @@ Key_Bind_f (void)
 		}
 	}
 
-	Key_In_Bind (imt, key, cmd);
+	Key_In_Bind ("imt_mod", key, cmd);
 }
 
-static void 
+static void
 Key_GIB_Bind_Get_f (void)
 {
-	const char *imt, *key, *cmd;
-	int t, k;
+	const char *key, *cmd;
+	int k;
 
-	if (GIB_Argc() != 2) {
+	if (GIB_Argc () != 2) {
 		GIB_USAGE ("key");
 		return;
 	}
 
-	imt = in_bind_imt->string;
-
 	key = OK_TranslateKeyName (GIB_Argv (1));
 
-	if ((t = Key_StringToIMTnum (imt)) == -1) {
-		GIB_Error ("bind", "bind::get: invalid imt %s", imt);
-		return;
-	}
-
 	if ((k = Key_StringToKeynum (key)) == -1) {
 		GIB_Error ("bind", "bind::get: invalid key %s", key);
 		return;
 	}
-	
-	if (!(cmd = Key_GetBinding (t, k)))
+
+	if (!(cmd = Key_GetBinding ("IMT_MOD", k)))
 		GIB_Return ("");
 	else
 		GIB_Return (cmd);
 }
 
+static void
+in_key_togglemenu_f (cvar_t *var)
+{
+	int         k;
+
+	if (!*var->string) {
+		key_togglemenu = QFK_ESCAPE;
+		return;
+	}
+	if ((k = Key_StringToKeynum (var->string)) == -1) {
+		k = QFK_ESCAPE;
+		Sys_Printf ("\"%s\" is not a valid key. setting to \"K_ESCAPE\"\n",
+					var->string);
+	}
+	key_togglemenu = k;
+}
 
 static void
-in_bind_imt_f (cvar_t *var)
+in_key_toggleconsole_f (cvar_t *var)
 {
-	if (Key_StringToIMTnum (var->string) == -1) {
-		Con_Printf ("\"%s\" is not a valid imt. setting to \"imt_default\"\n",
-					var->string);
-		Cvar_Set (var, "imt_default");
+	int         k;
+
+	if (!*var->string) {
+		key_toggleconsole = -1;
+		return;
 	}
+	if ((k = Key_StringToKeynum (var->string)) == -1) {
+		Sys_Printf ("\"%s\" is not a valid key. not setting\n",
+					var->string);
+		return;
+	}
+	key_toggleconsole = k;
 }
 
 static void
 Key_InputMappingTable_f (void)
 {
-	int         c, t;
+	int         c;
+	imt_t      *imt;
 
 	c = Cmd_Argc ();
 
 	if (c != 2) {
-		Con_Printf ("Current imt is %s\n", Key_IMTnumToString(game_target));
-		Con_Printf ("imt <imt> : set to a specific input mapping table\n");
+		Sys_Printf ("Current imt is %s\n", key_targets[key_game].active->name);
+		Sys_Printf ("imt <imt> : set to a specific input mapping table\n");
 		return;
 	}
 
-	t = Key_StringToIMTnum (Cmd_Argv (1));
-	if (t == -1) {
-		Con_Printf ("\"%s\" isn't a valid imt\n", Cmd_Argv (1));
+	imt = Key_FindIMT (Cmd_Argv (1));
+	if (!imt) {
+		Sys_Printf ("\"%s\" isn't a valid imt\n", Cmd_Argv (1));
 		return;
 	}
 
-	game_target = t;
+	key_targets[key_game].active = imt;
 }
 
-/*
-  Key_WriteBindings
+static void
+Key_IMT_Keydest_f (void)
+{
+	int         c;
+	imt_t      *imt;
+	const char *imt_name = 0;
+	const char *keydest;
+	keydest_t   kd;
+
+	c = Cmd_Argc ();
+	switch (c) {
+		case 3:
+			imt_name = Cmd_Argv (2);
+		case 2:
+			keydest = Cmd_Argv (1);
+			break;
+		default:
+			return;
+	}
+	for (kd = key_game; kd < key_last; kd++) {
+		if (!strcasecmp (keydest_names[kd], keydest)) {
+			break;
+		}
+	}
+	if (kd == key_last) {
+		Sys_Printf ("imt error: invalid keydest: %s\n", keydest);
+		return;
+	}
+
+	if (!imt_name) {
+		Sys_Printf ("Current imt is %s\n", key_targets[key_game].active->name);
+		Sys_Printf ("imt <imt> : set to a specific input mapping table\n");
+		return;
+	}
+
+	imt = key_target_find_imt (&key_targets[kd], imt_name);
+	if (!imt) {
+		Sys_Printf ("\"%s\" isn't an imt on %s\n", imt_name, keydest);
+		return;
+	}
+
+	key_targets[kd].active = imt;
+}
+
+static void __attribute__((format(printf,2,3)))
+key_printf (QFile *f, const char *fmt, ...)
+{
+	va_list     args;
+	static dstring_t *string;
+
+	if (!string) {
+		string = dstring_new ();
+	}
+	va_start (args, fmt);
+	dvsprintf (string, fmt, args);
+	va_end (args);
+
+	if (f) {
+		Qprintf (f, "%s", string->str);
+	} else {
+		Sys_Printf ("%s", string->str);
+	}
+}
+
+static void
+key_write_imt (QFile *f, keydest_t kd, imt_t *imt)
+{
+	int         i;
+	const char *bind;
+
+	if (!imt || imt->written) {
+		return;
+	}
+	imt->written = 1;
+	key_write_imt (f, kd, imt->chain);
+	if (imt->chain) {
+		key_printf (f, "imt_create %s %s %s\n", keydest_names[kd],
+					imt->name, imt->chain->name);
+	} else {
+		key_printf (f, "imt_create %s %s\n", keydest_names[kd], imt->name);
+	}
+	for (i = 0; i < QFK_LAST; i++) {
+		bind = imt->bindings[i].str;
+		if (bind) {
+			key_printf (f, "in_bind %s %s \"%s\"\n", imt->name,
+						Key_KeynumToString (i), bind);
+		}
+	}
+}
 
-  Writes lines containing "bind key value"
-*/
 void
 Key_WriteBindings (QFile *f)
 {
-	int			i, j;
-	const char	*bind;
+	keydest_t   kd;
+	imt_t      *imt;
 
-	for (j = 0; j < IMT_LAST; j++)
-		for (i = 0; i < QFK_LAST; i++)
-			if ((bind = Key_GetBinding(j, i))) {
-				if (f)
-					Qprintf (f, "in_bind %s %s \"%s\"\n",
-							 Key_IMTnumToString (j),
-							 Key_KeynumToString (i), bind);
-				else
-					Sys_Printf ("in_bind %s %s \"%s\"\n",
-							 Key_IMTnumToString (j),
-							 Key_KeynumToString (i), bind);
+	for (kd = key_unfocused; kd < key_last; kd++) {
+		for (imt = key_targets[kd].imts; imt; imt = imt->next) {
+			imt->written = 0;
+		}
+	}
+	key_printf (f, "imt_drop_all\n");
+	for (kd = key_unfocused; kd < key_last; kd++) {
+		if (key_targets[kd].imts) {
+			for (imt = key_targets[kd].imts; imt; imt = imt->next) {
+				key_write_imt (f, kd, imt);
 			}
+			key_printf (f, "imt_keydest %s %s\n", keydest_names[kd],
+						key_targets[kd].active->name);
+		}
+	}
 }
 
 static void
@@ -785,15 +1071,15 @@ keyhelp_f (void)
   Called by the system between frames for both key up and key down events
   Should NOT be called during an interrupt!
 */
-void
+VISIBLE void
 Key_Event (knum_t key, short unicode, qboolean down)
 {
-//  Con_Printf ("%d %d %d : %d\n", game_target, key_dest, key, down); //@@@
+//  Sys_Printf ("%d %d %d : %d\n", key_target, key_dest, key, down); //@@@
 
 	if (down) {
 		keydown[key]++;
 		if (keyhelp) {
-			Con_Printf ("Key name for that key is \"%s\"\n",
+			Sys_Printf ("Key name for that key is \"%s\"\n",
 						Key_KeynumToString (key));
 			keyhelp = 0;
 			return; // gobble the key
@@ -803,7 +1089,7 @@ Key_Event (knum_t key, short unicode, qboolean down)
 	}
 
 	// handle escape specially, so the user can never unbind it
-	if (unicode == '\x1b' || key == QFK_ESCAPE) {
+	if (key == key_togglemenu || key == key_toggleconsole) {
 		Key_Console (key, unicode);
 		return;
 	}
@@ -826,6 +1112,16 @@ Key_Event (knum_t key, short unicode, qboolean down)
 	}
 }
 
+VISIBLE void
+Key_FocusEvent (int gain)
+{
+	if (gain) {
+		Key_CallDestCallbacks (key_dest);
+	} else {
+		Key_CallDestCallbacks (key_unfocused);
+	}
+}
+
 void
 Key_ClearStates (void)
 {
@@ -838,11 +1134,54 @@ Key_ClearStates (void)
 	}
 }
 
+static struct {
+	keydest_t   kd;
+	const char *imt_name;
+	const char *chain_imt_name;
+} default_imts[] = {
+	{key_game,		"imt_mod",		0},
+	{key_game,		"imt_0",		"imt_mod"},
+	{key_game,		"imt_1",		"imt_0"},
+	{key_game,		"imt_2",		"imt_0"},
+	{key_game,		"imt_3",		"imt_0"},
+	{key_game,		"imt_4",		"imt_0"},
+	{key_game,		"imt_5",		"imt_0"},
+	{key_game,		"imt_6",		"imt_0"},
+	{key_game,		"imt_7",		"imt_0"},
+	{key_game,		"imt_8",		"imt_0"},
+	{key_game,		"imt_9",		"imt_0"},
+	{key_game,		"imt_10",		"imt_0"},
+	{key_game,		"imt_11",		"imt_0"},
+	{key_game,		"imt_12",		"imt_0"},
+	{key_game,		"imt_13",		"imt_0"},
+	{key_game,		"imt_14",		"imt_0"},
+	{key_game,		"imt_15",		"imt_0"},
+	{key_game,		"imt_16",		"imt_0"},
+	{key_demo,		"imt_demo",		0},
+	{key_console,	"imt_console",	0},
+	{key_message,	"imt_message",	0},
+	{key_menu,		"imt_menu",		0},
+	{key_last,		0,				0},
+};
+
+static void
+Key_CreateDefaultIMTs (void)
+{
+	int         i;
+
+	for (i = 0; default_imts[i].kd != key_last; i++) {
+		Key_CreateIMT (default_imts[i].kd, default_imts[i].imt_name,
+					   default_imts[i].chain_imt_name);
+	}
+}
+
 void
 Key_Init (cbuf_t *cb)
 {
 	cbuf = cb;
 
+	Key_CreateDefaultIMTs ();
+
 	OK_Init ();
 
 	// register our functions
@@ -854,7 +1193,20 @@ Key_Init (cbuf_t *cb)
 					"Remove the bind from the the selected key");
 	Cmd_AddCommand ("unbindall", Key_Unbindall_f,
 					"Remove all binds (USE CAUTIOUSLY!!!)");
+	Cmd_AddCommand ("in_clear", Key_In_Clear,
+					"Remove all binds from the specified imts");
 	Cmd_AddCommand ("imt", Key_InputMappingTable_f, "");
+	Cmd_AddCommand ("imt_keydest", Key_IMT_Keydest_f, "");
+	Cmd_AddCommand ("imt_create", Key_IMT_Create_f,
+					"create a new imt table:\n"
+					"    imt_create <keydest> <imt_name> [chain_name]\n"
+					"\n"
+					"The new table will be attached to the specified keydest\n"
+					"imt_name must not already exist.\n"
+					"If given, chain_name must already exist and be on "
+						"keydest.\n");
+	Cmd_AddCommand ("imt_drop_all", Key_IMT_Drop_All_f,
+					"delete all imt tables\n");
 	Cmd_AddCommand ("bind", Key_Bind_f, "wrapper for in_bind that uses "
 					"in_bind_imt for the imt parameter");
 	Cmd_AddCommand ("unbind", Key_Unbind_f,
@@ -863,7 +1215,7 @@ Key_Init (cbuf_t *cb)
 	Cmd_AddCommand ("bindlist", Key_Bindlist_f, "list all of the key bindings");
 	Cmd_AddCommand ("keyhelp", keyhelp_f, "display the keyname for the next "
 					"RECOGNIZED key-press. If the key pressed produces no "
-					"output, " PROGRAM " does not recognise that key.");
+					"output, " PACKAGE_NAME " does not recognise that key.");
 
 	GIB_Builtin_Add ("bind::get", Key_GIB_Bind_Get_f);
 
@@ -872,30 +1224,49 @@ Key_Init (cbuf_t *cb)
 void
 Key_Init_Cvars (void)
 {
-	in_bind_imt = Cvar_Get ("in_bind_imt", "imt_default", CVAR_ARCHIVE,
-							in_bind_imt_f, "imt parameter for the bind and "
-							"unbind wrappers to in_bind and in_unbind");
+	Cvar_Get ("in_key_togglemenu", "", CVAR_NONE, in_key_togglemenu_f,
+			  "Key for toggling the menu.");
+	Cvar_Get ("in_key_toggleconsole", "K_BACKQUOTE", CVAR_NONE,
+			  in_key_toggleconsole_f,
+			  "Key for toggling the console.");
 }
 
 const char *
-Key_GetBinding (imt_t imt, knum_t key)
+Key_GetBinding (const char *imt_name, knum_t key)
 {
-	return keybindings[imt][key].str;
+	imt_t      *imt;
+
+	imt = Key_FindIMT (imt_name);
+	if (imt) {
+		return imt->bindings[key].str;
+	}
+	return 0;
 }
 
-void
-Key_SetBinding (imt_t target, knum_t keynum, const char *binding)
+VISIBLE void
+Key_SetKeyDest(keydest_t kd)
 {
-	if (keynum == (knum_t) -1)
-		return;
-
-	// free old bindings
-	if (keybindings[target][keynum].str) {
-		free (keybindings[target][keynum].str);
-		keybindings[target][keynum].str = NULL;
-	}
-	// allocate memory for new binding
-	if (binding) {
-		keybindings[target][keynum].str = strdup(binding);
+	if ((int) kd < key_unfocused || kd >= key_last) {
+		Sys_Error ("Bad key_dest");
 	}
+	key_dest = kd;
+	Key_CallDestCallbacks (key_dest);
+}
+
+VISIBLE void
+Key_KeydestCallback (keydest_callback_t *callback)
+{
+	if (num_keydest_callbacks == max_keydest_callbacks) {
+		size_t size = (max_keydest_callbacks + KEYDEST_CALLBACK_CHUNK)
+					  * sizeof (keydest_callback_t *);
+		keydest_callbacks = realloc (keydest_callbacks, size);
+		if (!keydest_callbacks)
+			Sys_Error ("Too many keydest callbacks!");
+		max_keydest_callbacks += KEYDEST_CALLBACK_CHUNK;
+	}
+
+	if (!callback)
+		Sys_Error ("null keydest callback");
+
+	keydest_callbacks[num_keydest_callbacks++] = callback;
 }
