@@ -28,20 +28,22 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: host.c,v 1.147 2004/02/06 01:23:05 taniwha Exp $";
+#ifdef HAVE_UNISTD_H
+# include "unistd.h"
+#endif
 
 #include "QF/cbuf.h"
-#include "QF/idparse.h"
 #include "QF/cdaudio.h"
+#include "QF/draw.h"
+#include "QF/idparse.h"
 #include "QF/cmd.h"
 #include "QF/console.h"
 #include "QF/cvar.h"
-#include "QF/draw.h"
+#include "QF/image.h"
 #include "QF/input.h"
 #include "QF/keys.h"
 #include "QF/msg.h"
-#include "QF/plugin.h"
+#include "QF/png.h"
 #include "QF/progs.h"
 #include "QF/qargs.h"
 #include "QF/screen.h"
@@ -50,15 +52,15 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/vid.h"
 #include "QF/gib.h"
 
+#include "QF/plugin/console.h"
+#include "QF/plugin/vid_render.h"
+
 #include "buildnum.h"
 #include "chase.h"
 #include "compat.h"
 #include "host.h"
-#include "r_dynamic.h"
-#include "sbar.h"
 #include "server.h"
 #include "sv_progs.h"
-#include "view.h"
 
 
 /*
@@ -75,11 +77,6 @@ static plugin_list_t server_plugin_list[] = {
 		SERVER_PLUGIN_LIST
 };
 
-CLIENT_PLUGIN_PROTOS
-static plugin_list_t client_plugin_list[] = {
-		CLIENT_PLUGIN_LIST
-};
-
 qboolean	host_initialized;			// true if into command execution
 
 quakeparms_t host_parms;
@@ -97,21 +94,18 @@ double      oldcon_realtime;
 
 int			host_framecount;
 int			host_hunklevel;
+int         host_in_game;
 int			minimum_memory;
 
 client_t   *host_client;				// current client
 
 jmp_buf		host_abortserver;
 
-byte       *host_basepal;
-
-cvar_t     *fs_globalcfg;
-cvar_t     *fs_usercfg;
-
 cvar_t     *host_mem_size;
 
 cvar_t     *host_framerate;
 cvar_t     *host_speeds;
+cvar_t     *max_edicts;
 
 cvar_t     *sys_ticrate;
 cvar_t     *serverprofile;
@@ -131,7 +125,15 @@ cvar_t     *deathmatch;
 cvar_t     *pausable;
 
 cvar_t     *temp1;
+cvar_t     *cl_usleep;
 
+static int  cl_usleep_cache;
+
+static void
+cl_usleep_f (cvar_t *var)
+{
+	cl_usleep_cache = var->int_val;
+}
 
 
 void
@@ -146,7 +148,7 @@ Host_EndGame (const char *message, ...)
 	va_start (argptr, message);
 	dvsprintf (str, message, argptr);
 	va_end (argptr);
-	Con_DPrintf ("Host_EndGame: %s\n", str->str);
+	Sys_MaskPrintf (SYS_DEV, "Host_EndGame: %s\n", str->str);
 
 	if (sv.active)
 		Host_ShutdownServer (false);
@@ -182,7 +184,7 @@ Host_Error (const char *error, ...)
 
 	inerror = true;
 
-//	SCR_EndLoadingPlaque ();						// reenable screen updates
+	cl.loading = false;
 
 	va_start (argptr, error);
 	dvsprintf (str, error, argptr);
@@ -194,7 +196,7 @@ Host_Error (const char *error, ...)
 	if (cls.state == ca_dedicated)
 		Sys_Error ("Host_Error: %s", str->str);		// dedicated servers exit
 
-	Con_Printf ("Host_Error: %s\n", str->str);
+	Sys_Printf ("Host_Error: %s\n", str->str);
 
 	CL_Disconnect ();
 	cls.demonum = -1;
@@ -258,6 +260,8 @@ Host_InitLocal (void)
 	host_speeds =
 		Cvar_Get ("host_speeds", "0", CVAR_NONE, NULL,
 				"set for running times");
+	max_edicts = Cvar_Get ("max_edicts", "1024", CVAR_NONE, NULL,
+						   "maximum server edicts");
 
 	sys_ticrate = Cvar_Get ("sys_ticrate", "0.05", CVAR_NONE, NULL, "None");
 	serverprofile = Cvar_Get ("serverprofile", "0", CVAR_NONE, NULL, "None");
@@ -277,39 +281,16 @@ Host_InitLocal (void)
 	coop = Cvar_Get ("coop", "0", CVAR_NONE, NULL, "0 or 1");
 	pausable = Cvar_Get ("pausable", "1", CVAR_NONE, NULL, "None");
 	temp1 = Cvar_Get ("temp1", "0", CVAR_NONE, NULL, "None");
+	cl_usleep = Cvar_Get ("cl_usleep", "1", CVAR_ARCHIVE, cl_usleep_f,
+						  "Turn this on to save cpu when fps limited. "
+						  "May affect frame rate adversely depending on "
+						  "local machine/os conditions");
 
 	Host_FindMaxClients ();
 
 	host_time = 1.0;				// so a think at time 0 won't get called
 }
 
-/*
-	Host_WriteConfiguration
-
-	Writes key bindings and archived cvars to config.cfg
-*/
-static void
-Host_WriteConfiguration (void)
-{
-	QFile      *f;
-
-	// dedicated servers initialize the host but don't parse and set the
-	// config.cfg cvars
-	if (host_initialized && !isDedicated && cl_writecfg->int_val) {
-		char       *path = va ("%s/config.cfg", qfs_gamedir->dir.def);
-		f = QFS_WOpen (path, 0);
-		if (!f) {
-			Con_Printf ("Couldn't write config.cfg.\n");
-			return;
-		}
-
-		Key_WriteBindings (f);
-		Cvar_WriteVariables (f);
-
-		Qclose (f);
-	}
-}
-
 /*
 	SV_ClientPrintf
 
@@ -410,9 +391,10 @@ SV_DropClient (qboolean crash)
 			*sv_globals.self = saveSelf;
 		}
 
-		Con_Printf ("Client %s removed\n", host_client->name);
+		Sys_Printf ("Client %s removed\n", host_client->name);
 	}
 	// break the net connection
+	Sys_MaskPrintf (SYS_NET, "dropping client\n");
 	NET_Close (host_client->netconnection);
 	host_client->netconnection = NULL;
 
@@ -441,12 +423,12 @@ SV_DropClient (qboolean crash)
 /*
 	Host_ShutdownServer
 
-	This only happens at the end of a game, not between levels
+	This happens only at the end of a game, not between levels
 */
 void
 Host_ShutdownServer (qboolean crash)
 {
-	char        message[4];
+	byte        message[4];
 	double      start;
 	int         count, i;
 	sizebuf_t   buf;
@@ -457,7 +439,7 @@ Host_ShutdownServer (qboolean crash)
 	sv.active = false;
 
 	// stop all client sounds immediately
-	if (cls.state == ca_connected)
+	if (cls.state >= ca_connected)
 		CL_Disconnect ();
 
 	// flush any pending messages - like the score!!!
@@ -487,9 +469,9 @@ Host_ShutdownServer (qboolean crash)
 	buf.maxsize = 4;
 	buf.cursize = 0;
 	MSG_WriteByte (&buf, svc_disconnect);
-	count = NET_SendToAll (&buf, 5);
+	count = NET_SendToAll (&buf, 5.0);
 	if (count)
-		Con_Printf
+		Sys_Printf
 			("Host_ShutdownServer: NET_SendToAll failed for %u clients\n",
 			 count);
 
@@ -512,8 +494,9 @@ Host_ShutdownServer (qboolean crash)
 void
 Host_ClearMemory (void)
 {
-	Con_DPrintf ("Clearing memory\n");
-	D_FlushCaches ();
+	Sys_MaskPrintf (SYS_DEV, "Clearing memory\n");
+	if (viddef.flush_caches)
+		viddef.flush_caches ();
 	Mod_ClearAll ();
 	if (host_hunklevel)
 		Hunk_FreeToLowMark (host_hunklevel);
@@ -521,6 +504,8 @@ Host_ClearMemory (void)
 	cls.signon = 0;
 	memset (&sv, 0, sizeof (sv));
 	memset (&cl, 0, sizeof (cl));
+	if (r_data)
+		r_data->force_fullscreen = 0;
 }
 
 /*
@@ -528,7 +513,7 @@ Host_ClearMemory (void)
 
 	Returns false if the time is too short to run a frame
 */
-static qboolean
+static float
 Host_FilterTime (float time)
 {
 	float       timedifference;
@@ -547,7 +532,7 @@ Host_FilterTime (float time)
 	timedifference = (timescale / 72.0) - (realtime - oldrealtime);
 
 	if (!cls.timedemo && (timedifference > 0))
-		return false;                   // framerate is too high
+		return timedifference;                   // framerate is too high
 
 	host_frametime = realtime - oldrealtime;
 	oldrealtime = realtime;
@@ -560,13 +545,13 @@ Host_FilterTime (float time)
 	else	// don't allow really long or short frames
 		host_frametime = bound (0.001, host_frametime, 0.1);
 
-	return true;
+	return 0;
 }
 
 void
 Host_ServerFrame (void)
 {
-	*sv_globals.frametime = host_frametime;
+	*sv_globals.frametime = sv_frametime = host_frametime;
 
 	// set the time and clear the general datagram
 	SV_ClearDatagram ();
@@ -578,7 +563,7 @@ Host_ServerFrame (void)
 
 	// move things around and think
 	// always pause in single player if in console or menus
-	if (!sv.paused && (svs.maxclients > 1 || key_dest == key_game)) {
+	if (!sv.paused && (svs.maxclients > 1 || host_in_game)) {
 		SV_Physics ();
 		sv.time += host_frametime;
 	}
@@ -601,21 +586,17 @@ Host_ClientFrame (void)
 	host_time += host_frametime;
 
 	// fetch results from server
-	if (cls.state == ca_connected)
+	if (cls.state >= ca_connected)
 		CL_ReadFromServer ();
 
 	// update video
 	if (host_speeds->int_val)
 		time1 = Sys_DoubleTime ();
 
-	r_inhibit_viewmodel = (chase_active->int_val
-						   || (cl.stats[STAT_ITEMS] & IT_INVISIBILITY)
-						   || cl.stats[STAT_HEALTH] <= 0);
-	r_force_fullscreen = cl.intermission;
-	r_paused = cl.paused;
-	r_active = cls.state == ca_active;
-	r_view_model = &cl.viewent;
-	r_frametime = host_frametime;
+	r_data->inhibit_viewmodel = (chase_active->int_val
+								 || (cl.stats[STAT_ITEMS] & IT_INVISIBILITY)
+								 || cl.stats[STAT_HEALTH] <= 0);
+	r_data->frametime = host_frametime;
 
 	CL_UpdateScreen (cl.time);
 
@@ -623,11 +604,18 @@ Host_ClientFrame (void)
 		time2 = Sys_DoubleTime ();
 
 	// update audio
-	if (cls.signon == SIGNONS) {
-		S_Update (r_origin, vpn, vright, vup);
-		R_DecayLights (host_frametime);
+	if (cls.state == ca_active) {
+		mleaf_t    *l;
+		byte       *asl = 0;
+
+		l = Mod_PointInLeaf (r_data->origin, cl.worldmodel);
+		if (l)
+			asl = l->ambient_sound_level;
+		S_Update (r_data->origin, r_data->vpn, r_data->vright, r_data->vup,
+				  asl);
+		r_funcs->R_DecayLights (host_frametime);
 	} else
-		S_Update (vec3_origin, vec3_origin, vec3_origin, vec3_origin);
+		S_Update (vec3_origin, vec3_origin, vec3_origin, vec3_origin, 0);
 
 	CDAudio_Update ();
 
@@ -636,7 +624,7 @@ Host_ClientFrame (void)
 		time3 = Sys_DoubleTime ();
 		pass2 = (time2 - time1) * 1000;
 		pass3 = (time3 - time2) * 1000;
-		Con_Printf ("%3i tot %3i server %3i gfx %3i snd\n",
+		Sys_Printf ("%3i tot %3i server %3i gfx %3i snd\n",
 					pass1 + pass2 + pass3, pass1, pass2, pass3);
 	}
 }
@@ -650,6 +638,7 @@ static void
 _Host_Frame (float time)
 {
 	static int  first = 1;
+	float       sleeptime;
 
 	if (setjmp (host_abortserver))
 		return;							// something bad happened, or the
@@ -657,10 +646,18 @@ _Host_Frame (float time)
 
 	rand ();							// keep the random time dependent
 
+	if (cls.demo_capture)
+		time = 1.0 / 30;		//FIXME fixed 30fps atm
+
 	// decide the simulation time
-	if (!Host_FilterTime (time))
-		return;							// don't run too fast, or packets
-										// will flood out
+	if ((sleeptime = Host_FilterTime (time)) != 0) {
+		// don't run too fast, or packet will flood outs
+#ifdef HAVE_USLEEP
+		if (cl_usleep_cache && sleeptime > 0.002) // minimum sleep time
+			usleep ((unsigned long) (sleeptime * 1000000 / 2));
+#endif
+		return;
+	}
 
 	if (cls.state != ca_dedicated)
 		IN_ProcessEvents ();
@@ -699,6 +696,14 @@ _Host_Frame (float time)
 	else
 		host_time += host_frametime;	//FIXME is this needed? vcr stuff
 
+	if (cls.demo_capture) {
+		tex_t      *tex = r_funcs->SCR_CaptureBGR ();
+		WritePNGqfs (va ("%s/qfmv%06d.png", qfs_gamedir->dir.shots,
+						 cls.demo_capture++),
+					 tex->data, tex->width, tex->height);
+		free (tex);
+	}
+
 	host_framecount++;
 	fps_count++;
 }
@@ -735,7 +740,7 @@ Host_Frame (float time)
 			c++;
 	}
 
-	Con_Printf ("serverprofile: %2i clients %2i msec\n", c, m);
+	Sys_Printf ("serverprofile: %2i clients %2i msec\n", c, m);
 }
 
 
@@ -828,7 +833,7 @@ check_quakerc (void)
 }
 
 static void
-CL_Init_Memory (void)
+Host_Init_Memory (void)
 {
 	int         mem_parm = COM_CheckParm ("-mem");
 	int         mem_size;
@@ -839,9 +844,9 @@ CL_Init_Memory (void)
 	else
 		minimum_memory = MINIMUM_MEMORY_LEVELPAK;
 
-	host_mem_size = Cvar_Get ("host_mem_size", "16", CVAR_NONE, NULL,
+	host_mem_size = Cvar_Get ("host_mem_size", "40", CVAR_NONE, NULL,
 							  "Amount of memory (in MB) to allocate for the "
-							  PROGRAM " heap");
+							  PACKAGE_NAME " heap");
 	if (mem_parm)
 		Cvar_Set (host_mem_size, com_argv[mem_parm + 1]);
 
@@ -863,92 +868,45 @@ CL_Init_Memory (void)
 
 	Sys_PageIn (mem_base, mem_size);
 	Memory_Init (mem_base, mem_size);
+
+	Sys_Printf ("%4.1f megabyte heap\n", host_mem_size->value);
+}
+
+static void
+host_keydest_callback (keydest_t kd)
+{
+	host_in_game = kd == key_game;
 }
 
 void
 Host_Init (void)
 {
-	Con_Printf ("Host_Init\n");
+	Sys_Printf ("Host_Init\n");
 
 	host_cbuf = Cbuf_New (&id_interp);
 	cmd_source = src_command;
 
-	Cvar_Init_Hash ();
-	Cmd_Init_Hash ();
-	Cvar_Init ();
-	Sys_Init_Cvars ();
-
 	Sys_Init ();
-
-	Cmd_Init ();
 	GIB_Init (true);
+	COM_ParseConfig ();
 
-	// execute +set as early as possible
-	Cmd_StuffCmds (host_cbuf);
-	Cbuf_Execute_Sets (host_cbuf);
+	Host_Init_Memory ();
 
-	// execute the global configuration file if it exists
-	// would have been nice if Cmd_Exec_f could have been used, but it
-	// only reads from within the quake file system, and changing that is
-	// probably Not A Good Thing (tm).
-	fs_globalcfg = Cvar_Get ("fs_globalcfg", FS_GLOBALCFG,
-							 CVAR_ROM, NULL, "global configuration file");
-	Cmd_Exec_File (host_cbuf, fs_globalcfg->string, 0);
-	Cbuf_Execute_Sets (host_cbuf);
-
-	// execute +set again to override the config file
-	Cmd_StuffCmds (host_cbuf);
-	Cbuf_Execute_Sets (host_cbuf);
-
-	fs_usercfg = Cvar_Get ("fs_usercfg", FS_USERCFG, CVAR_ROM, NULL,
-						   "user configuration file");
-	Cmd_Exec_File (host_cbuf, fs_usercfg->string, 0);
-	Cbuf_Execute_Sets (host_cbuf);
-
-	// execute +set again to override the config file
-	Cmd_StuffCmds (host_cbuf);
-	Cbuf_Execute_Sets (host_cbuf);
-
-	CL_Init_Memory ();
-
-	pr_gametype = "netquake";
-
-	QFS_Init ("nq");
 	PI_Init ();
 
-	Chase_Init_Cvars ();
-	CL_InitCvars ();
-	IN_Init_Cvars ();
-	VID_Init_Cvars ();
-	S_Init_Cvars ();
-	Key_Init_Cvars ();
-	Con_Init_Cvars ();
+	Game_Init ();
+
 	PR_Init_Cvars ();
 	SV_Progs_Init_Cvars ();
-	R_Init_Cvars ();
-	R_Particles_Init_Cvars ();
-	Mod_Init_Cvars ();
-	Host_Skin_Init_Cvars ();
-	V_Init_Cvars ();
+
+	if (!isDedicated)
+		CL_InitCvars ();
 
 	PR_Init ();
 
-	V_Init ();
-	Game_Init ();
-	COM_Init ();
-
 	if (isDedicated) {
 		PI_RegisterPlugins (server_plugin_list);
 		Con_Init ("server");
-	} else {
-		PI_RegisterPlugins (client_plugin_list);
-		Con_Init ("client");
-	}
-	if (con_module) {
-		con_module->data->console->realtime = &con_realtime;
-		con_module->data->console->frametime = &con_frametime;
-		con_module->data->console->quit = Host_Quit_f;
-		con_module->data->console->cbuf = host_cbuf;
 	}
 
 	Host_InitVCR (&host_parms);
@@ -956,40 +914,21 @@ Host_Init (void)
 
 	NET_Init ();
 
-	W_LoadWadFile ("gfx.wad");
-	Key_Init (host_cbuf);
 	Mod_Init ();
 
-	CL_Demo_Init ();
+	Key_KeydestCallback (host_keydest_callback);
 
-	SV_Progs_Init ();
 	SV_Init ();
 
-	Con_Printf ("%4.1f megabyte heap\n", host_mem_size->value);
+	if (cls.state != ca_dedicated)
+		CL_Init (host_cbuf);
 
-	if (cls.state != ca_dedicated) {
-		host_basepal = (byte *) QFS_LoadHunkFile ("gfx/palette.lmp");
-		if (!host_basepal)
-			Sys_Error ("Couldn't load gfx/palette.lmp");
-		vid_colormap = (byte *) QFS_LoadHunkFile ("gfx/colormap.lmp");
-		if (!vid_colormap)
-			Sys_Error ("Couldn't load gfx/colormap.lmp");
-
-		VID_Init (host_basepal);
-		Draw_Init ();
-		SCR_Init ();
-		R_Init ();
-
-		S_Init (&cl.worldmodel, &viewentity, &host_frametime);
-
-		CDAudio_Init ();
-		Sbar_Init ();
-		CL_Init ();
-		IN_Init ();
-
-		CL_SetState (ca_disconnected);
+	if (con_module) {
+		con_module->data->console->realtime = &con_realtime;
+		con_module->data->console->frametime = &con_frametime;
+		con_module->data->console->quit = Host_Quit_f;
+		con_module->data->console->cbuf = host_cbuf;
 	}
-	Host_Skin_Init ();
 
 	CL_UpdateScreen (cl.time);
 	CL_UpdateScreen (cl.time);
@@ -1005,10 +944,11 @@ Host_Init (void)
 	Hunk_AllocName (0, "-HOST_HUNKLEVEL-");
 	host_hunklevel = Hunk_LowMark ();
 
-	Con_Printf ("\nVersion %s (build %04d)\n\n", VERSION,
+	Sys_Printf ("\nVersion %s (build %04d)\n\n", PACKAGE_VERSION,
 				build_number ());
 
-	Con_Printf ("\x80\x81\x81\x82 %s initialized \x80\x81\x81\x82\n", PROGRAM);
+	Sys_Printf ("\x80\x81\x81\x82 %s initialized \x80\x81\x81\x82\n",
+			    PACKAGE_NAME);
 
 	host_initialized = true;
 
@@ -1032,14 +972,10 @@ Host_Shutdown (void)
 	}
 	isdown = true;
 
-	Host_WriteConfiguration ();
 
 	NET_Shutdown ();
 	if (cls.state != ca_dedicated) {
-		CDAudio_Shutdown ();
-		S_Shutdown ();
-		IN_Shutdown ();
-		VID_Shutdown ();
+		CL_Shutdown ();
 	}
 	Con_Shutdown ();
 }
