@@ -28,35 +28,41 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sbar.c,v 1.67 2004/03/02 03:55:18 taniwha Exp $";
+#ifdef HAVE_STRING_H
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
 
 #include <time.h>
 
-#include "QF/console.h"
 #include "QF/cmd.h"
+#include "QF/console.h"
 #include "QF/cvar.h"
 #include "QF/draw.h"
 #include "QF/dstring.h"
 #include "QF/gib.h"
-#include "QF/plugin.h"
 #include "QF/screen.h"
 #include "QF/sys.h"
-#include "QF/vid.h"
 #include "QF/va.h"
+#include "QF/vid.h"
 #include "QF/view.h"
 #include "QF/wad.h"
 
+#include "QF/plugin/console.h"
+#include "QF/plugin/vid_render.h"
+
 #include "client.h"
 #include "compat.h"
 #include "game.h"
-#include "r_cvar.h"					// for scr_printspeed
 #include "sbar.h"
 #include "server.h"
 
 int         sb_updates;				// if >= vid.numpages, no update needed
 
 #define STAT_MINUS		10			// num frame for '-' stats digit
+
 qpic_t     *sb_nums[2][11];
 qpic_t     *sb_colon, *sb_slash;
 qpic_t     *sb_ibar;
@@ -79,6 +85,7 @@ qpic_t     *sb_face_invis_invuln;
 qboolean    sb_showscores;
 
 int         sb_lines;				// scan lines to draw
+qboolean	hudswap;
 
 qpic_t     *rsb_invbar[2];
 qpic_t     *rsb_weapons[5];
@@ -99,6 +106,8 @@ qpic_t     *hsb_items[2];			// MED 01/04/97 added hipnotic items array
 cvar_t     *hud_sbar;
 cvar_t     *hud_swap;
 cvar_t     *hud_scoreboard_gravity;
+cvar_t     *scr_centertime;
+cvar_t     *scr_printspeed;
 
 static view_t *sbar_view;
 static view_t *sbar_inventory_view;
@@ -111,10 +120,12 @@ static view_t *hud_frags_view;
 
 static view_t *overlay_view;
 static view_t *stuff_view;
+static view_t *main_view;
 
 static void
 hud_swap_f (cvar_t *var)
 {
+	hudswap = var->int_val;
 	if (var->int_val) {
 		hud_armament_view->gravity = grav_southwest;
 		hud_armament_view->children[0]->gravity = grav_northwest;
@@ -196,20 +207,16 @@ calc_sb_lines (cvar_t *var)
 static void
 hud_sbar_f (cvar_t *var)
 {
-	vid.recalc_refdef = true;
-	if (scr_viewsize)
-		calc_sb_lines (scr_viewsize);
-	r_lineadj = var->int_val ? sb_lines : 0;
-	if (con_module) {
-		if (var->int_val) {
-			view_remove (con_module->data->console->view,
-						 con_module->data->console->view->children[0]);
-			view_insert (con_module->data->console->view, sbar_view, 0);
-		} else {
-			view_remove (con_module->data->console->view,
-						 con_module->data->console->view->children[0]);
-			view_insert (con_module->data->console->view, hud_view, 0);
-		}
+	r_data->vid->recalc_refdef = true;
+	if (r_data->scr_viewsize)
+		calc_sb_lines (r_data->scr_viewsize);
+	r_data->lineadj = var->int_val ? sb_lines : 0;
+	if (var->int_val) {
+		view_remove (main_view, main_view->children[0]);
+		view_insert (main_view, sbar_view, 0);
+	} else {
+		view_remove (main_view, main_view->children[0]);
+		view_insert (main_view, hud_view, 0);
 	}
 }
 
@@ -218,87 +225,66 @@ viewsize_f (cvar_t *var)
 {
 	calc_sb_lines (var);
 	if (hud_sbar)
-		r_lineadj = hud_sbar->int_val ? sb_lines : 0;
+		r_data->lineadj = hud_sbar->int_val ? sb_lines : 0;
 }
 
 
 static int
 Sbar_ColorForMap (int m)
 {
-	return m + 8;						// FIXME: Might want this to be
-	// return (bound (0, m, 13) * 16) + 8;
+	return (bound (0, m, 13) * 16) + 8;
 }
 
-
-/*
-	Sbar_ShowScores
-
-	Tab key has been pressed, inform sbar it needs to show scores
-*/
 static void
 Sbar_ShowScores (void)
 {
 	if (sb_showscores)
 		return;
+
 	sb_showscores = true;
 	sb_updates = 0;
 }
 
-
-/*
-	Sbar_DontShowScores
-
-	Tab key up, show normal sbar again
-*/
 static void
 Sbar_DontShowScores (void)
 {
-	if (!sb_showscores)
-		return;
 	sb_showscores = false;
 	sb_updates = 0;
 }
 
-
-/*
-	Sbar_Changed
-
-	Call this to signal sbar to redraw next frame.
-*/
 void
 Sbar_Changed (void)
 {
 	sb_updates = 0;						// update next frame
 }
 
-
-static inline void
+static void
 draw_pic (view_t *view, int x, int y, qpic_t *pic)
 {
-	Draw_Pic (view->xabs + x, view->yabs + y, pic);
+	r_funcs->Draw_Pic (view->xabs + x, view->yabs + y, pic);
 }
 
 static inline void
 draw_cachepic (view_t *view, int x, int y, const char *name, int cent)
 {
-	qpic_t *pic = Draw_CachePic (name, true);
+	qpic_t *pic = r_funcs->Draw_CachePic (name, true);
 	if (cent)
 		x += (view->xlen - pic->width) / 2;
-	Draw_Pic (view->xabs + x, view->yabs + y, pic);
+	r_funcs->Draw_Pic (view->xabs + x, view->yabs + y, pic);
 }
 
 static inline void
 draw_subpic (view_t *view, int x, int y, qpic_t *pic,
 		     int srcx, int srcy, int width, int height)
 {
-	Draw_SubPic (view->xabs + x, view->yabs + y, pic,
+	r_funcs->Draw_SubPic (view->xabs + x, view->yabs + y, pic,
 				 srcx, srcy, width, height);
 }
 
 static inline void
 draw_transpic (view_t *view, int x, int y, qpic_t *pic)
 {
-	Draw_Pic (view->xabs + x, view->yabs + y, pic);
+	r_funcs->Draw_Pic (view->xabs + x, view->yabs + y, pic);
 }
 
 
@@ -307,31 +293,31 @@ draw_transpic (view_t *view, int x, int y, qpic_t *pic)
 static inline void
 draw_character (view_t *view, int x, int y, int c)
 {
-	Draw_Character (view->xabs + x, view->yabs + y, c);
+	r_funcs->Draw_Character (view->xabs + x, view->yabs + y, c);
 }
 
 static inline void
 draw_string (view_t *view, int x, int y, const char *str)
 {
-	Draw_String (view->xabs + x, view->yabs + y, str);
+	r_funcs->Draw_String (view->xabs + x, view->yabs + y, str);
 }
 
 static inline void
 draw_altstring (view_t *view, int x, int y, const char *str)
 {
-	Draw_AltString (view->xabs + x, view->yabs + y, str);
+	r_funcs->Draw_AltString (view->xabs + x, view->yabs + y, str);
 }
 
 static inline void
 draw_nstring (view_t *view, int x, int y, const char *str, int n)
 {
-	Draw_nString (view->xabs + x, view->yabs + y, str, n);
+	r_funcs->Draw_nString (view->xabs + x, view->yabs + y, str, n);
 }
 
 static inline void
 draw_fill (view_t *view, int x, int y, int w, int h, int col)
 {
-	Draw_Fill (view->xabs + x, view->yabs + y, w, h, col);
+	r_funcs->Draw_Fill (view->xabs + x, view->yabs + y, w, h, col);
 }
 
 static void
@@ -389,8 +375,131 @@ draw_smallnum (view_t *view, int x, int y, int n, int packed, int colored)
 
 static void
 draw_tile (view_t *view)
-{   
-	Draw_TileClear (view->xabs, view->yabs, view->xlen, view->ylen);
+{
+	r_funcs->Draw_TileClear (view->xabs, view->yabs, view->xlen, view->ylen);
+}
+
+static void
+draw_ammo_sbar (view_t *view)
+{
+	int         i, count;
+
+	// ammo counts
+	for (i = 0; i < 4; i++) {
+		count = cl.stats[STAT_SHELLS + i];
+		draw_smallnum (view, (6 * i + 1) * 8 + 2, 0, count, 0, 1);
+	}
+}
+
+static void
+draw_ammo_hud (view_t *view)
+{
+	int         i, count;
+
+	// ammo counts
+	for (i = 0; i < 4; i++) {
+		count = cl.stats[STAT_SHELLS + i];
+		draw_subpic (view, 0, i * 11, sb_ibar, 3 + (i * 48), 0, 42, 11);
+		draw_smallnum (view, 7, i * 11, count, 0, 1);
+	}
+}
+
+static int
+calc_flashon (float time, int mask)
+{
+	int         flashon;
+
+	flashon = (int) ((cl.time - time) * 10);
+	if (flashon < 0)
+		flashon = 0;
+	if (flashon >= 10) {
+		if (cl.stats[STAT_ACTIVEWEAPON] == mask)
+			flashon = 1;
+		else
+			flashon = 0;
+	} else
+		flashon = (flashon % 5) + 2;
+	return flashon;
+}
+
+static void
+draw_weapons_sbar (view_t *view)
+{
+	int         flashon, i;
+
+	for (i = 0; i < 7; i++) {
+		if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
+			flashon = calc_flashon (cl.item_gettime[i], IT_SHOTGUN << i);
+			draw_pic (view, i * 24, 0, sb_weapons[flashon][i]);
+			if (flashon > 1)
+				sb_updates = 0;			// force update to remove flash
+		}
+	}
+}
+
+static void
+draw_weapons_hud (view_t *view)
+{
+	int         flashon, i, x = 0;
+
+	if (view->parent->gravity == grav_southeast)
+		x = view->xlen - 24;
+
+	for (i = r_data->vid->conheight < 204; i < 7; i++) {
+		if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
+			flashon = calc_flashon (cl.item_gettime[i], IT_SHOTGUN << i);
+			draw_subpic (view, x, i * 16, sb_weapons[flashon][i], 0, 0, 24, 16);
+			if (flashon > 1)
+				sb_updates = 0;			// force update to remove flash
+		}
+	}
+}
+
+static void
+draw_items (view_t *view)
+{
+	float       time;
+	int         flashon = 0, i;
+
+	for (i = 0; i < 6; i++) {
+		if (cl.stats[STAT_ITEMS] & (1 << (17 + i))) {
+			time = cl.item_gettime[17 + i];
+			if (time && time > cl.time - 2 && flashon) {	// Flash frame
+				sb_updates = 0;
+			} else {
+				draw_pic (view, i * 16, 0, sb_items[i]);
+			}
+			if (time && time > cl.time - 2)
+				sb_updates = 0;
+		}
+	}
+}
+
+static void
+draw_sigils (view_t *view)
+{
+	float       time;
+	int         flashon = 0, i;
+
+	for (i = 0; i < 4; i++) {
+		if (cl.stats[STAT_ITEMS] & (1 << (28 + i))) {
+			time = cl.item_gettime[28 + i];
+			if (time && time > cl.time - 2 && flashon) {	// flash frame
+				sb_updates = 0;
+			} else {
+				draw_pic (view, i * 8, 0, sb_sigil[i]);
+			}
+			if (time && time > cl.time - 2)
+				sb_updates = 0;
+		}
+	}
+}
+
+static void
+draw_inventory_sbar (view_t *view)
+{
+	draw_pic (view, 0, 0, sb_ibar);
+	view_draw (view);
 }
 
 
@@ -410,7 +519,7 @@ Sbar_SortFrags (void)
 	// sort by frags
 	scoreboardlines = 0;
 	for (i = 0; i < cl.maxclients; i++) {
-		if (cl.scores[i].name[0]) {
+		if (cl.scores[i].name->value[0]) {
 			fragsort[scoreboardlines] = i;
 			scoreboardlines++;
 		}
@@ -455,126 +564,8 @@ draw_solo (view_t *view)
 
 	// draw level name
 	l = strlen (cl.levelname);
-	draw_string (view, 232 - l * 4, 12, cl.levelname);
-}
-
-static void
-draw_ammo_sbar (view_t *view) 
-{
-	int         i, count;
-
-	for (i = 0; i < 4; i++) {
-		count = cl.stats[STAT_SHELLS + i];
-		draw_smallnum (view, (6 * i + 1) * 8 - 2, 0, count, 0, 1);
-	}
-}
-
-static void
-draw_ammo_hud (view_t *view) 
-{
-	int         i, count;
-
-	for (i = 0; i < 4; i++) {
-		count = cl.stats[STAT_SHELLS + i];
-		draw_subpic (view, 0, i * 11, sb_ibar, 3 + (i * 48), 0, 42, 11);
-		draw_smallnum (view, 7, i * 11, count, 0, 1);
-	}
-}
-
-static int
-calc_flashon (float time, int mask)
-{
-	int         flashon;
-
-	flashon = (int) ((cl.time - time) * 10);
-	if (flashon < 0)
-		flashon = 0;
-	if (flashon >= 10) {
-		if (cl.stats[STAT_ACTIVEWEAPON] == mask)
-			flashon = 1;
-		else
-			flashon = 0;
-	} else
-		flashon = (flashon % 5) + 2;
-	return flashon;
-}
-
-static void
-draw_weapons_sbar (view_t *view)
-{
-	int         flashon, i;
-
-	// weapons
-	for (i = 0; i < 7; i++) {
-		if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
-			flashon = calc_flashon (cl.item_gettime[i], IT_SHOTGUN << i);
-			draw_pic (view, i * 24, 0, sb_weapons[flashon][i]);
-			if (flashon > 1)
-				sb_updates = 0;			// force update to remove flash
-		}
-	}
-}
-
-static void
-draw_weapons_hud (view_t *view)
-{
-	int         flashon, i;
-
-	for (i = vid.conheight < 204; i < 7; i++) {
-		if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
-			flashon = calc_flashon (cl.item_gettime[i], IT_SHOTGUN << i);
-			draw_subpic (view, 0, i * 16, sb_weapons[flashon][i], 0, 0, 24, 16);
-			if (flashon > 1)
-				sb_updates = 0;			// force update to remove flash
-		}
-	}
-}
-
-static void
-draw_items (view_t *view)
-{
-	float       time;
-	int         flashon = 0, i;
-
-	for (i = 0; i < 6; i++) {
-		if (cl.stats[STAT_ITEMS] & (1 << (17 + i))) {
-			time = cl.item_gettime[17 + i];
-			if (time && time > (cl.time - 2) && flashon) {	// Flash frame
-				sb_updates = 0;
-			} else {
-				draw_pic (view, i * 16, 0, sb_items[i]);
-			}
-			if (time && time > cl.time - 2)
-				sb_updates = 0;
-		}
-	}
-}
-
-static void
-draw_sigils (view_t *view)
-{
-	float       time;
-	int         flashon = 0, i;
-
-	for (i = 0; i < 4; i++) {
-		if (cl.stats[STAT_ITEMS] & (1 << (28 + i))) {
-			time = cl.item_gettime[28 + i];
-			if (time && time > cl.time - 2 && flashon) {	// flash frame
-				sb_updates = 0;
-			} else {
-				draw_pic (view, i * 8, 0, sb_sigil[i]);
-			}
-			if (time && time > cl.time - 2)
-				sb_updates = 0;
-		}
-	}
-}
-
-static void
-draw_inventory_sbar (view_t *view)
-{
-	draw_pic (view, 0, 0, sb_ibar);
-	view_draw (view);
+	l = 232 - l * 4;
+	draw_string (view, max (l, 152), 12, cl.levelname);
 }
 
 static void
@@ -598,14 +589,12 @@ draw_frags (view_t *view)
 	for (i = 0; i < l; i++) {
 		k = fragsort[i];
 		s = &cl.scores[k];
-		if (!s->name[0])
+		if (!s->name->value[0])
 			continue;
 
 		// draw background
-		top = s->colors & 0xf0;
-		bottom = (s->colors & 15) << 4;
-		top = Sbar_ColorForMap (top);
-		bottom = Sbar_ColorForMap (bottom);
+		top = Sbar_ColorForMap (s->topcolor);
+		bottom = Sbar_ColorForMap (s->bottomcolor);
 
 		draw_fill (view, x + 4, 1, 28, 4, top);
 		draw_fill (view, x + 4, 5, 28, 3, bottom);
@@ -614,6 +603,7 @@ draw_frags (view_t *view)
 
 		if (k == cl.viewentity - 1)
 			p = i;
+
 		x += 32;
 	}
 	if (p != -1) {
@@ -740,7 +730,7 @@ draw_rogue_weapons_hud (view_t *view)
 	int         flashon, i, j;
 	qpic_t     *pic;
 
-	for (i = vid.conheight < 204; i < 7; i++) {
+	for (i = r_data->vid->conheight < 204; i < 7; i++) {
 		if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
 			flashon = calc_flashon (cl.item_gettime[i], IT_SHOTGUN << i);
 			if (i >= 2) {
@@ -816,10 +806,8 @@ draw_rogue_face (view_t *view)
 
 	s = &cl.scores[cl.viewentity - 1];
 
-	top = (s->colors & 0xf0);
-	bottom = ((s->colors & 15) << 4);
-	top = Sbar_ColorForMap (top);
-	bottom = Sbar_ColorForMap (bottom);
+	top = Sbar_ColorForMap (s->topcolor);
+	bottom = Sbar_ColorForMap (s->bottomcolor);
 
 	draw_pic (view, 112, 0, rsb_teambord);
 	draw_fill (view, 113, 3, 22, 9, top);
@@ -845,7 +833,7 @@ draw_rogue_status (view_t *view)
 	else if (cl.stats[STAT_ITEMS] & RIT_ARMOR1)
 		draw_pic (view, 0, 0, sb_armor[0]);
 
-	// PGM 03/02/97 - fixed so color swatch only appears in CTF modes
+	// PGM 03/02/97 - fixed so color swatch appears in only CTF modes
 	if (cl.maxclients != 1 && teamplay->int_val > 3 && teamplay->int_val < 7)
 		draw_rogue_face (view);
 	else
@@ -997,19 +985,23 @@ draw_hipnotic_status (view_t *view)
 		draw_pic (view, 209, 12, sb_items[1]);
 }
 
-void
-Sbar_Draw (void)
+static void
+sbar_update_vis (void)
 {
 	qboolean    headsup;
 
+	if (r_data->scr_copyeverything)
+		Sbar_Changed ();
+
 	sbar_view->visible = 0;
 
-	headsup = !(hud_sbar->int_val || scr_viewsize->int_val < 100);
+	headsup = !(hud_sbar->int_val || r_data->scr_viewsize->int_val < 100);
 
-	if ((sb_updates >= vid.numpages) && !headsup)
+	if ((sb_updates >= r_data->vid->numpages) && !headsup)
 		return;
 
-	if (con_module && con_module->data->console->lines == vid.height)
+	if (con_module &&
+		con_module->data->console->lines == r_data->vid->conheight)
 		return;							// console is full screen
 
 	if (cls.state == ca_active
@@ -1023,9 +1015,15 @@ Sbar_Draw (void)
 
 	sbar_view->visible = 1;
 
-	scr_copyeverything = 1;
+	r_data->scr_copyeverything = 1;
 	sb_updates++;
+}
 
+void
+Sbar_Draw (void)
+{
+	sbar_update_vis ();
+	main_view->draw (main_view);
 }
 
 static void
@@ -1036,8 +1034,8 @@ Sbar_DeathmatchOverlay (view_t *view)
 	int         x, y;
 	scoreboard_t *s;
 
-	scr_copyeverything = 1;
-	scr_fullupdate = 0;
+	r_data->scr_copyeverything = 1;
+	r_data->scr_fullupdate = 0;
 
 	draw_cachepic (view, 0, 0, "gfx/ranking.lmp", 1);
 
@@ -1052,14 +1050,12 @@ Sbar_DeathmatchOverlay (view_t *view)
 	for (i = 0; i < l; i++) {
 		k = fragsort[i];
 		s = &cl.scores[k];
-		if (!s->name[0])
+		if (!s->name->value[0])
 			continue;
 
 		// draw background
-		top = s->colors & 0xf0;
-		bottom = (s->colors & 15) << 4;
-		top = Sbar_ColorForMap (top);
-		bottom = Sbar_ColorForMap (bottom);
+		top = Sbar_ColorForMap (s->topcolor);
+		bottom = Sbar_ColorForMap (s->bottomcolor);
 
 		draw_fill (view, x, y, 40, 4, top);
 		draw_fill (view, x, y + 4, 40, 4, bottom);
@@ -1070,7 +1066,7 @@ Sbar_DeathmatchOverlay (view_t *view)
 			draw_character (view, x - 4, y, 12);
 
 		// draw name
-		draw_string (view, x + 64, y, s->name);
+		draw_string (view, x + 64, y, s->name->value);
 
 		y += 10;
 	}
@@ -1117,7 +1113,7 @@ draw_time (view_t *view)
 static void
 draw_fps (view_t *view)
 {
-	static char   st[80];		//FIXME: overflow
+	static char   st[80];
 	double        t;
 	static double lastframetime;
 	static double lastfps;
@@ -1147,8 +1143,8 @@ draw_intermission (view_t *view)
 	int         dig;
 	int         num;
 
-	scr_copyeverything = 1;
-	scr_fullupdate = 0;
+	r_data->scr_copyeverything = 1;
+	r_data->scr_fullupdate = 0;
 
 	draw_cachepic (view, 64, 24, "gfx/complete.lmp", 0);
 
@@ -1174,8 +1170,8 @@ draw_intermission (view_t *view)
 void
 Sbar_IntermissionOverlay (void)
 {
-	scr_copyeverything = 1;
-	scr_fullupdate = 0;
+	r_data->scr_copyeverything = 1;
+	r_data->scr_fullupdate = 0;
 
 	if (cl.gametype == GAME_DEATHMATCH) {
 		Sbar_DeathmatchOverlay (overlay_view);
@@ -1237,7 +1233,7 @@ Sbar_DrawCenterString (view_t *view, int remaining)
 				break;
 		x = view->xabs + (view->xlen - l * 8) / 2;
 		for (j = 0; j < l; j++, x += 8) {
-			Draw_Character (x, y, start[j]);
+			r_funcs->Draw_Character (x, y, start[j]);
 			if (!remaining--)
 				return;
 		}
@@ -1257,10 +1253,7 @@ Sbar_FinaleOverlay (void)
 {
 	int         remaining;
 
-	if (key_dest != key_game)
-		return;
-
-	scr_copyeverything = 1;
+	r_data->scr_copyeverything = 1;
 
 	draw_cachepic (overlay_view, 0, 16, "gfx/finale.lmp", 1);
 	// the finale prints the characters one at a time
@@ -1271,15 +1264,12 @@ Sbar_FinaleOverlay (void)
 void
 Sbar_DrawCenterPrint (void)
 {
-	scr_copytop = 1;
+	r_data->scr_copytop = 1;
 
-	centertime_off -= r_frametime;
+	centertime_off -= r_data->frametime;
 	if (centertime_off <= 0)
 		return;
 
-	if (key_dest != key_game)
-		return;
-
 	Sbar_DrawCenterString (overlay_view, -1);
 }
 
@@ -1325,8 +1315,8 @@ init_sbar_views (void)
 	view->draw = draw_status;
 	view_add (sbar_view, view);
 
-	if (vid.width > 320) {
-		int         l = (vid.width - 320) / 2;
+	if (r_data->vid->conwidth > 320) {
+		int         l = (r_data->vid->conwidth - 320) / 2;
 
 		view = view_new (-l, 0, l, 48, grav_southwest);
 		view->draw = draw_tile;
@@ -1379,14 +1369,13 @@ init_hud_views (void)
 	if (hud_frags_view)
 		view_add (hud_inventory_view, hud_frags_view);
 
-	view = view_new (0, 0, vid.conwidth, 48, grav_south);
+	view = view_new (0, 0, r_data->vid->conwidth, 48, grav_south);
 	view_add (view, hud_view);
 	hud_view = view;
 
 	view_add (hud_view, hud_armament_view);
 
-	if (con_module)
-		view_insert (con_module->data->console->view, hud_view, 0);
+	view_insert (main_view, hud_view, 0);
 }
 
 static void
@@ -1431,8 +1420,8 @@ init_hipnotic_sbar_views (void)
 	view->draw = draw_hipnotic_status;
 	view_add (sbar_view, view);
 
-	if (vid.width > 320) {
-		int         l = (vid.width - 320) / 2;
+	if (r_data->vid->conwidth > 320) {
+		int         l = (r_data->vid->conwidth - 320) / 2;
 
 		view = view_new (-l, 0, l, 48, grav_southwest);
 		view->draw = draw_tile;
@@ -1457,8 +1446,9 @@ init_hipnotic_hud_views (void)
 
 	hud_view->resize_y = 1;
 
-	if (vid.conheight < 252) {
-		hud_armament_view = view_new (0, min (vid.conheight - 160, 48),
+	if (r_data->vid->conheight < 252) {
+		hud_armament_view = view_new (0,
+									  min (r_data->vid->conheight - 160, 48),
 									  66, 160, grav_southeast);
 	} else {
 		hud_armament_view = view_new (0, 48, 42, 204, grav_southeast);
@@ -1490,14 +1480,13 @@ init_hipnotic_hud_views (void)
 	if (hud_frags_view)
 		view_add (hud_inventory_view, hud_frags_view);
 
-	view = view_new (0, 0, vid.conwidth, 48, grav_south);
+	view = view_new (0, 0, r_data->vid->conwidth, 48, grav_south);
 	view_add (view, hud_view);
 	hud_view = view;
 
 	view_add (hud_view, hud_armament_view);
 
-	if (con_module)
-		view_insert (con_module->data->console->view, hud_view, 0);
+	view_insert (main_view, hud_view, 0);
 }
 
 static void
@@ -1538,8 +1527,8 @@ init_rogue_sbar_views (void)
 	view->draw = draw_rogue_status;
 	view_add (sbar_view, view);
 
-	if (vid.width > 320) {
-		int         l = (vid.width - 320) / 2;
+	if (r_data->vid->conwidth > 320) {
+		int         l = (r_data->vid->conwidth - 320) / 2;
 
 		view = view_new (-l, 0, l, 48, grav_southwest);
 		view->draw = draw_tile;
@@ -1588,38 +1577,43 @@ init_rogue_hud_views (void)
 	if (hud_frags_view)
 		view_add (hud_inventory_view, hud_frags_view);
 
-	view = view_new (0, 0, vid.conwidth, 48, grav_south);
+	view = view_new (0, 0, r_data->vid->conwidth, 48, grav_south);
 	view_add (view, hud_view);
 	hud_view = view;
 
 	view_add (hud_view, hud_armament_view);
 
-	if (con_module)
-		view_insert (con_module->data->console->view, hud_view, 0);
+	view_insert (main_view, hud_view, 0);
 }
 
 static void
 init_views (void)
 {
-	if (vid.conheight > 300)
+	main_view = view_new (0, 0, r_data->vid->conwidth, r_data->vid->conheight,
+						  grav_northwest);
+	if (con_module)
+		view_insert (con_module->data->console->view, main_view, 0);
+	main_view->resize_x = 1;	// get resized if the 2d view resizes
+	main_view->resize_y = 1;
+	main_view->visible = 0;		// but don't let the console draw our stuff
+	if (r_data->vid->conheight > 300)
 		overlay_view = view_new (0, 0, 320, 300, grav_center);
 	else
-		overlay_view = view_new (0, 0, 320, vid.conheight, grav_center);
+		overlay_view = view_new (0, 0, 320, r_data->vid->conheight,
+								 grav_center);
 	overlay_view->draw = draw_overlay;
 	overlay_view->visible = 0;
 
 	stuff_view = view_new (0, 48, 152, 16, grav_southwest);
 	stuff_view->draw = draw_stuff;
 
-	if (con_module) {
-		view_insert (con_module->data->console->view, overlay_view, 0);
-		view_insert (con_module->data->console->view, stuff_view, 0);
-	}
+	view_insert (main_view, overlay_view, 0);
+	view_insert (main_view, stuff_view, 0);
 
-	if (hipnotic) {
+	if (!strcmp (qfs_gamedir->hudtype, "hipnotic")) {
 		init_hipnotic_sbar_views ();
 		init_hipnotic_hud_views ();
-	} else if (rogue) {
+	} else if (!strcmp (qfs_gamedir->hudtype, "rogue")) {
 		init_rogue_sbar_views ();
 		init_rogue_hud_views ();
 	} else {
@@ -1637,11 +1631,12 @@ Sbar_GIB_Print_Center_f (void)
 		Sbar_CenterPrint (GIB_Argv(1));
 }
 
-/*
-	Sbar_Init
+static void
+sbar_keydest_callback (keydest_t kd)
+{
+	overlay_view->visible = kd == key_game;
+}
 
-	Initialize the status bar's data
-*/
 void
 Sbar_Init (void)
 {
@@ -1649,143 +1644,155 @@ Sbar_Init (void)
 
 	init_views ();
 
+	Key_KeydestCallback (sbar_keydest_callback);
+
 	for (i = 0; i < 10; i++) {
-		sb_nums[0][i] = Draw_PicFromWad (va ("num_%i", i));
-		sb_nums[1][i] = Draw_PicFromWad (va ("anum_%i", i));
+		sb_nums[0][i] = r_funcs->Draw_PicFromWad (va ("num_%i", i));
+		sb_nums[1][i] = r_funcs->Draw_PicFromWad (va ("anum_%i", i));
 	}
 
-	sb_nums[0][10] = Draw_PicFromWad ("num_minus");
-	sb_nums[1][10] = Draw_PicFromWad ("anum_minus");
+	sb_nums[0][10] = r_funcs->Draw_PicFromWad ("num_minus");
+	sb_nums[1][10] = r_funcs->Draw_PicFromWad ("anum_minus");
 
-	sb_colon = Draw_PicFromWad ("num_colon");
-	sb_slash = Draw_PicFromWad ("num_slash");
+	sb_colon = r_funcs->Draw_PicFromWad ("num_colon");
+	sb_slash = r_funcs->Draw_PicFromWad ("num_slash");
 
-	sb_weapons[0][0] = Draw_PicFromWad ("inv_shotgun");
-	sb_weapons[0][1] = Draw_PicFromWad ("inv_sshotgun");
-	sb_weapons[0][2] = Draw_PicFromWad ("inv_nailgun");
-	sb_weapons[0][3] = Draw_PicFromWad ("inv_snailgun");
-	sb_weapons[0][4] = Draw_PicFromWad ("inv_rlaunch");
-	sb_weapons[0][5] = Draw_PicFromWad ("inv_srlaunch");
-	sb_weapons[0][6] = Draw_PicFromWad ("inv_lightng");
+	sb_weapons[0][0] = r_funcs->Draw_PicFromWad ("inv_shotgun");
+	sb_weapons[0][1] = r_funcs->Draw_PicFromWad ("inv_sshotgun");
+	sb_weapons[0][2] = r_funcs->Draw_PicFromWad ("inv_nailgun");
+	sb_weapons[0][3] = r_funcs->Draw_PicFromWad ("inv_snailgun");
+	sb_weapons[0][4] = r_funcs->Draw_PicFromWad ("inv_rlaunch");
+	sb_weapons[0][5] = r_funcs->Draw_PicFromWad ("inv_srlaunch");
+	sb_weapons[0][6] = r_funcs->Draw_PicFromWad ("inv_lightng");
 
-	sb_weapons[1][0] = Draw_PicFromWad ("inv2_shotgun");
-	sb_weapons[1][1] = Draw_PicFromWad ("inv2_sshotgun");
-	sb_weapons[1][2] = Draw_PicFromWad ("inv2_nailgun");
-	sb_weapons[1][3] = Draw_PicFromWad ("inv2_snailgun");
-	sb_weapons[1][4] = Draw_PicFromWad ("inv2_rlaunch");
-	sb_weapons[1][5] = Draw_PicFromWad ("inv2_srlaunch");
-	sb_weapons[1][6] = Draw_PicFromWad ("inv2_lightng");
+	sb_weapons[1][0] = r_funcs->Draw_PicFromWad ("inv2_shotgun");
+	sb_weapons[1][1] = r_funcs->Draw_PicFromWad ("inv2_sshotgun");
+	sb_weapons[1][2] = r_funcs->Draw_PicFromWad ("inv2_nailgun");
+	sb_weapons[1][3] = r_funcs->Draw_PicFromWad ("inv2_snailgun");
+	sb_weapons[1][4] = r_funcs->Draw_PicFromWad ("inv2_rlaunch");
+	sb_weapons[1][5] = r_funcs->Draw_PicFromWad ("inv2_srlaunch");
+	sb_weapons[1][6] = r_funcs->Draw_PicFromWad ("inv2_lightng");
 
 	for (i = 0; i < 5; i++) {
-		sb_weapons[2 + i][0] = Draw_PicFromWad (va ("inva%i_shotgun", i + 1));
-		sb_weapons[2 + i][1] = Draw_PicFromWad (va ("inva%i_sshotgun", i + 1));
-		sb_weapons[2 + i][2] = Draw_PicFromWad (va ("inva%i_nailgun", i + 1));
-		sb_weapons[2 + i][3] = Draw_PicFromWad (va ("inva%i_snailgun", i + 1));
-		sb_weapons[2 + i][4] = Draw_PicFromWad (va ("inva%i_rlaunch", i + 1));
-		sb_weapons[2 + i][5] = Draw_PicFromWad (va ("inva%i_srlaunch", i + 1));
-		sb_weapons[2 + i][6] = Draw_PicFromWad (va ("inva%i_lightng", i + 1));
+		sb_weapons[2 + i][0] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_shotgun", i + 1));
+		sb_weapons[2 + i][1] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_sshotgun", i + 1));
+		sb_weapons[2 + i][2] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_nailgun", i + 1));
+		sb_weapons[2 + i][3] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_snailgun", i + 1));
+		sb_weapons[2 + i][4] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_rlaunch", i + 1));
+		sb_weapons[2 + i][5] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_srlaunch", i + 1));
+		sb_weapons[2 + i][6] =
+			r_funcs->Draw_PicFromWad (va ("inva%i_lightng", i + 1));
 	}
 
-	sb_ammo[0] = Draw_PicFromWad ("sb_shells");
-	sb_ammo[1] = Draw_PicFromWad ("sb_nails");
-	sb_ammo[2] = Draw_PicFromWad ("sb_rocket");
-	sb_ammo[3] = Draw_PicFromWad ("sb_cells");
+	sb_ammo[0] = r_funcs->Draw_PicFromWad ("sb_shells");
+	sb_ammo[1] = r_funcs->Draw_PicFromWad ("sb_nails");
+	sb_ammo[2] = r_funcs->Draw_PicFromWad ("sb_rocket");
+	sb_ammo[3] = r_funcs->Draw_PicFromWad ("sb_cells");
 
-	sb_armor[0] = Draw_PicFromWad ("sb_armor1");
-	sb_armor[1] = Draw_PicFromWad ("sb_armor2");
-	sb_armor[2] = Draw_PicFromWad ("sb_armor3");
+	sb_armor[0] = r_funcs->Draw_PicFromWad ("sb_armor1");
+	sb_armor[1] = r_funcs->Draw_PicFromWad ("sb_armor2");
+	sb_armor[2] = r_funcs->Draw_PicFromWad ("sb_armor3");
 
-	sb_items[0] = Draw_PicFromWad ("sb_key1");
-	sb_items[1] = Draw_PicFromWad ("sb_key2");
-	sb_items[2] = Draw_PicFromWad ("sb_invis");
-	sb_items[3] = Draw_PicFromWad ("sb_invuln");
-	sb_items[4] = Draw_PicFromWad ("sb_suit");
-	sb_items[5] = Draw_PicFromWad ("sb_quad");
+	sb_items[0] = r_funcs->Draw_PicFromWad ("sb_key1");
+	sb_items[1] = r_funcs->Draw_PicFromWad ("sb_key2");
+	sb_items[2] = r_funcs->Draw_PicFromWad ("sb_invis");
+	sb_items[3] = r_funcs->Draw_PicFromWad ("sb_invuln");
+	sb_items[4] = r_funcs->Draw_PicFromWad ("sb_suit");
+	sb_items[5] = r_funcs->Draw_PicFromWad ("sb_quad");
 
-	sb_sigil[0] = Draw_PicFromWad ("sb_sigil1");
-	sb_sigil[1] = Draw_PicFromWad ("sb_sigil2");
-	sb_sigil[2] = Draw_PicFromWad ("sb_sigil3");
-	sb_sigil[3] = Draw_PicFromWad ("sb_sigil4");
+	sb_sigil[0] = r_funcs->Draw_PicFromWad ("sb_sigil1");
+	sb_sigil[1] = r_funcs->Draw_PicFromWad ("sb_sigil2");
+	sb_sigil[2] = r_funcs->Draw_PicFromWad ("sb_sigil3");
+	sb_sigil[3] = r_funcs->Draw_PicFromWad ("sb_sigil4");
 
-	sb_faces[4][0] = Draw_PicFromWad ("face1");
-	sb_faces[4][1] = Draw_PicFromWad ("face_p1");
-	sb_faces[3][0] = Draw_PicFromWad ("face2");
-	sb_faces[3][1] = Draw_PicFromWad ("face_p2");
-	sb_faces[2][0] = Draw_PicFromWad ("face3");
-	sb_faces[2][1] = Draw_PicFromWad ("face_p3");
-	sb_faces[1][0] = Draw_PicFromWad ("face4");
-	sb_faces[1][1] = Draw_PicFromWad ("face_p4");
-	sb_faces[0][0] = Draw_PicFromWad ("face5");
-	sb_faces[0][1] = Draw_PicFromWad ("face_p5");
+	sb_faces[4][0] = r_funcs->Draw_PicFromWad ("face1");
+	sb_faces[4][1] = r_funcs->Draw_PicFromWad ("face_p1");
+	sb_faces[3][0] = r_funcs->Draw_PicFromWad ("face2");
+	sb_faces[3][1] = r_funcs->Draw_PicFromWad ("face_p2");
+	sb_faces[2][0] = r_funcs->Draw_PicFromWad ("face3");
+	sb_faces[2][1] = r_funcs->Draw_PicFromWad ("face_p3");
+	sb_faces[1][0] = r_funcs->Draw_PicFromWad ("face4");
+	sb_faces[1][1] = r_funcs->Draw_PicFromWad ("face_p4");
+	sb_faces[0][0] = r_funcs->Draw_PicFromWad ("face5");
+	sb_faces[0][1] = r_funcs->Draw_PicFromWad ("face_p5");
 
-	sb_face_invis = Draw_PicFromWad ("face_invis");
-	sb_face_invuln = Draw_PicFromWad ("face_invul2");
-	sb_face_invis_invuln = Draw_PicFromWad ("face_inv2");
-	sb_face_quad = Draw_PicFromWad ("face_quad");
+	sb_face_invis = r_funcs->Draw_PicFromWad ("face_invis");
+	sb_face_invuln = r_funcs->Draw_PicFromWad ("face_invul2");
+	sb_face_invis_invuln = r_funcs->Draw_PicFromWad ("face_inv2");
+	sb_face_quad = r_funcs->Draw_PicFromWad ("face_quad");
 
-	Cmd_AddCommand ("+showscores", Sbar_ShowScores, "No Description");
-	Cmd_AddCommand ("-showscores", Sbar_DontShowScores, "No Description");
+	Cmd_AddCommand ("+showscores", Sbar_ShowScores,
+					"Display information on everyone playing");
+	Cmd_AddCommand ("-showscores", Sbar_DontShowScores,
+					"Stop displaying information on everyone playing");
 
-	sb_sbar = Draw_PicFromWad ("sbar");
-	sb_ibar = Draw_PicFromWad ("ibar");
-	sb_scorebar = Draw_PicFromWad ("scorebar");
+	sb_sbar = r_funcs->Draw_PicFromWad ("sbar");
+	sb_ibar = r_funcs->Draw_PicFromWad ("ibar");
+	sb_scorebar = r_funcs->Draw_PicFromWad ("scorebar");
 
 	// MED 01/04/97 added new hipnotic weapons
-	if (hipnotic) {
-		hsb_weapons[0][0] = Draw_PicFromWad ("inv_laser");
-		hsb_weapons[0][1] = Draw_PicFromWad ("inv_mjolnir");
-		hsb_weapons[0][2] = Draw_PicFromWad ("inv_gren_prox");
-		hsb_weapons[0][3] = Draw_PicFromWad ("inv_prox_gren");
-		hsb_weapons[0][4] = Draw_PicFromWad ("inv_prox");
+	if (!strcmp (qfs_gamedir->hudtype, "hipnotic")) {
+		hsb_weapons[0][0] = r_funcs->Draw_PicFromWad ("inv_laser");
+		hsb_weapons[0][1] = r_funcs->Draw_PicFromWad ("inv_mjolnir");
+		hsb_weapons[0][2] = r_funcs->Draw_PicFromWad ("inv_gren_prox");
+		hsb_weapons[0][3] = r_funcs->Draw_PicFromWad ("inv_prox_gren");
+		hsb_weapons[0][4] = r_funcs->Draw_PicFromWad ("inv_prox");
 
-		hsb_weapons[1][0] = Draw_PicFromWad ("inv2_laser");
-		hsb_weapons[1][1] = Draw_PicFromWad ("inv2_mjolnir");
-		hsb_weapons[1][2] = Draw_PicFromWad ("inv2_gren_prox");
-		hsb_weapons[1][3] = Draw_PicFromWad ("inv2_prox_gren");
-		hsb_weapons[1][4] = Draw_PicFromWad ("inv2_prox");
+		hsb_weapons[1][0] = r_funcs->Draw_PicFromWad ("inv2_laser");
+		hsb_weapons[1][1] = r_funcs->Draw_PicFromWad ("inv2_mjolnir");
+		hsb_weapons[1][2] = r_funcs->Draw_PicFromWad ("inv2_gren_prox");
+		hsb_weapons[1][3] = r_funcs->Draw_PicFromWad ("inv2_prox_gren");
+		hsb_weapons[1][4] = r_funcs->Draw_PicFromWad ("inv2_prox");
 
 		for (i = 0; i < 5; i++) {
 			hsb_weapons[2 + i][0] =
-				Draw_PicFromWad (va ("inva%i_laser", i + 1));
+				r_funcs->Draw_PicFromWad (va ("inva%i_laser", i + 1));
 			hsb_weapons[2 + i][1] =
-				Draw_PicFromWad (va ("inva%i_mjolnir", i + 1));
+				r_funcs->Draw_PicFromWad (va ("inva%i_mjolnir", i + 1));
 			hsb_weapons[2 + i][2] =
-				Draw_PicFromWad (va ("inva%i_gren_prox", i + 1));
+				r_funcs->Draw_PicFromWad (va ("inva%i_gren_prox", i + 1));
 			hsb_weapons[2 + i][3] =
-				Draw_PicFromWad (va ("inva%i_prox_gren", i + 1));
-			hsb_weapons[2 + i][4] = Draw_PicFromWad (va ("inva%i_prox", i + 1));
+				r_funcs->Draw_PicFromWad (va ("inva%i_prox_gren", i + 1));
+			hsb_weapons[2 + i][4] =
+				r_funcs->Draw_PicFromWad (va ("inva%i_prox", i + 1));
 		}
 
-		hsb_items[0] = Draw_PicFromWad ("sb_wsuit");
-		hsb_items[1] = Draw_PicFromWad ("sb_eshld");
+		hsb_items[0] = r_funcs->Draw_PicFromWad ("sb_wsuit");
+		hsb_items[1] = r_funcs->Draw_PicFromWad ("sb_eshld");
 	}
 
 	// FIXME: MISSIONHUD
-	if (rogue) {
-		rsb_invbar[0] = Draw_PicFromWad ("r_invbar1");
-		rsb_invbar[1] = Draw_PicFromWad ("r_invbar2");
+	if (!strcmp (qfs_gamedir->hudtype, "rogue")) {
+		rsb_invbar[0] = r_funcs->Draw_PicFromWad ("r_invbar1");
+		rsb_invbar[1] = r_funcs->Draw_PicFromWad ("r_invbar2");
 
-		rsb_weapons[0] = Draw_PicFromWad ("r_lava");
-		rsb_weapons[1] = Draw_PicFromWad ("r_superlava");
-		rsb_weapons[2] = Draw_PicFromWad ("r_gren");
-		rsb_weapons[3] = Draw_PicFromWad ("r_multirock");
-		rsb_weapons[4] = Draw_PicFromWad ("r_plasma");
+		rsb_weapons[0] = r_funcs->Draw_PicFromWad ("r_lava");
+		rsb_weapons[1] = r_funcs->Draw_PicFromWad ("r_superlava");
+		rsb_weapons[2] = r_funcs->Draw_PicFromWad ("r_gren");
+		rsb_weapons[3] = r_funcs->Draw_PicFromWad ("r_multirock");
+		rsb_weapons[4] = r_funcs->Draw_PicFromWad ("r_plasma");
 
-		rsb_items[0] = Draw_PicFromWad ("r_shield1");
-		rsb_items[1] = Draw_PicFromWad ("r_agrav1");
+		rsb_items[0] = r_funcs->Draw_PicFromWad ("r_shield1");
+		rsb_items[1] = r_funcs->Draw_PicFromWad ("r_agrav1");
 
 		// PGM 01/19/97 - team color border
-		rsb_teambord = Draw_PicFromWad ("r_teambord");
+		rsb_teambord = r_funcs->Draw_PicFromWad ("r_teambord");
 		// PGM 01/19/97 - team color border
 
-		rsb_ammo[0] = Draw_PicFromWad ("r_ammolava");
-		rsb_ammo[1] = Draw_PicFromWad ("r_ammomulti");
-		rsb_ammo[2] = Draw_PicFromWad ("r_ammoplasma");
+		rsb_ammo[0] = r_funcs->Draw_PicFromWad ("r_ammolava");
+		rsb_ammo[1] = r_funcs->Draw_PicFromWad ("r_ammomulti");
+		rsb_ammo[2] = r_funcs->Draw_PicFromWad ("r_ammoplasma");
 	}
 
-	r_viewsize_callback = viewsize_f;
+	r_data->viewsize_callback = viewsize_f;
 	hud_sbar = Cvar_Get ("hud_sbar", "0", CVAR_ARCHIVE, hud_sbar_f,
-						 "status bar mode");
+						 "status bar mode: 0 = hud, 1 = oldstyle");
 	hud_swap = Cvar_Get ("hud_swap", "0", CVAR_ARCHIVE, hud_swap_f,
 						 "new HUD on left side?");
 	hud_scoreboard_gravity = Cvar_Get ("hud_scoreboard_gravity", "center",
@@ -1794,6 +1801,11 @@ Sbar_Init (void)
 									   "overlay: center, northwest, north, "
 									   "northeast, west, east, southwest, "
 									   "south, southeast");
+	scr_centertime = Cvar_Get ("scr_centertime", "2", CVAR_NONE, NULL, "How "
+							   "long in seconds screen hints are displayed");
+	scr_printspeed = Cvar_Get ("scr_printspeed", "8", CVAR_NONE, NULL,
+							   "How fast the text is displayed at the end of "
+							   "the single player episodes");
 
 	// register GIB builtins
 	GIB_Builtin_Add ("print::center", Sbar_GIB_Print_Center_f);
