@@ -28,20 +28,13 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sv_phys.c,v 1.30 2003/09/03 22:16:57 taniwha Exp $";
-
-#include "QF/console.h"
 #include "QF/cvar.h"
 #include "QF/sys.h"
 
-#include "host.h"
 #include "server.h"
 #include "sv_progs.h"
 #include "world.h"
 
-#define sv_frametime host_frametime
-
 /*
 	pushmove objects do not obey gravity, and do not interact with each
 	other or trigger fields, but block normal movement and push normal
@@ -57,14 +50,14 @@ static __attribute__ ((unused)) const char rcsid[] =
 	walking monsters are SOLID_SLIDEBOX and MOVETYPE_STEP
 	flying/floating monsters are SOLID_SLIDEBOX and MOVETYPE_FLY
 
-	solid_edge items only clip against bsp models.
+	solid_edge items clip against only bsp models.
 */
 
 cvar_t     *sv_friction;
 cvar_t     *sv_gravity;
-cvar_t     *sv_stopspeed;
+cvar_t     *sv_jump_any;
 cvar_t     *sv_maxvelocity;
-cvar_t     *sv_nostep;
+cvar_t     *sv_stopspeed;
 
 #define	MOVE_EPSILON	0.01
 #if 0
@@ -86,55 +79,47 @@ SV_CheckAllEnts (void)
 			continue;
 
 		if (SV_TestEntityPosition (check))
-			Con_Printf ("entity in invalid position\n");
+			Sys_Printf ("entity in invalid position\n");
 	}
 }
 #endif
 void
 SV_CheckVelocity (edict_t *ent)
 {
-#if 0
 	float       wishspeed;
-#endif
-	int         i;
+//	int         i;
 
 	// bound velocity
+#if 0
 	for (i = 0; i < 3; i++) {
 		if (IS_NAN (SVvector (ent, velocity)[i])) {
-			Con_Printf ("Got a NaN velocity on %s\n",
+			Sys_Printf ("Got a NaN velocity on %s\n",
 						PR_GetString (&sv_pr_state, SVstring (ent,
 															  classname)));
 			SVvector (ent, velocity)[i] = 0;
 		}
 		if (IS_NAN (SVvector (ent, origin)[i])) {
-			Con_Printf ("Got a NaN origin on %s\n",
+			Sys_Printf ("Got a NaN origin on %s\n",
 						PR_GetString (&sv_pr_state, SVstring (ent,
 															  classname)));
 			SVvector (ent, origin)[i] = 0;
 		}
-#if 1
-		if (SVvector (ent, velocity)[i] > sv_maxvelocity->value)
-			SVvector (ent, velocity)[i] = sv_maxvelocity->value;
-		else if (SVvector (ent, velocity)[i] < -sv_maxvelocity->value)
-			SVvector (ent, velocity)[i] = -sv_maxvelocity->value;
-#endif
 	}
-#if 0
+#endif
 	wishspeed = VectorLength (SVvector (ent, velocity));
 	if (wishspeed > sv_maxvelocity->value) {
 		VectorScale (SVvector (ent, velocity), sv_maxvelocity->value /
 					 wishspeed, SVvector (ent, velocity));
 	}
-#endif
 }
 
 /*
-  SV_RunThink
+	SV_RunThink
 
-  Runs thinking code if time.  There is some play in the exact time the think
-  function will be called, because it is called before any movement is done
-  in a frame.  Not used for pushmove objects, because they must be exact.
-  Returns false if the entity removed itself.
+	Runs thinking code if time.  There is some play in the exact time the think
+	function will be called, because it is called before any movement is done
+	in a frame.  Not used for pushmove objects, because they must be exact.
+	Returns false if the entity removed itself.
 */
 qboolean
 SV_RunThink (edict_t *ent)
@@ -156,7 +141,7 @@ SV_RunThink (edict_t *ent)
 
 		if (ent->free)
 			return false;
-	} while (0);
+	} while (SVfloat (ent, nextthink) > thinktime);
 
 	return true;
 }
@@ -188,10 +173,10 @@ SV_Impact (edict_t *e1, edict_t *e2)
 }
 
 /*
-  ClipVelocity
+	ClipVelocity
 
-  Slide off of the impacting object
-  returns the blocked flags (1 = floor, 2 = step / wall)
+	Slide off of the impacting object
+	returns the blocked flags (1 = floor, 2 = step / wall)
 */
 static int
 ClipVelocity (vec3_t in, vec3_t normal, vec3_t out, float overbounce)
@@ -217,6 +202,19 @@ ClipVelocity (vec3_t in, vec3_t normal, vec3_t out, float overbounce)
 	return blocked;
 }
 
+int
+SV_EntCanSupportJump (edict_t *ent)
+{
+	int         solid = SVfloat (ent, solid);
+	if (solid == SOLID_BSP)
+		return 1;
+	if (!sv_jump_any->int_val)
+		return 0;
+	if (solid == SOLID_NOT || solid == SOLID_SLIDEBOX)
+		return 0;
+	return 1;
+}
+
 #define	MAX_CLIP_PLANES	5
 
 /*
@@ -254,6 +252,10 @@ SV_FlyMove (edict_t *ent, float time, trace_t *steptrace)
 
 		VectorMultAdd (SVvector (ent, origin), time_left,
 					   SVvector (ent, velocity), end);
+		if (SVdata (ent)->add_grav) {
+			SVdata (ent)->add_grav = false;
+			SV_FinishGravity (ent, end);
+		}
 
 		trace = SV_Move (SVvector (ent, origin), SVvector (ent, mins),
 						 SVvector (ent, maxs), end, false, ent);
@@ -277,7 +279,7 @@ SV_FlyMove (edict_t *ent, float time, trace_t *steptrace)
 
 		if (trace.plane.normal[2] > 0.7) {
 			blocked |= 1;				// floor
-			if (SVfloat (trace.ent, solid) == SOLID_BSP) {
+			if (SV_EntCanSupportJump (trace.ent)) {
 				SVfloat (ent, flags) = (int) SVfloat (ent, flags) |
 					FL_ONGROUND;
 				SVentity (ent, groundentity) = EDICT_TO_PROG (&sv_pr_state,
@@ -344,13 +346,27 @@ SV_FlyMove (edict_t *ent, float time, trace_t *steptrace)
 void
 SV_AddGravity (edict_t *ent)
 {
-	float       ent_gravity;
+	float       ent_grav;
 
 	if (sv_fields.gravity != -1 && SVfloat (ent, gravity))
-		ent_gravity = SVfloat (ent, gravity);
+		ent_grav = SVfloat (ent, gravity);
 	else
-		ent_gravity = 1.0;
-	SVvector (ent, velocity)[2] -= ent_gravity * sv_gravity->value * sv_frametime;
+		ent_grav = 1.0;
+	SVvector (ent, velocity)[2] -= ent_grav * sv_gravity->value * sv_frametime;
+	SVdata (ent)->add_grav = true;
+}
+
+void
+SV_FinishGravity (edict_t *ent, vec3_t move)
+{
+	float       ent_grav;
+
+	if (sv_fields.gravity != -1 && SVfloat (ent, gravity))
+		ent_grav = SVfloat (ent, gravity);
+	else
+		ent_grav = 1.0;
+	ent_grav *= sv_gravity->value;
+	move[2] += ent_grav * sv_frametime * sv_frametime / 2;
 }
 
 /* PUSHMOVE */
@@ -365,22 +381,26 @@ SV_PushEntity (edict_t *ent, vec3_t push)
 {
 	trace_t     trace;
 	vec3_t      end;
+	vec_t      *e_origin, *e_mins, *e_maxs;
+	int         e_movetype, e_solid;
 
-	VectorAdd (SVvector (ent, origin), push, end);
+	e_origin = SVvector (ent, origin);
+	e_mins = SVvector (ent, mins);
+	e_maxs = SVvector (ent, maxs);
+	e_movetype = SVfloat (ent, movetype);
+	e_solid = SVfloat (ent, solid);
 
-	if (SVfloat (ent, movetype) == MOVETYPE_FLYMISSILE)
-		trace = SV_Move (SVvector (ent, origin), SVvector (ent, mins),
-						 SVvector (ent, maxs), end, MOVE_MISSILE, ent);
-	else if (SVfloat (ent, solid) == SOLID_TRIGGER
-			 || SVfloat (ent, solid) == SOLID_NOT)
-		// only clip against bmodels
-		trace = SV_Move (SVvector (ent, origin), SVvector (ent, mins),
-						 SVvector (ent, maxs), end, MOVE_NOMONSTERS, ent);
+	VectorAdd (e_origin, push, end);
+
+	if (e_movetype == MOVETYPE_FLYMISSILE)
+		trace = SV_Move (e_origin, e_mins, e_maxs, end, MOVE_MISSILE, ent);
+	else if (e_solid == SOLID_TRIGGER || e_solid == SOLID_NOT)
+		// clip against only bmodels
+		trace = SV_Move (e_origin, e_mins, e_maxs, end, MOVE_NOMONSTERS, ent);
 	else
-		trace = SV_Move (SVvector (ent, origin), SVvector (ent, mins),
-						 SVvector (ent, maxs), end, MOVE_NORMAL, ent);
+		trace = SV_Move (e_origin, e_mins, e_maxs, end, MOVE_NORMAL, ent);
 
-	VectorCopy (trace.endpos, SVvector (ent, origin));
+	VectorCopy (trace.endpos, e_origin);
 	SV_LinkEdict (ent, true);
 
 	if (trace.ent)
@@ -390,25 +410,47 @@ SV_PushEntity (edict_t *ent, vec3_t push)
 }
 
 static qboolean
-SV_Push (edict_t *pusher, vec3_t move)
+SV_Push (edict_t *pusher, const vec3_t tmove, const vec3_t amove)
 {
 	float       solid_save;
 	int         num_moved, i, e;
 	edict_t    *check, *block;
-	edict_t    *moved_edict[MAX_EDICTS];
-	vec3_t      entorig;
-	vec3_t      mins, maxs, pushorig;
-	vec3_t      moved_from[MAX_EDICTS];
+	edict_t   **moved_edict;
+	vec3_t      move, org, org2;
+	vec3_t      mins, maxs, pushtorig, pushaorig;
+	vec3_t     *moved_from;
+	vec3_t      forward = {1, 0, 0};
+	vec3_t      left    = {0, 1, 0};
+	vec3_t      up      = {0, 0, 1};
+	int         mark;
+	int         c_flags, c_movetype, c_groundentity, c_solid;
+	vec_t      *c_absmin, *c_absmax, *c_origin, *c_angles, *c_mins, *c_maxs;
+	vec_t      *p_origin, *p_angles;
 
-	VectorAdd (SVvector (pusher, absmin), move, mins);
-	VectorAdd (SVvector (pusher, absmax), move, maxs);
+	VectorAdd (SVvector (pusher, absmin), tmove, mins);
+	VectorAdd (SVvector (pusher, absmax), tmove, maxs);
 
-	VectorCopy (SVvector (pusher, origin), pushorig);
+	if (!VectorIsZero (amove)) {
+		vec3_t      a;
+		VectorSubtract (vec3_origin, amove, a);
+		AngleVectors (a, forward, left, up);
+		VectorNegate (left, left);	// AngleVectors is right-handed
+	}
+
+	p_origin = SVvector (pusher, origin);
+	p_angles = SVvector (pusher, angles);
+	VectorCopy (p_origin, pushtorig);
+	VectorCopy (p_angles, pushaorig);
 
 	// move the pusher to it's final position
-	VectorAdd (SVvector (pusher, origin), move, SVvector (pusher, origin));
+	VectorAdd (p_origin, tmove, p_origin);
+	VectorAdd (p_angles, amove, p_angles);
 	SV_LinkEdict (pusher, false);
 
+	mark = Hunk_LowMark ();
+	moved_edict = Hunk_Alloc (sv.num_edicts * sizeof (edict_t *));
+	moved_from = Hunk_Alloc (sv.num_edicts * sizeof (vec_t));
+
 	// see if any solid entities are inside the final position
 	num_moved = 0;
 	check = NEXT_EDICT (&sv_pr_state, sv.edicts);
@@ -416,38 +458,56 @@ SV_Push (edict_t *pusher, vec3_t move)
 		 e++, check = NEXT_EDICT (&sv_pr_state, check)) {
 		if (check->free)
 			continue;
-		if (SVfloat (check, movetype) == MOVETYPE_PUSH
-			|| SVfloat (check, movetype) == MOVETYPE_NONE
-			|| SVfloat (check, movetype) == MOVETYPE_NOCLIP)
+		c_movetype = SVfloat (check, movetype);
+		if (c_movetype == MOVETYPE_PUSH || c_movetype == MOVETYPE_NONE
+			|| c_movetype == MOVETYPE_NOCLIP)
+			continue;
+
+		// If the entity is in another solid, it's not free to move. Make the
+		// pusher non-solid to ensure it doesn't interfere with the check.
+		solid_save = SVfloat (pusher, solid);
+		SVfloat (pusher, solid) = SOLID_NOT;
+		block = SV_TestEntityPosition (check);
+		SVfloat (pusher, solid) = solid_save;
+		if (block)
 			continue;
 
 		// if the entity is standing on the pusher, it will definately be moved
-		if (!(((int) SVfloat (check, flags) & FL_ONGROUND)
-			  && PROG_TO_EDICT (&sv_pr_state,
-								SVentity (check, groundentity)) == pusher)) {
-			// check is NOT standing on pusher
-			if (SVvector (check, absmin)[0] >= maxs[0]
-				|| SVvector (check, absmin)[1] >= maxs[1]
-				|| SVvector (check, absmin)[2] >= maxs[2]
-				|| SVvector (check, absmax)[0] <= mins[0]
-				|| SVvector (check, absmax)[1] <= mins[1]
-				|| SVvector (check, absmax)[2] <= mins[2])
+		c_flags = SVfloat (check, flags);
+		c_groundentity = SVentity (check, groundentity);
+		if (!(c_flags & FL_ONGROUND)
+			  || PROG_TO_EDICT (&sv_pr_state, c_groundentity) != pusher) {
+			// The entity is NOT standing on pusher, so check whether the
+			// entity is inside the pusher's final position.
+			// FIXME what if the pusher is moving so fast it skips past the
+			// entity?
+			c_absmin = SVvector (check, absmin);
+			c_absmax = SVvector (check, absmax);
+			if (VectorCompCompare (c_absmin, >=, maxs)
+				|| VectorCompCompare (c_absmax, <=, mins))
 				continue;
 
-			// see if the ent's bbox is inside the pusher's final position
 			if (!SV_TestEntityPosition (check))
 				continue;
+			// The pusher and entity collide, so push the entity.
 		}
 		// remove the onground flag for non-players
-		if (SVfloat (check, movetype) != MOVETYPE_WALK)
-			SVfloat (check, flags) = (int) SVfloat (check, flags) &
-				~FL_ONGROUND;
+		if (c_movetype != MOVETYPE_WALK)
+			SVfloat (check, flags) = c_flags & ~FL_ONGROUND;
 
-		VectorCopy (SVvector (check, origin), entorig);
-		VectorCopy (SVvector (check, origin), moved_from[num_moved]);
+		c_origin = SVvector (check, origin);
+		VectorCopy (c_origin, moved_from[num_moved]);
 		moved_edict[num_moved] = check;
 		num_moved++;
 
+		// calculate destination position
+		VectorSubtract (c_origin, p_origin, org);
+		org2[0] = DotProduct (org, forward);
+		org2[1] = DotProduct (org, left);
+		org2[2] = DotProduct (org, up);
+		VectorSubtract (org2, org, move);
+		VectorAdd (move, tmove, move);
+
 		// try moving the contacted entity
 		solid_save = SVfloat (pusher, solid);
 		SVfloat (pusher, solid) = SOLID_NOT;
@@ -456,37 +516,43 @@ SV_Push (edict_t *pusher, vec3_t move)
 
 		block = SV_TestEntityPosition (check);
 		if (!block) {
+			c_angles = SVvector (check, angles);
+			VectorAdd (c_angles, amove, c_angles);
 			continue;
 		}
 		// if it is still inside the pusher, block
-		if (SVvector (check, mins)[0] == SVvector (check, maxs)[0]) {
-			continue;
-		}
-		if (SVfloat (check, solid) == SOLID_NOT
-			|| SVfloat (check, solid) == SOLID_TRIGGER) {	// corpse
-			SVvector (check, mins)[0] = SVvector (check, mins)[1] = 0;
-			VectorCopy (SVvector (check, mins), SVvector (check, maxs));
+		c_mins = SVvector (check, mins);
+		c_maxs = SVvector (check, maxs);
+		c_solid = SVfloat (check, solid);
+		if (c_mins[0] == c_maxs[0]
+			|| c_solid == SOLID_NOT || c_solid == SOLID_TRIGGER) {	// corpse
+			c_mins[0] = c_mins[1] = 0;
+			VectorCopy (c_mins, c_maxs);
+			SV_LinkEdict (check, false);
 			continue;
 		}
 
-		VectorCopy (entorig, SVvector (check, origin));
-		SV_LinkEdict (check, true);
-
-		VectorCopy (pushorig, SVvector (pusher, origin));
+		VectorCopy (pushtorig, p_origin);
+		VectorCopy (pushaorig, p_angles);
 		SV_LinkEdict (pusher, false);
 
 		// if the pusher has a "blocked" function, call it
 		// otherwise, just stay in place until the obstacle is gone
-		if (SVfunc (pusher, blocked)) {
+		if (SVfunc (pusher, blocked))
 			sv_pr_blocked (pusher, check);
-		}
+
 		// move back any entities we already moved
 		for (i = 0; i < num_moved; i++) {
-			VectorCopy (moved_from[i], SVvector (moved_edict[i], origin));
+			vec_t      *m_origin = SVvector (moved_edict[i], origin);
+			vec_t      *m_angles = SVvector (moved_edict[i], angles);
+			VectorCopy (moved_from[i], m_origin);
+			VectorSubtract (m_angles, amove, m_angles);
 			SV_LinkEdict (moved_edict[i], false);
 		}
+		Hunk_FreeToLowMark (mark);
 		return false;
 	}
+	Hunk_FreeToLowMark (mark);
 	return true;
 }
 
@@ -494,15 +560,19 @@ static void
 SV_PushMove (edict_t *pusher, float movetime)
 {
 	vec3_t      move;
+	vec3_t      amove;
 
-	if (VectorIsZero (SVvector (pusher, velocity))) {
+	if (VectorIsZero (SVvector (pusher, velocity))
+		&& VectorIsZero (SVvector (pusher, avelocity))) {
 		SVfloat (pusher, ltime) += movetime;
 		return;
 	}
 
 	VectorScale (SVvector (pusher, velocity), movetime, move);
+	//FIXME finish gravity
+	VectorScale (SVvector (pusher, avelocity), movetime, amove);
 
-	if (SV_Push (pusher, move))
+	if (SV_Push (pusher, move, amove))
 		SVfloat (pusher, ltime) += movetime;
 }
 
@@ -510,6 +580,8 @@ static void
 SV_Physics_Pusher (edict_t *ent)
 {
 	float       movetime, oldltime, thinktime;
+	float       l;
+	vec3_t      oldorg, move;
 
 	oldltime = SVfloat (ent, ltime);
 
@@ -527,13 +599,20 @@ SV_Physics_Pusher (edict_t *ent)
 	}
 
 	if (thinktime > oldltime && thinktime <= SVfloat (ent, ltime)) {
+		VectorCopy (SVvector (ent, origin), oldorg);
 		SVfloat (ent, nextthink) = 0;
 		*sv_globals.time = sv.time;
 		sv_pr_think (ent);
 		if (ent->free)
 			return;
-	}
+		VectorSubtract (SVvector (ent, origin), oldorg, move);
 
+		l = VectorLength (move);
+		if (l > (1.0 / 64.0)) {
+			VectorCopy (oldorg, SVvector (ent, origin));
+			SV_Push (ent, move, vec3_origin);	//FIXME angle
+		}
+	}
 }
 
 /*
@@ -617,6 +696,9 @@ SV_Physics_Toss (edict_t *ent)
 	if (!SV_RunThink (ent))
 		return;
 
+	if (SVvector (ent, velocity)[2] > 0)
+		SVfloat (ent, flags) = (int) SVfloat (ent, flags) & ~FL_ONGROUND;
+
 	// if onground, return without moving
 	if (((int) SVfloat (ent, flags) & FL_ONGROUND))
 		return;
@@ -634,6 +716,10 @@ SV_Physics_Toss (edict_t *ent)
 
 	// move origin
 	VectorScale (SVvector (ent, velocity), sv_frametime, move);
+	if (SVdata (ent)->add_grav) {
+		SVdata (ent)->add_grav = false;
+		SV_FinishGravity (ent, move);
+	}
 	trace = SV_PushEntity (ent, move);
 	if (trace.fraction == 1)
 		return;
@@ -673,6 +759,7 @@ SV_Physics_Toss (edict_t *ent)
 
 	This is also used for objects that have become still on the ground, but
 	will fall if the floor is pulled out from under them.
+	FIXME: is this true?
 */
 static void
 SV_Physics_Step (edict_t *ent)
@@ -702,7 +789,7 @@ SV_Physics_Step (edict_t *ent)
 	SV_CheckWaterTransition (ent);
 }
 
-static void
+void
 SV_ProgStartFrame (void)
 {
 	// let the progs know that a new frame has started
@@ -716,10 +803,11 @@ static void
 SV_RunEntity (edict_t *ent)
 {
 	if (sv_fields.lastruntime != -1) {
-		if (SVfloat (ent, lastruntime) == (float) realtime)
+		if (SVfloat (ent, lastruntime) == (float) sv.time)
 			return;
-		SVfloat (ent, lastruntime) = (float) realtime;
+		SVfloat (ent, lastruntime) = (float) sv.time;
 	}
+	SVdata (ent)->add_grav = false;
 
 	switch ((int) SVfloat (ent, movetype)) {
 		case MOVETYPE_PUSH:
@@ -746,7 +834,7 @@ SV_RunEntity (edict_t *ent)
 	}
 }
 
-static void
+void
 SV_RunNewmis (void)
 {
 	edict_t    *ent;
@@ -781,7 +869,8 @@ SV_Physics (void)
 		}
 
 		if (i > 0 && i <= svs.maxclients) {
-			SV_Physics_Client (ent, i);
+			if (svs.phys_client)
+				svs.phys_client (ent, i);
 			continue;
 		}
 
@@ -792,11 +881,11 @@ SV_Physics (void)
 	if (*sv_globals.force_retouch)
 		(*sv_globals.force_retouch)--;
 
-	if (EndFrame) {
+	if (sv_funcs.EndFrame) {
 		// let the progs know that the frame has ended
 		*sv_globals.self = EDICT_TO_PROG (&sv_pr_state, sv.edicts);
 		*sv_globals.other = EDICT_TO_PROG (&sv_pr_state, sv.edicts);
 		*sv_globals.time = sv.time;
-		PR_ExecuteProgram (&sv_pr_state, EndFrame);
+		PR_ExecuteProgram (&sv_pr_state, sv_funcs.EndFrame);
 	}
 }
