@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: world.c,v 1.26 2004/02/24 20:06:52 despair Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -50,6 +47,73 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "sv_progs.h"
 #include "world.h"
 
+#define always_inline inline __attribute__((__always_inline__))
+
+#define EDICT_LEAFS 32
+typedef struct edict_leaf_bucket_s {
+	struct edict_leaf_bucket_s *next;
+	edict_leaf_t edict_leafs[EDICT_LEAFS];
+} edict_leaf_bucket_t;
+
+static edict_leaf_bucket_t *edict_leaf_buckets;
+static edict_leaf_bucket_t **edict_leaf_bucket_tail = &edict_leaf_buckets;
+static edict_leaf_t *free_edict_leaf_list;
+
+static edict_leaf_t *
+alloc_edict_leaf (void)
+{
+	edict_leaf_bucket_t *bucket;
+	edict_leaf_t *el;
+	int         i;
+
+	if ((el = free_edict_leaf_list)) {
+		free_edict_leaf_list = el->next;
+		el->next = 0;
+		return el;
+	}
+
+	bucket = malloc (sizeof (edict_leaf_bucket_t));
+	bucket->next = 0;
+	*edict_leaf_bucket_tail = bucket;
+	edict_leaf_bucket_tail = &bucket->next;
+
+	for (el = bucket->edict_leafs, i = 0; i < EDICT_LEAFS - 1; i++, el++)
+		el->next = el + 1;
+	el->next = 0;
+	free_edict_leaf_list = bucket->edict_leafs;
+
+	return alloc_edict_leaf ();
+}
+
+static void
+free_edict_leafs (edict_leaf_t **edict_leafs)
+{
+	edict_leaf_t **el;
+
+	for (el = edict_leafs; *el; el = &(*el)->next)
+		;
+	*el = free_edict_leaf_list;
+	free_edict_leaf_list = *edict_leafs;
+	*edict_leafs = 0;
+}
+
+void
+SV_FreeAllEdictLeafs (void)
+{
+	edict_leaf_bucket_t *bucket;
+	edict_leaf_t *el;
+	int         i;
+
+	for (bucket = edict_leaf_buckets; bucket; bucket = bucket->next) {
+		for (el = bucket->edict_leafs, i = 0; i < EDICT_LEAFS - 1; i++, el++)
+			el->next = el + 1;
+		el->next = bucket->next ? bucket->next->edict_leafs : 0;
+	}
+	free_edict_leaf_list = 0;
+	if (edict_leaf_buckets)
+		free_edict_leaf_list = edict_leaf_buckets->edict_leafs;
+}
+
 /*
 	entities never clip against themselves, or their owner
 	line of sight checks trace->crosscontent, but bullets don't
@@ -70,8 +134,8 @@ typedef struct {
 /* HULL BOXES */
 
 static hull_t box_hull;
-static dclipnode_t box_clipnodes[6];
-static mplane_t box_planes[6];
+static mclipnode_t box_clipnodes[6];
+static plane_t box_planes[6];
 
 
 /*
@@ -81,7 +145,7 @@ static mplane_t box_planes[6];
 	can just be stored out and get a proper hull_t structure.
 */
 void
-SV_InitHull (hull_t *hull, dclipnode_t *clipnodes, mplane_t *planes)
+SV_InitHull (hull_t *hull, mclipnode_t *clipnodes, plane_t *planes)
 {
 	int			side, i;
 
@@ -89,6 +153,7 @@ SV_InitHull (hull_t *hull, dclipnode_t *clipnodes, mplane_t *planes)
 	hull->planes = planes;
 	hull->firstclipnode = 0;
 	hull->lastclipnode = 5;
+	hull->depth = 6;
 
 	for (i = 0; i < 6; i++) {
 		hull->clipnodes[i].planenum = i;
@@ -141,12 +206,12 @@ SV_HullForBox (const vec3_t mins, const vec3_t maxs)
 */
 hull_t *
 SV_HullForEntity (edict_t *ent, const vec3_t mins, const vec3_t maxs,
-				  vec3_t offset)
+				  vec3_t extents, vec3_t offset)
 {
 	int         hull_index = 0;
-	hull_t     *hull = 0;
+	hull_t     *hull = 0, **hull_list = 0;
 	model_t    *model;
-	vec3_t		hullmins, hullmaxs, size;
+	vec3_t      hullmins, hullmaxs, size;
 
 	if ((sv_fields.rotated_bbox != -1
 		 && SVinteger (ent, rotated_bbox))
@@ -159,34 +224,69 @@ SV_HullForEntity (edict_t *ent, const vec3_t mins, const vec3_t maxs,
 		else
 			hull_index = 2;
 	}
-	// decide which clipping hull to use, based on the size
 	if (sv_fields.rotated_bbox != -1
 		&& SVinteger (ent, rotated_bbox)) {
 		int h = SVinteger (ent, rotated_bbox) - 1;
-		hull = pf_hull_list[h]->hulls[hull_index];
+		hull_list = pf_hull_list[h]->hulls;
 	} if (SVfloat (ent, solid) == SOLID_BSP) {
 		// explicit hulls in the BSP model
 		if (SVfloat (ent, movetype) != MOVETYPE_PUSH)
-			Sys_Error ("SOLID_BSP without MOVETYPE_PUSH");
+			Sys_Error ("SOLID_BSP without MOVETYPE_PUSH: %d %s",
+					   NUM_FOR_EDICT (&sv_pr_state, ent),
+					   PR_GetString (&sv_pr_state,
+						   			 SVstring (ent, classname)));
 
 		model = sv.models[(int) SVfloat (ent, modelindex)];
 
 		if (!model || model->type != mod_brush)
-			Sys_Error ("SOLID_BSP with a non bsp model");
+			Sys_Error ("SOLID_BSP with a non bsp model: %d %s",
+					   NUM_FOR_EDICT (&sv_pr_state, ent),
+					   PR_GetString (&sv_pr_state,
+						   			 SVstring (ent, classname)));
 
-		hull = &model->hulls[hull_index];
+		hull_list = model->hull_list;
+	}
+	if (hull_list) {
+		// decide which clipping hull to use, based on the size
+		VectorSubtract (maxs, mins, size);
+		if (extents) {
+			VectorScale (size, 0.5, extents);
+		} else {
+			if (size[0] < 3)
+				hull_index = 0;
+			else if (size[0] <= 32)
+				hull_index = 1;
+			else
+				hull_index = 2;
+		}
+		hull = hull_list[hull_index];
 	}
 
 	if (hull) {
 		// calculate an offset value to center the origin
-		VectorSubtract (hull->clip_mins, mins, offset);
-		VectorAdd (offset, SVvector (ent, origin), offset);
-	} else {					// create a temp hull from bounding box sizes
-		VectorSubtract (SVvector (ent, mins), maxs, hullmins);
-		VectorSubtract (SVvector (ent, maxs), mins, hullmaxs);
-		hull = SV_HullForBox (hullmins, hullmaxs);
+		if (extents) {
+			VectorAdd (extents, mins, offset);
+			VectorSubtract (SVvector (ent, origin), offset, offset);
+		} else {
+			VectorSubtract (hull->clip_mins, mins, offset);
+			VectorAdd (offset, SVvector (ent, origin), offset);
+		}
+	} else {
+		// create a temp hull from bounding box sizes
+		if (extents) {
+			VectorCopy (SVvector (ent, mins), hullmins);
+			VectorCopy (SVvector (ent, maxs), hullmaxs);
 
-		VectorCopy (SVvector (ent, origin), offset);
+			//FIXME broken for map models (ie, origin always 0, 0, 0)
+			VectorAdd (extents, mins, offset);
+			VectorSubtract (SVvector (ent, origin), offset, offset);
+		} else {
+			VectorSubtract (SVvector (ent, mins), maxs, hullmins);
+			VectorSubtract (SVvector (ent, maxs), mins, hullmaxs);
+
+			VectorCopy (SVvector (ent, origin), offset);
+		}
+		hull = SV_HullForBox (hullmins, hullmaxs);
 	}
 
 	return hull;
@@ -251,21 +351,22 @@ link_t        **sv_link_prev;
 void
 SV_UnlinkEdict (edict_t *ent)
 {
-	if (!ent->area.prev)
+	free_edict_leafs (&SVdata (ent)->leafs);
+	if (!SVdata (ent)->area.prev)
 		return;							// not linked in anywhere
-	RemoveLink (&ent->area);
-	if (sv_link_next && *sv_link_next == &ent->area)
-		*sv_link_next = ent->area.next;
-	if (sv_link_prev && *sv_link_prev == &ent->area)
-		*sv_link_prev = ent->area.prev;
-	ent->area.prev = ent->area.next = NULL;
+	RemoveLink (&SVdata (ent)->area);
+	if (sv_link_next && *sv_link_next == &SVdata (ent)->area)
+		*sv_link_next = SVdata (ent)->area.next;
+	if (sv_link_prev && *sv_link_prev == &SVdata (ent)->area)
+		*sv_link_prev = SVdata (ent)->area.prev;
+	SVdata (ent)->area.prev = SVdata (ent)->area.next = NULL;
 }
 
 static void
 SV_TouchLinks (edict_t *ent, areanode_t *node)
 {
-	edict_t    *touch;
 	int         old_self, old_other;
+	edict_t    *touch;
 	link_t     *l, *next;
 
 	// touch linked edicts
@@ -310,23 +411,22 @@ SV_TouchLinks (edict_t *ent, areanode_t *node)
 static void
 SV_FindTouchedLeafs (edict_t *ent, mnode_t *node)
 {
-	int			leafnum, sides;
+	int			sides;
 	mleaf_t    *leaf;
-	mplane_t   *splitplane;
+	plane_t    *splitplane;
+	edict_leaf_t *edict_leaf;
 
 	if (node->contents == CONTENTS_SOLID)
 		return;
 
 	// add an efrag if the node is a leaf
 	if (node->contents < 0) {
-		if (ent->num_leafs == MAX_ENT_LEAFS)
-			return;
-
 		leaf = (mleaf_t *) node;
-		leafnum = leaf - sv.worldmodel->leafs - 1;
 
-		ent->leafnums[ent->num_leafs] = leafnum;
-		ent->num_leafs++;
+		edict_leaf = alloc_edict_leaf ();
+		edict_leaf->leafnum = leaf - sv.worldmodel->leafs - 1;
+		edict_leaf->next = SVdata (ent)->leafs;
+		SVdata (ent)->leafs = edict_leaf;
 		return;
 	}
 
@@ -348,7 +448,7 @@ SV_LinkEdict (edict_t *ent, qboolean touch_triggers)
 {
 	areanode_t *node;
 
-	if (ent->area.prev)
+	if (SVdata (ent)->area.prev)
 		SV_UnlinkEdict (ent);			// unlink from old position
 
 	if (ent == sv.edicts)
@@ -356,11 +456,25 @@ SV_LinkEdict (edict_t *ent, qboolean touch_triggers)
 	if (ent->free)
 		return;
 
-	// set the abs box
-	VectorAdd (SVvector (ent, origin), SVvector (ent, mins),
-			   SVvector (ent, absmin));
-	VectorAdd (SVvector (ent, origin), SVvector (ent, maxs),
-			   SVvector (ent, absmax));
+	if (SVfloat (ent, solid) == SOLID_BSP
+		&& !VectorIsZero (SVvector (ent, angles)) && ent != sv.edicts) {
+		float       m, v;
+		vec3_t      r;
+		m = DotProduct (SVvector (ent, mins), SVvector (ent, mins));
+		v = DotProduct (SVvector (ent, maxs), SVvector (ent, maxs));
+		if (m < v)
+			m = v;
+		m = sqrt (m);
+		VectorSet (m, m, m, r);
+		VectorSubtract (SVvector (ent, origin), r, SVvector (ent, absmin));
+		VectorAdd (SVvector (ent, origin), r, SVvector (ent, absmax));
+	} else {
+		// set the abs box
+		VectorAdd (SVvector (ent, origin), SVvector (ent, mins),
+				   SVvector (ent, absmin));
+		VectorAdd (SVvector (ent, origin), SVvector (ent, maxs),
+				   SVvector (ent, absmax));
+	}
 
 	// to make items easier to pick up and allow them to be grabbed off
 	// of shelves, the abs sizes are expanded
@@ -380,7 +494,7 @@ SV_LinkEdict (edict_t *ent, qboolean touch_triggers)
 	}
 
 	// link to PVS leafs
-	ent->num_leafs = 0;
+	free_edict_leafs (&SVdata (ent)->leafs);
 	if (SVfloat (ent, modelindex))
 		SV_FindTouchedLeafs (ent, sv.worldmodel->nodes);
 
@@ -402,9 +516,9 @@ SV_LinkEdict (edict_t *ent, qboolean touch_triggers)
 
 	// link it in
 	if (SVfloat (ent, solid) == SOLID_TRIGGER)
-		InsertLinkBefore (&ent->area, &node->trigger_edicts);
+		InsertLinkBefore (&SVdata (ent)->area, &node->trigger_edicts);
 	else
-		InsertLinkBefore (&ent->area, &node->solid_edicts);
+		InsertLinkBefore (&SVdata (ent)->area, &node->solid_edicts);
 
 	// if touch_triggers, touch all entities at this node and descend for more
 	if (touch_triggers)
@@ -413,17 +527,16 @@ SV_LinkEdict (edict_t *ent, qboolean touch_triggers)
 
 /* POINT TESTING IN HULLS */
 
-#ifndef USE_INTEL_ASM
 int
 SV_HullPointContents (hull_t *hull, int num, const vec3_t p)
 {
 	float        d;
-	dclipnode_t *node;
-	mplane_t    *plane;
+	mclipnode_t *node;
+	plane_t     *plane;
 
 	while (num >= 0) {
-		if (num < hull->firstclipnode || num > hull->lastclipnode)
-			Sys_Error ("SV_HullPointContents: bad node number");
+		//if (num < hull->firstclipnode || num > hull->lastclipnode)
+		//	Sys_Error ("SV_HullPointContents: bad node number");
 
 		node = hull->clipnodes + num;
 		plane = hull->planes + node->planenum;
@@ -440,7 +553,6 @@ SV_HullPointContents (hull_t *hull, int num, const vec3_t p)
 
 	return num;
 }
-#endif // !USE_INTEL_ASM
 
 int
 SV_PointContents (const vec3_t p)
@@ -478,110 +590,9 @@ SV_TestEntityPosition (edict_t *ent)
 
 	if (trace.startsolid)
 		return sv.edicts;
-
 	return NULL;
 }
 
-/* LINE TESTING IN HULLS */
-
-// 1/32 epsilon to keep floating point happy
-#define	DIST_EPSILON	(0.03125)
-
-qboolean
-SV_RecursiveHullCheck (hull_t *hull, int num, float p1f, float p2f,
-					   const vec3_t p1, const vec3_t p2, trace_t *trace)
-{
-	float       frac, midf, t1, t2;
-	int         side, i;
-	dclipnode_t *node;
-	mplane_t   *plane;
-	vec3_t      mid;
-
-	// check for empty
-	if (num < 0) {
-		if (num != CONTENTS_SOLID) {
-			trace->allsolid = false;
-			if (num == CONTENTS_EMPTY)
-				trace->inopen = true;
-			else
-				trace->inwater = true;
-		} else
-			trace->startsolid = true;
-		return true;					// empty
-	}
-
-	// find the point distances
-	node = hull->clipnodes + num;
-	plane = hull->planes + node->planenum;
-
-	if (plane->type < 3) {
-		t1 = p1[plane->type] - plane->dist;
-		t2 = p2[plane->type] - plane->dist;
-	} else {
-		t1 = DotProduct (plane->normal, p1) - plane->dist;
-		t2 = DotProduct (plane->normal, p2) - plane->dist;
-	}
-
-	if (t1 >= 0 && t2 >= 0)
-		return SV_RecursiveHullCheck (hull, node->children[0], p1f, p2f, p1,
-									  p2, trace);
-	if (t1 < 0 && t2 < 0)
-		return SV_RecursiveHullCheck (hull, node->children[1], p1f, p2f, p1,
-									  p2, trace);
-
-	side = (t1 < 0);
-	frac = t1 / (t1 - t2);
-//	frac = bound (0, frac, 1); // is this needed?
-
-	midf = p1f + (p2f - p1f) * frac;
-	for (i = 0; i < 3; i++)
-		mid[i] = p1[i] + frac * (p2[i] - p1[i]);
-
-	// move up to the node
-	if (!SV_RecursiveHullCheck (hull, node->children[side],
-								p1f, midf, p1, mid, trace))
-		return false;
-
-	if (SV_HullPointContents (hull, node->children[side ^ 1], mid)
-		!= CONTENTS_SOLID) {
-		// go past the node
-		return SV_RecursiveHullCheck (hull, node->children[side ^ 1], midf,
-									  p2f, mid, p2, trace);
-	}
-
-	if (trace->allsolid)
-		return false;					// never got out of the solid area
-
-	// the other side of the node is solid, this is the impact point
-	if (!side) {
-		VectorCopy (plane->normal, trace->plane.normal);
-		trace->plane.dist = plane->dist;
-	} else {
-		// invert plane paramterers
-		trace->plane.normal[0] = -plane->normal[0];
-		trace->plane.normal[1] = -plane->normal[1];
-		trace->plane.normal[2] = -plane->normal[2];
-		trace->plane.dist = -plane->dist;
-	}
-
-	// put the crosspoint DIST_EPSILON pixels on the near side to guarantee
-	// mid is on the correct side of the plane
-	if (side)
-		frac = (t1 + DIST_EPSILON) / (t1 - t2);
-	else
-		frac = (t1 - DIST_EPSILON) / (t1 - t2);
-	frac = bound (0, frac, 1);
-
-	midf = p1f + (p2f - p1f) * frac;
-	for (i = 0; i < 3; i++)
-		mid[i] = p1[i] + frac * (p2[i] - p1[i]);
-
-	trace->fraction = midf;
-	VectorCopy (mid, trace->endpos);
-
-	return false;
-}
-
 /*
 	SV_ClipMoveToEntity
 
@@ -589,33 +600,75 @@ SV_RecursiveHullCheck (hull_t *hull, int num, float p1f, float p2f,
 	eventually rotation) of the end points
 */
 static trace_t
-SV_ClipMoveToEntity (edict_t *touched, edict_t *mover, const vec3_t start,
+SV_ClipMoveToEntity (edict_t *touched, const vec3_t start,
 					 const vec3_t mins, const vec3_t maxs, const vec3_t end)
 {
 	hull_t     *hull;
 	trace_t     trace;
 	vec3_t      offset, start_l, end_l;
+	vec3_t      forward, right, up;
+	int         rot = 0;
+	vec3_t      temp;
 
 	// fill in a default trace
 	memset (&trace, 0, sizeof (trace_t));
 
 	trace.fraction = 1;
 	trace.allsolid = true;
+	trace.type = tr_point;
 	VectorCopy (end, trace.endpos);
 
 	// get the clipping hull
-	hull = SV_HullForEntity (touched, mins, maxs, offset);
+	hull = SV_HullForEntity (touched, mins, maxs,
+							 trace.type != tr_point ? trace.extents : 0,
+							 offset);
 
 	VectorSubtract (start, offset, start_l);
 	VectorSubtract (end, offset, end_l);
 
-	// trace a line through the apropriate clipping hull
-	SV_RecursiveHullCheck (hull, hull->firstclipnode, 0, 1, start_l, end_l,
-						   &trace);
+	if (SVfloat (touched, solid) == SOLID_BSP
+		&& !VectorIsZero (SVvector (touched, angles))
+		&& touched != sv.edicts) {
+		rot = 1;
+		AngleVectors (SVvector (touched, angles), forward, right, up);
+		VectorNegate (right, right);	// convert lhs to rhs
 
-	// fix trace up by the offset
-	if (trace.fraction != 1)
+		VectorCopy (start_l, temp);
+		start_l[0] = DotProduct (temp, forward);
+		start_l[1] = DotProduct (temp, right);
+		start_l[2] = DotProduct (temp, up);
+
+		VectorCopy (end_l, temp);
+		end_l[0] = DotProduct (temp, forward);
+		end_l[1] = DotProduct (temp, right);
+		end_l[2] = DotProduct (temp, up);
+	}
+
+	// trace a line through the apropriate clipping hull
+	MOD_TraceLine (hull, hull->firstclipnode, start_l, end_l, &trace);
+
+	// fix up trace by the rotation and offset
+	if (trace.fraction != 1) {
+		if (rot) {
+			vec_t       t;
+
+			// transpose the rotation matrix to get its inverse
+			t = forward[1]; forward[1] = right[0]; right[0] = t;
+			t = forward[2]; forward[2] = up[0]; up[0] = t;
+			t = right[2]; right[2] = up[1]; up[1] = t;
+
+			VectorCopy (trace.endpos, temp);
+			trace.endpos[0] = DotProduct (temp, forward);
+			trace.endpos[1] = DotProduct (temp, right);
+			trace.endpos[2] = DotProduct (temp, up);
+
+			VectorCopy (trace.plane.normal, temp);
+			trace.plane.normal[0] = DotProduct (temp, forward);
+			trace.plane.normal[1] = DotProduct (temp, right);
+			trace.plane.normal[2] = DotProduct (temp, up);
+		}
 		VectorAdd (trace.endpos, offset, trace.endpos);
+	}
 
 	// did we clip the move?
 	if (trace.fraction < 1 || trace.startsolid)
@@ -624,73 +677,149 @@ SV_ClipMoveToEntity (edict_t *touched, edict_t *mover, const vec3_t start,
 	return trace;
 }
 
+static always_inline int
+ctl_pretest_everything (edict_t *touch, moveclip_t *clip)
+{
+	if (touch->free)
+		return 0;
+	if (!((int) SVfloat (touch, flags) & FL_FINDABLE_NONSOLID)) {
+		if (SVfloat (touch, solid) == SOLID_NOT)
+			return 0;
+		if (SVfloat (touch, solid) == SOLID_TRIGGER)
+			return 0;
+	}
+	if (touch == clip->passedict)
+		return 0;
+	return 1;
+}
+
+static always_inline int
+ctl_pretest_triggers (edict_t *touch, moveclip_t *clip)
+{
+	if (SVfloat (touch, solid) != SOLID_TRIGGER)
+		return 0;
+	if (!((int) SVfloat (touch, flags) & FL_FINDABLE_NONSOLID))
+		return 0;
+	if (touch == clip->passedict)
+		return 0;
+	return 1;
+}
+
+static always_inline int
+ctl_pretest_other (edict_t *touch, moveclip_t *clip)
+{
+	if (SVfloat (touch, solid) == SOLID_NOT)
+		return 0;
+	if (touch == clip->passedict)
+		return 0;
+	if (SVfloat (touch, solid) == SOLID_TRIGGER)
+		Sys_Error ("Trigger in clipping list");
+
+	if (clip->type == MOVE_NOMONSTERS && SVfloat (touch, solid) != SOLID_BSP)
+		return 0;
+	return 1;
+}
+
+static always_inline int
+ctl_touch_test (edict_t *touch, moveclip_t *clip)
+{
+	if (clip->boxmins[0] > SVvector (touch, absmax)[0]
+		|| clip->boxmins[1] > SVvector (touch, absmax)[1]
+		|| clip->boxmins[2] > SVvector (touch, absmax)[2]
+		|| clip->boxmaxs[0] < SVvector (touch, absmin)[0]
+		|| clip->boxmaxs[1] < SVvector (touch, absmin)[1]
+		|| clip->boxmaxs[2] < SVvector (touch, absmin)[2])
+		return 0;
+
+	if (clip->passedict && SVvector (clip->passedict, size)[0]
+		&& !SVvector (touch, size)[0])
+		return 0;						// points never interact
+
+	// might intersect, so do an exact clip
+	if (clip->passedict) {
+		if (PROG_TO_EDICT (&sv_pr_state, SVentity (touch, owner))
+			== clip->passedict)
+			return 0;					// don't clip against own missiles
+		if (PROG_TO_EDICT (&sv_pr_state,
+						   SVentity (clip->passedict, owner)) == touch)
+			return 0;					// don't clip against owner
+	}
+	return 1;
+}
+
+static void
+ctl_do_clip (edict_t *touch, moveclip_t *clip, trace_t *trace)
+{
+	if ((int) SVfloat (touch, flags) & FL_MONSTER)
+		*trace = SV_ClipMoveToEntity (touch, clip->start,
+									  clip->mins2, clip->maxs2, clip->end);
+	else
+		*trace = SV_ClipMoveToEntity (touch, clip->start,
+									  clip->mins, clip->maxs, clip->end);
+	if (trace->allsolid || trace->startsolid
+		|| trace->fraction < clip->trace.fraction) {
+		trace->ent = touch;
+		if (clip->trace.startsolid) {
+			clip->trace = *trace;
+			clip->trace.startsolid = true;
+		} else
+			clip->trace = *trace;
+	} else if (trace->startsolid)
+		clip->trace.startsolid = true;
+}
+
 /*
 	SV_ClipToLinks
 
 	Mins and maxs enclose the entire area swept by the move
 */
 static void
-SV_ClipToLinks (areanode_t *node, moveclip_t * clip)
+SV_ClipToLinks (areanode_t *node, moveclip_t *clip)
 {
 	edict_t    *touch;
 	link_t     *l, *next;
 	trace_t		trace;
+	int         i;
 
-	// touch linked edicts
-	for (l = node->solid_edicts.next; l != &node->solid_edicts; l = next) {
-		next = l->next;
-		touch = EDICT_FROM_AREA (l);
-		if (SVfloat (touch, solid) == SOLID_NOT)
-			continue;
-		if (touch == clip->passedict)
-			continue;
-		if (SVfloat (touch, solid) == SOLID_TRIGGER)
-			Sys_Error ("Trigger in clipping list");
-
-		if (clip->type == MOVE_NOMONSTERS && SVfloat (touch, solid)
-			!= SOLID_BSP)
-			continue;
-
-		if (clip->boxmins[0] > SVvector (touch, absmax)[0]
-			|| clip->boxmins[1] > SVvector (touch, absmax)[1]
-			|| clip->boxmins[2] > SVvector (touch, absmax)[2]
-			|| clip->boxmaxs[0] < SVvector (touch, absmin)[0]
-			|| clip->boxmaxs[1] < SVvector (touch, absmin)[1]
-			|| clip->boxmaxs[2] < SVvector (touch, absmin)[2])
-			continue;
-
-		if (clip->passedict != 0 && SVvector (clip->passedict, size)[0]
-			&& !SVvector (touch, size)[0])
-			continue;					// points never interact
-
-		// might intersect, so do an exact clip
-		if (clip->trace.allsolid)
-			return;
-		if (clip->passedict) {
-			if (PROG_TO_EDICT (&sv_pr_state, SVentity (touch, owner))
-				== clip->passedict)
-				continue;				// don't clip against own missiles
-			if (PROG_TO_EDICT (&sv_pr_state,
-							   SVentity (clip->passedict, owner)) == touch)
-				continue;				// don't clip against owner
+	if (clip->type == TL_EVERYTHING) {
+		touch = NEXT_EDICT (&sv_pr_state, sv.edicts);
+		for (i = 1; i < sv.num_edicts; i++,
+			 touch = NEXT_EDICT (&sv_pr_state, touch)) {
+			if (clip->trace.allsolid)
+				return;
+			if (!ctl_pretest_everything (touch, clip))
+				continue;
+			if (!ctl_touch_test (touch, clip))
+				continue;
+			ctl_do_clip (touch, clip, &trace);
 		}
+	} else if (clip->type == TL_TRIGGERS) {
+		for (l = node->solid_edicts.next; l != &node->solid_edicts; l = next) {
+			next = l->next;
+			touch = EDICT_FROM_AREA (l);
 
-		if ((int) SVfloat (touch, flags) & FL_MONSTER)
-			trace = SV_ClipMoveToEntity (touch, clip->passedict, clip->start,
-										 clip->mins2, clip->maxs2, clip->end);
-		else
-			trace = SV_ClipMoveToEntity (touch, clip->passedict, clip->start,
-										 clip->mins, clip->maxs, clip->end);
-		if (trace.allsolid || trace.startsolid
-			|| trace.fraction < clip->trace.fraction) {
-			trace.ent = touch;
-			if (clip->trace.startsolid) {
-				clip->trace = trace;
-				clip->trace.startsolid = true;
-			} else
-				clip->trace = trace;
-		} else if (trace.startsolid)
-			clip->trace.startsolid = true;
+			if (clip->trace.allsolid)
+				return;
+			if (!ctl_pretest_triggers (touch, clip))
+				continue;
+			if (!ctl_touch_test (touch, clip))
+				continue;
+			ctl_do_clip (touch, clip, &trace);
+		}
+	} else {
+		// touch linked edicts
+		for (l = node->solid_edicts.next; l != &node->solid_edicts; l = next) {
+			next = l->next;
+			touch = EDICT_FROM_AREA (l);
+
+			if (clip->trace.allsolid)
+				return;
+			if (!ctl_pretest_other (touch, clip))
+				continue;
+			if (!ctl_touch_test (touch, clip))
+				continue;
+			ctl_do_clip (touch, clip, &trace);
+		}
 	}
 
 	// recurse down both sides
@@ -736,8 +865,7 @@ SV_Move (const vec3_t start, const vec3_t mins, const vec3_t maxs,
 	memset (&clip, 0, sizeof (moveclip_t));
 
 	// clip to world
-	clip.trace = SV_ClipMoveToEntity (sv.edicts, passedict, start,
-									  mins, maxs, end);
+	clip.trace = SV_ClipMoveToEntity (sv.edicts, start, mins, maxs, end);
 	clip.start = start;
 	clip.end = end;
 	clip.mins = mins;
@@ -805,7 +933,7 @@ SV_TestPlayerPosition (edict_t *ent, const vec3_t origin)
 
 		// get the clipping hull
 		hull = SV_HullForEntity (check, SVvector (ent, mins),
-								 SVvector (ent, maxs), offset);
+								 SVvector (ent, maxs), 0, offset);
 
 		VectorSubtract (origin, offset, offset);
 
