@@ -23,7 +23,6 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: cl_demo.h,v 1.7 2004/02/21 02:31:22 taniwha Exp $
 */
 
 #ifndef _CL_DEMO_H
@@ -33,15 +32,11 @@
 #include "qw/protocol.h"
 
 void CL_StopPlayback (void);
-qboolean CL_GetMessage (void);
+int CL_GetMessage (void);
 void CL_WriteDemoCmd (usercmd_t *pcmd);
 
-void CL_Stop_f (void);
-void CL_Record_f (void);
-void CL_Record (const char *argv1);
-void CL_ReRecord_f (void);
-void CL_PlayDemo_f (void);
-void CL_TimeDemo_f (void);
+void CL_StopRecording (void);
+void CL_Record (const char *argv1, int track);	// track ignored
 
 void CL_Demo_Init (void);
 
