@@ -23,17 +23,19 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: cl_ents.h,v 1.9 2002/10/02 21:56:44 taniwha Exp $
 */
 
 #ifndef _CL_ENTS_H
 #define _CL_ENTS_H
 
 #include "QF/qtypes.h"
+#include "QF/render.h"
 
 void CL_SetSolidPlayers (int playernum);
 void CL_ClearPredict (void);
 void CL_SetUpPlayerPrediction(qboolean dopred);
+void CL_TransformEntity (struct entity_s * ent, const vec3_t angles,
+						 qboolean force);
 void CL_EmitEntities (void);
 void CL_ClearProjectiles (void);
 void CL_ParseProjectiles (qboolean nail2);
@@ -45,4 +47,8 @@ void CL_Ents_Init (void);
 extern struct cvar_s   *cl_deadbodyfilter;
 extern struct cvar_s   *cl_gibfilter;
 
+extern entity_t cl_player_ents[];
+extern entity_t cl_flag_ents[];
+extern entity_t cl_packet_ents[];
+
 #endif
