@@ -26,7 +26,6 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: sv_qtv.h,v 1.2 2004/03/05 23:42:15 taniwha Exp $
 */
 
 #ifndef __sv_qtv_h
@@ -36,5 +35,9 @@ struct info_s;
 void SV_qtvConnect (int qport, struct info_s *info);
 int SV_qtvPacket (int qport);
 void SV_qtvCheckTimeouts (void);
+void SV_qtvSendMessages (void);
+void SV_qtvFinalMessage (const char *message);
+void SV_qtvChanging (void);
+void SV_qtvReconnect (void);
 
 #endif//__sv_qtv_h
