@@ -33,9 +33,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: cl_cam.c,v 1.55 2003/09/03 22:16:58 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -45,15 +42,15 @@ static __attribute__ ((unused)) const char rcsid[] =
 
 #include <math.h>
 
-#include "QF/console.h"
 #include "QF/cvar.h"
 #include "QF/msg.h"
 
+#include "chase.h"
 #include "cl_cam.h"
 #include "cl_input.h"
 #include "client.h"
 #include "compat.h"
-#include "pmove.h"
+#include "qw/pmove.h"
 #include "sbar.h"
 
 #define	PM_SPECTATORMAXSPEED 500
@@ -69,15 +66,6 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/mathlib.h"
 #include "world.h"
 
-vec3_t camera_origin = {0,0,0};
-vec3_t camera_angles = {0,0,0};
-vec3_t player_origin = {0,0,0};
-vec3_t player_angles = {0,0,0};
-
-cvar_t     *chase_back;
-cvar_t     *chase_up;
-cvar_t     *chase_right;
-cvar_t     *chase_active;
 cvar_t     *cl_hightrack;	// track high fragger
 cvar_t     *cl_chasecam;
 cvar_t     *cl_camera_maxpitch;
@@ -202,7 +190,7 @@ Cam_Lock (int playernum)
 	Sbar_Changed ();
 }
 
-static pmtrace_t
+static trace_t
 Cam_DoTrace (vec3_t vec1, vec3_t vec2)
 {
 #if 0
@@ -223,29 +211,29 @@ Cam_TryFlyby (player_state_t * self, player_state_t * player, vec3_t vec,
 			  qboolean checkvis)
 {
 	float       len;
-	pmtrace_t   trace;
+	trace_t     trace;
 	vec3_t      v;
 
 	vectoangles (vec, v);
 	VectorCopy (v, pmove.angles);
 	VectorNormalize (vec);
-	VectorMultAdd (player->origin, 800, vec, v);
+	VectorMultAdd (player->pls.origin, 800, vec, v);
 	// v is endpos
 	// fake a player move
-	trace = Cam_DoTrace (player->origin, v);
+	trace = Cam_DoTrace (player->pls.origin, v);
 	if ( /* trace.inopen || */ trace.inwater)
 		return 9999;
 	VectorCopy (trace.endpos, vec);
-	len = VectorDistance (trace.endpos, player->origin);
+	len = VectorDistance (trace.endpos, player->pls.origin);
 
 	if (len < 32 || len > 800)
 		return 9999;
 	if (checkvis) {
-		trace = Cam_DoTrace (self->origin, vec);
+		trace = Cam_DoTrace (self->pls.origin, vec);
 		if (trace.fraction != 1 || trace.inwater)
 			return 9999;
 
-		len = VectorDistance (trace.endpos, self->origin);
+		len = VectorDistance (trace.endpos, self->pls.origin);
 	}
 
 	return len;
@@ -256,14 +244,14 @@ static qboolean
 Cam_IsVisible (player_state_t * player, vec3_t vec)
 {
 	float       d;
-	pmtrace_t   trace;
+	trace_t     trace;
 	vec3_t      v;
 
-	trace = Cam_DoTrace (player->origin, vec);
+	trace = Cam_DoTrace (player->pls.origin, vec);
 	if (trace.fraction != 1 || /* trace.inopen || */ trace.inwater)
 		return false;
 	// check distance, don't let the player get too far away or too close
-	VectorSubtract (player->origin, vec, v);
+	VectorSubtract (player->pls.origin, vec, v);
 	d = VectorLength (v);
 
 	return (d > 16.0);
@@ -367,7 +355,7 @@ Cam_CheckHighTarget (void)
 	j = -1;
 	for (i = 0, max = -9999; i < MAX_CLIENTS; i++) {
 		s = &cl.players[i];
-		if (s->name[0] && !s->spectator && s->frags > max) {
+		if (s->name && s->name->value[0] && !s->spectator && s->frags > max) {
 			max = s->frags;
 			j = i;
 		}
@@ -403,7 +391,7 @@ Cam_Track (usercmd_t *cmd)
 		return;
 
 	if (locked
-		&& (!cl.players[spec_track].name[0]
+		&& (!cl.players[spec_track].name->value[0]
 			|| cl.players[spec_track].spectator)) {
 		locked = false;
 		if (cl_hightrack->int_val)
@@ -451,22 +439,22 @@ Cam_Track (usercmd_t *cmd)
 		cmd->forwardmove = cmd->sidemove = cmd->upmove = 0;
 
 		VectorCopy (player->viewangles, cl.viewangles);
-		VectorCopy (player->origin, desired_position);
-		if (memcmp (&desired_position, &self->origin,
+		VectorCopy (player->pls.origin, desired_position);
+		if (memcmp (&desired_position, &self->pls.origin,
 					sizeof (desired_position)) != 0) {
 			if (!cls.demoplayback) {
 				MSG_WriteByte (&cls.netchan.message, clc_tmove);
 				MSG_WriteCoordV (&cls.netchan.message, desired_position);
 			}
 			// move there locally immediately
-			VectorCopy (desired_position, self->origin);
+			VectorCopy (desired_position, self->pls.origin);
 		}
-		self->weaponframe = player->weaponframe;
+		self->pls.weaponframe = player->pls.weaponframe;
 
 	} else {
 		// Ok, move to our desired position and set our angles to view
 		// the player
-		VectorSubtract (desired_position, self->origin, vec);
+		VectorSubtract (desired_position, self->pls.origin, vec);
 		len = VectorLength (vec);
 		cmd->forwardmove = cmd->sidemove = cmd->upmove = 0;
 		if (len > 16) {					// close enough?
@@ -476,9 +464,9 @@ Cam_Track (usercmd_t *cmd)
 			}
 		}
 		// move there locally immediately
-		VectorCopy (desired_position, self->origin);
+		VectorCopy (desired_position, self->pls.origin);
 
-		VectorSubtract (player->origin, desired_position, vec);
+		VectorSubtract (player->pls.origin, desired_position, vec);
 		vectoangles (vec, cl.viewangles);
 		cl.viewangles[0] = -cl.viewangles[0];
 	}
@@ -531,7 +519,7 @@ Cam_SetView (void)
 	player = frame->playerstate + spec_track;
 	self = frame->playerstate + cl.playernum;
 
-	VectorSubtract (player->origin, cl.simorg, vec);
+	VectorSubtract (player->pls.origin, cl.simorg, vec);
 	if (cam_forceview) {
 		cam_forceview = false;
 		vectoangles (vec, cam_viewangles);
@@ -549,6 +537,7 @@ Cam_SetView (void)
 	}
 	VectorCopy (cam_viewangles, cl.viewangles);
 	VectorCopy (cl.viewangles, cl.simangles);
+	cl.simangles[ROLL] = 0;						// FIXME @@@
 }
 #endif
 
@@ -570,7 +559,7 @@ Cam_FinishMove (usercmd_t *cmd)
 		player = frame->playerstate + spec_track;
 		self = frame->playerstate + cl.playernum;
 
-		VectorSubtract (player->origin, self->origin, vec);
+		VectorSubtract (player->pls.origin, self->pls.origin, vec);
 		if (cam_forceview) {
 			cam_forceview = false;
 			vectoangles (vec, cam_viewangles);
@@ -623,7 +612,7 @@ Cam_FinishMove (usercmd_t *cmd)
 		}
 		oldbuttons |= BUTTON_JUMP;		// don't jump again until released
 	}
-//	Con_Printf ("Selecting track target...\n");
+//	Sys_Printf ("Selecting track target...\n");
 
 	if (locked && autocam)
 		end = (spec_track + 1) % MAX_CLIENTS;
@@ -632,7 +621,7 @@ Cam_FinishMove (usercmd_t *cmd)
 	i = end;
 	do {
 		s = &cl.players[i];
-		if (s->name[0] && !s->spectator) {
+		if (s->name && s->name->value[0] && !s->spectator) {
 			Cam_Lock (i);
 			ideal_track = i;
 			return;
@@ -642,12 +631,12 @@ Cam_FinishMove (usercmd_t *cmd)
 	// stay on same guy?
 	i = spec_track;
 	s = &cl.players[i];
-	if (s->name[0] && !s->spectator) {
+	if (s->name && s->name->value[0] && !s->spectator) {
 		Cam_Lock (i);
 		ideal_track = i;
 		return;
 	}
-	Con_Printf ("No target found ...\n");
+	Sys_Printf ("No target found ...\n");
 	autocam = locked = false;
 }
 
@@ -672,190 +661,4 @@ CL_Cam_Init_Cvars (void)
 	cl_hightrack = Cvar_Get ("cl_hightrack", "0", CVAR_NONE, NULL, "view the "
 							 "player who has the most frags while you are in "
 							 "spectator mode.");
-
-	chase_back = Cvar_Get ("chase_back", "100", CVAR_NONE, NULL, "None");
-	chase_up = Cvar_Get ("chase_up", "16", CVAR_NONE, NULL, "None");
-	chase_right = Cvar_Get ("chase_right", "0", CVAR_NONE, NULL, "None");
-	chase_active = Cvar_Get ("chase_active", "0", CVAR_NONE, NULL, "None");
-}
-
-static void
-TraceLine (vec3_t start, vec3_t end, vec3_t impact)
-{
-	pmtrace_t     trace;
-
-	memset (&trace, 0, sizeof (trace));
-	PM_RecursiveHullCheck (cl.worldmodel->hulls, 0, 0, 1, start, end, &trace);
-
-	VectorCopy (trace.endpos, impact);
-}
-
-void
-Chase_Update (void)
-{
-	float		pitch, yaw, fwd;
-	int			i;
-	vec3_t		forward, up, right, stop, dir;
-	usercmd_t	cmd;	// movement direction
-
-	// lazy camera, look toward player entity
-
-	if (chase_active->int_val == 2 || chase_active->int_val == 3)
-	{
-		// control camera angles with key/mouse/joy-look
-
-		camera_angles[PITCH] += cl.viewangles[PITCH] - player_angles[PITCH];
-		camera_angles[YAW] += cl.viewangles[YAW] - player_angles[YAW];
-		camera_angles[ROLL] += cl.viewangles[ROLL] - player_angles[ROLL];
-
-		if (chase_active->int_val == 2)
-		{
-			if (camera_angles[PITCH] < -60) camera_angles[PITCH] = -60;
-			if (camera_angles[PITCH] > 60) camera_angles[PITCH] = 60;
-		}
-
-		// move camera, it's not enough to just change the angles because
-		// the angles are automatically changed to look toward the player
-
-		if (chase_active->int_val == 3)
-			VectorCopy (r_refdef.vieworg, player_origin);
-
-		AngleVectors (camera_angles, forward, right, up);
-		VectorScale (forward, chase_back->value, forward);
-		VectorSubtract (player_origin, forward, camera_origin);
-
-		if (chase_active->int_val == 2)
-		{
-			VectorCopy (r_refdef.vieworg, player_origin);
-
-			// don't let camera get too low
-			if (camera_origin[2] < player_origin[2] + chase_up->value)
-				camera_origin[2] = player_origin[2] + chase_up->value;
-		}
-
-		// don't let camera get too far from player
-
-		VectorSubtract  (camera_origin, player_origin, dir);
-		VectorCopy      (dir, forward);
-		VectorNormalize (forward);
-
-		if (VectorLength (dir) > chase_back->value)
-		{
-			VectorScale (forward, chase_back->value, dir);
-			VectorAdd (player_origin, dir, camera_origin);
-		}
-
-		// check for walls between player and camera
-
-		VectorScale (forward, 8, forward);
-		VectorAdd (camera_origin, forward, camera_origin);
-		TraceLine (player_origin, camera_origin, stop);
-		if (VectorLength (stop) != 0)
-			VectorSubtract (stop, forward, camera_origin);
-
-		VectorSubtract  (camera_origin, r_refdef.vieworg, dir);
-		VectorCopy (dir, forward);
-		VectorNormalize (forward);
-
-		if (chase_active->int_val == 2)
-		{
-			if (dir[1] == 0 && dir[0] == 0)
-			{
-				// look straight up or down
-//				camera_angles[YAW] = r_refdef.viewangles[YAW];
-				if (dir[2] > 0)
-					camera_angles[PITCH] = 90;
-				else
-					camera_angles[PITCH] = 270;
-			}
-			else
-			{
-				yaw = (atan2 (dir[1], dir[0]) * 180 / M_PI);
-				if (yaw <   0) yaw += 360;
-				if (yaw < 180) yaw += 180;
-				else           yaw -= 180;
-				camera_angles[YAW] = yaw;
-
-				fwd = sqrt (dir[0] * dir[0] + dir[1] * dir[1]);
-				pitch = (atan2 (dir[2], fwd) * 180 / M_PI);
-				if (pitch < 0) pitch += 360;
-				camera_angles[PITCH] = pitch;
-			}
-		}
-
-		VectorCopy (camera_angles, r_refdef.viewangles); // rotate camera
-		VectorCopy (camera_origin, r_refdef.vieworg);    // move camera
-
-		// get basic movement from keyboard
-
-		memset (&cmd, 0, sizeof (cmd));
-//		VectorCopy (cl.viewangles, cmd.angles);
-
-		if (in_strafe.state & 1) {
-			cmd.sidemove += cl_sidespeed->value * CL_KeyState (&in_right);
-			cmd.sidemove -= cl_sidespeed->value * CL_KeyState (&in_left);
-		}
-		cmd.sidemove += cl_sidespeed->value * CL_KeyState (&in_moveright);
-		cmd.sidemove -= cl_sidespeed->value * CL_KeyState (&in_moveleft);
-
-		if (!(in_klook.state & 1)) {
-			cmd.forwardmove += cl_forwardspeed->value
-				* CL_KeyState (&in_forward);
-			cmd.forwardmove -= cl_backspeed->value * CL_KeyState (&in_back);
-		}
-		if (in_speed.state & 1) {
-			cmd.forwardmove *= cl_movespeedkey->value;
-			cmd.sidemove    *= cl_movespeedkey->value;
-		}
-
-		// mouse and joystick controllers add to movement
-		dir[1] = cl.viewangles[1] - camera_angles[1];  dir[0] = 0;  dir[2] = 0;
-		AngleVectors (dir, forward, right, up);
-		VectorScale  (forward, viewdelta.position[2] * m_forward->value,
-					  forward);
-		VectorScale  (right, viewdelta.position[0] * m_side->value, right);
-		VectorAdd    (forward, right, dir);
-		cmd.forwardmove += dir[0];
-		cmd.sidemove    -= dir[1];
-
-		dir[1] = camera_angles[1];  dir[0] = 0;  dir[2] = 0;
-		AngleVectors (dir, forward, right, up);
-
-		VectorScale (forward, cmd.forwardmove, forward);
-		VectorScale (right,   cmd.sidemove,    right);
-		VectorAdd   (forward, right, dir);
-
-		if (dir[1] || dir[0])
-		{
-			cl.viewangles[YAW] = (atan2 (dir[1], dir[0]) * 180 / M_PI);
-			if (cl.viewangles[YAW] <   0) cl.viewangles[YAW] += 360;
-//			if (cl.viewangles[YAW] < 180) cl.viewangles[YAW] += 180;
-//			else                          cl.viewangles[YAW] -= 180;
-		}
-
-		cl.viewangles[PITCH] = 0;
-
-		// remember the new angle to calculate the difference next frame
-		VectorCopy (cl.viewangles, player_angles);
-
-		return;
-	}
-
-	// regular camera, faces same direction as player
-
-	AngleVectors (cl.viewangles, forward, right, up);
-
-	// calc exact destination
-	for (i = 0; i < 3; i++)
-		camera_origin[i] = r_refdef.vieworg[i]
-			- forward[i] * chase_back->value - right[i] * chase_right->value;
-	camera_origin[2] += chase_up->value;
-
-	// check for walls between player and camera
-	TraceLine (r_refdef.vieworg, camera_origin, stop);
-	if (VectorLength (stop) != 0)
-		for (i = 0; i < 3; i++)
-			camera_origin[i] = stop[i] + forward[i] * 8;
-
-	VectorCopy (camera_origin, r_refdef.vieworg);
 }
