@@ -1,7 +1,7 @@
 /*
 	cl_demo.c
 
-	(description)
+	demo playback support
 
 	Copyright (C) 1996-1997  Id Software, Inc.
 
@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: cl_demo.c,v 1.81 2004/02/19 08:58:42 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -41,13 +38,17 @@ static __attribute__ ((unused)) const char rcsid[] =
 # include <unistd.h>
 #endif
 
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #include <time.h>
-	
+#include <math.h>
+
 #include "QF/cbuf.h"
-#include "QF/console.h"
 #include "QF/cmd.h"
 #include "QF/cvar.h"
+#include "QF/dstring.h"
+#include "QF/keys.h"
 #include "QF/msg.h"
 #include "QF/qendian.h"
 #include "QF/sys.h"
@@ -60,7 +61,7 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "client.h"
 #include "compat.h"
 #include "host.h"
-#include "pmove.h"
+#include "qw/pmove.h"
 
 typedef struct {
 	int         frames;
@@ -68,29 +69,32 @@ typedef struct {
 	double      fps;
 } td_stats_t;
 
-int     demo_timeframes_isactive;
-int     demo_timeframes_index;
-int     demotime_cached;
-float   nextdemotime;
-char    demoname[1024];
-double *demo_timeframes_array;
+int         demo_timeframes_isactive;
+int         demo_timeframes_index;
+static int  demotime_cached;
+static float cached_demotime;
+static byte cached_newtime;
+float       nextdemotime;
+char        demoname[1024];
+double     *demo_timeframes_array;
 #define CL_TIMEFRAMES_ARRAYBLOCK 4096
 
-int     timedemo_count;
-int     timedemo_runs;
+int         timedemo_count;
+int         timedemo_runs;
 td_stats_t *timedemo_data;
 
-
 static void CL_FinishTimeDemo (void);
 static void CL_TimeFrames_DumpLog (void);
 static void CL_TimeFrames_AddTimestamp (void);
 static void CL_TimeFrames_Reset (void);
 
-cvar_t     *demo_speed;
 cvar_t     *demo_gzip;
+cvar_t     *demo_speed;
 cvar_t     *demo_quit;
 cvar_t     *demo_timeframes;
 
+#define MAX_DEMMSG (MAX_MSGLEN + 8) //+8 for header
+
 /*
 	DEMO CODE
 
@@ -102,6 +106,31 @@ cvar_t     *demo_timeframes;
 */
 
 
+/*
+	CL_WriteDemoMessage
+
+	Dumps the current net message, prefixed by the length and view angles
+*/
+static void
+CL_WriteDemoMessage (sizebuf_t *msg)
+{
+	byte        c;
+	float       f;
+	int         len;
+
+	f = LittleFloat ((float) realtime);
+	Qwrite (cls.demofile, &f, sizeof (f));
+
+	c = dem_read;
+	Qwrite (cls.demofile, &c, sizeof (c));
+
+	len = LittleLong (msg->cursize);
+	Qwrite (cls.demofile, &len, 4);
+	Qwrite (cls.demofile, msg->data, msg->cursize);
+
+	Qflush (cls.demofile);
+}
+
 /*
 	CL_StopPlayback
 
@@ -116,6 +145,7 @@ CL_StopPlayback (void)
 	Qclose (cls.demofile);
 	cls.demofile = NULL;
 	CL_SetState (ca_disconnected);
+	cls.demo_capture = 0;
 	cls.demoplayback = 0;
 	cls.demoplayback2 = 0;
 	demotime_cached = 0;
@@ -125,6 +155,281 @@ CL_StopPlayback (void)
 		CL_FinishTimeDemo ();
 }
 
+void
+CL_StopRecording (void)
+{
+	// write a disconnect message to the demo file
+	SZ_Clear (net_message->message);
+	MSG_WriteLong (net_message->message, -1);  // -1 sequence means out of band
+	MSG_WriteByte (net_message->message, svc_disconnect);
+	MSG_WriteString (net_message->message, "EndOfDemo");
+	CL_WriteDemoMessage (net_message->message);
+
+	// finish up
+	Qclose (cls.demofile);
+	cls.demofile = NULL;
+	cls.demorecording = false;
+	Sys_Printf ("Completed demo\n");
+}
+
+#if 0
+static const char *dem_names[] = {
+	"dem_cmd",
+	"dem_read",
+	"dem_set",
+	"dem_multiple",
+	"dem_single",
+	"dem_stats",
+	"dem_all",
+	"dem_invalid",
+};
+#endif
+
+static void
+get_demotime (float *demotime, byte *newtime)
+{
+	// read the time from the packet
+	*newtime = 0;
+	if (demotime_cached) {
+		*demotime = cached_demotime;
+		*newtime = cached_newtime;
+		demotime_cached = 0;
+	} else {
+		if (cls.demoplayback2) {
+			Qread (cls.demofile, newtime, sizeof (*newtime));
+			*demotime = cls.basetime + (cls.prevtime + *newtime) * 0.001;
+		} else {
+			Qread (cls.demofile, demotime, sizeof (*demotime));
+			*demotime = LittleFloat (*demotime);
+			if (!nextdemotime)
+				realtime = nextdemotime = *demotime;
+		}
+	}
+}
+
+static void
+push_demotime (float demotime, byte newtime)
+{
+	// rewind back to time
+	demotime_cached = 1;
+	cached_demotime = demotime;
+	cached_newtime = newtime;
+}
+
+// decide if it is time to grab the next message
+static int
+check_next_demopacket (void)
+{
+	byte        newtime;
+	float       demotime;
+
+	get_demotime (&demotime, &newtime);
+
+	if (cls.timedemo) {
+		if (cls.td_lastframe < 0)
+			cls.td_lastframe = demotime;
+		else if (demotime > cls.td_lastframe) {
+			cls.td_lastframe = demotime;
+			push_demotime (demotime, newtime);
+			return 0;					// already read this frame's message
+		}
+		if (!cls.td_starttime && cls.state == ca_active) {
+			cls.td_starttime = Sys_DoubleTime ();
+			cls.td_startframe = host_framecount;
+		}
+		realtime = demotime;			// warp
+	} else if (!cl.paused && cls.state >= ca_onserver) {
+		// always grab until fully connected
+		if (!cls.demoplayback2 && realtime + 1.0 < demotime) {
+			// too far back
+			realtime = demotime - 1.0;
+			push_demotime (demotime, newtime);
+			return 0;
+		} else if (realtime < demotime) {
+			push_demotime (demotime, newtime);
+			return 0;					// don't need another message yet
+		}
+	} else
+		realtime = demotime;			// we're warping
+
+	if (realtime - nextdemotime > 0.0001) {
+		if (nextdemotime != demotime) {
+			if (cls.demoplayback2) {
+				cls.netchan.incoming_sequence++;
+				cls.netchan.incoming_acknowledged++;
+				cls.netchan.frame_latency = 0;
+			}
+		}
+	}
+	nextdemotime = demotime;
+
+	cls.prevtime += newtime;
+	return 1;
+}
+
+static int
+read_demopacket (void)
+{
+	int         r;
+
+	Qread (cls.demofile, &net_message->message->cursize, 4);
+	net_message->message->cursize =
+		LittleLong (net_message->message->cursize);
+	if (net_message->message->cursize > MAX_DEMMSG)
+		Host_Error ("Demo message > MAX_DEMMSG: %d/%d",
+					net_message->message->cursize, MAX_DEMMSG);
+	r = Qread (cls.demofile, net_message->message->data,
+			   net_message->message->cursize);
+	if (r != net_message->message->cursize) {
+		CL_StopPlayback ();
+		return 0;
+	}
+	return 1;
+}
+
+static int
+CL_GetDemoMessage (void)
+{
+	byte        c;
+	float       f;
+	int         r, i, j, tracknum;
+	usercmd_t  *pcmd;
+
+	if (!cls.demoplayback2)
+		nextdemotime = realtime;
+	if (realtime + 1.0 < nextdemotime)
+		realtime = nextdemotime - 1.0;
+
+nextdemomessage:
+	if (!check_next_demopacket ())
+		return 0;
+
+	if (cls.state < ca_demostart)
+		Host_Error ("CL_GetDemoMessage: cls.state != ca_active");
+
+	// get the msg type
+	Qread (cls.demofile, &c, sizeof (c));
+
+	switch (c & 7) {
+		case dem_cmd:
+			// user sent input
+			net_message->message->cursize = -1;
+			i = cls.netchan.outgoing_sequence & UPDATE_MASK;
+			pcmd = &cl.frames[i].cmd;
+			r = Qread (cls.demofile, pcmd, sizeof (*pcmd));
+			if (r != sizeof (*pcmd)) {
+				CL_StopPlayback ();
+				return 0;
+			}
+			// byte order stuff
+			for (j = 0; j < 3; j++)
+				pcmd->angles[j] = LittleFloat (pcmd->angles[j]);
+			pcmd->forwardmove = LittleShort (pcmd->forwardmove);
+			pcmd->sidemove = LittleShort (pcmd->sidemove);
+			pcmd->upmove = LittleShort (pcmd->upmove);
+			cl.frames[i].senttime = nextdemotime;	//FIXME was demotime
+			cl.frames[i].receivedtime = -1;	// we haven't gotten a reply yet
+			cls.netchan.outgoing_sequence++;
+			for (i = 0; i < 3; i++) {
+				Qread (cls.demofile, &f, 4);
+				cl.viewangles[i] = LittleFloat (f);
+			}
+			break;
+
+		case dem_read:
+readit:
+			// get the next message
+			if (read_demopacket () && cls.demoplayback2) {
+				tracknum = Cam_TrackNum ();
+
+				if (cls.lasttype == dem_multiple) {
+					if (tracknum == -1)
+						goto nextdemomessage;
+					if (!(cls.lastto & (1 << tracknum)))
+						goto nextdemomessage;
+				} else if (cls.lasttype == dem_single) {
+					if (tracknum == -1 || cls.lastto != spec_track)
+						goto nextdemomessage;
+				}
+			}
+			break;
+		case dem_set:
+			Qread (cls.demofile, &i, 4);
+			cls.netchan.outgoing_sequence = LittleLong (i);
+			Qread (cls.demofile, &i, 4);
+			cls.netchan.incoming_sequence = LittleLong (i);
+			if (cls.demoplayback2) {
+				cls.netchan.incoming_acknowledged =
+					cls.netchan.incoming_sequence;
+				goto nextdemomessage;
+			}
+			break;
+		case dem_multiple:
+			r = Qread (cls.demofile, &i, 4);
+			if (r != 4) {
+				CL_StopPlayback ();
+				return 0;
+			}
+			cls.lastto = LittleLong (i);
+			cls.lasttype = dem_multiple;
+			goto readit;
+		case dem_single:
+			cls.lastto = c >> 3;
+			cls.lasttype = dem_single;
+			goto readit;
+		case dem_stats:
+			cls.lastto = c >> 3;
+			cls.lasttype = dem_stats;
+			goto readit;
+		case dem_all:
+			cls.lastto = 0;
+			cls.lasttype = dem_all;
+			goto readit;
+		default:
+			Sys_Printf ("Corrupted demo.\n");
+			CL_StopPlayback ();
+			return 0;
+	}
+	return 1;
+}
+
+static int
+CL_GetPacket (void)
+{
+	return NET_GetPacket ();
+}
+
+/*
+	CL_GetMessage
+
+	Handles recording and playback of demos, on top of NET_ code
+*/
+int
+CL_GetMessage (void)
+{
+	if (cls.demoplayback) {
+		int         ret = CL_GetDemoMessage ();
+
+		if (!ret && demo_timeframes_isactive && cls.td_starttime) {
+			CL_TimeFrames_AddTimestamp ();
+		}
+		return ret;
+	}
+
+	if (!CL_GetPacket ())
+		return 0;
+
+	if (net_packetlog->int_val)
+		Log_Incoming_Packet (net_message->message->data,
+							 net_message->message->cursize, 1, 0);
+
+
+	if (cls.demorecording)
+		CL_WriteDemoMessage (net_message->message);
+
+	return 1;
+}
+
 /*
 	CL_WriteDemoCmd
 
@@ -163,297 +468,95 @@ CL_WriteDemoCmd (usercmd_t *pcmd)
 	Qflush (cls.demofile);
 }
 
-/*
-	CL_WriteDemoMessage
-
-	Dumps the current net message, prefixed by the length and view angles
-*/
-static void
-CL_WriteDemoMessage (sizebuf_t *msg)
-{
-	byte		c;
-	float		fl;
-	int			len;
-
-	if (!cls.demorecording)
-		return;
-
-	fl = LittleFloat ((float) realtime);
-	Qwrite (cls.demofile, &fl, sizeof (fl));
-
-	c = dem_read;
-	Qwrite (cls.demofile, &c, sizeof (c));
-
-	len = LittleLong (msg->cursize);
-	Qwrite (cls.demofile, &len, 4);
-	Qwrite (cls.demofile, msg->data, msg->cursize);
-
-	Qflush (cls.demofile);
-}
-
-#if 0
-static const char *dem_names[] = {
-	"dem_cmd",
-	"dem_read",
-	"dem_set",
-	"dem_multiple",
-	"dem_single",
-	"dem_stats",
-	"dem_all",
-	"dem_invalid",
-};
-#endif
-
-static qboolean
-CL_GetDemoMessage (void)
-{
-	byte		c, newtime;
-	float		demotime, f;
-	static float cached_demotime;
-	static byte cached_newtime;
-	int			r, i, j, tracknum;
-	usercmd_t  *pcmd;
-
-	if (!cls.demoplayback2)
-		nextdemotime = realtime;
-	if (realtime + 1.0 < nextdemotime)
-		realtime = nextdemotime - 1.0;
-
-nextdemomessage:
-	// read the time from the packet
-	newtime = 0;
-	if (demotime_cached) {
-		demotime = cached_demotime;
-		newtime = cached_newtime;
-		demotime_cached = 0;
-	} else {
-		if (cls.demoplayback2) {
-			Qread (cls.demofile, &newtime, sizeof (newtime));
-			demotime = cls.basetime + (cls.prevtime + newtime) * 0.001;
-		} else {
-			Qread (cls.demofile, &demotime, sizeof (demotime));
-			demotime = LittleFloat (demotime);
-			if (!nextdemotime)
-				realtime = nextdemotime = demotime;
-		}
-	}
-
-	// decide if it is time to grab the next message        
-	if (cls.timedemo) {
-		if (cls.td_lastframe < 0)
-			cls.td_lastframe = demotime;
-		else if (demotime > cls.td_lastframe) {
-			cls.td_lastframe = demotime;
-			// rewind back to time
-			demotime_cached = 1;
-			cached_demotime = demotime;
-			cached_newtime = newtime;
-			return 0;					// already read this frame's message
-		}
-		if (!cls.td_starttime && cls.state == ca_active) {
-			cls.td_starttime = Sys_DoubleTime ();
-			cls.td_startframe = host_framecount;
-		}
-		realtime = demotime;			// warp
-	} else if (!cl.paused && cls.state >= ca_onserver) {
-		// always grab until fully connected
-		if (!cls.demoplayback2 && realtime + 1.0 < demotime) {
-			// too far back
-			realtime = demotime - 1.0;
-			// rewind back to time
-			demotime_cached = 1;
-			cached_demotime = demotime;
-			cached_newtime = newtime;
-			return 0;
-		} else if (realtime < demotime) {
-			// rewind back to time
-			demotime_cached = 1;
-			cached_demotime = demotime;
-			cached_newtime = newtime;
-			return 0;					// don't need another message yet
-		}
-	} else
-		realtime = demotime;			// we're warping
-
-	if (realtime - nextdemotime > 0.0001) {
-		if (nextdemotime != demotime) {
-			if (cls.demoplayback2) {
-				cls.netchan.incoming_sequence++;
-				cls.netchan.incoming_acknowledged++;
-				cls.netchan.frame_latency = 0;
-			}
-		}
-	}
-	nextdemotime = demotime;
-
-	cls.prevtime += newtime;
-
-	if (cls.state < ca_demostart)
-		Host_Error ("CL_GetDemoMessage: cls.state != ca_active");
-
-	// get the msg type
-	Qread (cls.demofile, &c, sizeof (c));
-
-	switch (c & 7) {
-		case dem_cmd:
-			// user sent input
-			net_message->message->cursize = -1;
-			i = cls.netchan.outgoing_sequence & UPDATE_MASK;
-			pcmd = &cl.frames[i].cmd;
-			r = Qread (cls.demofile, pcmd, sizeof (*pcmd));
-			if (r != sizeof (*pcmd)) {
-				CL_StopPlayback ();
-				return 0;
-			}
-			// byte order stuff
-			for (j = 0; j < 3; j++)
-				pcmd->angles[j] = LittleFloat (pcmd->angles[j]);
-			pcmd->forwardmove = LittleShort (pcmd->forwardmove);
-			pcmd->sidemove = LittleShort (pcmd->sidemove);
-			pcmd->upmove = LittleShort (pcmd->upmove);
-			cl.frames[i].senttime = demotime;
-			cl.frames[i].receivedtime = -1;	// we haven't gotten a reply yet
-			cls.netchan.outgoing_sequence++;
-			for (i = 0; i < 3; i++) {
-				Qread (cls.demofile, &f, 4);
-				cl.viewangles[i] = LittleFloat (f);
-			}
-			break;
-
-		case dem_read:
-readit:
-			// get the next message
-			Qread (cls.demofile, &net_message->message->cursize, 4);
-			net_message->message->cursize = LittleLong
-				(net_message->message->cursize);
-			if (net_message->message->cursize > MAX_MSGLEN + 8) //+8 for header
-				Host_Error ("Demo message > MAX_MSGLEN + 8: %d/%d",
-							net_message->message->cursize, MAX_MSGLEN + 8);
-			r = Qread (cls.demofile, net_message->message->data,
-					   net_message->message->cursize);
-			if (r != net_message->message->cursize) {
-				CL_StopPlayback ();
-				return 0;
-			}
-
-			if (cls.demoplayback2) {
-				tracknum = Cam_TrackNum ();
-
-				if (cls.lasttype == dem_multiple) {
-					if (tracknum == -1)
-						goto nextdemomessage;
-					if (!(cls.lastto & (1 << tracknum)))
-						goto nextdemomessage;
-				} else if (cls.lasttype == dem_single) {
-					if (tracknum == -1 || cls.lastto != spec_track)
-						goto nextdemomessage;
-				}
-			}
-			break;
-
-		case dem_set:
-			Qread (cls.demofile, &i, 4);
-			cls.netchan.outgoing_sequence = LittleLong (i);
-			Qread (cls.demofile, &i, 4);
-			cls.netchan.incoming_sequence = LittleLong (i);
-			if (cls.demoplayback2) {
-				cls.netchan.incoming_acknowledged =
-					cls.netchan.incoming_sequence;
-				goto nextdemomessage;
-			}
-			break;
-
-		case dem_multiple:
-			r = Qread (cls.demofile, &i, 4);
-			if (r != 4) {
-				CL_StopPlayback ();
-				return 0;
-			}
-			cls.lastto = LittleLong (i);
-			cls.lasttype = dem_multiple;
-			goto readit;
-
-		case dem_single:
-			cls.lastto = c >> 3;
-			cls.lasttype = dem_single;
-			goto readit;
-
-		case dem_stats:
-			cls.lastto = c >> 3;
-			cls.lasttype = dem_stats;
-			goto readit;
-
-		case dem_all:
-			cls.lastto = 0;
-			cls.lasttype = dem_all;
-			goto readit;
-
-		default:
-			Con_Printf ("Corrupted demo.\n");
-			CL_StopPlayback ();
-			return 0;
-	}
-
-	return 1;
-}
-
-/*
-	CL_GetMessage
-
-	Handles recording and playback of demos, on top of NET_ code
-*/
-qboolean
-CL_GetMessage (void)
-{
-	if (cls.demoplayback) {
-		qboolean    ret = CL_GetDemoMessage ();
-
-		if (!ret && demo_timeframes_isactive && cls.td_starttime) {
-			CL_TimeFrames_AddTimestamp ();
-		}
-		return ret;
-	}
-
-	if (!NET_GetPacket ())
-		return false;
-
-	if (net_packetlog->int_val)
-		Log_Incoming_Packet (net_message->message->data,
-							 net_message->message->cursize, 1);
-
-	CL_WriteDemoMessage (net_message->message);
-
-	return true;
-}
-
 /*
 	CL_Stop_f
 
 	stop recording a demo
 */
-void
+static void
 CL_Stop_f (void)
 {
 	if (!cls.demorecording) {
-		Con_Printf ("Not recording a demo.\n");
+		Sys_Printf ("Not recording a demo.\n");
 		return;
 	}
-	// write a disconnect message to the demo file
-	SZ_Clear (net_message->message);
-	MSG_WriteLong (net_message->message, -1);  // -1 sequence means out of band
-	MSG_WriteByte (net_message->message, svc_disconnect);
-	MSG_WriteString (net_message->message, "EndOfDemo");
-	CL_WriteDemoMessage (net_message->message);
-
-	// finish up
-	Qclose (cls.demofile);
-	cls.demofile = NULL;
-	cls.demorecording = false;
-	Con_Printf ("Completed demo\n");
+	CL_StopRecording ();
 }
 
 /*
-	CL_WriteDemoMessage
+	CL_Record_f
+
+	record <demoname>
+*/
+static void
+CL_Record_f (void)
+{
+	if (Cmd_Argc () > 2) {
+		// we use a demo name like year-month-day-hours-minutes-mapname.qwd
+		// if there is no argument
+		Sys_Printf ("record [demoname]\n");
+		return;
+	}
+
+	if (cls.demoplayback || cls.state != ca_active) {
+		Sys_Printf ("You must be connected to record.\n");
+		return;
+	}
+
+	if (cls.demorecording)
+		CL_Stop_f ();
+	if (Cmd_Argc () == 2)
+		CL_Record (Cmd_Argv (1), -1);
+	else
+		CL_Record (0, -1);
+}
+
+/*
+	CL_ReRecord_f
+
+	record <demoname>
+*/
+static void
+CL_ReRecord_f (void)
+{
+	dstring_t  *name;
+	int			c;
+
+	c = Cmd_Argc ();
+	if (c != 2) {
+		Sys_Printf ("rerecord <demoname>\n");
+		return;
+	}
+
+	if (!cls.servername || !cls.servername->str) {
+		Sys_Printf ("No server to which to reconnect...\n");
+		return;
+	}
+
+	if (cls.demorecording)
+		CL_Stop_f ();
+
+	name = dstring_newstr ();
+	dsprintf (name, "%s/%s", qfs_gamedir->dir.def, Cmd_Argv (1));
+
+	// open the demo file
+	QFS_DefaultExtension (name, ".qwd");
+
+	cls.demofile = QFS_WOpen (name->str, 0);
+	if (!cls.demofile) {
+		Sys_Printf ("ERROR: couldn't open.\n");
+	} else {
+		Sys_Printf ("recording to %s.\n", name->str);
+		cls.demorecording = true;
+
+		CL_Disconnect ();
+		CL_BeginServerConnect ();
+	}
+	dstring_delete (name);
+}
+
+/*
+	CL_WriteRecordDemoMessage
 
 	Dumps the current net message, prefixed by the length and view angles
 */
@@ -509,79 +612,51 @@ CL_WriteSetDemoMessage (void)
 	Qflush (cls.demofile);
 }
 
-/*
-	CL_Record_f
-
-	record <demoname> <server>
-*/
-
-void
-CL_Record (const char *argv1)
+static dstring_t *
+demo_default_name (const char *argv1)
 {
-	char        buf_data[MAX_MSGLEN + 10];	// + 10 for header
-	char        name[MAX_OSPATH];
-	char       *s;
+	dstring_t  *name;
+	const char *mapname;
+	int         mapname_len;
 	char        timestring[20];
-	char        mapname[MAX_OSPATH];
+	time_t      tim;
 
-	int         n, i, j, l=0;
-	size_t      k;
+	name = dstring_new ();
+
+	if (argv1) {
+		dsprintf (name, "%s/%s", qfs_gamedir->dir.def, argv1);
+		return name;
+	}
+
+	// Get time to a useable format
+	time (&tim);
+	strftime (timestring, 19, "%Y-%m-%d-%H-%M", localtime (&tim));
+
+	// the leading path-name is to be removed from cl.worldmodel->name
+	mapname = QFS_SkipPath (cl.worldmodel->name);
+
+	// the map name is cut off after any "." because this would prevent
+	// an extension being appended
+	for (mapname_len = 0; mapname[mapname_len]; mapname_len++)
+		if (mapname[mapname_len] == '.')
+			break;
+
+	dsprintf (name, "%s/%s-%.*s", qfs_gamedir->dir.def, timestring,
+			  mapname_len, mapname);
+	return name;
+}
+
+static void
+demo_start_recording (int track)
+{
+	byte        buf_data[MAX_MSGLEN + 10];	// + 10 for header
+	char       *s;
+	int         n, i, j;
 	int         seq = 1;
 	entity_t   *ent;
 	entity_state_t *es, blankes;
 	player_info_t *player;
 	sizebuf_t   buf;
-	time_t      tim;
-
-	if (!argv1) {
-		// Get time to a useable format
-		time (&tim);
-		strftime (timestring, 19, "%Y-%m-%d-%H-%M", localtime (&tim));
-
-		// the leading path-name is to be removed from cl.worldmodel->name
-		for (k = 0; k <= strlen (cl.worldmodel->name); k++) {
-			if (cl.worldmodel->name[k] == '/') {
-				l = 0;
-				mapname[l] = '\0';
-				continue;
-			}
-			mapname[l] = cl.worldmodel->name[k];
-			l++;
-		}
-
-		// the map name is cut off after any "." because this would prevent
-		// ".qwd" from being appended
-
-		for (k = 0; k <= strlen (mapname); k++)
-			if (mapname[k] == '.')
-				mapname[k] = '\0';
-
-		snprintf (name, sizeof (name), "%s/%s-%s", qfs_gamedir->dir.def,
-				  timestring, mapname);
-	} else {
-		snprintf (name, sizeof (name), "%s/%s", qfs_gamedir->dir.def, argv1);
-	}
-
-	// open the demo file
-#ifdef HAVE_ZLIB
-	if (demo_gzip->int_val) {
-		QFS_DefaultExtension (name, ".qwd.gz");
-		cls.demofile = QFS_WOpen (name, demo_gzip->int_val);
-	} else
-#endif
-	{
-		QFS_DefaultExtension (name, ".qwd");
-		cls.demofile = QFS_WOpen (name, 0);
-	}
-	if (!cls.demofile) {
-		Con_Printf ("ERROR: couldn't open.\n");
-		return;
-	}
-
-	Con_Printf ("recording to %s.\n", name);
-	cls.demorecording = true;
-
-/*-------------------------------------------------*/
 
 	// serverdata
 	// send the info about the new client to all connected clients
@@ -679,15 +754,13 @@ CL_Record (const char *argv1)
 		SZ_Clear (&buf);
 	}
 	// spawnstatic
-	for (i = 0; i < cl.num_statics; i++) {
-		ent = cl_static_entities + i;
-
+	for (ent = cl_static_entities; ent; ent = ent->unext) {
 		MSG_WriteByte (&buf, svc_spawnstatic);
 
-		for (j = 1; j < MAX_MODELS; j++)
+		for (j = 1; j < cl.nummodels; j++)
 			if (ent->model == cl.model_precache[j])
 				break;
-		if (j == MAX_MODELS)
+		if (j == cl.nummodels)
 			MSG_WriteByte (&buf, 0);
 		else
 			MSG_WriteByte (&buf, j);
@@ -709,7 +782,7 @@ CL_Record (const char *argv1)
 	// baselines
 	memset (&blankes, 0, sizeof (blankes));
 	for (i = 0; i < MAX_EDICTS; i++) {
-		es = cl_baselines + i;
+		es = qw_entstates.baseline + i;
 
 		if (memcmp (es, &blankes, sizeof (blankes))) {
 			MSG_WriteByte (&buf, svc_spawnbaseline);
@@ -773,7 +846,7 @@ CL_Record (const char *argv1)
 	for (i = 0; i < MAX_LIGHTSTYLES; i++) {
 		MSG_WriteByte (&buf, svc_lightstyle);
 		MSG_WriteByte (&buf, (char) i);
-		MSG_WriteString (&buf, r_lightstyle[i].map);
+		MSG_WriteString (&buf, cl.lightstyle[i].map);
 	}
 
 	for (i = 0; i < MAX_CL_STATS; i++) {
@@ -786,20 +859,6 @@ CL_Record (const char *argv1)
 		}
 	}
 
-#if 0
-	MSG_WriteByte (&buf, svc_updatestatlong);
-	MSG_WriteByte (&buf, STAT_TOTALMONSTERS);
-	MSG_WriteLong (&buf, cl.stats[STAT_TOTALMONSTERS]);
-
-	MSG_WriteByte (&buf, svc_updatestatlong);
-	MSG_WriteByte (&buf, STAT_SECRETS);
-	MSG_WriteLong (&buf, cl.stats[STAT_SECRETS]);
-
-	MSG_WriteByte (&buf, svc_updatestatlong);
-	MSG_WriteByte (&buf, STAT_MONSTERS);
-	MSG_WriteLong (&buf, cl.stats[STAT_MONSTERS]);
-#endif
-
 	// get the client to check and download skins
 	// when that is completed, a begin command will be issued
 	MSG_WriteByte (&buf, svc_stufftext);
@@ -813,99 +872,161 @@ CL_Record (const char *argv1)
 }
 
 void
-CL_Record_f (void)
+CL_Record (const char *argv1, int track)
 {
-	if (Cmd_Argc () > 2) {
-		// we use a demo name like year-month-day-hours-minutes-mapname.qwd
-		// if there is no argument
-		Con_Printf ("record [demoname]\n");
-		return;
-	}
+	dstring_t  *name;
 
-	if (cls.state != ca_active) {
-		Con_Printf ("You must be connected to record.\n");
-		return;
-	}
-
-	if (cls.demorecording)
-		CL_Stop_f ();
-	if (Cmd_Argc () == 2)
-		CL_Record (Cmd_Argv (1));
-	else
-		CL_Record (0);
-}
-
-/*
-	CL_ReRecord_f
-
-	record <demoname>
-*/
-void
-CL_ReRecord_f (void)
-{
-	char		name[MAX_OSPATH];
-	int			c;
-
-	c = Cmd_Argc ();
-	if (c != 2) {
-		Con_Printf ("rerecord <demoname>\n");
-		return;
-	}
-
-	if (!*cls.servername) {
-		Con_Printf ("No server to reconnect to...\n");
-		return;
-	}
-
-	if (cls.demorecording)
-		CL_Stop_f ();
-
-	snprintf (name, sizeof (name), "%s/%s",
-			  qfs_gamedir->dir.def, Cmd_Argv (1));
+	name = demo_default_name (argv1);
 
 	// open the demo file
-	QFS_DefaultExtension (name, ".qwd");
+#ifdef HAVE_ZLIB
+	if (demo_gzip->int_val) {
+		QFS_DefaultExtension (name, ".qwd.gz");
+		cls.demofile = QFS_WOpen (name->str, demo_gzip->int_val);
+	} else
+#endif
+	{
+		QFS_DefaultExtension (name, ".qwd");
+		cls.demofile = QFS_WOpen (name->str, 0);
+	}
 
-	cls.demofile = QFS_WOpen (name, 0);
 	if (!cls.demofile) {
-		Con_Printf ("ERROR: couldn't open.\n");
+		Sys_Printf ("ERROR: couldn't open.\n");
+		dstring_delete (name);
 		return;
 	}
 
-	Con_Printf ("recording to %s.\n", name);
+	Sys_Printf ("recording to %s.\n", name->str);
+	dstring_delete (name);
 	cls.demorecording = true;
 
-	CL_Disconnect ();
-	CL_BeginServerConnect ();
+	demo_start_recording (track);
+}
+
+static inline uint32_t
+get_ulong (const byte *buf)
+{
+	return buf[0] | (buf[1] << 8) | (buf[2] << 16) | (buf[3] << 24);
+}
+
+static inline float
+get_float (const byte *buf)
+{
+	union {
+		uint32_t    u;
+		float       f;
+	} uf;
+	uf.u = get_ulong (buf);
+	return uf.f;
+}
+
+static int
+demo_check_qwd_mvd (void)
+{
+	byte        buf[22];
+	size_t      bytes;
+	int         c, ret = 0;
+	float       f;
+	uint32_t    u;
+
+	bytes = Qread (cls.demofile, buf, sizeof (buf));
+	if (bytes != sizeof (buf))
+		goto done;
+
+	if ((f = get_float (buf + 0)) >= 0 && !isinf (f) && !isnan (f)
+		&& buf[4] == dem_read && get_ulong (buf + 5) <= MAX_DEMMSG
+		&& get_ulong (buf + 9) == 1 && get_ulong (buf + 13) == 1
+		&& buf[17] == svc_serverdata
+		&& (u = get_ulong (buf + 18)) >= 26 && u <= PROTOCOL_VERSION) {
+		ret = 1;
+		goto done;
+	}
+	if ((f = get_float (buf + 0)) >= 0 && !isinf (f) && !isnan (f)
+		&& buf[4] == dem_read && get_ulong (buf + 5) == 5
+		&& get_ulong (buf + 9) == 0xffffffff && buf[13] == S2C_CONNECTION) {
+		//FIXME for now, assuming this is qwd (recorded by qizmo)
+		//the particular sequence seems to be:
+		//  dem_read S2C_CONNECTION
+		//  dem_cmd
+		//  dem_read real start packet (with print, even:P)
+		ret = 1;
+		goto done;
+	}
+	if (buf[0] != 0 || (buf[1] != dem_read && buf[1] != dem_all)
+		|| (u = get_ulong (buf + 2)) > MAX_DEMMSG)
+		goto done;
+	Qseek (cls.demofile, 6, SEEK_SET);
+	net_message->message->cursize = u;
+	bytes = Qread (cls.demofile, net_message->message->data, u);
+	if (bytes != u)
+		goto done;
+	MSG_BeginReading (net_message);
+	while (!ret) {
+		if (net_message->badread)
+			goto done;
+		c = MSG_ReadByte (net_message);
+		switch (c) {
+			case svc_print:
+				MSG_ReadString (net_message);
+				break;
+			case svc_serverdata:
+				if (MSG_ReadLong (net_message) != PROTOCOL_VERSION)
+					goto done;
+				ret = 2;
+				break;
+			default:
+				goto done;
+		}
+	}
+done:
+	Qseek (cls.demofile, 0, SEEK_SET);
+	return ret;
 }
 
 static void
 CL_StartDemo (void)
 {
-	char		name[MAX_OSPATH];
+	dstring_t  *name;
+	int         type;
 
 	// open the demo file
-	strncpy (name, demoname, sizeof (name));
-	QFS_DefaultExtension (name, ".qwd");
+	name = dstring_strdup (demoname);
 
-	Con_Printf ("Playing demo from %s.\n", name);
-	QFS_FOpenFile (name, &cls.demofile);
+	QFS_DefaultExtension (name, ".mvd");
+	QFS_FOpenFile (name->str, &cls.demofile);
 	if (!cls.demofile) {
-		Con_Printf ("ERROR: couldn't open.\n");
+		dstring_copystr (name, demoname);
+		QFS_DefaultExtension (name, ".qwd");
+		QFS_FOpenFile (name->str, &cls.demofile);
+	}
+
+	if (!cls.demofile) {
+		Sys_Printf ("ERROR: couldn't open.\n");
 		cls.demonum = -1;				// stop demo loop
+		dstring_delete (name);
 		return;
 	}
+	if (!(type = demo_check_qwd_mvd ())) {
+		Sys_Printf ("%s is not a valid .qwd or .mvd file.\n", name->str);
+		cls.demonum = -1;				// stop demo loop
+		dstring_delete (name);
+		Qclose (cls.demofile);
+		cls.demofile = 0;
+		return;
+	}
+	Sys_Printf ("Playing demo from %s.\n", name->str);
 
 	cls.demoplayback = true;
+	Key_SetKeyDest (key_demo);
 	net_blocksend = 1;
-	if (strequal (QFS_FileExtension (name), ".mvd")) {
+	if (type == 2) {
 		cls.demoplayback2 = true;
-		Con_Printf ("mvd\n");
+		Sys_MaskPrintf (SYS_DEV, "mvd\n");
 	} else {
-		Con_Printf ("qwd\n");
+		Sys_MaskPrintf (SYS_DEV, "qwd\n");
 	}
 	CL_SetState (ca_demostart);
-	Netchan_Setup (&cls.netchan, net_from, 0, NC_SEND_QPORT);
+	Netchan_Setup (&cls.netchan, net_from, 0, NC_QPORT_SEND);
 	realtime = 0;
 	cls.findtrack = true;
 	cls.lasttype = 0;
@@ -915,6 +1036,8 @@ CL_StartDemo (void)
 	demotime_cached = 0;
 	nextdemotime = 0;
 	CL_ClearPredict ();
+
+	dstring_delete (name);
 }
 
 /*
@@ -922,18 +1045,34 @@ CL_StartDemo (void)
 
 	play [demoname]
 */
-void
+static void
 CL_PlayDemo_f (void)
 {
-	if (Cmd_Argc () != 2) {
-		Con_Printf ("play <demoname> : plays a demo\n");
-		return;
+	switch (Cmd_Argc ()) {
+		case 1:
+			if (!demoname[0])
+				goto playdemo_error;
+			// fall through
+		case 2:
+			cls.demo_capture = 0;
+			break;
+		case 3:
+			if (!strcmp (Cmd_Argv (2), "rec")) {
+				cls.demo_capture = 1;
+				break;
+			}
+			// fall through
+		default:
+playdemo_error:
+			Sys_Printf ("play <demoname> : plays a demo\n");
+			return;
 	}
 	timedemo_runs = timedemo_count = 1;	// make sure looped timedemos stop
 	// disconnect from server
 	CL_Disconnect ();
 
-	strncpy (demoname, Cmd_Argv (1), sizeof (demoname));
+	if (Cmd_Argc () > 1)
+		strncpy (demoname, Cmd_Argv (1), sizeof (demoname));
 	CL_StartDemo ();
 }
 
@@ -967,8 +1106,8 @@ sqr (double x)
 static void
 CL_FinishTimeDemo (void)
 {
-	float		time;
-	int			frames;
+	int         frames;
+	float       time;
 
 	cls.timedemo = false;
 
@@ -977,20 +1116,22 @@ CL_FinishTimeDemo (void)
 	time = Sys_DoubleTime () - cls.td_starttime;
 	if (!time)
 		time = 1;
-	Con_Printf ("%i frame%s %.4g seconds %.4g fps\n", frames,
+	Sys_Printf ("%i frame%s %.4g seconds %.4g fps\n", frames,
 				frames == 1 ? "" : "s", time, frames / time);
 
 	CL_TimeFrames_DumpLog ();
 	demo_timeframes_isactive = 0;
 
 	timedemo_count--;
-	timedemo_data[timedemo_count].frames = time;
-	timedemo_data[timedemo_count].time = time;
-	timedemo_data[timedemo_count].fps = frames / time;
+	if (timedemo_data) {
+		timedemo_data[timedemo_count].frames = frames;
+		timedemo_data[timedemo_count].time = time;
+		timedemo_data[timedemo_count].fps = frames / time;
+	}
 	if (timedemo_count > 0) {
 		CL_StartTimeDemo ();
 	} else {
-		if (--timedemo_runs > 0) {
+		if (--timedemo_runs > 0 && timedemo_data) {
 			double      average = 0;
 			double      variance = 0;
 			double      min, max;
@@ -1006,13 +1147,11 @@ CL_FinishTimeDemo (void)
 			for (i = 0; i < timedemo_runs; i++)
 				variance += sqr (timedemo_data[i].fps - average);
 			variance /= timedemo_runs;
-			Con_Printf ("timedemo stats for %d runs:\n", timedemo_runs);
-			Con_Printf ("  average fps: %.3f\n", average);
-			Con_Printf ("  min/max fps: %.3f/%.3f\n", min, max);
-			Con_Printf ("std deviation: %.3f fps\n", sqrt (variance));
+			Sys_Printf ("timedemo stats for %d runs:\n", timedemo_runs);
+			Sys_Printf ("  average fps: %.3f\n", average);
+			Sys_Printf ("  min/max fps: %.3f/%.3f\n", min, max);
+			Sys_Printf ("std deviation: %.3f fps\n", sqrt (variance));
 		}
-		free (timedemo_data);
-		timedemo_data = 0;
 		if (demo_quit->int_val)
 			Cbuf_InsertText (cl_cbuf, "quit\n");
 	}
@@ -1023,28 +1162,30 @@ CL_FinishTimeDemo (void)
 
 	timedemo [demoname]
 */
-void
+static void
 CL_TimeDemo_f (void)
 {
+	int         count = 1;
+
 	if (Cmd_Argc () < 2 || Cmd_Argc () > 3) {
-		Con_Printf ("timedemo <demoname> [count]: gets demo speeds\n");
+		Sys_Printf ("timedemo <demoname> [count]: gets demo speeds\n");
 		return;
 	}
 	timedemo_runs = timedemo_count = 1;	// make sure looped timedemos stop
 	// disconnect from server
 	CL_Disconnect ();
 
-	if (Cmd_Argc () == 3) {
-		timedemo_count = atoi (Cmd_Argv (2));
-	} else {
-		timedemo_count = 1;
-	}
-	timedemo_runs = timedemo_count = max (timedemo_count, 1);
-	if (timedemo_data)
+	if (Cmd_Argc () == 3)
+		count = atoi (Cmd_Argv (2));
+	if (timedemo_data) {
 		free (timedemo_data);
+		timedemo_data = 0;
+	}
 	timedemo_data = calloc (timedemo_runs, sizeof (td_stats_t));
 	strncpy (demoname, Cmd_Argv (1), sizeof (demoname));
 	CL_StartTimeDemo ();
+	timedemo_runs = timedemo_count = max (count, 1);
+	timedemo_data = calloc (timedemo_runs, sizeof (td_stats_t));
 }
 
 void
@@ -1065,6 +1206,16 @@ CL_Demo_Init (void)
 						  "automaticly quit after a timedemo has finished");
 	demo_timeframes = Cvar_Get ("demo_timeframes", "0", CVAR_NONE, NULL,
 								"write timestamps for every frame");
+	Cmd_AddCommand ("record", CL_Record_f, "Record a demo, if no filename "
+					"argument is given\n"
+					"the demo will be called Year-Month-Day-Hour-Minute-"
+					"Mapname");
+	Cmd_AddCommand ("rerecord", CL_ReRecord_f, "Rerecord a demo on the same "
+					"server");
+	Cmd_AddCommand ("stop", CL_Stop_f, "Stop recording a demo");
+	Cmd_AddCommand ("playdemo", CL_PlayDemo_f, "Play a recorded demo");
+	Cmd_AddCommand ("timedemo", CL_TimeDemo_f, "Play a demo as fast as your "
+					"hardware can. Useful for benchmarking.");
 }
 
 static void
@@ -1093,17 +1244,17 @@ static void
 CL_TimeFrames_DumpLog (void)
 {
 	const char *filename = "timeframes.txt";
-	int			i;
-	long		frame;
-	QFile	   *outputfile;
+	int         i;
+	long        frame;
+	QFile      *outputfile;
 
 	if (demo_timeframes_isactive == 0)
 		return;
 
-	Con_Printf ("Dumping Timed Frames log: %s\n", filename);
+	Sys_Printf ("Dumping Timed Frames log: %s\n", filename);
 	outputfile = QFS_Open (filename, "w");
 	if (!outputfile) {
-		Con_Printf ("Could not open: %s\n", filename);
+		Sys_Printf ("Could not open: %s\n", filename);
 		return;
 	}
 	for (i = 1; i < demo_timeframes_index; i++) {
