@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sv_main.c,v 1.280 2004/04/27 21:58:21 snax Exp $";
-
 #ifdef HAVE_WINDOWS_H
 # include "winquake.h"
 #endif
@@ -71,7 +68,6 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/dstring.h"
 #include "QF/model.h"
 #include "QF/msg.h"
-#include "QF/plugin.h"
 #include "QF/qargs.h"
 #include "QF/quakefs.h"
 #include "QF/sys.h"
@@ -79,18 +75,21 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/ver_check.h"
 #include "QF/zone.h"
 
-#include "bothdefs.h"
+#include "QF/plugin/console.h"
+
+#include "qw/bothdefs.h"
 #include "buildnum.h"
 #include "compat.h"
 #include "crudefile.h"
 #include "game.h"
 #include "netchan.h"
-#include "pmove.h"
+#include "qw/pmove.h"
 #include "server.h"
 #include "sv_demo.h"
 #include "sv_progs.h"
 #include "sv_gib.h"
 #include "sv_qtv.h"
+#include "sv_recorder.h"
 
 SERVER_PLUGIN_PROTOS
 static plugin_list_t server_plugin_list[] = {
@@ -122,9 +121,6 @@ qboolean    rcon_from_user;
 double      netdosexpire[DOSFLOODCMDS] = { 1, 1, 2, 0.9, 1, 5 };
 double      netdosvalues[DOSFLOODCMDS] = { 12, 1, 3, 1, 1, 1 };
 
-cvar_t     *fs_globalcfg;
-cvar_t     *fs_usercfg;
-
 cvar_t     *sv_mem_size;
 
 cvar_t     *sv_console_plugin;
@@ -140,7 +136,7 @@ cvar_t     *sv_maxtic;					// physics time tic
 
 cvar_t     *sv_netdosprotect;			// tone down DoS from quake servers
 
-cvar_t     *timeout;					// seconds without any message
+cvar_t     *sv_timeout;					// seconds without any message
 cvar_t     *zombietime;					// seconds to sink messages after
 										// disconnect
 
@@ -197,6 +193,7 @@ int         sv_net_initialized;
 
 const char *client_info_filters[] = {  // Info keys needed by client
 	"name",
+	"chat",
 	"topcolor",
 	"bottomcolor",
 	"rate",
@@ -247,7 +244,7 @@ SV_Shutdown (void)
 		Qclose (sv_fraglogfile);
 		sv_fraglogfile = NULL;
 	}
-	if (sv.demorecording)
+	if (sv.recording_demo)
 		SV_Stop (0);
 
 	NET_Shutdown ();
@@ -264,28 +261,23 @@ static void
 SV_Error (const char *error, va_list argptr)
 {
 	static qboolean inerror = false;
+	dstring_t  *string;
 
 	if (inerror)
 		return;
 
 	inerror = true;
+	string = dstring_new ();
+	dvsprintf (string, error, argptr);
+
+	Sys_Printf ("%s\n", string->str);
 
 	if (sv_net_initialized) {
-		dstring_t  *string = dstring_new ();
-
-		dvsprintf (string, error, argptr);
 		dstring_insertstr (string, 0, "server crashed: ");
 		dstring_appendstr (string, "\n");
 		SV_FinalMessage (string->str);
-		dstring_delete (string);
-	}
-
-	if (con_module) {
-		con_module->functions->console->pC_Print (error, argptr);
-		con_module->functions->console->pC_Print ("\n", argptr);
-	} else {
-		Sys_Print (stderr, error, argptr);
 	}
+	dstring_delete (string);
 }
 
 /*
@@ -312,6 +304,7 @@ SV_FinalMessage (const char *message)
 		if (cl->state >= cs_connected)
 			Netchan_Transmit (&cl->netchan, net_message->message->cursize,
 							  net_message->message->data);
+	SV_qtvFinalMessage (message);
 }
 
 /*
@@ -373,7 +366,7 @@ SV_DropClient (client_t *drop)
 
 	// send notification to all remaining clients
 	SV_FullClientUpdate (drop, &sv.reliable_datagram);
-	
+
 	// Trigger GIB event
 	if (sv_client_disconnect_e->func)
 		GIB_Event_Callback (sv_client_disconnect_e, 1,
@@ -391,7 +384,7 @@ SV_CalcPing (client_t *cl)
 		return cl->ping;
 	ping = 0;
 	count = 0;
-	for (frame = cl->frames, i = 0; i < UPDATE_BACKUP; i++, frame++) {
+	for (frame = cl->delta.frames, i = 0; i < UPDATE_BACKUP; i++, frame++) {
 		if (frame->ping_time > 0) {
 			ping += frame->ping_time;
 			count++;
@@ -456,16 +449,16 @@ SV_FullClientUpdate (client_t *client, sizebuf_t *buf)
 	Writes all update values to a client's reliable stream
 */
 void
-SV_FullClientUpdateToClient (client_t *client, client_t *cl)
+SV_FullClientUpdateToClient (client_t *client, backbuf_t *backbuf)
 {
 	if (client->state < cs_connected && client->state != cs_server)
 		return;
-	MSG_ReliableCheckBlock (&cl->backbuf, 24 + client->userinfo->cursize);
-	if (cl->backbuf.num_backbuf) {
-		SV_FullClientUpdate (client, &cl->backbuf.backbuf);
-		MSG_Reliable_FinishWrite (&cl->backbuf);
+	MSG_ReliableCheckBlock (backbuf, 24 + Info_CurrentSize (client->userinfo));
+	if (backbuf->num_backbuf) {
+		SV_FullClientUpdate (client, &backbuf->backbuf);
+		MSG_Reliable_FinishWrite (backbuf);
 	} else
-		SV_FullClientUpdate (client, &cl->netchan.message);
+		SV_FullClientUpdate (client, &backbuf->netchan->message);
 }
 
 /* CONNECTIONLESS COMMANDS */
@@ -495,7 +488,7 @@ CheckForFlood (flood_enum_t cmdtype)
 	oldest = 0;
 
 	if (firsttime) {
-		memset (floodstatus, sizeof (flood_t) * DOSFLOODCMDS * DOSFLOODIP, 0);
+		memset (floodstatus, 0, sizeof (flood_t) * DOSFLOODCMDS * DOSFLOODIP);
 
 		firsttime = false;
 	}
@@ -582,7 +575,7 @@ SVC_Status (void)
 }
 
 /* SV_CheckLog */
-#define	LOG_HIGHWATER	4096
+#define	LOG_HIGHWATER	(MAX_DATAGRAM - 100)
 #define	LOG_FLUSH		10*60
 
 static void
@@ -636,8 +629,8 @@ SVC_Log (void)
 		return;
 	}
 
-	Con_DPrintf ("sending log %i to %s\n", svs.logsequence - 1,
-				 NET_AdrToString (net_from));
+	Sys_MaskPrintf (SYS_DEV, "sending log %i to %s\n", svs.logsequence - 1,
+					NET_AdrToString (net_from));
 
 //	snprintf (data, sizeof (data), "stdlog %i\n", svs.logsequence - 1);
 //	strncat (data,  (char *) svs.log_buf[((svs.logsequence - 1) & 1)],
@@ -706,7 +699,7 @@ SVC_GetChallenge (void)
 	}
 
 	if (sv_extensions->int_val) {
-		extended = " QF qtv";
+		extended = " QF qtv EXT";
 	}
 
 	// send it to the client
@@ -749,6 +742,8 @@ SV_AllocClient (int spectator, int server)
 			svs.num_clients++;
 			memset (cl, 0, sizeof (client_t));
 			cl->userid = userid++;	// so every client gets a unique id
+			if (userid < 0)			// reserve -ve userids for special purposes
+				userid = 0;
 			cl->edict = EDICT_NUM (&sv_pr_state, (cl - svs.clients) + 1);
 			return cl;
 		}
@@ -768,9 +763,11 @@ SVC_DirectConnect (void)
 	info_t     *userinfo = 0;
 	const char *s;
 	client_t   *cl, *newcl;
-	int         challenge, qport, version, i, qtv = 0;
+	int         challenge, version, i, qtv = 0;
+	int         qport;
 	netadr_t    adr;
 	qboolean    spectator;
+	client_frame_t *frames;
 
 	if (CheckForFlood (FLOOD_CONNECT))
 		return;
@@ -898,15 +895,17 @@ SVC_DirectConnect (void)
 	// build a new connection
 	// accept the new client
 	// this is the only place a client_t is ever initialized
+	frames = newcl->delta.frames;
 	for (i = 0; i < UPDATE_BACKUP; i++) {
-		newcl->frames[i].entities.entities = cl_entities[newcl-svs.clients][i];
+		frames[i].entities.entities = cl_entities[newcl-svs.clients][i];
 		memset (cl_entities[newcl-svs.clients][i], 0,
 				sizeof (cl_entities[newcl-svs.clients][i]));
 	}
+	newcl->delta.client = newcl;
 
 	Netchan_OutOfBandPrint (adr, "%c", S2C_CONNECTION);
 
-	Netchan_Setup (&newcl->netchan, adr, qport, NC_READ_QPORT);
+	Netchan_Setup (&newcl->netchan, adr, qport, NC_QPORT_READ);
 	newcl->backbuf.netchan = &newcl->netchan;
 	newcl->backbuf.name = newcl->name;
 
@@ -918,7 +917,7 @@ SVC_DirectConnect (void)
 	newcl->datagram.data = newcl->datagram_buf;
 	newcl->datagram.maxsize = sizeof (newcl->datagram_buf);
 
-	// spectator mode can ONLY be set at join time
+	// spectator mode can be set ONLY at join time
 	newcl->spectator = spectator;
 
 	// parse some info from the info strings
@@ -1027,7 +1026,7 @@ SVC_RemoteCommand (void)
 					   do_cmd ? "issued" : "attempted");
 		}
 	}
-	
+
 	if (do_cmd) {
 		command = Cmd_Args (2);
 
@@ -1116,7 +1115,7 @@ SV_ConnectionlessPacket (void)
 	will match any value, so you can specify an entire class C network with
 	"addip 192.246.40".
 
-	Removeip will only remove an address specified exactly the same way.
+	Removeip will remove only an address specified exactly the same way.
 	You cannot addip a subnet, then removeip a single host.
 
 	listip
@@ -1133,7 +1132,7 @@ SV_ConnectionlessPacket (void)
 	prohibited from entering the game.  This is the default setting.
 
 	If 0, then only addresses matching the list will be allowed.  This lets
-	you easily set up a private game, or a game that only allows players
+	you easily set up a private game, or a game that allows only players
 	from your local network.
 */
 
@@ -1160,7 +1159,7 @@ static void
 SV_GenerateIPMasks (void)
 {
 	int i;
-	unsigned long int j = 0xFFFFFFFF;
+	unsigned int j = 0xFFFFFFFF;
 
 	for (i = 32; i >= 0; i--) {
 		ipmasks[i] = htonl (j);
@@ -1218,7 +1217,10 @@ SV_MaskIPCompare (byte *a, byte *b, int mask)
 #endif
 
 	for (i = 0; i < intcount; i++) {
-		if ((((unsigned int *)a)[i] & ipmasks[mask > 32 ? 32 : mask]) != ((unsigned int *)b)[i])
+		unsigned int x, y;
+		memcpy (&x, a + i * 4, 4);
+		memcpy (&y, b + i * 4, 4);
+		if ((x & ipmasks[mask > 32 ? 32 : mask]) != y)
 			return false;
 		if ((mask -= 32) < 0)
 			mask = 0;
@@ -1262,9 +1264,9 @@ static qboolean
 SV_StringToFilter (const char *address, ipfilter_t *f)
 {
 #ifdef HAVE_IPV6
-	byte		b[16] = {};
+	byte		b[16] = {0};
 #else
-	byte        b[4] = {};
+	byte        b[4] = {0};
 #endif
 	int			mask = 0;
 	unsigned int i;
@@ -1317,8 +1319,8 @@ SV_StringToFilter (const char *address, ipfilter_t *f)
 		if (mask == -1) {
 			if (sv_filter_automask->int_val) {
 				mask = sizeof (b) * 8;
-				i = sizeof (b) - 1;
-				while (i >= 0 && !b[i]) {
+				i = sizeof (b);
+				while (i > 0 && !b[i - 1]) {
 					mask -= 8;
 					i--;
 				}
@@ -1749,15 +1751,14 @@ SV_ReadPackets (void)
 {
 	//NOTE star volatile, not volatile star
 	client_t   *volatile cl;			// * volatile for longjmp
-	int         qport, i;
-	qboolean    good;
+	int         i;
+	int         qport;
 	double      until;
 
-	good = false;
 	while (NET_GetPacket ()) {
 		if (net_packetlog->int_val)
 			Log_Incoming_Packet (net_message->message->data,
-								 net_message->message->cursize, 1);
+								 net_message->message->cursize, 1, 1);
 		if (SV_FilterIP (net_from.ip, &until)) {
 			SV_SendBan (until);			// tell them we aren't listening...
 			continue;
@@ -1769,7 +1770,8 @@ SV_ReadPackets (void)
 		}
 
 		if (net_message->message->cursize < 11) {
-			SV_Printf ("%s: Runt packet\n", NET_AdrToString (net_from));
+			SV_Printf ("%s: Runt packet: %d\n", NET_AdrToString (net_from),
+					   net_message->message->cursize);
 			continue;
 		}
 		// read the qport out of the message so we can fix up
@@ -1788,13 +1790,13 @@ SV_ReadPackets (void)
 			if (cl->netchan.qport != qport)
 				continue;
 			if (cl->netchan.remote_address.port != net_from.port) {
-				Con_DPrintf ("SV_ReadPackets: fixing up a translated port\n");
+				Sys_MaskPrintf (SYS_DEV, "SV_ReadPackets: fixing up a "
+								"translated port\n");
 				cl->netchan.remote_address.port = net_from.port;
 			}
 			if (Netchan_Process (&cl->netchan)) {
 				// this is a valid, sequenced packet, so process it
 				svs.stats.packets++;
-				good = true;
 				cl->send_message = true;	// reply at end of frame
 				if (cl->state != cs_zombie) {
 					SV_ExecuteClientMessage (cl);
@@ -1832,7 +1834,7 @@ SV_CheckTimeouts (void)
 	float       droptime;
 	int         nclients, i;
 
-	droptime = realtime - timeout->value;
+	droptime = realtime - sv_timeout->value;
 	nclients = 0;
 
 	for (i = 0, cl = svs.clients; i < MAX_CLIENTS; i++, cl++) {
@@ -1859,17 +1861,6 @@ SV_CheckTimeouts (void)
 	}
 }
 
-/*
-  SV_GetConsoleCommands
-
-  Add them exactly as if they had been typed at the console
-*/
-static void
-SV_GetConsoleCommands (void)
-{
-	Con_ProcessInput ();
-}
-
 /*
 	SV_CheckVars
 */
@@ -1903,7 +1894,7 @@ SV_CheckVars (void)
 	SV_GarbageCollect
 
 	Run string GC on progs every pr_gc_interval frames
-	
+
 	//snax: run QFobject GC as well
 */
 static void
@@ -1962,11 +1953,10 @@ SV_Frame (float time)
 	// get packets
 	SV_ReadPackets ();
 
-	// check for commands typed to the host
-	SV_GetConsoleCommands ();
+	// check for typing at the console
+	Con_ProcessInput ();
 
 	// process gib threads
-
 	GIB_Thread_Execute ();
 
 	// process console commands
@@ -1978,10 +1968,13 @@ SV_Frame (float time)
 	SV_SendClientMessages ();
 
 	demo_start = Sys_DoubleTime ();
-	SV_SendDemoMessage ();
+	if (sv.recorders)
+		SVR_SendMessages ();
 	demo_end = Sys_DoubleTime ();
 	svs.stats.demo += demo_end - demo_start;
 
+	SV_qtvSendMessages ();
+
 	// send a heartbeat to the master if needed
 	Master_Heartbeat ();
 
@@ -2000,8 +1993,8 @@ SV_Frame (float time)
 		svs.stats.packets = 0;
 		svs.stats.count = 0;
 		svs.stats.demo = 0;
+		Con_DrawConsole ();
 	}
-	Con_ProcessInput ();	//XXX evil hack to get the cursor in the right place
 }
 
 static void
@@ -2035,6 +2028,18 @@ maxclients_f (cvar_t *var)
 	}
 }
 
+static void
+gamedir_f (int phase)
+{
+	if (!phase)
+		return;
+	if (qfs_gamedir->gamedir)
+		Info_SetValueForStarKey (svs.info, "*gamedir",
+								 qfs_gamedir->gamedir, 0);
+	else
+		Info_RemoveKey (svs.info, "*gamedir");
+}
+
 static void
 SV_InitLocal (void)
 {
@@ -2089,14 +2094,15 @@ SV_InitLocal (void)
 							  "equal-to or less-then the maxclients value");
 	hostname = Cvar_Get ("hostname", "unnamed", CVAR_SERVERINFO, Cvar_Info,
 						 "Report or sets the server name");
-	deathmatch = Cvar_Get ("deathmatch", "1", CVAR_SERVERINFO, Cvar_Info, 
+	deathmatch = Cvar_Get ("deathmatch", "1", CVAR_SERVERINFO, Cvar_Info,
 						   "Sets the rules for weapon and item respawning. "
 						   "1 Does not leave weapons on the map. You can "
 						   "pickup weapons and items and they will respawn, "
-						   "2 Leaves weapons on the map. You can only pick up "
-						   "a weapon once. Picked up items will not respawn, "
-						   "3 Leaves weapons on the map. You can only pick up "
-						   "a weapon once. Picked up items will respawn.");
+						   "2 Leaves weapons on the map. You can pick up a "
+						   "weapon only once. Picked up items will not "
+						   "respawn, 3 Leaves weapons on the map. You can "
+						   "pick up a weapon only once. Picked up items will "
+						   "respawn.");
 	coop = Cvar_Get ("coop", "0", CVAR_NONE, NULL, "co-op mode for progs that "
 					 "support it");
 	skill = Cvar_Get ("skill", "0", CVAR_NONE, NULL, "skill setting for progs "
@@ -2105,8 +2111,8 @@ SV_InitLocal (void)
 					  "Spawn the player entity");
 	watervis = Cvar_Get ("watervis", "0", CVAR_SERVERINFO, Cvar_Info,
 						 "Set nonzero to enable r_wateralpha on clients");
-	timeout = Cvar_Get ("timeout", "65", CVAR_NONE, NULL, "Sets the amount of "
-						"time in seconds before a client is considered "
+	sv_timeout = Cvar_Get ("timeout", "65", CVAR_NONE, NULL, "Sets the amount "
+						"of time in seconds before a client is considered "
 						"disconnected if the server does not receive a "
 						"packet");
 	zombietime = Cvar_Get ("zombietime", "2", CVAR_NONE, NULL, "The number of "
@@ -2121,7 +2127,8 @@ SV_InitLocal (void)
 							  "clients");
 	sv_gravity = Cvar_Get ("sv_gravity", "800", CVAR_NONE, NULL,
 						   "Sets the global value for the amount of gravity");
-	sv_stopspeed = Cvar_Get ("sv_stopspeed", "100", CVAR_NONE, NULL, 
+	sv_jump_any = Cvar_Get ("sv_jump_any", "1", CVAR_NONE, NULL, "None");
+	sv_stopspeed = Cvar_Get ("sv_stopspeed", "100", CVAR_NONE, NULL,
 							 "Sets the value that determines how fast the "
 							 "player should come to a complete stop");
 	sv_maxspeed = Cvar_Get ("sv_maxspeed", "320", CVAR_NONE, NULL,
@@ -2158,7 +2165,7 @@ SV_InitLocal (void)
 							  "Time/date stamps in log entries");
 	sv_timefmt = Cvar_Get ("sv_timefmt", "[%b %e %X] ", CVAR_NONE, NULL,
 						   "Time/date format to use");
-	filterban = Cvar_Get ("filterban", "1", CVAR_NONE, NULL, 
+	filterban = Cvar_Get ("filterban", "1", CVAR_NONE, NULL,
 						  "Determines the rules for the IP list "
 						  "0 Only IP addresses on the Ban list will be "
 						  "allowed onto the server, 1 Only IP addresses NOT "
@@ -2204,7 +2211,7 @@ SV_InitLocal (void)
 								 "Handling of double entity remove.  "
 								 "0 is silently ignore, 1 prints a "
 								 "traceback, and 2 gives an error.\n"
-								 "Only works if debugging is available "
+								 "works Only if debugging is available "
 								 "and enabled");
 	// DoS protection
 	Cmd_AddCommand ("netdosexpire", SV_netDoSexpire_f, "FIXME: part of DoS "
@@ -2230,7 +2237,7 @@ SV_InitLocal (void)
 							 !sv_highchars->int_val);
 
 	// Brand server as QF, with appropriate QSG standards version  --KB
-	Info_SetValueForStarKey (svs.info, "*qf_version", VERSION,
+	Info_SetValueForStarKey (svs.info, "*qf_version", PACKAGE_VERSION,
 							 !sv_highchars->int_val);
 	Info_SetValueForStarKey (svs.info, "*qsg_version", QW_QSG_VERSION,
 							 !sv_highchars->int_val);
@@ -2326,7 +2333,7 @@ SV_ExtractFromUserinfo (client_t *cl)
 	}
 	*r = '\0';
 
-	// empty (or whitespace only) name
+	// empty (or only whitespace) name
 	if (!*newname)
 		badname = true;
 
@@ -2389,7 +2396,7 @@ SV_ExtractFromUserinfo (client_t *cl)
 		}
 
 		// finally, report it to all our friends
-		if (*cl->name)
+		if (!cl->spectator && *cl->name)
 			SV_BroadcastPrintf (PRINT_HIGH, "%s changed name to %s\n",
 								cl->name, newname);
 		strcpy (cl->name, newname);
@@ -2450,7 +2457,7 @@ SV_Init_Memory (void)
 
 	sv_mem_size = Cvar_Get ("sv_mem_size", "8", CVAR_NONE, NULL,
 							"Amount of memory (in MB) to allocate for the "
-							PROGRAM " heap");
+							PACKAGE_NAME " heap");
 	if (mem_parm)
 		Cvar_Set (sv_mem_size, com_argv[mem_parm + 1]);
 
@@ -2481,55 +2488,24 @@ SV_Init (void)
 
 	Sys_RegisterShutdown (SV_Shutdown);
 
-	Cvar_Init_Hash ();
-	Cmd_Init_Hash ();
-	Cvar_Init ();
-	Sys_Init_Cvars ();
-
 	Sys_Init ();
+	GIB_Init (true);
+	COM_ParseConfig ();
 
 	Cvar_Get ("cmd_warncmd", "1", CVAR_NONE, NULL, NULL);
 
-	Cmd_Init ();
-	GIB_Init (true);
-
-	// execute +set as early as possible
-	Cmd_StuffCmds (sv_cbuf);
-	Cbuf_Execute_Sets (sv_cbuf);
-
 	// snax: Init experimental object system and run a test
 	//Object_Init();
 
-	// execute the global configuration file if it exists
-	// would have been nice if Cmd_Exec_f could have been used, but it
-	// only reads from within the quake file system, and changing that is
-	// probably Not A Good Thing (tm).
-	fs_globalcfg = Cvar_Get ("fs_globalcfg", FS_GLOBALCFG,
-							 CVAR_ROM, 0, "global configuration file");
-	Cmd_Exec_File (sv_cbuf, fs_globalcfg->string, 0);
-	Cbuf_Execute_Sets (sv_cbuf);
-
-	// execute +set again to override the config file
-	Cmd_StuffCmds (sv_cbuf);
-	Cbuf_Execute_Sets (sv_cbuf);
-
-	fs_usercfg = Cvar_Get ("fs_usercfg", FS_USERCFG,
-						   CVAR_ROM, 0, "user configuration file");
-	Cmd_Exec_File (sv_cbuf, fs_usercfg->string, 0);
-	Cbuf_Execute_Sets (sv_cbuf);
-
-	// execute +set again to override the config file
-	Cmd_StuffCmds (sv_cbuf);
-	Cbuf_Execute_Sets (sv_cbuf);
-
 	SV_Init_Memory ();
 
+	QFS_GamedirCallback (gamedir_f);
 	svs.maxclients = MAX_CLIENTS;
 	svs.info = Info_ParseString ("", MAX_SERVERINFO_STRING, 0);
 	localinfo = Info_ParseString ("", 0, 0);	// unlimited
 	SV_InitOperatorCommands ();
 	SV_GIB_Init ();
-	
+
 	QFS_Init ("qw");
 	PI_Init ();
 
@@ -2544,7 +2520,6 @@ SV_Init (void)
 	Sys_SetErrPrintf (SV_Error);
 
 	Game_Init_Cvars ();
-	COM_Init_Cvars ();
 	Mod_Init_Cvars ();
 	Netchan_Init_Cvars ();
 	Pmove_Init_Cvars ();
@@ -2555,8 +2530,9 @@ SV_Init (void)
 	Cmd_StuffCmds (sv_cbuf);
 	Cbuf_Execute_Sets (sv_cbuf);
 
+	SV_Sbar_Init ();
+
 	Game_Init ();
-	COM_Init ();
 
 	PR_Init ();
 	SV_Progs_Init ();
@@ -2567,6 +2543,7 @@ SV_Init (void)
 	SV_InitLocal ();
 	Pmove_Init ();
 
+	SVR_Init ();
 	Demo_Init ();
 
 	Hunk_AllocName (0, "-HOST_HUNKLEVEL-");
@@ -2580,11 +2557,12 @@ SV_Init (void)
 	SV_Printf ("%4.1f megabyte heap\n", sv_mem_size->value);
 
 	SV_Printf ("\n");
-	SV_Printf ("%s server, Version %s (build %04d)\n", PROGRAM, VERSION,
-				build_number ());
+	SV_Printf ("%s server, Version %s (build %04d)\n",
+			   PACKAGE_NAME, PACKAGE_VERSION,
+			   build_number ());
 	SV_Printf ("\n");
 
-	SV_Printf ("<==> %s initialized <==>\n", PROGRAM);
+	SV_Printf ("<==> %s initialized <==>\n", PACKAGE_NAME);
 
 	// process command line arguments
 	Cmd_Exec_File (sv_cbuf, fs_usercfg->string, 0);
