@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sv_pr_cmds.c,v 1.137 2004/04/06 22:02:45 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -51,10 +48,9 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "compat.h"
 #include "crudefile.h"
 #include "server.h"
-#include "sv_demo.h"
 #include "sv_gib.h"
-#include "sv_pr_cmds.h"
 #include "sv_progs.h"
+#include "sv_recorder.h"
 #include "world.h"
 
 /* BUILT-IN FUNCTIONS */
@@ -69,15 +65,15 @@ static __attribute__ ((unused)) const char rcsid[] =
 	error (value)
 	// void (string e) error
 */
-void
+static void
 PF_error (progs_t *pr)
 {
 	const char *s;
 	edict_t    *ed;
 
 	s = PF_VarString (pr, 0);
-	SV_Printf ("======SERVER ERROR in %s:\n%s\n",
-				PR_GetString (pr, pr->pr_xfunction->s_name), s);
+	Sys_Printf ("======SERVER ERROR in %s:\n%s\n",
+				PR_GetString (pr, pr->pr_xfunction->descriptor->s_name), s);
 	ed = PROG_TO_EDICT (pr, *sv_globals.self);
 	ED_Print (pr, ed);
 
@@ -93,30 +89,29 @@ PF_error (progs_t *pr)
 	objerror (value)
 	// void (string e) objerror
 */
-void
+static void
 PF_objerror (progs_t *pr)
 {
 	const char *s;
 	edict_t    *ed;
 
 	s = PF_VarString (pr, 0);
-	SV_Printf ("======OBJECT ERROR in %s:\n%s\n",
-				PR_GetString (pr, pr->pr_xfunction->s_name), s);
+	Sys_Printf ("======OBJECT ERROR in %s:\n%s\n",
+				PR_GetString (pr, pr->pr_xfunction->descriptor->s_name), s);
 	ed = PROG_TO_EDICT (pr, *sv_globals.self);
 	ED_Print (pr, ed);
 	ED_Free (pr, ed);
 
-	Sys_Error ("Program error");
+	PR_RunError (pr, "object error");
 }
 
 /*
 	PF_makevectors
 
 	Writes new values for v_forward, v_up, and v_right based on angles
-	makevectors (vector)
-	// void (entity e) makevectors
+	void (vector angles) makevectors
 */
-void
+static void
 PF_makevectors (progs_t *pr)
 {
 	AngleVectors (P_VECTOR (pr, 0), *sv_globals.v_forward,
@@ -135,7 +130,7 @@ PF_makevectors (progs_t *pr)
 	setorigin (entity, origin)
 	// void (entity e, vector o) setorigin
 */
-void
+static void
 PF_setorigin (progs_t *pr)
 {
 	edict_t    *e;
@@ -155,7 +150,7 @@ PF_setorigin (progs_t *pr)
 	setsize (entity, minvector, maxvector)
 	// void (entity e, vector min, vector max) setsize
 */
-void
+static void
 PF_setsize (progs_t *pr)
 {
 	edict_t    *e;
@@ -173,11 +168,11 @@ PF_setsize (progs_t *pr)
 /*
 	PF_setmodel
 
-	setmodel(entity, model)
+	setmodel (entity, model)
 	// void (entity e, string m) setmodel
 	Also sets size, mins, and maxs for inline bmodels
 */
-void
+static void
 PF_setmodel (progs_t *pr)
 {
 	edict_t    *e;
@@ -217,7 +212,7 @@ PF_setmodel (progs_t *pr)
 	bprint (value)
 	// void (string s) bprint
 */
-void
+static void
 PF_bprint (progs_t *pr)
 {
 	const char *s;
@@ -237,7 +232,7 @@ PF_bprint (progs_t *pr)
 	sprint (clientent, value)
 	// void (entity client, string s) sprint
 */
-void
+static void
 PF_sprint (progs_t *pr)
 {
 	const char *s;
@@ -248,7 +243,7 @@ PF_sprint (progs_t *pr)
 	level = P_FLOAT (pr, 1);
 
 	if (entnum < 1 || entnum > MAX_CLIENTS) {
-		SV_Printf ("tried to sprint to a non-client\n");
+		Sys_Printf ("tried to sprint to a non-client\n");
 		return;
 	}
 
@@ -264,13 +259,13 @@ PF_sprint (progs_t *pr)
 
 /*
 	PF_centerprint
-	// void (...) centerprint
 
 	single print to a specific client
 
 	centerprint (clientent, value)
+	// void (...) centerprint
 */
-void
+static void
 PF_centerprint (progs_t *pr)
 {
 	const char *s;
@@ -280,7 +275,7 @@ PF_centerprint (progs_t *pr)
 	entnum = P_EDICTNUM (pr, 0);
 
 	if (entnum < 1 || entnum > MAX_CLIENTS) {
-		SV_Printf ("tried to sprint to a non-client\n");
+		Sys_Printf ("tried to sprint to a non-client\n");
 		return;
 	}
 
@@ -294,10 +289,11 @@ PF_centerprint (progs_t *pr)
 	MSG_ReliableWrite_Begin (&cl->backbuf, svc_centerprint, 2 + strlen (s));
 	MSG_ReliableWrite_String (&cl->backbuf, s);
 
-	if (sv.demorecording) {
-		DemoWrite_Begin (dem_single, entnum - 1, 2 + strlen (s));
-		MSG_WriteByte (&demo.dbuf->sz, svc_centerprint);
-		MSG_WriteString (&demo.dbuf->sz, s);
+	if (sv.recorders) {
+		sizebuf_t  *dbuf;
+		dbuf = SVR_WriteBegin (dem_single, entnum - 1, 2 + strlen (s));
+		MSG_WriteByte (dbuf, svc_centerprint);
+		MSG_WriteString (dbuf, s);
 	}
 }
 
@@ -305,7 +301,7 @@ PF_centerprint (progs_t *pr)
 	PF_ambientsound
 	// void (vector pos, string samp, float vol, float atten) ambientsound
 */
-void
+static void
 PF_ambientsound (progs_t *pr)
 {
 	const char **check;
@@ -325,7 +321,7 @@ PF_ambientsound (progs_t *pr)
 			break;
 
 	if (!*check) {
-		SV_Printf ("no precache: %s\n", samp);
+		Sys_Printf ("no precache: %s\n", samp);
 		return;
 	}
 
@@ -340,7 +336,6 @@ PF_ambientsound (progs_t *pr)
 
 /*
 	PF_sound
-	// void (entity e, float chan, string samp) sound
 
 	Each entity can have eight independant sound sources, like voice,
 	weapon, feet, etc.
@@ -350,8 +345,9 @@ PF_ambientsound (progs_t *pr)
 
 	An attenuation of 0 will play full volume everywhere in the level.
 	Larger attenuations will drop off.
+	// void (entity e, float chan, string samp) sound
 */
-void
+static void
 PF_sound (progs_t *pr)
 {
 	const char *sample;
@@ -378,7 +374,7 @@ PF_sound (progs_t *pr)
 	traceline (vector1, vector2, tryents)
 	// float (vector v1, vector v2, float tryents) traceline
 */
-void
+static void
 PF_traceline (progs_t *pr)
 {
 	float      *v1, *v2;
@@ -391,6 +387,9 @@ PF_traceline (progs_t *pr)
 	nomonsters = P_FLOAT (pr, 2);
 	ent = P_EDICT (pr, 3);
 
+	if (sv_antilag->int_val == 2)
+		nomonsters |= MOVE_LAGGED;
+
 	trace = SV_Move (v1, vec3_origin, vec3_origin, v2, nomonsters, ent);
 
 	*sv_globals.trace_allsolid = trace.allsolid;
@@ -401,6 +400,7 @@ PF_traceline (progs_t *pr)
 	VectorCopy (trace.endpos, *sv_globals.trace_endpos);
 	VectorCopy (trace.plane.normal, *sv_globals.trace_plane_normal);
 	*sv_globals.trace_plane_dist = trace.plane.dist;
+
 	if (trace.ent)
 		*sv_globals.trace_ent = EDICT_TO_PROG (pr, trace.ent);
 	else
@@ -408,19 +408,20 @@ PF_traceline (progs_t *pr)
 }
 
 /*
-	PF_checkmove
-	// void (vector start, vector mins, vector maxs, vector end, float type, entity passent) checkmove
+	PF_tracebox
+	// void (vector start, vector mins, vector maxs, vector end, float type,
+	//       entity passent) tracebox
 
 	Wrapper around SV_Move, this makes PF_movetoground and PF_traceline
 	redundant.
 */
 static void
-PF_checkmove (progs_t *pr)
+PF_tracebox (progs_t *pr)
 {
-	edict_t	*ent;
-	float	*start, *end, *mins, *maxs;
-	int		type;
-	trace_t	trace;
+	edict_t    *ent;
+	float      *start, *end, *mins, *maxs;
+	int         type;
+	trace_t     trace;
 
 	start = P_VECTOR (pr, 0);
 	mins = P_VECTOR (pr, 1);
@@ -453,14 +454,14 @@ PF_checkmove (progs_t *pr)
 	FIXME: make work...
 	scalar checkpos (entity, vector)
 */
-void
+static void __attribute__ ((used))
 PF_checkpos (progs_t *pr)
 {
 }
 
 byte        checkpvs[MAX_MAP_LEAFS / 8];
 
-int
+static int
 PF_newcheckclient (progs_t *pr, int check)
 {
 	byte       *pvs;
@@ -514,17 +515,19 @@ int         c_invis, c_notvis;
 
 /*
 	PF_checkclient
-	// entity () clientlist
 
 	Returns a client (or object that has a client enemy) that would be a
 	valid target.
 
 	If there are more than one valid options, they are cycled each frame
 
-	If (self.origin + self.viewofs) is not in the PVS of the current target, it
-	is not returned at all.
+	If (self.origin + self.viewofs) is not in the PVS of the current target,
+	it is not returned at all.
+
+	name checkclient ()
+	// entity () clientlist
 */
-void
+static void
 PF_checkclient (progs_t *pr)
 {
 	edict_t    *ent, *self;
@@ -537,7 +540,7 @@ PF_checkclient (progs_t *pr)
 		sv.lastcheck = PF_newcheckclient (pr, sv.lastcheck);
 		sv.lastchecktime = sv.time;
 	}
-	// return check if it might be visible  
+	// return check if it might be visible
 	ent = EDICT_NUM (pr, sv.lastcheck);
 	if (ent->free || SVfloat (ent, health) <= 0) {
 		RETURN_EDICT (pr, sv.edicts);
@@ -566,7 +569,7 @@ PF_checkclient (progs_t *pr)
 	stuffcmd (clientent, value)
 	// void (entity client, string s) stuffcmd
 */
-void
+static void
 PF_stuffcmd (progs_t *pr)
 {
 	const char *str;
@@ -599,14 +602,16 @@ PF_stuffcmd (progs_t *pr)
 
 	p = strrchr (buf, '\n');
 	if (p) {
-		char t = p[1];
+		char        t = p[1];
+		int         len = (p - buf) + 2;
 		p[1] = 0;
-		MSG_ReliableWrite_Begin (&cl->backbuf, svc_stufftext, 2 + p - buf);
+		MSG_ReliableWrite_Begin (&cl->backbuf, svc_stufftext, len + 1);
 		MSG_ReliableWrite_String (&cl->backbuf, buf);
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 2 + strlen (buf));
-			MSG_WriteByte (&demo.dbuf->sz, svc_stufftext);
-			MSG_WriteString (&demo.dbuf->sz, buf);
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, len + 1);
+			MSG_WriteByte (dbuf, svc_stufftext);
+			MSG_WriteString (dbuf, buf);
 		}
 		p[1] = t;
 		strcpy (buf, p + 1);		// safe because this is a downward, in
@@ -616,13 +621,13 @@ PF_stuffcmd (progs_t *pr)
 
 /*
 	PF_localcmd
-	// void (string s) localcmd
 
 	Inserts text into the server console's execution buffer
 
 	localcmd (string)
+	// void (string s) localcmd
 */
-void
+static void
 PF_localcmd (progs_t *pr)
 {
 	const char       *str;
@@ -639,37 +644,33 @@ PF_localcmd (progs_t *pr)
 	findradius (origin, radius)
 	// entity (vector org, float rad) findradius
 */
-void
+static void
 PF_findradius (progs_t *pr)
 {
 	edict_t    *ent, *chain;
-	float       rad;
-	//float      *eorigin;
-	float      *emins, *emaxs, *org;
+	float       rsqr;
+	vec_t      *emins, *emaxs, *org;
 	int         i, j;
 	vec3_t      eorg;
 
 	chain = (edict_t *) sv.edicts;
 
 	org = P_VECTOR (pr, 0);
-	rad = P_FLOAT (pr, 1);
-	rad *= rad;					// Square early, sqrt never
+	rsqr = P_FLOAT (pr, 1);
+	rsqr *= rsqr;					// Square early, sqrt never
 
 	ent = NEXT_EDICT (pr, sv.edicts);
 	for (i = 1; i < sv.num_edicts; i++, ent = NEXT_EDICT (pr, ent)) {
 		if (ent->free)
 			continue;
-		if (SVfloat (ent, solid) == SOLID_NOT)
+		if (SVfloat (ent, solid) == SOLID_NOT
+			&& !((int) SVfloat (ent, flags) & FL_FINDABLE_NONSOLID))
 			continue;
-		//eorigin = SVvector (ent, origin);
-		//emins = SVvector (ent, mins);
-		//emaxs = SVvector (ent, maxs);
 		emins = SVvector (ent, absmin);
 		emaxs = SVvector (ent, absmax);
 		for (j = 0; j < 3; j++)
 			eorg[j] = org[j] - 0.5 * (emins[j] + emaxs[j]);
-			//eorg[j] = org[j] - eorigin[j] - 0.5 * (emins[j] + emaxs[j]);
-		if (DotProduct (eorg, eorg) > rad)
+		if (DotProduct (eorg, eorg) > rsqr)
 			continue;
 
 		SVentity (ent, chain) = EDICT_TO_PROG (pr, chain);
@@ -680,7 +681,7 @@ PF_findradius (progs_t *pr)
 }
 
 // entity () spawn
-void
+static void
 PF_Spawn (progs_t *pr)
 {
 	edict_t    *ed;
@@ -692,23 +693,30 @@ PF_Spawn (progs_t *pr)
 cvar_t *pr_double_remove;
 
 // void (entity e) remove
-void
+static void
 PF_Remove (progs_t *pr)
 {
 	edict_t    *ed;
 
 	ed = P_EDICT (pr, 0);
+	if (NUM_FOR_EDICT (pr, ed) < *pr->reserved_edicts) {
+		if (pr_double_remove->int_val == 1) {
+			PR_DumpState (pr);
+			Sys_Printf ("Reserved entity remove\n");
+		} else // == 2
+			PR_RunError (pr, "Reserved entity remove\n");
+	}
 	if (ed->free && pr_double_remove->int_val) {
 		if (pr_double_remove->int_val == 1) {
 			PR_DumpState (pr);
-			SV_Printf ("Double entity remove\n");
+			Sys_Printf ("Double entity remove\n");
 		} else // == 2
 			PR_RunError (pr, "Double entity remove\n");
 	}
 	ED_Free (pr, ed);
 }
 
-void
+static void
 PR_CheckEmptyString (progs_t *pr, const char *s)
 {
 	if (s[0] <= ' ')
@@ -723,7 +731,7 @@ do_precache (progs_t *pr, const char **cache, int max, const char *name,
 	char       *s;
 
 	if (sv.state != ss_loading)
-		PR_RunError (pr, "%s: Precache can only be done in spawn functions",
+		PR_RunError (pr, "%s: Precache can be done only in spawn functions",
 					 func);
 
 	PR_CheckEmptyString (pr, name);
@@ -740,7 +748,7 @@ do_precache (progs_t *pr, const char **cache, int max, const char *name,
 			char *c = Hunk_Alloc (strlen (s) + 1);
 			strcpy (c, s);
 			cache[i] = c; // blah, const
-			Con_DPrintf ("%s: %3d %s\n", func, i, s);
+			Sys_MaskPrintf (SYS_DEV, "%s: %3d %s\n", func, i, s);
 			return;
 		}
 		if (!strcmp (cache[i], s))
@@ -751,16 +759,16 @@ do_precache (progs_t *pr, const char **cache, int max, const char *name,
 
 // string (string s) precache_file
 // string (string s) precache_file2
-void
+static void
 PF_precache_file (progs_t *pr)
-{										// precache_file is only used to copy 
-										// files with qcc, it does nothing
+{
+	// precache_file is used only to copy files with qcc, it does nothing
 	R_INT (pr) = P_INT (pr, 0);
 }
 
 // void (string s) precache_sound
 // string (string s) precache_sound2
-void
+static void
 PF_precache_sound (progs_t *pr)
 {
 	do_precache (pr, sv.sound_precache, MAX_SOUNDS, P_GSTRING (pr, 0),
@@ -770,13 +778,12 @@ PF_precache_sound (progs_t *pr)
 
 // void (string s) precache_model
 // string (string s) precache_model2
-void
+static void
 PF_precache_model (progs_t *pr)
 {
-	R_INT (pr) = P_INT (pr, 0);
-
 	do_precache (pr, sv.model_precache, MAX_MODELS, P_GSTRING (pr, 0),
 				 "precache_model");
+	R_INT (pr) = P_INT (pr, 0);
 }
 
 /*
@@ -785,10 +792,9 @@ PF_precache_model (progs_t *pr)
 	float (float yaw, float dist) walkmove
 	// float (float yaw, float dist) walkmove
 */
-void
+static void
 PF_walkmove (progs_t *pr)
 {
-	dfunction_t *oldf;
 	edict_t    *ent;
 	float       yaw, dist;
 	int         oldself;
@@ -810,13 +816,11 @@ PF_walkmove (progs_t *pr)
 	move[2] = 0;
 
 	// save program state, because SV_movestep may call other progs
-	oldf = pr->pr_xfunction;
 	oldself = *sv_globals.self;
 
 	R_FLOAT (pr) = SV_movestep (ent, move, true);
 
 	// restore program state
-	pr->pr_xfunction = oldf;
 	*sv_globals.self = oldself;
 }
 
@@ -826,7 +830,7 @@ PF_walkmove (progs_t *pr)
 	void () droptofloor
 	// float () droptofloor
 */
-void
+static void
 PF_droptofloor (progs_t *pr)
 {
 	edict_t    *ent;
@@ -858,7 +862,7 @@ PF_droptofloor (progs_t *pr)
 	void (float style, string value) lightstyle
 	// void (float style, string value) lightstyle
 */
-void
+static void
 PF_lightstyle (progs_t *pr)
 {
 	const char *val;
@@ -882,16 +886,17 @@ PF_lightstyle (progs_t *pr)
 			MSG_ReliableWrite_Char (&cl->backbuf, style);
 			MSG_ReliableWrite_String (&cl->backbuf, val);
 		}
-	if (sv.demorecording) {
-		DemoWrite_Begin (dem_all, 0, strlen (val) + 3);
-		MSG_WriteByte (&demo.dbuf->sz, svc_lightstyle);
-		MSG_WriteByte (&demo.dbuf->sz, style);
-		MSG_WriteString (&demo.dbuf->sz, val);
+	if (sv.recorders) {
+		sizebuf_t  *dbuf;
+		dbuf = SVR_WriteBegin (dem_all, 0, strlen (val) + 3);
+		MSG_WriteByte (dbuf, svc_lightstyle);
+		MSG_WriteByte (dbuf, style);
+		MSG_WriteString (dbuf, val);
 	}
 }
 
 // float (entity e) checkbottom
-void
+static void
 PF_checkbottom (progs_t *pr)
 {
 	edict_t    *ent;
@@ -902,7 +907,7 @@ PF_checkbottom (progs_t *pr)
 }
 
 // float (vector v) pointcontents
-void
+static void
 PF_pointcontents (progs_t *pr)
 {
 	float      *v;
@@ -916,12 +921,12 @@ cvar_t     *sv_aim;
 
 /*
 	PF_aim
-	// vector (entity e, float speed) aim
 
 	Pick a vector for the player to shoot along
 	vector aim (entity, missilespeed)
+	// vector (entity e, float speed) aim
 */
-void
+static void
 PF_aim (progs_t *pr)
 {
 	const char *noaim;
@@ -949,6 +954,7 @@ PF_aim (progs_t *pr)
 	}
 
 	speed = P_FLOAT (pr, 1);
+	(void) speed; //FIXME
 
 	VectorCopy (SVvector (ent, origin), start);
 	start[2] += 20;
@@ -1011,9 +1017,9 @@ PF_aim (progs_t *pr)
 
 /*
 	PF_changeyaw
-	// void () ChangeYaw
 
 	This was a major timewaster in progs, so it was converted to C
+	// void () ChangeYaw
 */
 void
 PF_changeyaw (progs_t *pr)
@@ -1055,7 +1061,7 @@ PF_changeyaw (progs_t *pr)
 #define	MSG_INIT		3				// write to the init string
 #define	MSG_MULTICAST	4				// for multicast ()
 
-static sizebuf_t  *
+static sizebuf_t *
 WriteDest (progs_t *pr)
 {
 	int         dest;
@@ -1073,7 +1079,7 @@ WriteDest (progs_t *pr)
 
 		case MSG_INIT:
 			if (sv.state != ss_loading)
-				PR_RunError (pr, "PF_Write_*: MSG_INIT can only be written in "
+				PR_RunError (pr, "PF_Write_*: MSG_INIT can be written in only "
 							 "spawn functions");
 			return &sv.signon;
 
@@ -1102,36 +1108,38 @@ Write_GetClient (progs_t *pr)
 }
 
 // void (float to, ...) WriteBytes
-void
+static void
 PF_WriteBytes (progs_t *pr)
 {
 	int         i, p;
+	int         count = pr->pr_argc - 1;
+	byte        buf[MAX_PARMS];
+
+	for (i = 0; i < count; i++) {
+		p = P_FLOAT (pr, i + 1);
+		buf[i] = p;
+	}
 
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
 		client_t   *cl = Write_GetClient (pr);
 
-		if (cl->state != cs_server)
-			MSG_ReliableCheckBlock (&cl->backbuf, pr->pr_argc);
-		if (sv.demorecording)
-			DemoWrite_Begin (dem_single, cl - svs.clients, pr->pr_argc);
-		for (i = 1; i < pr->pr_argc; i++) {
-			p = P_FLOAT (pr, i);
-			if (cl->state != cs_server)
-				MSG_ReliableWrite_Byte (&cl->backbuf, p);
-			if (sv.demorecording)
-				MSG_WriteByte (&demo.dbuf->sz, p);
+		if (cl->state != cs_server) {
+			MSG_ReliableCheckBlock (&cl->backbuf, count);
+			MSG_ReliableWrite_SZ (&cl->backbuf, buf, count);
+		}
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, count);
+			SZ_Write (dbuf, buf, count);
 		}
 	} else {
 		sizebuf_t  *msg = WriteDest (pr);
-		for (i = 1; i < pr->pr_argc; i++) {
-			p = P_FLOAT (pr, i);
-			MSG_WriteByte (msg, p);
-		}
+		SZ_Write (msg, buf, count);
 	}
 }
 
 // void (float to, float f) WriteByte
-void
+static void
 PF_WriteByte (progs_t *pr)
 {
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
@@ -1141,16 +1149,17 @@ PF_WriteByte (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 1);
 			MSG_ReliableWrite_Byte (&cl->backbuf, P_FLOAT (pr, 1));
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 1);
-			MSG_WriteByte (&demo.dbuf->sz, P_FLOAT (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 1);
+			MSG_WriteByte (dbuf, P_FLOAT (pr, 1));
 		}
 	} else
 		MSG_WriteByte (WriteDest (pr), P_FLOAT (pr, 1));
 }
 
 // void (float to, float f) WriteChar
-void
+static void
 PF_WriteChar (progs_t *pr)
 {
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
@@ -1160,16 +1169,17 @@ PF_WriteChar (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 1);
 			MSG_ReliableWrite_Char (&cl->backbuf, P_FLOAT (pr, 1));
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 1);
-			MSG_WriteByte (&demo.dbuf->sz, P_FLOAT (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 1);
+			MSG_WriteByte (dbuf, P_FLOAT (pr, 1));
 		}
 	} else
 		MSG_WriteByte (WriteDest (pr), P_FLOAT (pr, 1));
 }
 
 // void (float to, float f) WriteShort
-void
+static void
 PF_WriteShort (progs_t *pr)
 {
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
@@ -1179,16 +1189,17 @@ PF_WriteShort (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 2);
 			MSG_ReliableWrite_Short (&cl->backbuf, P_FLOAT (pr, 1));
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 2);
-			MSG_WriteShort (&demo.dbuf->sz, P_FLOAT (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 2);
+			MSG_WriteShort (dbuf, P_FLOAT (pr, 1));
 		}
 	} else
 		MSG_WriteShort (WriteDest (pr), P_FLOAT (pr, 1));
 }
 
 // void (float to, float f) WriteLong
-void
+static void
 PF_WriteLong (progs_t *pr)
 {
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
@@ -1198,16 +1209,17 @@ PF_WriteLong (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 4);
 			MSG_ReliableWrite_Long (&cl->backbuf, P_FLOAT (pr, 1));
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 4);
-			MSG_WriteLong (&demo.dbuf->sz, P_FLOAT (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 4);
+			MSG_WriteLong (dbuf, P_FLOAT (pr, 1));
 		}
 	} else
 		MSG_WriteLong (WriteDest (pr), P_FLOAT (pr, 1));
 }
 
 // void (float to, float f) WriteAngle
-void
+static void
 PF_WriteAngle (progs_t *pr)
 {
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
@@ -1217,16 +1229,17 @@ PF_WriteAngle (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 1);
 			MSG_ReliableWrite_Angle (&cl->backbuf, P_FLOAT (pr, 1));
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 1);
-			MSG_WriteAngle (&demo.dbuf->sz, P_FLOAT (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 1);
+			MSG_WriteAngle (dbuf, P_FLOAT (pr, 1));
 		}
 	} else
 		MSG_WriteAngle (WriteDest (pr), P_FLOAT (pr, 1));
 }
 
 // void (float to, float f) WriteCoord
-void
+static void
 PF_WriteCoord (progs_t *pr)
 {
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
@@ -1236,16 +1249,17 @@ PF_WriteCoord (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 2);
 			MSG_ReliableWrite_Coord (&cl->backbuf, P_FLOAT (pr, 1));
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 2);
-			MSG_WriteCoord (&demo.dbuf->sz, P_FLOAT (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 2);
+			MSG_WriteCoord (dbuf, P_FLOAT (pr, 1));
 		}
 	} else
 		MSG_WriteCoord (WriteDest (pr), P_FLOAT (pr, 1));
 }
 
 // void (float to, vector v) WriteAngleV
-void
+static void
 PF_WriteAngleV (progs_t *pr)
 {
 	float      *ang = P_VECTOR (pr, 1);
@@ -1257,16 +1271,17 @@ PF_WriteAngleV (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 1);
 			MSG_ReliableWrite_AngleV (&cl->backbuf, ang);
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 1);
-			MSG_WriteAngleV (&demo.dbuf->sz, ang);
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 1);
+			MSG_WriteAngleV (dbuf, ang);
 		}
 	} else
 		MSG_WriteAngleV (WriteDest (pr), ang);
 }
 
 // void (float to, vector v) WriteCoordV
-void
+static void
 PF_WriteCoordV (progs_t *pr)
 {
 	float      *coord = P_VECTOR (pr, 1);
@@ -1278,56 +1293,61 @@ PF_WriteCoordV (progs_t *pr)
 			MSG_ReliableCheckBlock (&cl->backbuf, 2);
 			MSG_ReliableWrite_CoordV (&cl->backbuf, coord);
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 2);
-			MSG_WriteCoordV (&demo.dbuf->sz, coord);
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 2);
+			MSG_WriteCoordV (dbuf, coord);
 		}
 	} else
 		MSG_WriteCoordV (WriteDest (pr), coord);
 }
 
 // void (float to, string s) WriteString
-void
+static void
 PF_WriteString (progs_t *pr)
 {
+	const char *str = P_GSTRING (pr, 1);
+
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
 		client_t   *cl = Write_GetClient (pr);
 
 		if (cl->state != cs_server) {
-			MSG_ReliableCheckBlock (&cl->backbuf,
-									1 + strlen (P_GSTRING (pr, 1)));
-			MSG_ReliableWrite_String (&cl->backbuf, P_GSTRING (pr, 1));
+			MSG_ReliableCheckBlock (&cl->backbuf, 1 + strlen (str));
+			MSG_ReliableWrite_String (&cl->backbuf, str);
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients,
-							 1 + strlen (P_GSTRING (pr, 1)));
-			MSG_WriteString (&demo.dbuf->sz, P_GSTRING (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 1 + strlen (str));
+			MSG_WriteString (dbuf, str);
 		}
 	} else
-		MSG_WriteString (WriteDest (pr), P_GSTRING (pr, 1));
+		MSG_WriteString (WriteDest (pr), str);
 }
 
 // void (float to, entity s) WriteEntity
-void
+static void
 PF_WriteEntity (progs_t *pr)
 {
+	int         ent = P_EDICTNUM (pr, 1);
+
 	if (P_FLOAT (pr, 0) == MSG_ONE) {
 		client_t   *cl = Write_GetClient (pr);
 
 		if (cl->state != cs_server) {
 			MSG_ReliableCheckBlock (&cl->backbuf, 2);
-			MSG_ReliableWrite_Short (&cl->backbuf, P_EDICTNUM (pr, 1));
+			MSG_ReliableWrite_Short (&cl->backbuf, ent);
 		}
-		if (sv.demorecording) {
-			DemoWrite_Begin (dem_single, cl - svs.clients, 2);
-			MSG_WriteShort (&demo.dbuf->sz, P_EDICTNUM (pr, 1));
+		if (sv.recorders) {
+			sizebuf_t  *dbuf;
+			dbuf = SVR_WriteBegin (dem_single, cl - svs.clients, 2);
+			MSG_WriteShort (dbuf, ent);
 		}
 	} else
-		MSG_WriteShort (WriteDest (pr), P_EDICTNUM (pr, 1));
+		MSG_WriteShort (WriteDest (pr), ent);
 }
 
 // void (entity e) makestatic
-void
+static void
 PF_makestatic (progs_t *pr)
 {
 	const char *model;
@@ -1355,7 +1375,7 @@ PF_makestatic (progs_t *pr)
 }
 
 // void (entity e) setspawnparms
-void
+static void
 PF_setspawnparms (progs_t *pr)
 {
 	client_t   *client;
@@ -1378,7 +1398,7 @@ PF_setspawnparms (progs_t *pr)
 }
 
 // void (string s) changelevel
-void
+static void
 PF_changelevel (progs_t *pr)
 {
 	const char *s;
@@ -1399,7 +1419,7 @@ PF_changelevel (progs_t *pr)
 	logfrag (killer, killee)
 	// void (entity killer, entity killee) logfrag
 */
-void
+static void
 PF_logfrag (progs_t *pr)
 {
 	const char *s;
@@ -1417,9 +1437,9 @@ PF_logfrag (progs_t *pr)
 		char buf[16];
 		char type1[2], type2[2];
 		int u1, u2;
-		
+
 		type1[1] = type2[1] = 0;
-		if (e1 < 1 || e1 > MAX_CLIENTS || 
+		if (e1 < 1 || e1 > MAX_CLIENTS ||
 			svs.clients[e1 - 1].state == cs_server) {
 			type1[0] = 'e';
 			u1 = e1;
@@ -1438,7 +1458,7 @@ PF_logfrag (progs_t *pr)
 		}
 
 		snprintf(buf, sizeof(buf), "%d", u2);
-		
+
 		GIB_Event_Callback (sv_frag_e, 4, type1, va ("%d", u1), type2, buf);
 	}
 
@@ -1450,7 +1470,7 @@ PF_logfrag (progs_t *pr)
 
 	SZ_Print (&svs.log[svs.logsequence & 1], s);
 	if (sv_fraglogfile) {
-		Qprintf (sv_fraglogfile, s);
+		Qprintf (sv_fraglogfile, "%s", s);
 		Qflush (sv_fraglogfile);
 	}
 }
@@ -1461,7 +1481,7 @@ PF_logfrag (progs_t *pr)
 	string (entity e, string key) infokey
 	// string (entity e, string key) infokey
 */
-void
+static void
 PF_infokey (progs_t *pr)
 {
 	const char *key, *value;
@@ -1505,7 +1525,7 @@ PF_infokey (progs_t *pr)
 	void (vector where, float set) multicast
 	// void (vector where, float set) multicast
 */
-void
+static void
 PF_multicast (progs_t *pr)
 {
 	float      *o;
@@ -1628,7 +1648,7 @@ PF_hullpointcontents (progs_t *pr)
 	hull_t     *hull;
 	vec3_t      offset;
 
-	hull = SV_HullForEntity (edict, mins, maxs, offset);
+	hull = SV_HullForEntity (edict, mins, maxs, 0, offset);
 	VectorSubtract (point, offset, offset);
 	R_INT (pr) = SV_HullPointContents (hull, 0, offset);
 }
@@ -1784,13 +1804,6 @@ PF_rotate_bbox (progs_t *pr)
 	}
 }
 
-// void (entity e, vector min, vector max) setabssize
-void
-PF_Fixme (progs_t *pr)
-{
-	PR_RunError (pr, "Unimplemented builtin function called");
-}
-
 // float () checkextension
 static void
 PF_checkextension (progs_t *pr)
@@ -1954,6 +1967,7 @@ static builtin_t builtins[] = {
 	{"aim",					PF_aim,					44},
 
 	{"localcmd",			PF_localcmd,			46},
+
 	{"changeyaw",			PF_changeyaw,			49},
 
 	{"writebyte",			PF_WriteByte,			52},
@@ -1983,14 +1997,14 @@ static builtin_t builtins[] = {
 	{"logfrag",				PF_logfrag,				79},
 	{"infokey",				PF_infokey,				80},
 	{"multicast",			PF_multicast,			82},
+
 	{"testentitypos",		PF_testentitypos,		QF 92},
 	{"hullpointcontents",	PF_hullpointcontents,	QF 93},
 	{"getboxbounds",		PF_getboxbounds,		QF 94},
 	{"getboxhull",			PF_getboxhull,			QF 95},
 	{"freeboxhull",			PF_freeboxhull,			QF 96},
 	{"rotate_bbox",			PF_rotate_bbox,			QF 97},
-
-	{"checkmove",			PF_checkmove,			QF 98},
+	{"tracebox",			PF_tracebox,			QF 98},
 	{"checkextension",		PF_checkextension,		QF 99},
 	{"setinfokey",			PF_setinfokey,			QF 102},
 	{"cfopen",				PF_cfopen,				QF 103},
@@ -2006,6 +2020,9 @@ static builtin_t builtins[] = {
 	{"SV_SetPing",			PR_SV_SetPing,			-1},
 	{"SV_UserCmd",			PR_SV_UserCmd,			-1},
 	{"SV_Spawn",			PR_SV_Spawn,			-1},
+
+	{"EntityParseFunction", ED_EntityParseFunction,	-1},
+
 	{0}
 };
 
@@ -2023,5 +2040,4 @@ SV_PR_Cmds_Init ()
 	bi->proc = PF_sv_cvar;
 
 	PR_RegisterBuiltins (&sv_pr_state, builtins);
-};
-// void (float step) movetogoal
+}
