@@ -28,9 +28,6 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] = 
-	"$Id: sv_user.c,v 1.173 2004/04/27 03:45:26 taniwha Exp $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -56,15 +53,16 @@ static __attribute__ ((unused)) const char rcsid[] =
 #include "QF/sys.h"
 #include "QF/va.h"
 
+#include "qw/msg_ucmd.h"
 #include "qw/msg_ucmd.h"
 
-#include "bothdefs.h"
+#include "qw/bothdefs.h"
 #include "compat.h"
-#include "pmove.h"
+#include "qw/pmove.h"
 #include "server.h"
-#include "sv_demo.h"
-#include "sv_progs.h"
 #include "sv_gib.h"
+#include "sv_progs.h"
+#include "sv_recorder.h"
 #include "world.h"
 
 typedef struct ucmd_s {
@@ -81,6 +79,16 @@ edict_t    *sv_player;
 
 usercmd_t   cmd;
 
+cvar_t     *sv_antilag;
+cvar_t     *sv_antilag_frac;
+
+cvar_t     *sv_accelerate;
+cvar_t     *sv_airaccelerate;
+cvar_t     *sv_maxspeed;
+cvar_t     *sv_spectatormaxspeed;
+cvar_t     *sv_wateraccelerate;
+cvar_t     *sv_waterfriction;
+
 cvar_t     *sv_allowfake;
 
 cvar_t     *cl_rollspeed;
@@ -98,10 +106,44 @@ cvar_t     *sv_timekick_interval;
 cvar_t     *sv_timecheck_fuzz;
 cvar_t     *sv_timecheck_decay;
 
+cvar_t     *sv_http_url_base;
+
 static void OutofBandPrintf (netadr_t where, const char *fmt, ...) __attribute__ ((format (printf, 2, 3)));
 
 //	USER STRINGCMD EXECUTION host_client and sv_player will be valid.
 
+void
+SV_WriteWorldVars (netchan_t *netchan)
+{
+
+	// send full levelname
+	MSG_WriteString (&netchan->message,
+					 PR_GetString (&sv_pr_state,
+								   SVstring (sv.edicts, message)));
+
+	// send the movevars
+	MSG_WriteFloat (&netchan->message, movevars.gravity);
+	MSG_WriteFloat (&netchan->message, movevars.stopspeed);
+	MSG_WriteFloat (&netchan->message, movevars.maxspeed);
+	MSG_WriteFloat (&netchan->message, movevars.spectatormaxspeed);
+	MSG_WriteFloat (&netchan->message, movevars.accelerate);
+	MSG_WriteFloat (&netchan->message, movevars.airaccelerate);
+	MSG_WriteFloat (&netchan->message, movevars.wateraccelerate);
+	MSG_WriteFloat (&netchan->message, movevars.friction);
+	MSG_WriteFloat (&netchan->message, movevars.waterfriction);
+	MSG_WriteFloat (&netchan->message, movevars.entgravity);
+
+	// send music
+	MSG_WriteByte (&netchan->message, svc_cdtrack);
+	MSG_WriteByte (&netchan->message, SVfloat (sv.edicts, sounds));
+
+	// send server info string
+	MSG_WriteByte (&netchan->message, svc_stufftext);
+	MSG_WriteString (&netchan->message,
+					 va ("fullserverinfo \"%s\"\n",
+						 Info_MakeString (svs.info, 0)));
+}
+
 /*
 	SV_New_f
 
@@ -111,31 +153,30 @@ static void OutofBandPrintf (netadr_t where, const char *fmt, ...) __attribute__
 static void
 SV_New_f (void *unused)
 {
-	const char *gamedir;
 	int         playernum;
+	const char *gamedir;
 
 	if (host_client->state == cs_spawned)
 		return;
 
-	host_client->state = cs_connected;
-	host_client->connection_started = realtime;
-
-	// send the info about the new client to all connected clients
-//	SV_FullClientUpdate (host_client, &sv.reliable_datagram);
-//	host_client->sendinfo = true;
-
 	gamedir = Info_ValueForKey (svs.info, "*gamedir");
 	if (!gamedir[0])
 		gamedir = "qw";
 
+	host_client->state = cs_connected;
+	host_client->connection_started = realtime;
+
+	// send the info about the new client to all connected clients
+
 // NOTE:  This doesn't go through MSG_ReliableWrite since it's before the
 // user spawns.  These functions are written to not overflow
 	if (host_client->backbuf.num_backbuf) {
-		SV_Printf ("WARNING %s: [SV_New] Back buffered (%d0, clearing\n",
+		SV_Printf ("WARNING %s: [SV_New] Back buffered (%d), clearing\n",
 					host_client->name, host_client->netchan.message.cursize);
 		host_client->backbuf.num_backbuf = 0;
 		SZ_Clear (&host_client->netchan.message);
 	}
+
 	// send the serverdata
 	MSG_WriteByte (&host_client->netchan.message, svc_serverdata);
 	MSG_WriteLong (&host_client->netchan.message, PROTOCOL_VERSION);
@@ -147,43 +188,38 @@ SV_New_f (void *unused)
 		playernum |= 128;
 	MSG_WriteByte (&host_client->netchan.message, playernum);
 
-	// send full levelname
-	MSG_WriteString (&host_client->netchan.message,
-					 PR_GetString (&sv_pr_state,
-								   SVstring (sv.edicts, message)));
-
-	// send the movevars
-	MSG_WriteFloat (&host_client->netchan.message, movevars.gravity);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.stopspeed);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.maxspeed);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.spectatormaxspeed);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.accelerate);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.airaccelerate);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.wateraccelerate);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.friction);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.waterfriction);
-	MSG_WriteFloat (&host_client->netchan.message, movevars.entgravity);
-
-	// send music
-	MSG_WriteByte (&host_client->netchan.message, svc_cdtrack);
-	MSG_WriteByte (&host_client->netchan.message, SVfloat (sv.edicts, sounds));
-
-	// send server info string
-	MSG_WriteByte (&host_client->netchan.message, svc_stufftext);
-	MSG_WriteString (&host_client->netchan.message,
-					 va ("fullserverinfo \"%s\"\n",
-						 Info_MakeString (svs.info, 0)));
+	SV_WriteWorldVars (&host_client->netchan);
 
 	// Trigger GIB connection event
 	if (sv_client_connect_e->func)
-		GIB_Event_Callback (sv_client_connect_e, 1, va ("%u",
-														host_client->userid));
+		GIB_Event_Callback (sv_client_connect_e, 1,
+							va ("%u", host_client->userid));
+}
+
+void
+SV_WriteSoundlist (netchan_t *netchan, int n)
+{
+	const char **s;
+
+	MSG_WriteByte (&netchan->message, svc_soundlist);
+	MSG_WriteByte (&netchan->message, n);
+	for (s = sv.sound_precache + 1 + n;
+		 *s && netchan->message.cursize < (MAX_MSGLEN / 2);
+		 s++, n++)
+		MSG_WriteString (&netchan->message, *s);
+
+	MSG_WriteByte (&netchan->message, 0);
+
+	// next msg
+	if (*s)
+		MSG_WriteByte (&netchan->message, n);
+	else
+		MSG_WriteByte (&netchan->message, 0);
 }
 
 static void
 SV_Soundlist_f (void *unused)
 {
-	const char **s;
 	unsigned    n;
 
 	if (host_client->state != cs_connected) {
@@ -206,31 +242,38 @@ SV_Soundlist_f (void *unused)
 // NOTE:  This doesn't go through MSG_ReliableWrite since it's before the
 // user spawns.  These functions are written to not overflow
 	if (host_client->backbuf.num_backbuf) {
-		SV_Printf ("WARNING %s: [SV_Soundlist] Back buffered (%d0, clearing",
+		SV_Printf ("WARNING %s: [SV_Soundlist] Back buffered (%d), clearing",
 					host_client->name, host_client->netchan.message.cursize);
 		host_client->backbuf.num_backbuf = 0;
 		SZ_Clear (&host_client->netchan.message);
 	}
 
-	MSG_WriteByte (&host_client->netchan.message, svc_soundlist);
-	MSG_WriteByte (&host_client->netchan.message, n);
-	for (s = sv.sound_precache + 1 + n;
-		 *s && host_client->netchan.message.cursize < (MAX_MSGLEN / 2);
-		 s++, n++) MSG_WriteString (&host_client->netchan.message, *s);
+	SV_WriteSoundlist (&host_client->netchan, n);
+}
 
-	MSG_WriteByte (&host_client->netchan.message, 0);
+void
+SV_WriteModellist (netchan_t *netchan, int n)
+{
+	const char **s;
+
+	MSG_WriteByte (&netchan->message, svc_modellist);
+	MSG_WriteByte (&netchan->message, n);
+	for (s = sv.model_precache + 1 + n;
+		 *s && netchan->message.cursize < (MAX_MSGLEN / 2);
+		 s++, n++)
+		MSG_WriteString (&netchan->message, *s);
+	MSG_WriteByte (&netchan->message, 0);
 
 	// next msg
 	if (*s)
-		MSG_WriteByte (&host_client->netchan.message, n);
+		MSG_WriteByte (&netchan->message, n);
 	else
-		MSG_WriteByte (&host_client->netchan.message, 0);
+		MSG_WriteByte (&netchan->message, 0);
 }
 
 static void
 SV_Modellist_f (void *unused)
 {
-	const char **s;
 	unsigned    n;
 
 	if (host_client->state != cs_connected) {
@@ -259,18 +302,7 @@ SV_Modellist_f (void *unused)
 		SZ_Clear (&host_client->netchan.message);
 	}
 
-	MSG_WriteByte (&host_client->netchan.message, svc_modellist);
-	MSG_WriteByte (&host_client->netchan.message, n);
-	for (s = sv.model_precache + 1 + n;
-		 *s && host_client->netchan.message.cursize < (MAX_MSGLEN / 2);
-		 s++, n++) MSG_WriteString (&host_client->netchan.message, *s);
-	MSG_WriteByte (&host_client->netchan.message, 0);
-
-	// next msg
-	if (*s)
-		MSG_WriteByte (&host_client->netchan.message, n);
-	else
-		MSG_WriteByte (&host_client->netchan.message, 0);
+	SV_WriteModellist (&host_client->netchan, n);
 }
 
 static void
@@ -300,7 +332,7 @@ SV_PreSpawn_f (void *unused)
 		// should be three numbers following containing checksums
 		check = atoi (Cmd_Argv (3));
 
-//      Con_DPrintf ("Client check = %d\n", check);
+//      Sys_MaskPrintf (SYS_DEV, , "Client check = %d\n", check);
 
 		if (sv_mapcheck->int_val && check != sv.worldmodel->checksum &&
 			check != sv.worldmodel->checksum2) {
@@ -313,7 +345,6 @@ SV_PreSpawn_f (void *unused)
 			SV_DropClient (host_client);
 			return;
 		}
-		host_client->checksum = check;
 	}
 
 	host_client->prespawned = true;
@@ -348,21 +379,61 @@ SV_Spawn (client_t *client)
 	memset (&ent->v, 0, sv_pr_state.progs->entityfields * 4);
 	SVfloat (ent, colormap) = NUM_FOR_EDICT (&sv_pr_state, ent);
 	SVfloat (ent, team) = 0; // FIXME
-	SVstring (ent, netname) = PR_SetString (&sv_pr_state, host_client->name);
+	SVstring (ent, netname) = PR_SetString (&sv_pr_state, client->name);
 
-	host_client->entgravity = 1.0;
+	client->entgravity = 1.0;
 	if (sv_fields.gravity != -1)
 		SVfloat (ent, gravity) = 1.0;
-	host_client->maxspeed = sv_maxspeed->value;
+	client->maxspeed = sv_maxspeed->value;
 	if (sv_fields.maxspeed != -1)
 		SVfloat (ent, maxspeed) = sv_maxspeed->value;
 }
 
+void
+SV_WriteSpawn1 (backbuf_t *backbuf, int n)
+{
+	int         i;
+	client_t   *client;
+
+	// normally this could overflow, but no need to check due to backbuf
+	for (i = n, client = svs.clients + n; i < MAX_CLIENTS; i++, client++)
+		SV_FullClientUpdateToClient (client, backbuf);
+
+	// send all current light styles
+	for (i = 0; i < MAX_LIGHTSTYLES; i++) {
+		MSG_ReliableWrite_Begin (backbuf, svc_lightstyle,
+								 3 + (sv.lightstyles[i] ?
+									  strlen (sv.lightstyles[i]) : 1));
+		MSG_ReliableWrite_Byte (backbuf, (char) i);
+		MSG_ReliableWrite_String (backbuf, sv.lightstyles[i]);
+	}
+}
+
+void
+SV_WriteSpawn2 (backbuf_t *backbuf)
+{
+
+	MSG_ReliableWrite_Begin (backbuf, svc_updatestatlong, 6);
+	MSG_ReliableWrite_Byte (backbuf, STAT_TOTALSECRETS);
+	MSG_ReliableWrite_Long (backbuf, *sv_globals.total_secrets);
+
+	MSG_ReliableWrite_Begin (backbuf, svc_updatestatlong, 6);
+	MSG_ReliableWrite_Byte (backbuf, STAT_TOTALMONSTERS);
+	MSG_ReliableWrite_Long (backbuf, *sv_globals.total_monsters);
+
+	MSG_ReliableWrite_Begin (backbuf, svc_updatestatlong, 6);
+	MSG_ReliableWrite_Byte (backbuf, STAT_SECRETS);
+	MSG_ReliableWrite_Long (backbuf, *sv_globals.found_secrets);
+
+	MSG_ReliableWrite_Begin (backbuf, svc_updatestatlong, 6);
+	MSG_ReliableWrite_Byte (backbuf, STAT_MONSTERS);
+	MSG_ReliableWrite_Long (backbuf, *sv_globals.killed_monsters);
+}
+
 static void
 SV_Spawn_f (void *unused)
 {
-	int         i, n;
-	client_t   *client;
+	int         n;
 
 	if (host_client->state != cs_connected) {
 		SV_Printf ("Spawn not valid -- already spawned\n");
@@ -398,45 +469,17 @@ SV_Spawn_f (void *unused)
 	SZ_Clear (&host_client->netchan.message);
 
 	// send current status of all other players
-
-	// normally this could overflow, but no need to check due to backbuf
-	for (i = n, client = svs.clients + n; i < MAX_CLIENTS; i++, client++)
-		SV_FullClientUpdateToClient (client, host_client);
-
-	// send all current light styles
-	for (i = 0; i < MAX_LIGHTSTYLES; i++) {
-		MSG_ReliableWrite_Begin (&host_client->backbuf, svc_lightstyle,
-								 3 + (sv.lightstyles[i] ?
-									  strlen (sv.lightstyles[i]) : 1));
-		MSG_ReliableWrite_Byte (&host_client->backbuf, (char) i);
-		MSG_ReliableWrite_String (&host_client->backbuf, sv.lightstyles[i]);
-	}
+	SV_WriteSpawn1 (&host_client->backbuf, n);
 
 	SV_Spawn (host_client);
 
+	SV_WriteSpawn2 (&host_client->backbuf);
+
 //
 // force stats to be updated
 //
 	memset (host_client->stats, 0, sizeof (host_client->stats));
 
-	MSG_ReliableWrite_Begin (&host_client->backbuf, svc_updatestatlong, 6);
-	MSG_ReliableWrite_Byte (&host_client->backbuf, STAT_TOTALSECRETS);
-	MSG_ReliableWrite_Long (&host_client->backbuf, *sv_globals.total_secrets);
-
-	MSG_ReliableWrite_Begin (&host_client->backbuf, svc_updatestatlong, 6);
-	MSG_ReliableWrite_Byte (&host_client->backbuf, STAT_TOTALMONSTERS);
-	MSG_ReliableWrite_Long (&host_client->backbuf,
-							*sv_globals.total_monsters);
-
-	MSG_ReliableWrite_Begin (&host_client->backbuf, svc_updatestatlong, 6);
-	MSG_ReliableWrite_Byte (&host_client->backbuf, STAT_SECRETS);
-	MSG_ReliableWrite_Long (&host_client->backbuf, *sv_globals.found_secrets);
-
-	MSG_ReliableWrite_Begin (&host_client->backbuf, svc_updatestatlong, 6);
-	MSG_ReliableWrite_Byte (&host_client->backbuf, STAT_MONSTERS);
-	MSG_ReliableWrite_Long (&host_client->backbuf,
-							*sv_globals.killed_monsters);
-
 	// get the client to check and download skins
 	// when that is completed, a begin command will be issued
 	MSG_ReliableWrite_Begin (&host_client->backbuf, svc_stufftext, 8);
@@ -571,7 +614,8 @@ static void
 SV_NextDownload_f (void *unused)
 {
 	byte        buffer[768];		// FIXME protocol limit? could be bigger?
-	int         percent, size, r;
+	int         percent, size;
+	size_t      r;
 
 	if (!host_client->download)
 		return;
@@ -641,7 +685,7 @@ SV_NextUpload (void)
 			net_message->readcount, size);
 	net_message->readcount += size;
 
-	Con_DPrintf ("UPLOAD: %d received\n", size);
+	Sys_MaskPrintf (SYS_DEV, "UPLOAD: %d received\n", size);
 
 	if (percent != 100) {
 		MSG_ReliableWrite_Begin (&host_client->backbuf, svc_stufftext, 8);
@@ -674,7 +718,7 @@ SV_BeginDownload_f (void *unused)
 {
 	const char *name;
 	dstring_t  *realname;
-	int			size, zip;
+	int			http, size, zip;
 	QFile	   *file;
 
 	name = Cmd_Argv (1);
@@ -693,7 +737,7 @@ SV_BeginDownload_f (void *unused)
 			!allow_download_sounds->int_val)
 		// now maps (note special case for maps, must not be in pak)
 		|| (strncmp (name, "maps/", 5) == 0 && !allow_download_maps->int_val)
-		// MUST be in a subdirectory    
+		// MUST be in a subdirectory
 		|| !strstr (name, "/")) {		// don't allow anything with .. path
 		MSG_ReliableWrite_Begin (&host_client->backbuf, svc_download, 4);
 		MSG_ReliableWrite_Short (&host_client->backbuf, -1);
@@ -707,6 +751,8 @@ SV_BeginDownload_f (void *unused)
 	}
 
 	zip = strchr (Info_ValueForKey (host_client->userinfo, "*cap"), 'z') != 0;
+	http = sv_http_url_base->string[0]
+			&& strchr (Info_ValueForKey (host_client->userinfo, "*cap"), 'h');
 
 	realname = dstring_newstr ();
 	size = _QFS_FOpenFile (name, &file, realname, !zip);
@@ -726,20 +772,41 @@ SV_BeginDownload_f (void *unused)
 
 		SV_Printf ("Couldn't download %s to %s\n", name, host_client->name);
 		MSG_ReliableWrite_Begin (&host_client->backbuf, svc_download, 4);
-		MSG_ReliableWrite_Short (&host_client->backbuf, -1);
+		MSG_ReliableWrite_Short (&host_client->backbuf, DL_NOFILE);
 		MSG_ReliableWrite_Byte (&host_client->backbuf, 0);
 		dstring_delete (realname);
 		return;
 	}
 
-	if (zip && strcmp (realname->str, name)) {
-		SV_Printf ("download renamed to %s\n", realname->str);
-		MSG_ReliableWrite_Begin (&host_client->backbuf, svc_download,
-								 strlen (realname->str) + 5);
-		MSG_ReliableWrite_Short (&host_client->backbuf, -2);
+	if (http) {
+		int         size;
+		int         ren = zip && strcmp (realname->str, name);
+		SV_Printf ("http redirect: %s/%s\n", sv_http_url_base->string,
+				   realname->str);
+		size = ren ? strlen (realname->str) * 2 : strlen (name);
+		size += strlen (sv_http_url_base->string) + 7;
+		MSG_ReliableWrite_Begin (&host_client->backbuf, svc_download, size);
+		MSG_ReliableWrite_Short (&host_client->backbuf, DL_HTTP);
 		MSG_ReliableWrite_Byte (&host_client->backbuf, 0);
-		MSG_ReliableWrite_String (&host_client->backbuf, realname->str);
-		MSG_Reliable_FinishWrite (&host_client->backbuf);
+		MSG_ReliableWrite_String (&host_client->backbuf,
+								  va ("%s/%s", sv_http_url_base->string,
+									  ren ? realname->str : name));
+		MSG_ReliableWrite_String (&host_client->backbuf,
+								  ren ? realname->str : "");
+		if (host_client->download) {
+			Qclose (host_client->download);
+			host_client->download = NULL;
+		}
+	} else {
+		if (zip && strcmp (realname->str, name)) {
+			SV_Printf ("download renamed to %s\n", realname->str);
+			MSG_ReliableWrite_Begin (&host_client->backbuf, svc_download,
+									 strlen (realname->str) + 5);
+			MSG_ReliableWrite_Short (&host_client->backbuf, DL_RENAME);
+			MSG_ReliableWrite_Byte (&host_client->backbuf, 0);
+			MSG_ReliableWrite_String (&host_client->backbuf, realname->str);
+			MSG_Reliable_FinishWrite (&host_client->backbuf);
+		}
 	}
 	dstring_delete (realname);
 
@@ -757,6 +824,7 @@ SV_Say (qboolean team)
 	const char *t1 = 0, *t2, *type, *fmt;
 	client_t   *client;
 	int			tmp, j, cls = 0;
+	sizebuf_t  *dbuf;
 
 	if (Cmd_Argc () < 2)
 		return;
@@ -822,8 +890,9 @@ SV_Say (qboolean team)
 
 	if (sv_funcs.ChatMessage) {
 		PR_PushFrame (&sv_pr_state);
+		PR_RESET_PARAMS (&sv_pr_state);
 		P_STRING (&sv_pr_state, 0) = PR_SetTempString (&sv_pr_state, p);
-		G_FLOAT (&sv_pr_state, 1) = (float) team;
+		P_FLOAT (&sv_pr_state, 1) = (float) team;
 
 		*sv_globals.time = sv.time;
 		*sv_globals.self = EDICT_TO_PROG (&sv_pr_state, sv_player);
@@ -877,7 +946,7 @@ SV_Say (qboolean team)
 		SV_ClientPrintf (0, client, PRINT_CHAT, "%s", text->str);
 	}
 
-	if (!sv.demorecording || !cls) {
+	if (!sv.recorders || !cls) {
 		dstring_delete (text);
 		return;
 	}
@@ -885,13 +954,13 @@ SV_Say (qboolean team)
 	// player
 	if (!team && ((host_client->spectator && sv_spectalk->value)
 				  || !host_client->spectator)) {
-		DemoWrite_Begin (dem_all, 0, strlen (text->str) + 3);
+		dbuf = SVR_WriteBegin (dem_all, 0, strlen (text->str) + 3);
 	} else {
-		DemoWrite_Begin (dem_multiple, cls, strlen (text->str) + 3);
+		dbuf = SVR_WriteBegin (dem_multiple, cls, strlen (text->str) + 3);
 	}
-	MSG_WriteByte (&demo.dbuf->sz, svc_print);
-	MSG_WriteByte (&demo.dbuf->sz, PRINT_CHAT);
-	MSG_WriteString (&demo.dbuf->sz, text->str);
+	MSG_WriteByte (dbuf, svc_print);
+	MSG_WriteByte (dbuf, PRINT_CHAT);
+	MSG_WriteString (dbuf, text->str);
 	dstring_delete (text);
 }
 
@@ -1117,6 +1186,7 @@ void
 SV_SetUserinfo (client_t *client, const char *key, const char *value)
 {
 	char       *oldvalue = 0;
+	int         send_changes = 1;
 
 	if (sv_setinfo_e->func || sv_funcs.UserInfoChanged)
 		oldvalue = strdup (Info_ValueForKey (client->userinfo, key));
@@ -1140,17 +1210,19 @@ SV_SetUserinfo (client_t *client, const char *key, const char *value)
 		PR_PushFrame (&sv_pr_state);
 		*sv_globals.time = sv.time;
 		*sv_globals.self = EDICT_TO_PROG (&sv_pr_state, client->edict);
+		PR_RESET_PARAMS (&sv_pr_state);
 		P_STRING (&sv_pr_state, 0) = PR_SetTempString (&sv_pr_state, key);
 		P_STRING (&sv_pr_state, 1) = PR_SetTempString (&sv_pr_state, oldvalue);
 		P_STRING (&sv_pr_state, 2) = PR_SetTempString (&sv_pr_state, value);
 		PR_ExecuteProgram (&sv_pr_state, sv_funcs.UserInfoChanged);
 		PR_PopFrame (&sv_pr_state);
+		send_changes = !R_FLOAT (&sv_pr_state);
 	}
 
 	if (oldvalue)
 		free (oldvalue);
 
-	if (Info_FilterForKey (key, client_info_filters)) {
+	if (send_changes && Info_FilterForKey (key, client_info_filters)) {
 		MSG_WriteByte (&sv.reliable_datagram, svc_setinfo);
 		MSG_WriteByte (&sv.reliable_datagram, client - svs.clients);
 		MSG_WriteString (&sv.reliable_datagram, key);
@@ -1189,11 +1261,13 @@ SV_SetInfo_f (void *unused)
 	if (sv_funcs.UserInfoCallback) {
 		PR_PushFrame (&sv_pr_state);
 		*sv_globals.self = EDICT_TO_PROG (&sv_pr_state, sv_player);
+		PR_RESET_PARAMS (&sv_pr_state);
 		P_STRING (&sv_pr_state, 0) = PR_SetTempString (&sv_pr_state, key);
 		P_STRING (&sv_pr_state, 1) = PR_SetTempString (&sv_pr_state, value);
 		PR_ExecuteProgram (&sv_pr_state, sv_funcs.UserInfoCallback);
 		PR_PopFrame (&sv_pr_state);
-		return;
+		if (R_FLOAT (&sv_pr_state))
+			return;
 	}
 
 	SV_SetUserinfo (host_client, key, value);
@@ -1232,7 +1306,7 @@ ucmd_t      ucmds[] = {
 	{"drop",		SV_Drop_f,			0, 0},
 	{"pings",		SV_Pings_f,			0, 0},
 
-// issued by hand at client consoles    
+// issued by hand at client consoles
 	{"rate",		SV_Rate_f,			0, 0},
 	{"kill",		SV_Kill_f,			1, 1},
 	{"pause",		SV_Pause_f,			1, 0},
@@ -1261,11 +1335,11 @@ static void
 call_qc_hook (void *qc_hook)
 {
 	*sv_globals.self = EDICT_TO_PROG (&sv_pr_state, sv_player);
-	PR_ExecuteProgram (&sv_pr_state, (func_t) (long) qc_hook);
+	PR_ExecuteProgram (&sv_pr_state, (func_t) (intptr_t) qc_hook);
 }
 
 static const char *
-ucmds_getkey (void *_a, void *unused)
+ucmds_getkey (const void *_a, void *unused)
 {
 	ucmd_t *a = (ucmd_t*)_a;
 	return a->name;
@@ -1291,15 +1365,15 @@ ucmds_free (void *_c, void *unused)
 	pointer.
 */
 
-static unsigned long
-ucmd_get_hash (void *_a, void *data)
+static uintptr_t
+ucmd_get_hash (const void *_a, void *data)
 {
 	ucmd_t *a = (ucmd_t*)_a;
 	return Hash_String (a->name);
 }
 
 static int
-ucmd_compare (void *a, void *b, void *data)
+ucmd_compare (const void *a, const void *b, void *data)
 {
 	return a == b;
 }
@@ -1330,22 +1404,22 @@ SV_AddUserCommand (const char *name, void (*func) (void *userdata), int flags,
 	cmd->userdata = userdata;
 	cmd->on_free = on_free;
 	Hash_Add (ucmd_table, cmd);
-	
+
 	return cmd;
 }
 
 /*
-	Removes a user command added with SV_AddUserCommand.  Returns true if 
+	Removes a user command added with SV_AddUserCommand.  Returns true if
 	successful, false if not.
 */
 int
 SV_RemoveUserCommand (void *cmd)
 {
 	void *ele = Hash_DelElement(ucmd_table, cmd);
-	
+
 	if (!ele)
 		return 0;
-	
+
 	Hash_Free (ucmd_table, ele);
 	return 1;
 }
@@ -1355,11 +1429,11 @@ PF_AddUserCommand (progs_t *pr)
 {
 	const char *name = P_GSTRING (pr, 0);
 	ucmd_t     *cmd;
-	cmd = SV_AddUserCommand (name, call_qc_hook, 
+	cmd = SV_AddUserCommand (name, call_qc_hook,
 							 P_INT (pr, 2) ? UCMD_NO_REDIRECT : 0,
-							 (void *) (long) P_FUNCTION (pr, 1),
+							 (void *) (intptr_t) P_FUNCTION (pr, 1),
 							 NULL);
-	
+
 	if (!cmd)
 		SV_Printf ("%s already a user command\n", name);
 }
@@ -1391,7 +1465,7 @@ SV_ExecuteUserCommand (const char *s)
 	u = (ucmd_t*) Hash_Find (ucmd_table, sv_args->argv[0]->str);
 
 	if (!u) {
-		if (ucmd_unknown && !ucmd_unknown ()) {
+		if (!ucmd_unknown || !ucmd_unknown ()) {
 			SV_BeginRedirect (RD_CLIENT);
 			SV_Printf ("Bad user command: %s\n", sv_args->argv[0]->str);
 			SV_EndRedirect ();
@@ -1473,6 +1547,7 @@ AddLinksToPmove (areanode_t *node)
 			pmove.numphysent++;
 
 			VectorCopy (SVvector (check, origin), pe->origin);
+			VectorCopy (SVvector (check, angles), pe->angles);
 			pe->info = NUM_FOR_EDICT (&sv_pr_state, check);
 
 			if (sv_fields.rotated_bbox != -1
@@ -1733,7 +1808,7 @@ SV_RunCmd (usercmd_t *ucmd, qboolean inside)
 
 		// touch other objects
 		for (i = 0; i < pmove.numtouch; i++) {
-			n = pmove.physents[pmove.touchindex[i]].info;
+			n = pmove.touchindex[i]->info;
 			ent = EDICT_NUM (&sv_pr_state, n);
 			if (!SVfunc (ent, touch) || (playertouch[n / 8] & (1 << (n % 8))))
 				continue;
@@ -1778,23 +1853,43 @@ SV_ExecuteClientMessage (client_t *cl)
 	client_frame_t *frame;
 	int         checksumIndex, seq_hash, c;
 	usercmd_t   oldest, oldcmd, newcmd;
-	qboolean    move_issued = false;	// only allow one move command
+	qboolean    move_issued = false;	// allow only one move command
 	vec3_t      o;
 
-	// calc ping time
-	frame = &cl->frames[cl->netchan.incoming_acknowledged & UPDATE_MASK];
-	frame->ping_time = realtime - frame->senttime;
-
 	// make sure the reply sequence number matches the incoming
-	// sequence number 
+	// sequence number
 	if (cl->netchan.incoming_sequence >= cl->netchan.outgoing_sequence)
 		cl->netchan.outgoing_sequence = cl->netchan.incoming_sequence;
 	else
 		cl->send_message = false;		// don't reply, sequences have slipped
+
+	// setup delta information
+	cl->delta.cur_frame = cl->netchan.incoming_acknowledged & UPDATE_MASK;
+	cl->delta.out_frame = cl->netchan.outgoing_sequence & UPDATE_MASK;
+	cl->delta.in_frame = cl->netchan.incoming_sequence & UPDATE_MASK;
+
+	// calc ping time
+	frame = &cl->delta.frames[cl->delta.cur_frame];
+	frame->ping_time = realtime - frame->senttime;
+
+	cl->laggedents_count = 0;
+	if (sv_antilag->int_val) {
+		int         i;
+
+		for (i = 0; i < MAX_CLIENTS; i++) {
+			cl->laggedents[i].present = frame->playerpresent[i];
+			if (cl->laggedents[i].present) {
+				VectorCopy(frame->playerpositions[i],
+						   cl->laggedents[i].laggedpos);
+			}
+		}
+		cl->laggedents_count = MAX_CLIENTS;
+		cl->laggedents_frac = sv_antilag_frac->value;
+	}
+
 	// save time for ping calculations
-	cl->frames[cl->netchan.outgoing_sequence & UPDATE_MASK].senttime =
-		realtime;
-	cl->frames[cl->netchan.outgoing_sequence & UPDATE_MASK].ping_time = -1;
+	cl->delta.frames[cl->delta.out_frame].senttime = realtime;
+	cl->delta.frames[cl->delta.out_frame].ping_time = -1;
 
 	host_client = cl;
 	sv_player = host_client->edict;
@@ -1804,7 +1899,7 @@ SV_ExecuteClientMessage (client_t *cl)
 
 	// mark time so clients will know how much to predict other players
 	cl->localtime = sv.time;
-	cl->delta_sequence = -1;			// no delta unless requested
+	cl->delta.delta_sequence = -1;			// no delta unless requested
 	while (1) {
 		if (net_message->badread) {
 			SV_Printf ("SV_ReadClientMessage: badread\n");
@@ -1826,7 +1921,7 @@ SV_ExecuteClientMessage (client_t *cl)
 				break;
 
 			case clc_delta:
-				cl->delta_sequence = MSG_ReadByte (net_message);
+				cl->delta.delta_sequence = MSG_ReadByte (net_message);
 				break;
 
 			case clc_move:
@@ -1841,9 +1936,9 @@ SV_ExecuteClientMessage (client_t *cl)
 				// read loss percentage
 				cl->lossage = MSG_ReadByte (net_message);
 
-				MSG_ReadDeltaUsercmd (&nullcmd, &oldest);
-				MSG_ReadDeltaUsercmd (&oldest, &oldcmd);
-				MSG_ReadDeltaUsercmd (&oldcmd, &newcmd);
+				MSG_ReadDeltaUsercmd (net_message, &nullcmd, &oldest);
+				MSG_ReadDeltaUsercmd (net_message, &oldest, &oldcmd);
+				MSG_ReadDeltaUsercmd (net_message, &oldcmd, &newcmd);
 
 				if (cl->state != cs_spawned)
 					break;
@@ -1856,24 +1951,25 @@ SV_ExecuteClientMessage (client_t *cl)
 											  checksumIndex - 1, seq_hash);
 
 				if (calculatedChecksum != checksum) {
-					Con_DPrintf
-						("Failed command checksum for %s(%d) (%d != %d)\n",
-						 cl->name, cl->netchan.incoming_sequence, checksum,
-						 calculatedChecksum);
+					Sys_MaskPrintf (SYS_DEV,
+									"Failed command checksum for %s(%d) "
+									"(%d != %d)\n",
+									cl->name, cl->netchan.incoming_sequence,
+									checksum, calculatedChecksum);
 					return;
 				}
 
 				if (!sv.paused) {
 					SV_PreRunCmd ();
 
-					if (net_drop < 20) {
-						while (net_drop > 2) {
+					if (cl->netchan.net_drop < 20) {
+						while (cl->netchan.net_drop > 2) {
 							SV_RunCmd (&cl->lastcmd, 0);
-							net_drop--;
+							cl->netchan.net_drop--;
 						}
-						if (net_drop > 1)
+						if (cl->netchan.net_drop > 1)
 							SV_RunCmd (&oldest, 0);
-						if (net_drop > 0)
+						if (cl->netchan.net_drop > 0)
 							SV_RunCmd (&oldcmd, 0);
 					}
 					SV_RunCmd (&newcmd, 0);
@@ -1892,7 +1988,7 @@ SV_ExecuteClientMessage (client_t *cl)
 
 			case clc_tmove:
 				MSG_ReadCoordV (net_message, o);
-				// only allowed by spectators
+				// allowed for only spectators
 				if (host_client->spectator) {
 					VectorCopy (o, SVvector (sv_player, origin));
 					SV_LinkEdict (sv_player, false);
@@ -1923,8 +2019,18 @@ SV_UserInit (void)
 	cl_rollangle = Cvar_Get ("cl_rollangle", "2", CVAR_NONE, NULL, "How much "
 							 "a player's screen tilts when strafing");
 
-	sv_allowfake = Cvar_Get ("sv_allowfake", "2", CVAR_NONE, NULL, "Allow 'fake' messages (FuhQuake $\\). 1 = " 
-								"always, 2 = only say_team");
+	sv_antilag = Cvar_Get ("sv_antilag", "1", CVAR_SERVERINFO, Cvar_Info,
+						   "Attempt to backdate impacts to compensate for "
+						   "lag. 0=completely off. 1=mod-controlled. "
+						   "2=forced, which might break certain uses of "
+						   "traceline.");
+	sv_antilag_frac = Cvar_Get ("sv_antilag_frac", "1", CVAR_SERVERINFO,
+								Cvar_Info,
+								"FIXME something to do with sv_antilag");
+
+	sv_allowfake = Cvar_Get ("sv_allowfake", "2", CVAR_NONE, NULL,
+							 "Allow 'fake' messages (FuhQuake $\\). 1 = "
+							 "always, 2 = only say_team");
 	sv_spectalk = Cvar_Get ("sv_spectalk", "1", CVAR_NONE, NULL, "Toggles "
 							"the ability of spectators to talk to players");
 	sv_mapcheck = Cvar_Get ("sv_mapcheck", "1", CVAR_NONE, NULL, "Toggle the "
@@ -1948,22 +2054,29 @@ SV_UserInit (void)
 								   "\"forgiven\".");
 	sv_kickfake = Cvar_Get ("sv_kickfake", "0", CVAR_NONE, NULL,
 							"Kick users sending to send fake talk messages");
+	sv_http_url_base = Cvar_Get ("sv_http_url_base", "", CVAR_NONE, NULL,
+								 "set to base url for http redirects of "
+								 "downloaded files");
 }
 
 static void
 OutofBandPrintf (netadr_t where, const char *fmt, ...)
 {
-	char		send[1024];
+	// 0 for davsprintf
+	static const char header[] = {0xff, 0xff, 0xff, 0xff, A2C_PRINT, 0};
+	static dstring_t *send = 0;
 	va_list		argptr;
+	int         len;
 
-	send[0] = 0xff;
-	send[1] = 0xff;
-	send[2] = 0xff;
-	send[3] = 0xff;
-	send[4] = A2C_PRINT;
+	if (!send)
+		send = dstring_new ();
+	send->size = sizeof (header);
+	dstring_adjust (send);
+	memcpy (send->str, header, sizeof (header));
 	va_start (argptr, fmt);
-	vsnprintf (send + 5, sizeof (send) - 5, fmt, argptr);
+	davsprintf (send, fmt, argptr);
 	va_end (argptr);
 
-	Netchan_SendPacket (strlen (send) + 1, send, where);
+	len = min (send->size, 1024);
+	Netchan_SendPacket (len, send->str, where);
 }
