@@ -1,14 +1,14 @@
-#include "gui/Point.h"
+#include "gui/Size.h"
 #include "draw.h"
 
 @interface Frame : Object
 {
-	QPic picture;
+	QPic *picture;
 	float duration;
 }
 - (id) initWithFile: (string) file duration: (float) time;
 - (void) dealloc;
-- (Point) size;
+- (Size) size;
 - (float) duration;
-- (void) draw: (integer) x :(integer) y;
+- (void) draw: (int) x :(int) y;
 @end
