@@ -13,13 +13,13 @@
 
 - (void) dealloc
 {
-	[picture dealloc];
+	[picture release];
 	[super dealloc];
 }
 
-- (Point) size
+- (Size) size
 {
-	return [[Point alloc] initWithComponents :[picture width] :[picture height]];
+	return makeSize ([picture width], [picture height]);
 }
 
 - (float) duration
@@ -27,7 +27,7 @@
 	return duration;
 }
 
-- (void) draw: (integer) x :(integer) y
+- (void) draw: (int) x :(int) y
 {
 	[picture draw :x :y];
 }
