@@ -1,6 +1,6 @@
 #ifndef __world_h
 #define __world_h
 
-@extern integer deathmatch;
+@extern int deathmatch;
 
 #endif//__world_h
