@@ -2,18 +2,18 @@
 #include "World.h"
 
 id		world;
-integer	deathmatch;
+int	deathmatch;
 
 #define MAX_BODIES 8
 
 @interface BodyQueue: Object
 {
-	entity [MAX_BODIES]	bodies;
-	integer				head;
+	entity	bodies[MAX_BODIES];
+	int	head;
 }
 
 - (id) init;
-- (void) addEntity: (GameEntity)ent;
+- (void) addEntity: (GameEntity *)ent;
 
 @end
 
@@ -21,24 +21,25 @@ integer	deathmatch;
 
 - (id) init
 {
-	local integer	i;
+	local int	i;
 	self = [super init];
 
-	self.head = NIL;
+	head = nil;
 
 	for (i = 0; i < MAX_BODIES; i++) {
-		local GameEntity	ent = NIL;
+		local GameEntity*	ent = nil;
 		ent = [[GameEntity alloc] init];
-		self.bodies[i] = ent.ent;
+		bodies[i] = ent.ent;
 	}
 #if 0
 	for (i = 0; i < MAX_BODIES; i++) {
-		self.bodies[i] = [[[GameEntity alloc] init] ent];
+		bodies[i] = [[[GameEntity alloc] init] ent];
 	}
 #endif
+	return self;
 }
 
-- (void) addEntity: (GameEntity)ent
+- (void) addEntity: (GameEntity*)ent
 {
 	local entity	be = bodies[head++];
 	local entity	e = [ent ent];
@@ -63,7 +64,7 @@ integer	deathmatch;
 }
 
 - (void) spawn: (entity)ent;
-- (void) copyToBodyQueue: (GameEntity)ent;
+- (void) copyToBodyQueue: (GameEntity *)ent;
 
 @end
 
@@ -75,7 +76,7 @@ integer	deathmatch;
 	bodyque = [[BodyQueue alloc] init];
 }
 
-- (void) copyToBodyQueue: (GameEntity)ent
+- (void) copyToBodyQueue: (GameEntity *)ent
 {
 	[bodyque addEntity: ent];
 }
