@@ -5,14 +5,14 @@
 
 @implementation View
 
-- (id) initWithComponents: (integer)x : (integer)y : (integer)w : (integer)h
+- (id) initWithComponents: (int)x : (int)y : (int)w : (int)h
 {
 	self = [self init];
 	xpos = xabs = x;
 	ypos = yabs = y;
 	xlen = w;
 	ylen = h;
-	parent = NIL;
+	parent = nil;
 	return self;
 }
 
@@ -27,15 +27,32 @@
 	return [self initWithOrigin:aRect.origin size:aRect.size];
 }
 
-- (void) setBasePos: (integer) x y: (integer) y
+- (id) canFocus: (int) cf
 {
-	local Point point = [[Point alloc] initWithComponents:x :y];
-	[self setBasePos:point];
-	[point dealloc];
+	flags |= 1;
+	return self;
 }
 
-- (void) setBasePos: (Point)pos
+- (int) canFocus
 {
+	return flags & 1;
+}
+
+- (Point) basePos
+{
+	return makePoint (xabs, yabs);
+}
+
+- (void) setBasePos: (int) x y: (int) y
+{
+	local Point point = {x, y};
+	xabs = xpos + x;
+	yabs = ypos + y;
+}
+
+- (void) setBasePosFromView: (View *) view
+{
+	Point pos = [view basePos];
 	xabs = xpos + pos.x;
 	yabs = ypos + pos.y;
 }
@@ -44,4 +61,9 @@
 {
 }
 
+- (int) keyEvent:(int)key unicode:(int)unicode down:(int)down
+{
+	return 0;
+}
+
 @end
