@@ -18,11 +18,15 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <errno.h>
 #include <string.h>
 #include <math.h>
@@ -31,7 +35,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "QF/bspfile.h"
 #include "QF/cmd.h"
 #include "QF/cvar.h"
+#include "QF/mathlib.h"
 #include "QF/pcx.h"
+#include "QF/png.h"
+#include "QF/quakefs.h"
 #include "QF/quakeio.h"
 #include "QF/sys.h"
 #include "QF/zone.h"
@@ -70,7 +77,7 @@ typedef struct edge_extra_t {
 	long        ref_faces[MAX_REF_FACES];	// which faces are referenced
 	dvertex_t   ref_faces_normal[MAX_REF_FACES];	// normal of referenced
 													// faces
-	int         ref_faces_area[MAX_REF_FACES];	// area of the referenced faces 
+	int         ref_faces_area[MAX_REF_FACES];	// area of the referenced faces
 } edge_extra_t;
 
 typedef unsigned char eightbit;
@@ -78,6 +85,7 @@ typedef unsigned char eightbit;
 typedef struct options_t {
 	char       *bspf_name;
 	char       *outf_name;
+	int         outf_type;
 
 	float       scaledown;
 	float       z_pad;
@@ -142,14 +150,16 @@ plotpoint (image_t *image, long xco, long yco, unsigned int color)
 {
 	unsigned int bigcol = 0;
 
-	if (xco < 0 || xco > image->width || yco < 0 || yco > image->height)
+	if (image->width <= 0 || image->height <= 0
+		|| xco < 0 || xco >= image->width
+		|| yco < 0 || yco >= image->height)
 		return;
 
 	bigcol = (unsigned int) image->image[yco * image->width + xco];
 
 	bigcol += color;
 
-	bigcol = bound (0, bigcol, 255);
+	bigcol = min (bigcol, 255);
 
 	image->image[yco * image->width + xco] = (eightbit) bigcol;
 
@@ -227,6 +237,7 @@ def_options (struct options_t *opt)
 
 	locopt.bspf_name = NULL;
 	locopt.outf_name = NULL;
+	locopt.outf_type = 0;
 
 	locopt.scaledown = 4.0;
 	locopt.image_pad = 16.0;
@@ -382,7 +393,17 @@ get_options (struct options_t *opt, int argc, char *argv[])
 			if (locopt.bspf_name == NULL) {
 				locopt.bspf_name = arg;
 			} else if (locopt.outf_name == NULL) {
+				const char *ext;
 				locopt.outf_name = arg;
+				ext = QFS_FileExtension (locopt.outf_name);
+				if (strcmp (ext, ".pcx") == 0)
+					locopt.outf_type = 0;
+				else if (strcmp (ext, ".png") == 0)
+					locopt.outf_type = 1;
+				else {
+					printf ("Unknown output format: %s\n", ext);
+					exit (1);
+				}
 			} else {
 				printf ("Unknown option: %s\n", arg);
 				show_help ();
@@ -572,23 +593,14 @@ render_map (bsp_t *bsp)
 
 				e1 = &edgelist[abs (ledges[j])];
 				e2 = &edgelist[abs (ledges[k])];
+				//FIXME verify directions
 				if (ledges[j] > 0) {
-					v0.X = vertexlist[e1->v[0]].X - vertexlist[e1->v[1]].X;
-					v0.Y = vertexlist[e1->v[0]].Y - vertexlist[e1->v[1]].Y;
-					v0.Z = vertexlist[e1->v[0]].Z - vertexlist[e1->v[1]].Z;
-
-					v1.X = vertexlist[e2->v[0]].X - vertexlist[e2->v[1]].X;
-					v1.Y = vertexlist[e2->v[0]].Y - vertexlist[e2->v[1]].Y;
-					v1.Z = vertexlist[e2->v[0]].Z - vertexlist[e2->v[1]].Z;
+					SUB (vertexlist[e1->v[0]], vertexlist[e1->v[1]], v0);
+					SUB (vertexlist[e2->v[0]], vertexlist[e2->v[1]], v1);
 				} else {
 					/* negative index, therefore walk in reverse order */
-					v0.X = vertexlist[e1->v[1]].X - vertexlist[e1->v[0]].X;
-					v0.Y = vertexlist[e1->v[1]].Y - vertexlist[e1->v[0]].Y;
-					v0.Z = vertexlist[e1->v[1]].Z - vertexlist[e1->v[0]].Z;
-
-					v1.X = vertexlist[e2->v[1]].X - vertexlist[e2->v[0]].X;
-					v1.Y = vertexlist[e2->v[1]].Y - vertexlist[e2->v[0]].Y;
-					v1.Z = vertexlist[e2->v[1]].Z - vertexlist[e2->v[0]].Z;
+					SUB (vertexlist[e1->v[1]], vertexlist[e1->v[0]], v0);
+					SUB (vertexlist[e2->v[1]], vertexlist[e2->v[0]], v1);
 				}
 
 				/* cross product */
@@ -655,7 +667,7 @@ render_map (bsp_t *bsp)
 				vertexlist[i].Z = tempf;
 				break;
 
-			case -2:					/* -Y -- (+x <--> -x; -y out of screen, 
+			case -2:					/* -Y -- (+x <--> -x; -y out of screen,
 										   +z up) */
 				vertexlist[i].X = -vertexlist[i].X;
 				tempf = vertexlist[i].Z;
@@ -663,14 +675,14 @@ render_map (bsp_t *bsp)
 				vertexlist[i].Y = tempf;;
 				break;
 
-			case 2:					/* +Y -- (-x <--> +x; +y out of screen, 
+			case 2:					/* +Y -- (-x <--> +x; +y out of screen,
 										   +z up) */
 				tempf = vertexlist[i].Z;
 				vertexlist[i].Z = -vertexlist[i].Y;
 				vertexlist[i].Y = tempf;;
 				break;
 
-			case -3:					/* -Z -- negate X and Z (ie. 180 rotate 
+			case -3:					/* -Z -- negate X and Z (ie. 180 rotate
 										   along Y axis) */
 				vertexlist[i].X = -vertexlist[i].X;
 				vertexlist[i].Z = -vertexlist[i].Z;
@@ -719,10 +731,10 @@ render_map (bsp_t *bsp)
 	{
 		long        width, height;
 
-		width = (maxX - minX) / options.scaledown;
+		width = (maxX - minX + 1) / options.scaledown;
 		width += (options.image_pad + options.z_pad) * 2;
 
-		height = (maxY - minY) / options.scaledown;
+		height = (maxY - minY + 1) / options.scaledown;
 		height += (options.image_pad + options.z_pad) * 2;
 
 		image = create_image (width, height);
@@ -816,7 +828,7 @@ render_map (bsp_t *bsp)
 			Zoffset1 = (options.z_pad * (vert2->Z - midZ) / (maxZ - minZ));
 
 			offs0 = options.image_pad + options.z_pad + (Zoffset0 * Z_Xdir);
-			offs1 = options.image_pad + options.z_pad + (Zoffset1 * Z_Xdir);
+			offs1 = options.image_pad + options.z_pad + (Zoffset1 * Z_Ydir);
 
 			bresline (image, (vert1->X - minX) / options.scaledown + offs0,
 							 (vert1->Y - minY) / options.scaledown + offs0,
@@ -873,11 +885,60 @@ render_map (bsp_t *bsp)
 	return image;
 }
 
+static void
+write_png (image_t *image)
+{
+	byte       *data, *in, *out, b;
+	int         size = image->width * image->height;
+
+	out = data = malloc (size * 3);
+	for (in = image->image; in - image->image < size; in++) {
+		b = *in;
+		*out++ = b;
+		*out++ = b;
+		*out++ = b;
+	}
+	WritePNG (options.outf_name, data, image->width, image->height);
+}
+
+static void
+write_pcx (image_t *image)
+{
+	pcx_t      *pcx;
+	int         pcx_len, i;
+	byte        palette[768];
+	QFile      *outfile;
+
+	outfile = Qopen (options.outf_name, "wb");
+	if (outfile == NULL) {
+		fprintf (stderr, "Error opening output file %s.\n", options.outf_name);
+
+		exit (1);
+	}
+
+	// quick and dirty greyscale palette
+	for (i = 0; i < 256; i++) {
+		palette[i * 3 + 0] = i;
+		palette[i * 3 + 1] = i;
+		palette[i * 3 + 2] = i;
+	}
+
+	Sys_Init ();
+
+	Memory_Init (malloc (MEMSIZE), MEMSIZE);
+	pcx = EncodePCX (image->image, image->width, image->height,
+					 image->width, palette, false, &pcx_len);
+	if (Qwrite (outfile, pcx, pcx_len) != pcx_len) {
+		fprintf (stderr, "Error writing to %s\n", options.outf_name);
+		exit (1);
+	}
+	Qclose (outfile);
+}
+
 int
 main (int argc, char *argv[])
 {
 	QFile      *bspfile;
-	QFile      *outfile;
 	bsp_t      *bsp;
 	image_t    *image;
 
@@ -901,43 +962,20 @@ main (int argc, char *argv[])
 	Qclose (bspfile);
 
 	image = render_map (bsp);
+	BSP_Free (bsp);
 
 	/* Write image */
 
-	outfile = Qopen (options.outf_name, "wb");
-	if (outfile == NULL) {
-		fprintf (stderr, "Error opening output file %s.\n", options.outf_name);
-
-		return 1;
-	} else {
-		pcx_t      *pcx;
-		int         pcx_len, i;
-		byte        palette[768];
-
-		// quick and dirty greyscale palette
-		for (i = 0; i < 256; i++) {
-			palette[i * 3 + 0] = i;
-			palette[i * 3 + 1] = i;
-			palette[i * 3 + 2] = i;
-		}
-
-		Cvar_Init_Hash ();
-		Cmd_Init_Hash ();
-		Cvar_Init ();
-		Sys_Init_Cvars ();
-		Cmd_Init ();
-
-		Memory_Init (malloc (MEMSIZE), MEMSIZE);
-		pcx = EncodePCX (image->image, image->width, image->height,
-						 image->width, palette, false, &pcx_len);
-		if (Qwrite (outfile, pcx, pcx_len) != pcx_len) {
-			fprintf (stderr, "Error writing to %s\n", options.outf_name);
-			return 1;
-		}
+	switch (options.outf_type) {
+		case 0:
+			write_pcx (image);
+			break;
+		case 1:
+			write_png (image);
+			break;
 	}
 
 	printf ("File written to %s.\n", options.outf_name);
-	Qclose (outfile);
 
 	/* Close, done! */
 
