@@ -28,10 +28,19 @@
 #endif
 
 #include "QF/sys.h"
+#include "QF/va.h"
 
 #include "compat.h"
+
+#include "brush.h"
 #include "bsp5.h"
+#include "draw.h"
 #include "options.h"
+#include "surfaces.h"
+
+/**	\addtogroup qfbsp_brush
+*/
+//@{
 
 int         numbrushplanes;
 plane_t     planes[MAX_MAP_PLANES];
@@ -41,13 +50,25 @@ mface_t     faces[MAX_FACES];	// beveled clipping hull can generate many extra
 
 static entity_t *CurrentEntity;
 
-/*
-	CheckFace
+brush_t *
+AllocBrush (void)
+{
+	brush_t    *b;
 
-	Note: this will not catch 0 area polygons
+	b = malloc (sizeof (brush_t));
+	memset (b, 0, sizeof (brush_t));
+
+	return b;
+}
+
+/**	Check the face for validity.
+
+	\param f		The face to check.
+
+	\note Does not catch 0 area polygons.
 */
 static void
-CheckFace (face_t *f)
+CheckFace (const face_t *f)
 {
 	int		 i, j;
 	vec_t	*p1, *p2;
@@ -71,8 +92,7 @@ CheckFace (face_t *f)
 		j = i + 1 == f->points->numpoints ? 0 : i + 1;
 
 		// check the point is on the face plane
-		d = DotProduct (p1, planes[f->planenum].normal)
-			- planes[f->planenum].dist;
+		d = PlaneDiff (p1, &planes[f->planenum]);
 
 		// point off plane autofix
 		if (d < -ON_EPSILON || d > ON_EPSILON)
@@ -104,20 +124,30 @@ CheckFace (face_t *f)
 	}
 }
 
+/**	Initialize the bounding box of the brush set.
+
+	\param bs		The brush set of which to initialize the bounding box.
+*/
 static void
 ClearBounds (brushset_t *bs)
 {
 	int		i, j;
 
-	for (j = 0; j < NUM_HULLS; j++)
+	for (j = 0; j < NUM_HULLS; j++) {
 		for (i = 0; i < 3; i++) {
 			bs->mins[i] = BOGUS_RANGE;
 			bs->maxs[i] = -BOGUS_RANGE;
 		}
+	}
 }
 
+/**	Grow the bounding box of the brush set to include the vector.
+
+	\param bs		The brush set of which to grown the bounding box.
+	\param v		The vector to be included in the bounding box.
+*/
 static void
-AddToBounds (brushset_t *bs, vec3_t v)
+AddToBounds (brushset_t *bs, const vec3_t v)
 {
 	int		i;
 
@@ -130,9 +160,10 @@ AddToBounds (brushset_t *bs, vec3_t v)
 }
 
 int
-PlaneTypeForNormal (vec3_t normal)
+PlaneTypeForNormal (const vec3_t normal)
 {
 	float	ax, ay, az;
+	int     type;
 
 	// NOTE: should these have an epsilon around 1.0?
 	if (normal[0] == 1.0)
@@ -142,77 +173,89 @@ PlaneTypeForNormal (vec3_t normal)
 	if (normal[2] == 1.0)
 		return PLANE_Z;
 	if (normal[0] == -1.0 || normal[1] == -1.0 || normal[2] == -1.0)
-		Sys_Error ("PlaneTypeForNormal: not a canonical vector");
+		Sys_Error ("PlaneTypeForNormal: not a canonical vector (%g %g %g)",
+				   normal[0], normal[1], normal[2]);
 
 	ax = fabs(normal[0]);
 	ay = fabs(normal[1]);
 	az = fabs(normal[2]);
 
-	if (ax >= ay && ax >= az)
-		return PLANE_ANYX;
-	if (ay >= ax && ay >= az)
-		return PLANE_ANYY;
-	return PLANE_ANYZ;
+	if (az >= ax && az >= ay)
+		type = PLANE_ANYZ;
+	else if (ay >= ax && ay >= az)
+		type = PLANE_ANYY;
+	else
+		type = PLANE_ANYX;
+	if (normal[type - PLANE_ANYX] < 0)
+		Sys_Error ("PlaneTypeForNormal: not a canonical vector (%g %g %g) %d",
+				   normal[0], normal[1], normal[2], type);
+	return type;
 }
 
 #define	DISTEPSILON		0.01
 #define	ANGLEEPSILON	0.00001
 
-static void
+int
 NormalizePlane (plane_t *dp)
 {
-	vec_t	ax, ay, az;
+	vec_t       ax, ay, az;
+	int         flip = 0;
 
+	// Make axis aligned planes point to +inf.
 	if (dp->normal[0] == -1.0) {
 		dp->normal[0] = 1.0;
 		dp->dist = -dp->dist;
+		flip = 1;
 	} else if (dp->normal[1] == -1.0) {
 		dp->normal[1] = 1.0;
 		dp->dist = -dp->dist;
+		flip = 1;
 	} else if (dp->normal[2] == -1.0) {
 		dp->normal[2] = 1.0;
 		dp->dist = -dp->dist;
+		flip = 1;
 	}
 
+	// For axis aligned planes, set the plane type and ensure the normal
+	// vector is mathematically correct.
 	if (dp->normal[0] == 1.0) {
 		dp->type = PLANE_X;
 		dp->normal[1] = dp->normal[2] = 0.0;
-		return;
+		return flip;
 	}
 	if (dp->normal[1] == 1.0) {
 		dp->type = PLANE_Y;
 		dp->normal[0] = dp->normal[2] = 0.0;
-		return;
+		return flip;
 	}
 	if (dp->normal[2] == 1.0) {
 		dp->type = PLANE_Z;
 		dp->normal[0] = dp->normal[1] = 0.0;
-		return;
+		return flip;
 	}
 
+	// Find out with which axis the plane is most aligned.
 	ax = fabs (dp->normal[0]);
 	ay = fabs (dp->normal[1]);
 	az = fabs (dp->normal[2]);
 
-	if (ax >= ay && ax >= az)
-		dp->type = PLANE_ANYX;
+	if (az >= ax && az >= ay)
+		dp->type = PLANE_ANYZ;
 	else if (ay >= ax && ay >= az)
 		dp->type = PLANE_ANYY;
 	else
-		dp->type = PLANE_ANYZ;
+		dp->type = PLANE_ANYX;
+	// Make the plane's normal point towards +inf along its primary axis.
 	if (dp->normal[dp->type - PLANE_ANYX] < 0) {
 		VectorNegate (dp->normal, dp->normal);
 		dp->dist = -dp->dist;
+		flip = 1;
 	}
+	return flip;
 }
 
-/*
-	FindPlane
-
-	Returns a global plane number and the side that will be the front
-*/
 int
-FindPlane (plane_t *dplane, int *side)
+FindPlane (const plane_t *dplane, int *side)
 {
 	int		 i;
 	plane_t	*dp, pl;
@@ -231,10 +274,8 @@ FindPlane (plane_t *dplane, int *side)
 
 	dp = planes;
 	for (i = 0; i < numbrushplanes; i++, dp++) {
-		vec3_t t;
-		VectorSubtract (dp->normal, pl.normal, t);
 		dot = DotProduct (dp->normal, pl.normal);
-		if (dot > 1.0 - ANGLEEPSILON 
+		if (dot > 1.0 - ANGLEEPSILON
 			&& fabs(dp->dist - pl.dist) < DISTEPSILON) {	// regular match
 			return i;
 		}
@@ -257,6 +298,11 @@ FindPlane (plane_t *dplane, int *side)
 vec3_t      brush_mins, brush_maxs;
 face_t     *brush_faces;
 
+/**	Find the entity with the matching target name.
+
+	\param targetname The target name for which to search.
+	\return			The matching entity or NULL if not found.
+*/
 static entity_t *
 FindTargetEntity (const char *targetname)
 {
@@ -270,6 +316,9 @@ FindTargetEntity (const char *targetname)
 }
 
 #define	ZERO_EPSILON	0.001
+
+/**	Create the faces of the active brush.
+*/
 static void
 CreateBrushFaces (void)
 {
@@ -281,7 +330,7 @@ CreateBrushFaces (void)
 	winding_t  *w;
 	vec3_t      offset, point;
 
-	offset[0] = offset[1] = offset[2] = 0;
+	VectorZero (offset);
 	brush_mins[0] = brush_mins[1] = brush_mins[2] = BOGUS_RANGE;
 	brush_maxs[0] = brush_maxs[1] = brush_maxs[2] = -BOGUS_RANGE;
 
@@ -291,16 +340,14 @@ CreateBrushFaces (void)
 	if (rotate) {
 		entity_t   *FoundEntity;
 		const char *searchstring;
-		char       text[20];
 
 		searchstring = ValueForKey (CurrentEntity, "target");
 		FoundEntity = FindTargetEntity (searchstring);
 		if (FoundEntity)
 			GetVectorForKey (FoundEntity, "origin", offset);
 
-		snprintf (text, sizeof (text), "%g %g %g",
-				  offset[0], offset[1], offset[2]);
-		SetKeyValue (CurrentEntity, "origin", text);
+		SetKeyValue (CurrentEntity, "origin",
+					 va ("%g %g %g", VectorExpand (offset)));
 	}
 
 	for (i = 0; i < numbrushfaces; i++) {
@@ -365,9 +412,9 @@ CreateBrushFaces (void)
 			min = brush_mins[2];
 
 		max = brush_maxs[0];
-		if (max > brush_maxs[1])
+		if (max < brush_maxs[1])
 			max = brush_maxs[1];
-		if (max > brush_maxs[2])
+		if (max < brush_maxs[2])
 			max = brush_maxs[2];
 
 		delta = fabs(max);
@@ -394,8 +441,8 @@ vec3_t      hull_size[3][2] = {
 	{{-32, -32, -64}, {32, 32, 24}}
 };
 
-#define	MAX_HULL_POINTS	64
-#define	MAX_HULL_EDGES	128
+#define	MAX_HULL_POINTS	256
+#define	MAX_HULL_EDGES	MAX_HULL_POINTS * 2
 
 int         num_hull_points;
 vec3_t      hull_points[MAX_HULL_POINTS];
@@ -404,7 +451,7 @@ int         num_hull_edges;
 int         hull_edges[MAX_HULL_EDGES][2];
 
 static void
-AddBrushPlane (plane_t *plane)
+AddBrushPlane (const plane_t *plane)
 {
 	float       l;
 	int         i;
@@ -434,7 +481,7 @@ AddBrushPlane (plane_t *plane)
 	vertexes can be put on the front side
 */
 static void
-TestAddPlane (plane_t *plane)
+TestAddPlane (const plane_t *plane)
 {
 	int         c, i;
 	int         counts[3];
@@ -490,7 +537,7 @@ TestAddPlane (plane_t *plane)
 	Doesn't add if duplicated
 */
 static int
-AddHullPoint (vec3_t p, int hullnum)
+AddHullPoint (const vec3_t p, int hullnum)
 {
 	int			i, x, y, z;
 	vec_t	   *c;
@@ -526,7 +573,7 @@ AddHullPoint (vec3_t p, int hullnum)
 	Creates all of the hull planes around the given edge, if not done already
 */
 static void
-AddHullEdge (vec3_t p1, vec3_t p2, int hullnum)
+AddHullEdge (const vec3_t p1, const vec3_t p2, int hullnum)
 {
 	int         pt1, pt2, a, b, c, d, e, i;
 	plane_t     plane;
@@ -628,13 +675,13 @@ ExpandBrush (int hullnum)
 
 	Converts a mapbrush to a bsp brush
 */
-static brush_t    *
-LoadBrush (mbrush_t *mb, int hullnum)
+static brush_t *
+LoadBrush (const mbrush_t *mb, int hullnum)
 {
 	brush_t    *b;
-	char       *name;
+	const char *name;
 	int         contents;
-	mface_t    *f;
+	const mface_t *f;
 
 	// check texture name for attributes
 	if (mb->faces->texinfo < 0) {
@@ -643,7 +690,7 @@ LoadBrush (mbrush_t *mb, int hullnum)
 			return NULL;
 		contents = CONTENTS_EMPTY;
 	} else {
-		name = miptex[bsp->texinfo[mb->faces->texinfo].miptex];
+		name = miptexnames[bsp->texinfo[mb->faces->texinfo].miptex];
 
 		if (!strcasecmp (name, "clip") && hullnum == 0)
 			return NULL;		// "clip" brushes don't show up in the draw hull
@@ -705,7 +752,7 @@ LoadBrush (mbrush_t *mb, int hullnum)
 }
 
 static void
-Brush_DrawAll (brushset_t *bs)
+Brush_DrawAll (const brushset_t *bs)
 {
 	brush_t    *b;
 	face_t     *f;
@@ -769,3 +816,5 @@ Brush_LoadEntity (entity_t *ent, int hullnum)
 
 	return bset;
 }
+
+//@}
