@@ -21,10 +21,11 @@
 # include "config.h"
 #endif
 
-static __attribute__ ((unused)) const char rcsid[] =
-	"$Id: nodraw.c,v 1.6 2003/01/15 15:31:34 taniwha Exp $";
+#include "draw.h"
 
-#include "bsp5.h"
+/**	\addtogroup qfbsp_draw
+*/
+//@{
 
 
 void
@@ -33,12 +34,12 @@ Draw_ClearBounds (void)
 }
 
 void
-Draw_AddToBounds (vec3_t v)
+Draw_AddToBounds (const vec3_t v)
 {
 }
 
 void
-Draw_DrawFace (face_t *f)
+Draw_DrawFace (const struct visfacet_s *f)
 {
 }
 
@@ -63,31 +64,33 @@ Draw_SetBlack (void)
 }
 
 void
-DrawPoint (vec3_t v)
+DrawPoint (const vec3_t v)
 {
 }
 
 void
-DrawLeaf (node_t *l, int color)
+DrawLeaf (const struct node_s *l, int color)
 {
 }
 
 void
-DrawBrush (brush_t *b)
+DrawBrush (const struct brush_s *b)
 {
 }
 
 void
-DrawWinding (winding_t *w)
+DrawWinding (const struct winding_s *w)
 {
 }
 
 void
-DrawTri (vec3_t p1, vec3_t p2, vec3_t p3)
+DrawTri (const vec3_t p1, const vec3_t p2, const vec3_t p3)
 {
 }
 
 void
-DrawPortal (portal_t *portal)
+DrawPortal (const struct portal_s *portal)
 {
 }
+
+//@}
