@@ -26,19 +26,18 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: idstuff.h,v 1.4 2003/01/06 18:28:11 taniwha Exp $
 */
 
 #ifndef __idstuff_h
 #define __idstuff_h
 
-struct def_s;
+#include "QF/pr_comp.h"
 
 //XXX eww :/
-void PrecacheSound (struct def_s *e, int ch);
-void PrecacheModel (struct def_s *e, int ch);
-void PrecacheFile (struct def_s *e, int ch);
+void PrecacheSound (const char *, int ch);
+void PrecacheModel (const char *, int ch);
+void PrecacheFile (const char *, int ch);
 int  WriteFiles (const char *sourcedir);
-int  WriteProgdefs (const char *filename);
+int  WriteProgdefs (dprograms_t *progs, const char *filename);
 
 #endif//__idstuff_h
