@@ -26,13 +26,20 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: linker.h,v 1.6 2002/11/01 18:05:12 taniwha Exp $
 */
 
 #ifndef __linker_h
 #define __linker_h
 
+struct qfo_s;
+struct type_s;
+
 void linker_begin (void);
+int linker_add_string (const char *str);
+void linker_add_def (const char *name, struct type_s *type, unsigned flags,
+					 void *val);
+struct qfo_def_s *linker_find_def (const char *name);
+int linker_add_qfo (struct qfo_s *qfo);
 int linker_add_object_file (const char *filename);
 int linker_add_lib (const char *libname);
 void linker_add_path (const char *path);
