@@ -23,7 +23,6 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: options.h,v 1.2 2002/09/21 21:27:38 taniwha Exp $
 */
 
 #ifndef __options_h
@@ -36,7 +35,7 @@ typedef struct {
 	int         threads;
 	qboolean    minimal;
 	int         level;
-	char       *bspfile;
+	struct dstring_s *bspfile;
 } options_t;
 
 extern options_t options;
