@@ -1,4 +1,4 @@
-/* 
+/*
 	wad.h
 
 	wadfile tool (definitions)
@@ -25,7 +25,6 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: wad.h,v 1.1 2004/01/10 08:05:25 taniwha Exp $
 */
 
 #ifndef __wad_h
@@ -49,7 +48,23 @@ typedef struct {
 	int			verbosity;		// 0=silent
 	qboolean	compress;		// for the future
 	qboolean	pad;			// pad area of files to 4-byte boundary
+	qboolean	nomip;			// exclude mipmaps from output textures.
 	char		*wadfile;		// wad file to read/write/test
 } options_t;
 
+extern struct tex_s *image;
+extern byte default_palette[];
+extern byte *lumpbuffer, *lump_p;
+extern struct dstring_s *lumpname;
+
+struct script_s;
+void GrabPalette (struct script_s *script);
+void GrabColormap (struct script_s *script);
+void GrabPic (struct script_s *script);
+void GrabMip (struct script_s *script);
+void GrabRaw (struct script_s *script);
+void GrabColormap2 (struct script_s *script);
+
+void process_wad_script (const char *name);
+
 #endif	// __wad_h
